c Last modification RG on 22 April 2016
c------------------------------------------------------------------------------c
c   This subroutine invokes all necessary  subroutines and functions           c
c   and returns the renormalized self-energy at the order alpha_s alpha_t      c
c   calculated in the gaugeless limit with vanishing external momentum         c
c------------------------------------------------------------------------------c
      
      subroutine SelfCollect2loop(renfin)
      
      implicit none
      
      double complex renfin(5,5)
      
      double precision Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
     
      double precision me,mmu,mtau,mu,md,ms,mc,mt,mb,Ye(3),Yd(3),Yu(3)
      
      double precision ZH5(5,5),ZH(6,6)
     
      double complex UM(2,2),UP(2,2),NN(5,5),ZE(6,6),ZD(6,6),ZU(6,6)
      
      double complex nZU(6,6)
      
      double precision nMSu(6)
      
      double precision absM3,phiM3
      
      double complex g2thh(5),g2tha(5),g2uh(2,2,5),g2u2h(2,5,2,5),
     &               g4u(2,2,2,2),gLtFgSu(2),gRtFgSu(2),g4dL,
     &               g2u2dL(2,2),gLbFgdL
      
      double complex part1(5,5),part2(5,5),part3(5,5),part4(5,5),
     &               Eself2loop(5,5),CTdiafin
     
      double precision dthu,dthd,dths,dtad,dtas,dMHpm2,dv,dM(5,5),
     &                 dMrot(5,5)
      
      double complex dmtfin,dmtdiv1,dYtfin,dYtdiv1
      double complex dMstfin(2,2),dMstdiv1(2,2),
     &               dAtfin,dAtdiv1,dphiAtfin,dphiAtdiv1
      
      integer i,j,k,l
      
      common /SMinput/ Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      common /Yukawa/ me,mmu,mtau,mu,md,ms,mc,mt,mb,Ye,Yd,Yu
      common /rot/ ZH5,ZH,UM,UP,NN,ZE,ZD,ZU
      common /nSu/ nMSu,nZU
      common /gluino/ absM3,phiM3
      common/qcdcoup/ g2thh,g2tha,g2uh,g2u2h,g4u,gLtFgSu,gRtFgSu,g4dL,
     &                 g2u2dL,gLbFgdL
      common/qcdcount/ dmtfin,dmtdiv1,dMstfin,dMstdiv1,dYtfin,dYtdiv1,
     &                 dAtfin,dAtdiv1,dphiAtfin,dphiAtdiv1
     
c calculatiing the higgs, qcd-couplings in the gaugeless limit
      call qcdcouplings
      
c calculate all the genuine two-loop diagrams for the neutral Higgs bosons
      call HiggsSelf2loopGluonTop(g3,mt,g2tha,g2thh,part1)
      
      call HiggsSelf2loopGluonStop(g3,nMSu,g2uh,g2u2h,part2)

      call HiggsSelf2loopOnlyStops(nMSu,g2uh,g2u2h,g4u,part3)
     
      call HiggsSelf2loopGluino(mt,nMSu,absM3,g2tha,g2thh,
     .                      g2u2h,g2uh,gLtFgSu,gRtFgSu,part4)

      Eself2loop=part1+part2+part3+part4
      
c calculate counterterms of the top/stop sector
      call topsector(dmtfin,dmtdiv1,dYtfin,dYtdiv1)
      call stopsector(dmtfin,dmtdiv1,dMstfin,dMstdiv1,
     &                      dAtfin,dAtdiv1,dphiAtfin,dphiAtdiv1)
     
c calculate counterterm couplings
      call dqcdcouplings
    
c calcluate all two-loop OS counterterms 
      call Calc2loopCounter(dMHpm2,dthu,dthd,dths,dtad,dtas,dv)

c calcualte two-loop counterterm mass matrix
      call counterM2loop(dMHpm2,dthu,dthd,dths,dtad,dtas,dv,dM)
      
      dMrot=0d0
      do i=1,5
      do j=1,5
      do k=1,5
      do l=1,5
      dMrot(i,j)=dMrot(i,j)+ZH5(i,k)*(dM(k,l))*ZH5(j,l)
      end do
      end do
      end do
      end do
      
c build two-loop renormalized self energy
      do i=1,5
      do j=1,5
      renfin(i,j)=CTdiafin(i,j)+Eself2loop(i,j)-dMrot(i,j)
      end do
      end do

      return
      end 
      
      
c---------------------------------------------------------------------------------------------c   
      
c all contributions from only tops and gluon in the loops (diagrams se3+se6)
      subroutine HiggsSelf2loopGluonTop(g3,mt,g2tha,g2thh,self)
      
      implicit none
      
      double precision g3,mt
      double complex g2tha(5),g2thh(5)
      double complex self(5,5)
     
      double precision Qs
      double precision lnmt
      integer i,j
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      common /scale/ Qs
      
      lnmt=Log(mt**2/Qs**2)
      
      do i=1,5
      do j=1,5
      self(i,j)=-(g3**2*mt**2*
     -     ((18+4*lnmt*(-5+3*lnmt)+Pi**2)*g2tha(i)*g2tha(j)-
     -      (14+12*lnmt*(-1+3*lnmt)+3*Pi**2)*g2thh(i)*g2thh(j)))
     -     /(16.*Pi**4)
      end do
      end do
     
      return
      end
      
      
 
c all contributions from only stops and gluon in the loops (se3b+se7+se11)
      subroutine HiggsSelf2loopGluonStop(g3,nMSu,g2uh,g2u2h,self)
      
      implicit none
      
      double precision g3,nMSu(6)
      double complex g2uh(2,2,5),g2u2h(2,5,2,5)
      double complex self(5,5)
      
      double precision F1,F2
      double precision f1MSu5,f1MSu6,f2MSu5,f2MSu6
      integer i,j
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      f1Msu5=F1(nMSu(5)**2)
      f1Msu6=F1(nMSu(6)**2)
      f2Msu5=F2(nMSu(5)**2)
      f2Msu6=F2(nMSu(6)**2)
      
      do i=1,5
      do j=1,5
      self(i,j)=(g3**2*((0,1)*(f1MSu5*g2u2h(1,i,1,j) +
     -          f1MSu6*g2u2h(2,i,2,j))-f2MSu5*
     -          g2uh(1,1,i)*g2uh(1,1,j)-f2MSu6*
     -          g2uh(2,2,i)*g2uh(2,2,j)+((-f1MSu5+
     -          f1MSu6)*(g2uh(1,2,j)*g2uh(2,1,i)+
     -          g2uh(1,2,i)*g2uh(2,1,j)))/
     -          (nMSu(5)**2-nMSu(6)**2)))/(128.*Pi**4)
      end do
      end do

      
      return
      end
      
      
      subroutine HiggsSelf2loopOnlyStops(nMSu,g2uh,g2u2h,g4u,self)
      
      implicit none
      
      double precision nMSu(6)
      double complex g2uh(2,2,5),g2u2h(2,5,2,5),g4u(2,2,2,2)
      double complex self(5,5)
      
      double precision F3,F4,F5,F6,F7,F8,F9,F10
      double precision f355,f356,f365,f366,f455,f456,f465,f466,
     .                 f565,f556,f655,f656,f665,f666,f756,f765,
     .                 f856,f865,f956,f1055,f1056,f1066
      double complex top1(5,5),top2(5,5),top3(5,5)
      
      integer i,j
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      f355=F3(nMSu(5)**2,nMSu(5)**2)
      f356=F3(nMSu(5)**2,nMSu(6)**2)
      f365=F3(nMSu(6)**2,nMSu(5)**2)
      f366=F3(nMSu(6)**2,nMSu(6)**2)
      
      f455=F4(nMSu(5)**2,nMSu(5)**2)
      f456=F4(nMSu(5)**2,nMSu(6)**2)
      f465=F4(nMSu(6)**2,nMSu(5)**2)
      f466=F4(nMSu(6)**2,nMSu(6)**2)
      
      f556=F5(nMSu(5)**2,nMSu(6)**2)
      f565=F5(nMSu(6)**2,nMSu(5)**2)
      
      f655=F6(nMSu(5)**2,nMSu(5)**2)
      f656=F6(nMSu(5)**2,nMSu(6)**2)
      f665=F6(nMSu(6)**2,nMSu(5)**2)
      f666=F6(nMSu(6)**2,nMSu(6)**2)
      
      f756=F7(nMSu(5)**2,nMSu(6)**2)
      f765=F7(nMSu(6)**2,nMSu(5)**2)
      
      f856=F8(nMSu(5)**2,nMSu(6)**2)
      f865=F8(nMSu(6)**2,nMSu(5)**2)
      
      f956=F9(nMSu(5)**2,nMSu(6)**2)
      
      f1055=F10(nMSu(5)**2,nMSu(5)**2)
      f1056=F10(nMSu(5)**2,nMSu(6)**2)
      f1066=F10(nMSu(6)**2,nMSu(6)**2)
      
      
      
      do i=1,5
      do j=1,5
      top1(i,j)=((0,1)*3/512*
     -      (4*f355*g2uh(1,1,i)*g2uh(1,1,j)*g4u(1,1,1,1)+
     -      f365*g2uh(1,1,i)*g2uh(1,1,j)*(3*g4u(1,2,1,2)+g4u(1,2,2,1))+
     -      f356*g2uh(2,2,i)*g2uh(2,2,j)*(3*g4u(1,2,1,2)+g4u(2,1,1,2))+
     -      4*f366*g2uh(2,2,i)*g2uh(2,2,j)*g4u(2,2,2,2)+
     -      (4*f456*g2uh(2,1,i)*g2uh(2,2,j)*g4u(1,1,1,2)+
     -      4*f565*g2uh(1,1,j)*g2uh(1,2,i)*g4u(1,2,1,1)+
     -      3*f456*g2uh(1,2,i)*g2uh(2,1,j)*g4u(1,2,1,2)+
     -      4*f556*g2uh(2,1,i)*g2uh(2,2,j)*g4u(1,2,2,2)+
     -      4*g2uh(1,1,i)*g2uh(2,1,j)*
     -      (f565*g4u(1,1,1,2)+f465*g4u(1,2,2,2))+
     -      f456*g2uh(1,2,i)*g2uh(2,1,j)*g4u(2,1,1,2)+
     -      4*f465*g2uh(1,1,j)*g2uh(1,2,i)*g4u(2,2,1,2)+
     -      g2uh(1,2,j)*(g2uh(2,1,i)*(4*f565*g4u(1,1,1,1)+
     -      3*f465*g4u(1,2,1,2)+f465*g4u(1,2,2,1))+
     -      4*g2uh(2,2,i)*(f456*g4u(1,2,1,1)+f556*g4u(2,2,1,2)))+
     -      4*f556*g2uh(1,2,i)*g2uh(2,1,j)*g4u(2,2,2,2))/
     -      (nMSu(5)**2-nMSu(6)**2)**2))/Pi**4
     
      top2(i,j)=(g2u2h(2,i,1,j)*(-4*f765*g4u(1,1,1,2)+
     -      4*f756*g4u(1,2,2,2))+g2u2h(1,i,2,j)*
     -      (-4*f765*g4u(1,2,1,1)+4*f756*g4u(2,2,1,2))+
     -      (g2u2h(1,i,1,j)*(4*f655*g4u(1,1,1,1)+
     -      3*f665*g4u(1,2,1,2)+f665*g4u(1,2,2,1))+
     -      g2u2h(2,i,2,j)*(3*f656*g4u(1,2,1,2)+f656*g4u(2,1,1,2)+
     -      4*f666*g4u(2,2,2,2)))*(nMSu(5)**2-nMSu(6)**2))/
     -      (512.*Pi**4*(nMSu(5)**2-nMSu(6)**2))
     
      top3(i,j)=((0,1)/512*(f956*(g2uh(2,1,i)*
     -      (4*g2uh(2,1,j)*g4u(1,1,2,2)+
     -      g2uh(1,2,j)*(g4u(1,2,1,2)+3*g4u(1,2,2,1)))+
     -      g2uh(1,2,i)*(g2uh(2,1,j)*(g4u(1,2,1,2)+3*g4u(2,1,1,2))+
     -      4*g2uh(1,2,j)*g4u(2,2,1,1)))+(nMSu(5)**2-nMSu(6)**2)*
     -      (4*f856*((g2uh(1,1,j)*g2uh(2,1,i)+g2uh(1,1,i)*g2uh(2,1,j))*
     -      g4u(1,1,1,2)+(g2uh(1,1,j)*g2uh(1,2,i)+
     -      g2uh(1,1,i)*g2uh(1,2,j))*g4u(1,2,1,1))-
     -      4*f865*(g2uh(2,1,j)*g2uh(2,2,i)+g2uh(2,1,i)*g2uh(2,2,j))*
     -      g4u(1,2,2,2) -4*f865*(g2uh(1,2,j)*g2uh(2,2,i)+g2uh(1,2,i)*
     -      g2uh(2,2,j))*g4u(2,2,1,2)+(g2uh(1,1,i)*
     -      (4*f1055*g2uh(1,1,j)*g4u(1,1,1,1)+
     -      f1056*g2uh(2,2,j)*(3*g4u(1,2,1,2)+g4u(1,2,2,1)))+
     -      g2uh(2,2,i)*(f1056*g2uh(1,1,j)*(3*g4u(1,2,1,2)+
     -      g4u(2,1,1,2))+4*f1066*g2uh(2,2,j)*g4u(2,2,2,2)))*
     -      (nMSu(5)**2-nMSu(6)**2))))/
     -      (Pi**4*(nMSu(5)**2-nMSu(6)**2)**2)
     
      end do
      end do
     
      do i=1,5
      do j=1,5
      self(i,j)=top1(i,j)+top1(j,i)+top2(i,j)+top3(i,j)
      end do
      end do
      
      return
      end
      
      
      
      subroutine HiggsSelf2loopGluino(mt,nMSu,absM3,g2tha,g2thh,
     .                       g2u2h,g2uh,gLtFgSu,gRtFgSu,self)
      
      implicit none
      
      double precision mt,nMSu(6),absM3
      double complex g2tha(5),g2thh(5),g2u2h(2,5,2,5),
     &               gLtFgSu(2),gRtFgSu(2),g2uh(2,2,5)
      double complex self(5,5)
      
      double complex top1(5,5),top2(5,5),top3(5,5),top4(5,5)
      double complex TFI1,TFI2,TFI3,TFI4,TFI5,TFI6,TFI7,TFI8,
     &               TFI9,TFI10,TFI11,TFI12,TFI13,TFI14,TFI15
      double complex TFI1num,TFI2num,TFI3num,TFI4num,TFI5a,
     &               TFI5b,TFI6a,TFI6b,TFI7a,TFI7b,TFI11ab,TFI10a,
     &               TFI10b,TFI11abm12ab,TFI12ab,TFI6ab,TFI6abmTFI11ab,
     &               TFI8a,TFI8b,TFI8m9a,TFI8m9b,TFI6abpTFI11ab,
     &               TFI8p9a,TFI8p9b,TFI910a,TFI910b,TFI9a,TFI9b,TFI1a,
     &               TFI1b,TFI12a,TFI12b,TFI13a,TFI13b,TFI14a,TFI14b,
     &               TFI15a,TFI15b,TFI132a,TFI132b,TFIcom1,TFIcom1b,
     &               TFIcom2,TFIcom2b,TFI2a,TFI2b
      double complex Ifin
      double complex Ifina,Ifinb
      
      integer i,j,ind3
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
c both Higgs coupling to tops and one stop Gluino loop
      top1=0d0
      do ind3=1,2
      
      TFI1num=TFI1(mt**2,absM3**2,nMSu(4+ind3)**2)
      TFI2num=TFI2(mt**2,absM3**2,nMSu(4+ind3)**2)
      TFI3num=TFI3(mt**2,absM3**2,nMSu(4+ind3)**2)
      TFI4num=TFI4(mt**2,absM3**2,nMSu(4+ind3)**2)
      
      
      do i=1,5
      do j=1,5
      top1(i,j)=top1(i,j)+
     -      (Abs(gLtFgSu(ind3))**2*
     -      ((mt**2*TFI2num - TFI4num)*g2tha(i)*g2tha(j)+
     -      (3*mt**2*TFI2num+TFI4num)*g2thh(i)*g2thh(j))+
     -      Abs(gRtFgSu(ind3))**2*
     -      ((mt**2*TFI2num - TFI4num)*g2tha(i)*g2tha(j)+
     -      (3*mt**2*TFI2num+TFI4num)*g2thh(i)*g2thh(j))+
     -      absM3*mt*(CONJG(gRtFgSu(ind3))*
     -      ((mt**2*TFI1num - TFI3num)*
     -      g2tha(i)*(g2tha(j)+g2thh(j))+
     -      g2thh(i)*(mt**2*TFI1num*g2tha(j) - TFI3num*g2tha(j)+
     -      mt**2*TFI1num*g2thh(j)+3*TFI3num*g2thh(j)))*gLtFgSu(ind3)+
     -      CONJG(gLtFgSu(ind3))*((mt**2*TFI1num - TFI3num)*g2tha(i)*
     -      (g2tha(j) - g2thh(j))+g2thh(i)*
     -      ((-(mt**2*TFI1num)+TFI3num)*g2tha(j)+
     -      (mt**2*TFI1num+3*TFI3num)*g2thh(j)))*gRtFgSu(ind3)))/
     -      (4.*Pi**8)
      end do
      end do
      end do
      

c Higgs-Higgs-stop-stop coupling+Gluino-top loop
      Ifina=Ifin(mt**2,absM3**2,nMSu(5)**2)
      Ifinb=Ifin(mt**2,absM3**2,nMSu(6)**2)
      TFI5a=TFI5(mt**2,absM3**2,nMSu(5)**2)
      TFI5b=TFI5(mt**2,absM3**2,nMSu(6)**2)
      TFI6a=TFI6(mt**2,absM3**2,nMSu(5)**2)
      TFI6b=TFI6(mt**2,absM3**2,nMSu(6)**2)
      TFI7a=TFI7(mt**2,absM3**2,nMSu(5)**2)
      TFI7b=TFI7(mt**2,absM3**2,nMSu(6)**2)
      
      top2=0d0
      do i=1,5
      do j=1,5
      top2(i,j)=top2(i,j)+
     -      ((0,-0.125)*(CONJG(gLtFgSu(2))*g2u2h(2,i,1,j)*
     -      ((TFI5a - TFI5b)*gLtFgSu(1)+
     -      absM3*(Ifina - Ifinb)*mt*gRtFgSu(1))+
     -      CONJG(gRtFgSu(2))*g2u2h(2,i,1,j)*
     -      (absM3*(Ifina - Ifinb)*mt*gLtFgSu(1)+
     -      (TFI5a - TFI5b)*gRtFgSu(1))+
     -      CONJG(gLtFgSu(1))*g2u2h(1,i,2,j)*
     -      ((TFI5a - TFI5b)*gLtFgSu(2)+
     -      absM3*(Ifina - Ifinb)*mt*gRtFgSu(2))+
     -      CONJG(gRtFgSu(1))*g2u2h(1,i,2,j)*
     -      (absM3*(Ifina - Ifinb)*mt*gLtFgSu(2)+
     -      (TFI5a - TFI5b)*gRtFgSu(2))+g2u2h(1,i,1,j)*
     -      (TFI7a*(Abs(gLtFgSu(1))**2+Abs(gRtFgSu(1))**2)+
     -      absM3*mt*TFI6a*(CONJG(gRtFgSu(1))*gLtFgSu(1)+
     -      CONJG(gLtFgSu(1))*gRtFgSu(1)))*(nMSu(5)**2 - nMSu(6)**2)+
     -      g2u2h(2,i,2,j)*(TFI7b*
     -      (Abs(gLtFgSu(2))**2+Abs(gRtFgSu(2))**2)+
     -      absM3*mt*TFI6b*(CONJG(gRtFgSu(2))*gLtFgSu(2)+
     -      CONJG(gLtFgSu(2))*gRtFgSu(2)))*
     -      (nMSu(5)**2 - nMSu(6)**2)))/
     -      (Pi**8*(nMSu(5)**2 - nMSu(6)**2))
      end do
      end do
      
      
      TFI6ab=TFI6(absM3**2,nMSu(5)**2,mt**2)-
     .       TFI6(absM3**2,nMSu(6)**2,mt**2)
      TFI11ab=TFI11(mt**2,nMSu(5)**2,absM3**2)-
     .        TFI11(mt**2,nMSu(6)**2,absM3**2)
      TFI6abpTFI11ab=mt**2*TFI6ab+TFI11ab
      TFI6abmTFI11ab=mt**2*TFI6ab-TFI11ab
      
      TFI8a=TFI8(mt**2,nMSu(5)**2,absM3**2)
      TFI8b=TFI8(mt**2,nMSu(6)**2,absM3**2)
      TFI9a=TFI9(mt**2,nMSu(5)**2,absM3**2)
      TFI9b=TFI9(mt**2,nMSu(6)**2,absM3**2)
      TFI10a=TFI10(mt**2,nMSu(5)**2,absM3**2)
      TFI10b=TFI10(mt**2,nMSu(6)**2,absM3**2)
      TFI8p9a=mt**2*TFI8a+TFI9a
      TFI8p9b=mt**2*TFI8b+TFI9b
      TFI8m9a=mt**2*TFI8a-TFI9a
      TFI8m9b=mt**2*TFI8b-TFI9b
      TFI910a=TFI9a-TFI10a
      TFI910b=TFI9b-TFI10b
      TFI12ab=TFI12(mt**2,nMSu(5)**2,absM3**2)-
     .        TFI12(mt**2,nMSu(6)**2,absM3**2)
      TFI11abm12ab=TFI11ab-TFI12ab
      
      top3=0d0
      do i=1,5
      do j=1,5
      top3(i,j)=top3(i,j)+
     -      (g2uh(1,1,j)*(absM3*TFI8m9a*CONJG(gRtFgSu(1))*g2tha(i)*
     -      gLtFgSu(1)+g2thh(i)*(2*mt*TFI910a*(Abs(gLtFgSu(1))**2 +
     -      Abs(gRtFgSu(1))**2) +absM3*TFI8p9a*CONJG(gRtFgSu(1))*
     -      gLtFgSu(1))+absM3*CONJG(gLtFgSu(1))*
     -      (-(TFI8m9a*g2tha(i))+TFI8p9a*g2thh(i))*gRtFgSu(1)) +
     -      g2uh(2,2,j)*(absM3*TFI8m9b*CONJG(gRtFgSu(2))*g2tha(i)*
     -      gLtFgSu(2)+g2thh(i)*(2*mt*TFI910b*(Abs(gLtFgSu(2))**2 +
     -      Abs(gRtFgSu(2))**2)+absM3*TFI8p9b*CONJG(gRtFgSu(2))*
     -      gLtFgSu(2))+absM3*CONJG(gLtFgSu(2))*
     -      (-(TFI8m9b*g2tha(i))+TFI8p9b*g2thh(i))*gRtFgSu(2))+
     -      (CONJG(gRtFgSu(2))*g2uh(2,1,j)*(absM3*(TFI6abmTFI11ab*
     -      g2tha(i)+TFI6abpTFI11ab*g2thh(i))*gLtFgSu(1)+
     -      2*mt*TFI11abm12ab*g2thh(i)*gRtFgSu(1)) +CONJG(gLtFgSu(2))*
     -      g2uh(2,1,j)*(2*mt*TFI11abm12ab*g2thh(i)*gLtFgSu(1)+
     -      absM3*(-(TFI6abmTFI11ab*g2tha(i))+TFI6abpTFI11ab*
     -      g2thh(i))*gRtFgSu(1))+g2uh(1,2,j)*(CONJG(gRtFgSu(1))*
     -      (absM3*(TFI6abmTFI11ab*g2tha(i)+TFI6abpTFI11ab*g2thh(i))*
     -      gLtFgSu(2)+2*mt*TFI11abm12ab*g2thh(i)*gRtFgSu(2))+
     -      CONJG(gLtFgSu(1))*(2*mt*TFI11abm12ab*g2thh(i)*gLtFgSu(2)+
     -      absM3*(-(TFI6abmTFI11ab*g2tha(i))+TFI6abpTFI11ab*
     -      g2thh(i))*gRtFgSu(2))))
     -      /(nMSu(5)**2 - nMSu(6)**2))/(8.*Pi**8)
      end do
      end do
      
      
      TFI1a=TFI1(nMSu(5)**2,mt**2,absM3**2)
      TFI1b=TFI1(nMSu(6)**2,mt**2,absM3**2)
      TFI2a=TFI2(nMSu(5)**2,mt**2,absM3**2)
      TFI2b=TFI2(nMSu(6)**2,mt**2,absM3**2)
      TFI5a=TFI5(nMSu(5)**2,mt**2,absM3**2)
      TFI5b=TFI5(nMSu(6)**2,mt**2,absM3**2)
      TFI6a=TFI6(absM3**2,mt**2,nMSu(5)**2)
      TFI6b=TFI6(absM3**2,mt**2,nMSu(6)**2)
      TFI12a=TFI12(nMSu(5)**2,mt**2,absM3**2)
      TFI12b=TFI12(nMSu(6)**2,mt**2,absM3**2)
      TFI13a=TFI13(nMSu(5)**2,mt**2,absM3**2)
      TFI13b=TFI13(nMSu(6)**2,mt**2,absM3**2)
      TFI14a=TFI14(nMSu(5)**2,mt**2,absM3**2)
      TFI14b=TFI14(nMSu(6)**2,mt**2,absM3**2)
      TFI15a=TFI15(nMSu(5)**2,mt**2,absM3**2)
      TFI15b=TFI15(nMSu(6)**2,mt**2,absM3**2)
      TFIcom1=(nMSu(5)**2-nMSu(6)**2)*(TFI12a-TFI15a)+TFI14a-TFI14b-
     &        TFI5a+TFI5b
      TFIcom1b=(nMSu(5)**2-nMSu(6)**2)*(TFI12b-TFI15b)+TFI14a-TFI14b-
     &        TFI5a+TFI5b
      TFIcom2=Ifina-Ifinb+(nMSu(6)**2-nMSu(5)**2)*TFI6a
      TFIcom2b=Ifina-Ifinb+(nMSu(6)**2-nMSu(5)**2)*TFI6b
      TFI132a=TFI13a-TFI2a
      TFI132b=TFI13b-TFI2b
      
      top4=0d0
      do i=1,5
      do j=1,5
      top4(i,j)=top4(i,j)+
     -      (g2uh(1,1,i)*g2uh(1,1,j)*
     -      (TFI132a*(Abs(gLtFgSu(1))**2+Abs(gRtFgSu(1))**2)+
     -      absM3*mt*TFI1a*(CONJG(gRtFgSu(1))*gLtFgSu(1) +
     -      CONJG(gLtFgSu(1))*gRtFgSu(1)))+g2uh(2,2,i)*g2uh(2,2,j)*
     -      (TFI132b*(Abs(gLtFgSu(2))**2+Abs(gRtFgSu(2))**2)+
     -      absM3*mt*TFI1b*(CONJG(gRtFgSu(2))*gLtFgSu(2)+
     -      CONJG(gLtFgSu(2))*gRtFgSu(2)))+(absM3*mt*TFIcom2*
     -      (-(g2uh(1,2,i)*g2uh(2,1,j)*(CONJG(gRtFgSu(1))*gLtFgSu(1) +
     -      CONJG(gLtFgSu(1))*gRtFgSu(1))) - g2uh(1,1,j)*g2uh(2,1,i)*
     -      (CONJG(gRtFgSu(2))*gLtFgSu(1)+CONJG(gLtFgSu(2))*
     -      gRtFgSu(1)) - g2uh(1,1,i)*g2uh(1,2,j)*(CONJG(gRtFgSu(1))*
     -      gLtFgSu(2)+CONJG(gLtFgSu(1))*gRtFgSu(2)))+absM3*mt*
     -      TFIcom2b*(g2uh(2,1,j)*g2uh(2,2,i)*(CONJG(gRtFgSu(2))*
     -      gLtFgSu(1)+CONJG(gLtFgSu(2))*gRtFgSu(1))+
     -      g2uh(1,2,i)*g2uh(2,2,j)*(CONJG(gRtFgSu(1))*gLtFgSu(2) +
     -      CONJG(gLtFgSu(1))*gRtFgSu(2))+g2uh(1,2,j)*g2uh(2,1,i)*
     -      (CONJG(gRtFgSu(2))*gLtFgSu(2)+CONJG(gLtFgSu(2))*
     -      gRtFgSu(2)))+TFIcom1*(-((Abs(gLtFgSu(1))**2+
     -      Abs(gRtFgSu(1))**2)*g2uh(1,2,i)*g2uh(2,1,j)) - g2uh(1,1,j)*
     -      g2uh(2,1,i)*(CONJG(gLtFgSu(2))*gLtFgSu(1)+
     -      CONJG(gRtFgSu(2))*gRtFgSu(1)) - g2uh(1,1,i)*g2uh(1,2,j)*
     -      (CONJG(gLtFgSu(1))*gLtFgSu(2)+CONJG(gRtFgSu(1))*
     -      gRtFgSu(2)))+TFIcom1b*((Abs(gLtFgSu(2))**2+
     -      Abs(gRtFgSu(2))**2)*g2uh(1,2,j)*g2uh(2,1,i)+g2uh(2,1,j)*
     -      g2uh(2,2,i)*(CONJG(gLtFgSu(2))*gLtFgSu(1)+
     -      CONJG(gRtFgSu(2))*gRtFgSu(1))+g2uh(1,2,i)*g2uh(2,2,j)*
     -      (CONJG(gLtFgSu(1))*gLtFgSu(2)+CONJG(gRtFgSu(1))*
     -      gRtFgSu(2))))/(nMSu(5)**2 - nMSu(6)**2)**2)/(4.*Pi**8)
      end do
      end do
      top4=(top4+CONJG(top4))/2d0
      
      do i=1,5
      do j=1,5
      self(i,j)=top1(i,j)+top2(i,j)+top3(i,j)+top3(j,i)+top4(j,i)
      end do
      end do

      return
      end
      
c -----------------------------------------------------------------------------------------c
c       calculates the finite part of the counterterm inserted diagrams                    c
c       of the order alpha_s alpha_t for the neutral Higgs bosons                          c
c------------------------------------------------------------------------------------------c
      
      
      double complex function CTdiafin(i,j)
      
      implicit none
      
      integer i,j
     
      double precision me,mmu,mtau,mu,md,ms,mc,mt,mb,Ye(3),Yd(3),Yu(3)
     
      double complex dg2thhfin(5),dg2thafin(5),dg2uhfin(2,2,5),
     &               dg2u2hfin(2,5,2,5),
     &               dg2thhdiv1(5),dg2thadiv1(5),dg2uhdiv1(2,2,5),
     &               dg2u2hdiv1(2,5,2,5)
     
      double complex g2thh(5),g2tha(5),g2uh(2,2,5),g2u2h(2,5,2,5),
     &               g4u(2,2,2,2),gLtFgSu(2),gRtFgSu(2),g4dL,
     &               g2u2dL(2,2),gLbFgdL
    
      double complex dmtfin,dmtdiv1,dMstfin(2,2),dMstdiv1(2,2),
     &               dYtfin,dYtdiv1,dAtfin,dAtdiv1,dphiAtfin,dphiAtdiv1
     
      double precision nMSu(6)
      
      double complex nZU(6,6)
      
      double precision lnmt,lnSu5,lnSu6
      
      double precision Qs
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
     
     
      common/dqcdcoup/ dg2thhfin,dg2thafin,dg2uhfin,dg2u2hfin,
     &                 dg2thhdiv1,dg2thadiv1,dg2uhdiv1,dg2u2hdiv1
      common /Yukawa/ me,mmu,mtau,mu,md,ms,mc,mt,mb,Ye,Yd,Yu
      common/qcdcoup/ g2thh,g2tha,g2uh,g2u2h,g4u,gLtFgSu,gRtFgSu,g4dL,
     &                 g2u2dL,gLbFgdL
      common/qcdcount/ dmtfin,dmtdiv1,dMstfin,dMstdiv1,dYtfin,dYtdiv1,
     &                 dAtfin,dAtdiv1,dphiAtfin,dphiAtdiv1
      common /nSu/ nMSu,nZU
      common /scale/ Qs
      
      lnmt=Log(mt**2/Qs**2)
      lnSu5=Log(nMSu(5)**2/Qs**2)
      lnSu6=Log(nMSu(6)**2/Qs**2)
      
      CTdiafin=(-192*(dmtfin - dmtdiv1*lnmt)*mt*g2thh(i)*g2thh(j) - 
     -   4*mt**2*((12-12*lnmt+6*lnmt**2+Pi**2)*(dg2thadiv1(j)*g2tha(i)+
     -   dg2thadiv1(i)*g2tha(j)-dg2thhdiv1(j)*g2thh(i)-dg2thhdiv1(i)*
     -   g2thh(j))-12*(-1+lnmt)*(dg2thafin(j)*g2tha(i)+dg2thafin(i)*
     -   g2tha(j)-dg2thhfin(j)*g2thh(i) - dg2thhfin(i)*g2thh(j)))+
     -   8*mt*((6*lnmt**2+Pi**2)*(-(dmtdiv1*g2tha(i)*g2tha(j))+mt*
     -   dg2thhdiv1(j)*g2thh(i)+(mt*dg2thhdiv1(i)+3*dmtdiv1*g2thh(i))*
     -   g2thh(j))-12*lnmt*(-(dmtfin*g2tha(i)*g2tha(j))+mt*
     -   dg2thhfin(j)*g2thh(i)+(mt*dg2thhfin(i)+3*dmtfin*g2thh(i))*
     -   g2thh(j)))+(0,1)*(6*lnSu5**2*dMstdiv1(1,1)+Pi**2*
     -   dMstdiv1(1,1)-12*lnSu5*dMstfin(1,1))*g2u2h(1,i,1,j)+
     -   (0,1)*(6*lnSu6**2*dMstdiv1(2,2)+Pi**2*dMstdiv1(2,2) - 
     -   12*lnSu6*dMstfin(2,2))*g2u2h(2,i,2,j)-(6*lnSu5**2+Pi**2)*
     -   (dg2uhdiv1(1,1,j)*g2uh(1,1,i)+dg2uhdiv1(1,1,i)*g2uh(1,1,j))+
     -    12*lnSu5*(dg2uhfin(1,1,j)*g2uh(1,1,i)+dg2uhfin(1,1,i)*
     -    g2uh(1,1,j))-(6*lnSu6**2+Pi**2)*(dg2uhdiv1(2,2,j)*
     -    g2uh(2,2,i)+dg2uhdiv1(2,2,i)*g2uh(2,2,j))+12*lnSu6*
     -    (dg2uhfin(2,2,j)*g2uh(2,2,i)+dg2uhfin(2,2,i)*g2uh(2,2,j))+
     -    (12*(-(lnSu5*dMstdiv1(1,1))+dMstfin(1,1))*g2uh(1,1,i)*
     -    g2uh(1,1,j))/nMSu(5)**2+(0,1)*((12 - 12*lnSu5+6*lnSu5**2+
     -    Pi**2)*dg2u2hdiv1(1,i,1,j)-12*(-1+lnSu5)*dg2u2hfin(1,i,1,j))*
     -    nMSu(5)**2+(12*(-(lnSu6*dMstdiv1(2,2))+dMstfin(2,2))*
     -    g2uh(2,2,i)*g2uh(2,2,j))/nMSu(6)**2+(0,1)*((12 - 12*lnSu6+6*
     -    lnSu6**2+Pi**2)*dg2u2hdiv1(2,i,2,j) - 12*(-1+lnSu6)*
     -    dg2u2hfin(2,i,2,j))*nMSu(6)**2+(6*(g2uh(1,2,j)*g2uh(2,1,i)+
     -    g2uh(1,2,i)*g2uh(2,1,j))*(2*dMstfin(2,2)*((-1+lnSu5-lnSu6)*
     -    nMSu(5)**2+nMSu(6)**2)+dMstdiv1(2,2)*((-2+2*lnSu5-lnSu5**2+
     -    lnSu6**2)*nMSu(5)**2-2*(-1+lnSu6)*nMSu(6)**2)))/(nMSu(5)**2-
     -    nMSu(6)**2)**2 -(6*(-2*(dMstfin(1,2)*g2uh(1,2,j)*g2uh(2,2,i)+
     -    dMstfin(2,1)*g2uh(2,1,j)*g2uh(2,2,i)+(dMstfin(1,2)*
     -    g2uh(1,2,i)+dMstfin(2,1)*g2uh(2,1,i))*g2uh(2,2,j))*((-1+
     -    lnSu5-lnSu6)*nMSu(5)**2+nMSu(6)**2)+(dMstdiv1(1,2)*
     -    g2uh(1,2,j)*g2uh(2,2,i)+dMstdiv1(2,1)*g2uh(2,1,j)*
     -    g2uh(2,2,i)+(dMstdiv1(1,2)*g2uh(1,2,i)+dMstdiv1(2,1)*
     -    g2uh(2,1,i))*g2uh(2,2,j))*((2 - 2*lnSu5+lnSu5**2 - lnSu6**2)*
     -    nMSu(5)**2+2*(-1+lnSu6)*nMSu(6)**2)))/(nMSu(5)**2 - 
     -    nMSu(6)**2)**2+(6*(g2uh(1,2,j)*g2uh(2,1,i)+g2uh(1,2,i)*
     -    g2uh(2,1,j))*(2*dMstfin(1,1)*(nMSu(5)**2+(-1 - lnSu5+lnSu6)*
     -    nMSu(6)**2)+dMstdiv1(1,1)*(-2*(-1+lnSu5)*nMSu(5)**2+
     -    (-2+lnSu5**2+2*lnSu6 - lnSu6**2)*nMSu(6)**2)))/(nMSu(5)**2 - 
     -    nMSu(6)**2)**2-(6*(-2*(g2uh(1,1,j)*(dMstfin(1,2)*g2uh(1,2,i)+
     -    dMstfin(2,1)*g2uh(2,1,i))+g2uh(1,1,i)*(dMstfin(1,2)*
     -    g2uh(1,2,j)+dMstfin(2,1)*g2uh(2,1,j)))*(nMSu(5)**2+(-1 - 
     -    lnSu5+lnSu6)*nMSu(6)**2)+(g2uh(1,1,j)*(dMstdiv1(1,2)*
     -    g2uh(1,2,i)+dMstdiv1(2,1)*g2uh(2,1,i))+g2uh(1,1,i)*
     -    (dMstdiv1(1,2)*g2uh(1,2,j)+dMstdiv1(2,1)*g2uh(2,1,j)))*
     -    (2*(-1+lnSu5)*nMSu(5)**2+(2 - lnSu5**2 - 2*lnSu6+lnSu6**2)*
     -    nMSu(6)**2)))/(nMSu(5)**2-nMSu(6)**2)**2+((0,1)*(-12*
     -    (dMstfin(1,2)*g2u2h(1,i,2,j)+dMstfin(2,1)*g2u2h(2,i,1,j))*
     -    ((-1+lnSu5)*nMSu(5)**2 - (-1+lnSu6)*nMSu(6)**2)+
     -    (dMstdiv1(1,2)*g2u2h(1,i,2,j)+dMstdiv1(2,1)*g2u2h(2,i,1,j))*
     -    ((12 - 12*lnSu5+6*lnSu5**2+Pi**2)*nMSu(5)**2 -(12 - 12*lnSu6+
     -    6*lnSu6**2+Pi**2)*nMSu(6)**2)))/(nMSu(5)**2 - nMSu(6)**2) - 
     -    (-12*(dg2uhfin(2,1,j)*g2uh(1,2,i)+dg2uhfin(2,1,i)*g2uh(1,2,j)+
     -    dg2uhfin(1,2,j)*g2uh(2,1,i)+dg2uhfin(1,2,i)*g2uh(2,1,j))*
     -    ((-1+lnSu5)*nMSu(5)**2 - (-1+lnSu6)*nMSu(6)**2)+
     -    (dg2uhdiv1(2,1,j)*g2uh(1,2,i)+dg2uhdiv1(2,1,i)*g2uh(1,2,j)+
     -    dg2uhdiv1(1,2,j)*g2uh(2,1,i)+dg2uhdiv1(1,2,i)*g2uh(2,1,j))*
     -    ((12 - 12*lnSu5+6*lnSu5**2+Pi**2)*nMSu(5)**2 - 
     -    (12 - 12*lnSu6+6*lnSu6**2+Pi**2)*nMSu(6)**2))/
     -    (nMSu(5)**2 - nMSu(6)**2))/(64.*Pi**2)

      
      return
      end
      

c-------------------------------------------------------------------------c
c Calculates all OS counterterms of the order alpha_s alpha_t
c-------------------------------------------------------------------------c
      subroutine Calc2loopCounter(dMHpm2,dthu,dthd,dths,dtad,dtas,dv)
      
      implicit none
      
      double precision dMHpm2,dthu,dthd,dths,dtad,dtas,dv
      
      double precision Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      
      double precision abslambda, abskappa, absAlambda, absAkappa,vs,
     &                 mueff,tanb,beta
     
      double precision me,mmu,mtau,mu,md,ms,mc,mt,mb,Ye(3),Yd(3),Yu(3)
      
      double precision absM1, absM2, absAu(3), absAd(3), absAe(3),  
     &                   ml2(3), mer2(3), mq2(3), mdr2(3), mur2(3)
      
      double complex nZU(6,6),nZD(6,6)
      
      double precision nMSu(6),nMSd(6)
      
      double precision absM3,phiM3
      
      double precision phiu, phis, phil, phik, phiAk, phiAl, phiM1,
     &                phiM2, phiAu(3), phiAd(3), phiAe(3)
      
      double complex g2thh(5),g2tha(5),g2uh(2,2,5),g2u2h(2,5,2,5),
     &               g4u(2,2,2,2),gLtFgSu(2),gRtFgSu(2),g4dL,
     &               g2u2dL(2,2),gLbFgdL
      
      double complex gudLHp(2),g2u2Hp(2,2),g2dL2Hp
     
      double complex part1,part2,part3,part4,CTdiafincha
      
      double complex dvfunc
      
      double precision dv2loop
      
      common /SMinput/ Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      common /Higgsinput/ abslambda, abskappa, absAlambda, absAkappa,
     &                    vs,mueff,tanb,beta
      common /Yukawa/ me,mmu,mtau,mu,md,ms,mc,mt,mb,Ye,Yd,Yu
      common /softbreak/ absM1, absM2, absAu, absAd, absAe,  
     &                   ml2, mer2, mq2, mdr2, mur2
      common /nSu/ nMSu,nZU
      common /nSd/ nMSd,nZD
      common /gluino/ absM3,phiM3
      common /phases/ phiu, phis, phil, phik, phiAk, phiAl, phiM1,
     &                phiM2, phiAu, phiAd, phiAe
      common/qcdcoup/ g2thh,g2tha,g2uh,g2u2h,g4u,gLtFgSu,gRtFgSu,g4dL,
     &                 g2u2dL,gLbFgdL
      common/ qcdchacoup / gudLHp,g2u2Hp,g2dL2Hp
      common/ dv2loop /dv2loop
      

c calculate all tadpoles
      call tadpoles2loop(dthu,dthd,dths,dtad,dtas)
      
c genuine two-loop contributions to charged Higgs boson self-energy
      call HiggsSelf2loopGluonTop_cha(g3,mt,beta,v,part1)
      
      call HiggsSelf2loopGluonStop_cha(g3,nMSu,sqrt(mq2(3)),gudLHp,
     &                                 g2u2Hp,g2dL2Hp,part2)
      
      call HiggsSelf2loopOnlyStops_cha(nMSu,sqrt(mq2(3)),gudLHp,g2u2Hp,
     &                                 g2dL2Hp,g4u,g4dL,g2u2dL,part3)
      
      call HiggsSelf2loopGluino_cha(mt,nMSu,absM3,sqrt(mq2(3)),gudLHp,
     &                           g2u2Hp,g2dL2Hp,gLbFgdL,gRtFgSu,gLtFgSu,
     &                           beta,v,phiu,part4)

c counterterm of charged Higgs boson mass set
      dMHpm2=Realpart(part1+part2+part3+part4+CTdiafincha())
      
c contributions of alpha_s alpha_t to dv in the gaugeless limit
      dv=Realpart(dvfunc())
      dv2loop=dv
     
      return
      end
      
c-----------------------------------------------------------------------------c


c genuine two-loop contributions to self-energy of charged Higgs boson
c of order alpha_s alpha_t with gluons and tops in the loops
      subroutine HiggsSelf2loopGluonTop_cha(g3,mt,beta,v,self)
      
      implicit none
      
      double precision g3,mt,beta,v
      double complex self
     
      double precision Qs
      double precision lnmt
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      common /scale/ Qs
      
      lnmt=Log(mt**2/Qs**2)

      self=-(g3**2*mt**4*(18+4*lnmt*(-5+3*lnmt)+Pi**2)/
     -            tan(beta)**2)/(16.*Pi**4*v**2)
     
      return
      end
      
c genuine two-loop contributions to self-energy of charged Higgs boson
c of order alpha_s alpha_t with gluons and stops in the loops
      subroutine HiggsSelf2loopGluonStop_cha(g3,nMSu,mQ3,gudLHp,g2u2Hp,
     &                                       g2dL2Hp,self)
      
      implicit none
      
      double precision g3,nMSu(6),mQ3
      double complex gudLHp(2),g2u2Hp(2,2),g2dL2Hp
      double complex self
      
      double precision F1
      double precision f1MSu5,f1MSu6,f1mQ3
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      f1Msu5=F1(nMSu(5)**2)
      f1Msu6=F1(nMSu(6)**2)
      f1mQ3=F1(mQ3**2)
      
      self=(g3**2*(((f1mQ3-f1MSu5)*Abs(gudLHp(1))**2)/
     -     (mQ3**2-nMSu(5)**2)+((0,1)*((0,-1)*(f1mQ3-f1MSu6)*
     -     Abs(gudLHp(2))**2+(f1mQ3*g2dL2Hp+f1MSu5*g2u2Hp(1,1)+
     -     f1MSu6*g2u2Hp(2,2))*(mQ3**2-nMSu(6)**2)))/
     -     (mQ3**2-nMSu(6)**2)))/(128.*Pi**4)


      
      return
      end
      

c genuine two-loop contributions to self-energy of charged Higgs boson
c of order alpha_s alpha_t with only tops in the loops
      subroutine HiggsSelf2loopOnlyStops_cha(nMSu,mQ3,gudLHp,g2u2Hp,
     &                                       g2dL2Hp,g4u,g4dL,g2u2dL,self)
      
      implicit none
      
      double precision nMSu(6),mQ3
      double complex gudLHp(2),g2u2Hp(2,2),g2dL2Hp,g4u(2,2,2,2),
     &               g4dL,g2u2dL(2,2)
      double complex self
      
      double precision F5,F6,F7,F17,F18,F19
      
      double precision f5Q5,f5Q6,f55Q,f56Q,f17Q56,f17Q65,f18Q56,f18Q65
      double precision f6QQ,f655,f666,f656,f665,f756,f765
      double precision f1956Q,f195Q6,f1965Q,f196Q5
     
      double complex top1,top2,top3
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      f5Q6=F5(mQ3**2,nMSu(6)**2)/(mQ3**2-nMSu(6)**2)**2
      f5Q5=F5(mQ3**2,nMSu(5)**2)/(mQ3**2-nMSu(5)**2)**2
      f55Q=F5(nMSu(5)**2,mQ3**2)/(mQ3**2-nMSu(5)**2)**2
      f56Q=F5(nMSu(6)**2,mQ3**2)/(mQ3**2-nMSu(6)**2)**2
      f17Q56=F17(mQ3**2,nMSu(5)**2,nMSu(6)**2)
      f17Q65=F17(mQ3**2,nMSu(6)**2,nMSu(5)**2)
      f18Q56=F18(mQ3**2,nMSu(5)**2,nMSu(6)**2)
      f18Q65=F18(mQ3**2,nMSu(6)**2,nMSu(5)**2)
      
      top1=((0,-0.005859375)*(Abs(gudLHp(1))**2*
     -     (4*f55Q*g4dL+4*f5Q5*g4u(1,1,1,1)+3*f18Q65*g4u(1,2,1,2)+
     -     f18Q65*g4u(1,2,2,1))+Abs(gudLHp(2))**2*(4*f56Q*g4dL+
     -     3*f18Q56*g4u(1,2,1,2)+f18Q56*g4u(2,1,1,2)+
     -     4*f5Q6*g4u(2,2,2,2))+4*(CONJG(gudLHp(2))*
     -     (f17Q65*g4u(1,2,1,1)+f17Q56*g4u(2,2,1,2))*gudLHp(1)+
     -     CONJG(gudLHp(1))*(f17Q65*g4u(1,1,1,2)+f17Q56*g4u(1,2,2,2))*
     -     gudLHp(2))))/Pi**4
     
      
      f6QQ=F6(mQ3**2,mQ3**2)
      f655=F6(nMSu(5)**2,nMSu(5)**2)
      f666=F6(nMSu(6)**2,nMSu(6)**2)
      f656=F6(nMSu(5)**2,nMSu(6)**2)
      f665=F6(nMSu(6)**2,nMSu(5)**2)
      f756=F7(nMSu(5)**2,nMSu(6)**2)/(nMSu(5)**2-nMSu(6)**2)
      f765=F7(nMSu(6)**2,nMSu(5)**2)/(nMSu(6)**2-nMSu(5)**2)
      
      top2=(4*f6QQ*g2dL2Hp*g4dL+4*f655*g2u2Hp(1,1)*g4u(1,1,1,1)+
     -    4*f765*g2u2Hp(2,1)*g4u(1,1,1,2)+
     -    4*f765*g2u2Hp(1,2)*g4u(1,2,1,1)+
     -    f665*g2u2Hp(1,1)*(3*g4u(1,2,1,2)+g4u(1,2,2,1))+
     -    4*f756*g2u2Hp(2,1)*g4u(1,2,2,2)+
     -    f656*g2u2Hp(2,2)*(3*g4u(1,2,1,2)+g4u(2,1,1,2))+
     -    4*f756*g2u2Hp(1,2)*g4u(2,2,1,2)+
     -    4*f666*g2u2Hp(2,2)*g4u(2,2,2,2))/(512.*Pi**4)
     
      
      f1956Q=F19(nMSu(5)**2,nMSu(6)**2,mQ3**2)
      f195Q6=F19(nMSu(5)**2,mQ3**2,nMSu(6)**2)
      f1965Q=F19(nMSu(6)**2,nMSu(5)**2,mQ3**2)
      f196Q5=F19(nMSu(6)**2,mQ3**2,nMSu(5)**2)
      
      top3=((0,-0.015625)*((f1965Q - f196Q5)*Abs(gudLHp(1))**2*
     -     g2u2dL(1,1)*(mQ3**2 - nMSu(6)**2)**3+
     -     (mQ3**2 - nMSu(5)**2)**2*((f1956Q - f195Q6)*
     -     Abs(gudLHp(2))**2*g2u2dL(2,2)*(mQ3**2 - nMSu(5)**2)+
     -     f1956Q*(CONJG(gudLHp(2))*g2u2dL(2,1)*gudLHp(1)+
     -     CONJG(gudLHp(1))*g2u2dL(1,2)*gudLHp(2))*
     -     (mQ3**2 - nMSu(6)**2))))/(Pi**4*(mQ3 - nMSu(5))**3*
     -     (mQ3+nMSu(5))**3*(mQ3 - nMSu(6))**3*(mQ3+nMSu(6))**3)

      self=top1+top2+top3
      return
      end
      
      
c genuine two-loop contributions to self-energy of charged Higgs boson
c of order alpha_s alpha_t with gluinos in the loops
      subroutine HiggsSelf2loopGluino_cha(mt,nMSu,absM3,mQ3,gudLHp,
     &                          g2u2Hp,g2dL2Hp,gLbFgdL,gRtFgSu,gLtFgSu,
     &                          beta,v,phiu,self)
     
      implicit none
     
      double precision mt,nMSu(6),absM3,mQ3
      double complex gudLHp(2),g2u2Hp(2,2),g2dL2Hp,gLbFgdL,gRtFgSu(2),
     &               gLtFgSu(2)
      double precision beta,v,phiu
      double complex self
      
      double complex EPhiu
      double complex Ifin
      double precision F20,F21,F22
      double complex TFI5,TFI6,TFI7,TFI11,TFI12,TFI14,TFI15,TFI16,
     &               TFI17,TFI18
      double complex IfinQtM3,Ifin0QM3,IfinSutM3,Ifin0SuM3
      double complex TFI16num,TFI5numSu,TFI11numSu,TFI12numSu,
     &                TFI15numSu,TFI17numSu,TFI16numSu
      double complex Abbr1,Abbr1b,Abbr2,Abbr2b,Abbr3,Abbr4
      double complex TFI18QM35,TFI18QM36,TFI145tM3,TFI14QtM3,TFI55tM3,
     &               TFI5QtM3,TFI125tM3,TFI155tM3,TFI146tM3,TFI56tM3, 
     &               TFI126tM3,TFI156tM3,Ifin5tM3,TFI6M3t5,Ifin6tM3,
     &               TFI6M3t6,TFI5tM35,TFI5tM36,TFI6tM35,TFI6tM36,
     &               TFI7tM35,TFI7tM36
      double precision f20Qt,f20tSu,f21M3Q,f21M3Su,f22QM3
      
      double complex top1,top2,top3,top4
      
      integer ind1

      double precision pi,Qs
      parameter (pi = 2d0*Asin(1d0))

      Qs=700d0
      EPhiu=exp((0,1)*phiu)
      
      
      IfinQtM3=Ifin(mQ3**2,mt**2,absM3**2)
      Ifin0QM3=Ifin(mQ3**2,absM3**2,0d0)
      Ifin5tM3=Ifin(nMSu(5)**2,mt**2,absM3**2)
      Ifin6tM3=Ifin(nMSu(6)**2,mt**2,absM3**2)
      f20Qt=F20(mQ3**2,mt**2)
      f21M3Q=F21(absM3**2,mQ3**2)
      TFI16num=TFI16(mt**2, absM3**2, mQ3**2)
      
      top1=0d0
      top3=0d0
      do ind1=1,2
      if(ind1.eq.1) then
      IfinSutM3=Ifin5tM3
      else
      IfinSutM3=Ifin6tM3
      end if
      Ifin0SuM3=Ifin(nMSu(ind1+4)**2,absM3**2,0d0)
      f20tSu=F20(mt**2,nMSu(ind1+4)**2)
      f21M3Su=F21(absM3**2,nMSu(ind1+4)**2)
      TFI5numSu=TFI5(mt**2, absM3**2, nMSu(ind1+4)**2)
      TFI11numSu=TFI11(mt**2, absM3**2, nMSu(ind1+4)**2)
      TFI12numSu=TFI12(mt**2, absM3**2, nMSu(ind1+4)**2)
      TFI15numSu=TFI15(mt**2, absM3**2, nMSu(ind1+4)**2)
      TFI16numSu=TFI16(mt**2, absM3**2, nMSu(ind1+4)**2)
      TFI17numSu=TFI17(mt**2, absM3**2, nMSu(ind1+4)**2)
      
      top1=top1+
     -    (1/tan(beta)**2*(-(gLbFgdL*(6*Ifin0QM3*(-absM3**2+mQ3**2)*
     -    mt**2+absM3**2*f21M3Q*mQ3**2*(2*absM3**2+mt**2)*Pi**4+
     -    12*TFI16num)*CONJG(gLbFgdL))+24*absM3*mt**3*(IfinSutM3 - 
     -    TFI11numSu)*(CONJG(gRtFgSu(ind1))*gLtFgSu(ind1)+
     -    CONJG(gLtFgSu(ind1))*gRtFgSu(ind1))+4*Abs(gLtFgSu(ind1))**2*
     -    (6*TFI16numSu - 6*mt**2*TFI17numSu+absM3**4*F21M3Su*Pi**4*
     -    nMSu(4+ind1)**2)+2*mt**2*Abs(gRtFgSu(ind1))**2*
     -    (-6*(absM3**2*Ifin0SuM3+2*mt**2*TFI12numSu - 2*TFI5numSu)+
     -    (6*Ifin0SuM3+absM3**2*F21M3Su*Pi**4)*nMSu(4+ind1)**2)))/
     -    (96.*mt**2*Pi**8*v**2)
      
      top3=top3+((0,1)/48d0*gLbFgdL*CONJG(gudLHp(ind1))/tan(beta)*
     -     (12*absM3*(IfinQtM3 - IfinSutM3)*mt*gLtFgSu(ind1)+
     -      gRtFgSu(ind1)*(-f20Qt+f20tSu - 6*absM3**2*Ifin0QM3+
     -      6*absM3**2*Ifin0SuM3+6*absM3**2*IfinQtM3- 
     -      6*absM3**2*IfinSutM3+6*Ifin0QM3*mQ3**2-6*IfinQtM3*mQ3**2+
     -      6*IfinQtM3*mt**2-6*IfinSutM3*mt**2-6*(Ifin0SuM3-IfinSutM3)*
c MMM changed 21 Aug 2018   
c     -     nMSu(4+ind1)**2)))/(Sqrt(2d0)*Sqrt(EPhiu)*Pi**8*v*
     -     nMSu(4+ind1)**2)))/(Sqrt(2d0)*exp((0,1) *phiu/2d0)*Pi**8*v*
c end MMM changed 21 Aug 2018      
     -     (mQ3**2 - nMSu(4+ind1)**2))
      end do
      top3=top3+CONJG(top3)
      
      
      TFI5tM35=TFI5(mt**2,absM3**2,nMSu(5)**2)
      TFI5tM36=TFI5(mt**2,absM3**2,nMSu(6)**2)
      TFI6tM35=TFI6(mt**2,absM3**2,nMSu(5)**2)
      TFI6tM36=TFI6(mt**2,absM3**2,nMSu(6)**2)
      TFI7tM35=TFI7(mt**2,absM3**2,nMSu(5)**2)
      TFI7tM36=TFI7(mt**2,absM3**2,nMSu(6)**2)
      f22QM3=F22(mQ3**2,absM3**2)
      
      top2=((0,1)/48d0*(g2dL2Hp*gLbFgdL*(f22QM3+6*Ifin0QM3)*
     -     CONJG(gLbFgdL) - 6*g2u2Hp(1,1)*(CONJG(gLtFgSu(1))*(TFI7tM35*
     -     gLtFgSu(1)+absM3*mt*TFI6tM35*gRtFgSu(1))+CONJG(gRtFgSu(1))*
     -     (absM3*mt*TFI6tM35*gLtFgSu(1)+TFI7tM35*gRtFgSu(1))) - 
     -     6*g2u2Hp(2,2)*(CONJG(gLtFgSu(2))*(TFI7tM36*gLtFgSu(2)+
     -     absM3*mt*TFI6tM36*gRtFgSu(2))+CONJG(gRtFgSu(2))*(absM3*mt*
     -     TFI6tM36*gLtFgSu(2)+TFI7tM36*gRtFgSu(2)))-(6*g2u2Hp(2,1)*
     -     (CONJG(gLtFgSu(2))*((TFI5tM35 - TFI5tM36)*gLtFgSu(1)+
     -     absM3*(Ifin5tM3 - Ifin6tM3)*mt*gRtFgSu(1))+
     -     CONJG(gRtFgSu(2))*(absM3*(Ifin5tM3 - Ifin6tM3)*mt*
     -     gLtFgSu(1)+(TFI5tM35 - TFI5tM36)*gRtFgSu(1))))/
     -     (nMSu(5)**2 - nMSu(6)**2) - (6*g2u2Hp(1,2)*
     -     (CONJG(gLtFgSu(1))*((TFI5tM35 - TFI5tM36)*gLtFgSu(2)+
     -     absM3*(Ifin5tM3 - Ifin6tM3)*mt*gRtFgSu(2))+
     -     CONJG(gRtFgSu(1))*(absM3*(Ifin5tM3 - Ifin6tM3)*mt*
     -     gLtFgSu(2)+(TFI5tM35 - TFI5tM36)*gRtFgSu(2))))/
     -     (nMSu(5)**2 - nMSu(6)**2)))/Pi**8
      
      
      TFI18QM35=TFI18(mQ3**2,absM3**2,nMSu(5)**2)
      TFI18QM36=TFI18(mQ3**2,absM3**2,nMSu(6)**2)
      TFI145tM3=TFI14(nMSu(5)**2,mt**2,absM3**2)
      TFI14QtM3=TFI14(mQ3**2,mt**2,absM3**2)
      TFI55tM3=TFI5(nMSu(5)**2,mt**2,absM3**2)
      TFI5QtM3=TFI5(mQ3**2,mt**2,absM3**2)
      TFI125tM3=TFI12(nMSu(5)**2,mt**2,absM3**2)
      TFI155tM3=TFI15(nMSu(5)**2,mt**2,absM3**2)
      TFI146tM3=TFI14(nMSu(6)**2,mt**2,absM3**2)
      TFI56tM3=TFI5(nMSu(6)**2,mt**2,absM3**2)
      TFI126tM3=TFI12(nMSu(6)**2,mt**2,absM3**2)
      TFI156tM3=TFI15(nMSu(6)**2,mt**2,absM3**2)
      TFI6M3t5=TFI6(absM3**2,mt**2,nMSu(5)**2)
      TFI6M3t6=TFI6(absM3**2,mt**2,nMSu(6)**2)
      Abbr1=-TFI145tM3+TFI14QtM3+TFI55tM3 - TFI5QtM3+
     -  (TFI125tM3 - TFI155tM3)*(mQ3 - nMSu(5))*(mQ3+nMSu(5))
      Abbr1b=-TFI146tM3+TFI14QtM3+TFI56tM3 - TFI5QtM3+
     -  (TFI126tM3 - TFI156tM3)*(mQ3 - nMSu(6))*(mQ3+nMSu(6))
      Abbr2=Ifin5tM3 - IfinQtM3+TFI6M3t5*(mQ3**2 - (nMSu(5))**2)
      Abbr2b=Ifin6tM3 - IfinQtM3+TFI6M3t6*(mQ3**2 - (nMSu(6))**2)
      Abbr3=mQ3**2*(TFI145tM3 - TFI146tM3 - TFI55tM3+TFI56tM3)+
     -      TFI146tM3*nMSu(5)**2-TFI14QtM3*nMSu(5)**2-
     -      TFI56tM3*nMSu(5)**2+TFI5QtM3*nMSu(5)**2 -TFI145tM3*
     -      nMSu(6)**2+TFI14QtM3*nMSu(6)**2+TFI55tM3*nMSu(6)**2 - 
     -      TFI5QtM3*nMSu(6)**2
      Abbr4=Ifin6tM3*(-mQ3**2+nMSu(5)**2)+Ifin5tM3*(mQ3**2 - 
     -      nMSu(6)**2)+IfinQtM3*(-nMSu(5)**2+nMSu(6)**2)
    
      top4=((-12*Abbr1*Abs(gLtFgSu(1)*gudLHp(1))**2)/
     -     (mQ3**2 - nMSu(5)**2)**2+(12*Abs(gudLHp(1))**2*(-(Abbr1*
     -     Abs(gRtFgSu(1))**2)+Abbr2*absM3*mt*(CONJG(gRtFgSu(1))*
     -     gLtFgSu(1)+CONJG(gLtFgSu(1))*gRtFgSu(1))))/
     -     (mQ3**2 - nMSu(5)**2)**2-(12*Abbr1b*Abs(gLtFgSu(2)*
     -     gudLHp(2))**2)/(mQ3**2 - nMSu(6)**2)**2+
     -     (12*Abs(gudLHp(2))**2*(-(Abbr1b*Abs(gRtFgSu(2))**2)+
     -     Abbr2b*absM3*mt*(CONJG(gRtFgSu(2))*gLtFgSu(2)+
     -     CONJG(gLtFgSu(2))*gRtFgSu(2))))/(mQ3**2 - nMSu(6)**2)**2+
     -     (12*CONJG(gRtFgSu(1))*CONJG(gudLHp(2))*(Abbr4*absM3*mt*
     -     gLtFgSu(2)+Abbr3*gRtFgSu(2))*gudLHp(1))/
     -     ((mQ3**2 - nMSu(5)**2)*(mQ3**2 - nMSu(6)**2)*
     -     (nMSu(5)**2 - nMSu(6)**2))+(12*CONJG(gLtFgSu(1))*
     -     CONJG(gudLHp(2))*(Abbr3*gLtFgSu(2)+Abbr4*absM3*mt*
     -     gRtFgSu(2))*gudLHp(1))/((mQ3**2 - nMSu(5)**2)*
     -     (mQ3**2 - nMSu(6)**2)*(nMSu(5)**2 - nMSu(6)**2))+
     -     (12*CONJG(gudLHp(1))*(CONJG(gRtFgSu(2))*(Abbr4*absM3*mt*
     -     gLtFgSu(1)+Abbr3*gRtFgSu(1))+CONJG(gLtFgSu(2))*(Abbr3*
     -     gLtFgSu(1)+Abbr4*absM3*mt*gRtFgSu(1)))*gudLHp(2))/
     -     ((mQ3**2 - nMSu(5)**2)*(mQ3**2 - nMSu(6)**2)*
     -     (nMSu(5)**2 - nMSu(6)**2))+Abs(gLbFgdL)**2*((TFI18QM35*
     -     CONJG(gudLHp(1))*gudLHp(1))/(mQ3**2 - nMSu(5)**2)**2+
     -     (TFI18QM36*CONJG(gudLHp(2))*gudLHp(2))/
     -     (mQ3**2 - nMSu(6)**2)**2))/(96.*Pi**8)
    
      self=top1+top2+top3+top4
     
      return 
      end
      
c----------------------------------------------------------------------------------------c


c counterterm inserted diagrams for self-energy of charged Higgs boson
      double complex function CTdiafincha()
      
      implicit none
      
      double precision Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      
      double precision abslambda, abskappa, absAlambda, absAkappa,vs,
     &                 mueff,tanb,beta
     
      double precision absM1, absM2, absAu(3), absAd(3), absAe(3),  
     &                 ml2(3), mer2(3), mq2(3), mdr2(3), mur2(3)

      double precision me,mmu,mtau,mu,md,ms,mc,mt,mb,Ye(3),Yd(3),Yu(3)
      
      double complex nZU(6,6)
      
      double precision nMSu(6)
      
      double precision nMSd(6)
      
      double complex dmQ32fin,dmQ32div1,dMsbfin(2,2),dMsbdiv1(2,2)
      
      double complex gudLHp(2),g2u2Hp(2,2),g2dL2Hp
      
      double complex gudHp(2,2),g2d2Hp(2,2)
      
      double complex dgudLHpfin(2),dg2u2Hpfin(2,2),dg2dL2Hpfin,
     &               dgudLHpdiv1(2),dg2u2Hpdiv1(2,2),dg2dL2Hpdiv1
     
      double complex dgudHpfin(2,2),dg2d2Hpfin(2,2),
     &               dgudHpdiv1(2,2),dg2d2Hpdiv1(2,2)
     
      double complex dmtfin,dmtdiv1,dMstfin(2,2),dMstdiv1(2,2),
     &               dYtfin,dYtdiv1,dAtfin,dAtdiv1,dphiAtfin,dphiAtdiv1
     
      double precision Qs
      
      double precision Ae,A0
      double complex B0,Be
      
      double complex ct1,ct2,ct3,ct4
      
      integer ind3,ind4,ind5
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      common /SMinput/ Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      common /Higgsinput/ abslambda, abskappa, absAlambda, absAkappa,
     &                    vs,mueff,tanb,beta
      common /softbreak/ absM1, absM2, absAu, absAd, absAe,  
     &                   ml2, mer2, mq2, mdr2, mur2
      common /Yukawa/ me,mmu,mtau,mu,md,ms,mc,mt,mb,Ye,Yd,Yu
      common /nSu/ nMSu,nZU
      common/ qcdchacoup / gudLHp,g2u2Hp,g2dL2Hp
      common/ dqcdchacoup / dgudLHpfin,dg2u2Hpfin,dg2dL2Hpfin,
     &                      dgudLHpdiv1,dg2u2Hpdiv1,dg2dL2Hpdiv1
      common/qcdcount/ dmtfin,dmtdiv1,dMstfin,dMstdiv1,dYtfin,dYtdiv1,
     &                 dAtfin,dAtdiv1,dphiAtfin,dphiAtdiv1
      common /scale/ Qs
     
      
      dmQ32fin=Abs(nZU(5,5))**2*dMstfin(1,1)
     &      +Abs(nZU(5,6))**2*dMstfin(2,2)
     &      -nZU(6,6)*CONJG(nZU(5,6))*dMstfin(1,2)
     &      -nZU(5,6)*CONJG(nZU(6,6))*dMstfin(2,1)
     &      -2*mt*dmtfin
      dmQ32div1=Abs(nZU(5,5))**2*dMstdiv1(1,1)
     &      +Abs(nZU(5,6))**2*dMstdiv1(2,2)
     &      -nZU(6,6)*CONJG(nZU(5,6))*dMstdiv1(1,2)
     &      -nZU(5,6)*CONJG(nZU(6,6))*dMstdiv1(2,1)
     &      -2*mt*dmtdiv1
     
      
      nMSd(5)=Sqrt(mq2(3))
      nMSd(6)=123.45678d0
      dMsbfin=0d0
      dMsbfin(1,1)=dmQ32fin
      dMsbdiv1=0d0
      dMsbdiv1(1,1)=dmQ32div1
      g2d2Hp=0d0
      g2d2Hp(1,1)=g2dL2Hp
      dg2d2Hpfin=0d0
      dg2d2Hpfin(1,1)=dg2dL2Hpfin
      dg2d2Hpdiv1=0d0
      dg2d2Hpdiv1(1,1)=dg2dL2Hpdiv1
      gudHp=0d0
      gudHp(1,1)=gudLHp(1)
      gudHp(2,1)=gudLHp(2)
      dgudHpfin=0d0
      dgudHpfin(1,1)=dgudLHpfin(1)
      dgudHpfin(2,1)=dgudLHpfin(2)
      dgudHpdiv1=0d0
      dgudHpdiv1(1,1)=dgudLHpdiv1(1)
      dgudHpdiv1(2,1)=dgudLHpdiv1(2)

      
      ct1=(3*Cos(beta)**2*Yu(3)*(-(mt**3*(dYtfin*
     -     B0(0d0,0d0,mt**2,Qs**2)+dYtdiv1*Be(0d0,0d0,mt**2))) - 
     -     (-(dmtdiv1*A0(mt**2))+mt**2*(- dmtfin+dmtfin*
     -     B0(0d0,0d0,mt**2,Qs**2)+dmtdiv1*Be(0d0,0d0,mt**2)))*
     -     Yu(3)))/(4.*mt*Pi**2)
     
     
      ct2=0d0
      ct3=0d0
      ct4=0d0
      do ind3=1,2
      ct2=ct2+((0,3)/16.*(Ae(nMSd(4+ind3)**2)*dg2d2Hpdiv1(ind3,ind3)+
     -      A0(nMSd(4+ind3)**2)*dg2d2Hpfin(ind3,ind3)+
     -      Ae(nMSu(4+ind3)**2)*dg2u2Hpdiv1(ind3,ind3)+
     -      A0(nMSu(4+ind3)**2)*dg2u2Hpfin(ind3,ind3)))/Pi**2
     
      do ind4=1,2
      ct3=ct3+(3*(CONJG(gudHp(ind4,ind3))*(
     -   B0(0d0,nMSd(4+ind3)**2,nMSu(4+ind4)**2,Qs**2)*
     -   dgudHpfin(ind4,ind3))+
     -   (0,1)*((Be(0d0,nMSd(4+ind3)**2,nMSd(4+ind4)**2)
     -   *dMsbdiv1(ind4,ind3)+
     -   B0(0d0,nMSd(4+ind3)**2,nMSd(4+ind4)**2,Qs**2)*
     -   dMsbfin(ind4,ind3))*g2d2Hp(ind4,ind3)+
     -   (Be(0d0,nMSu(4+ind3)**2,nMSu(4+ind4)**2)*dMstdiv1(ind4,ind3)+
     -   B0(0d0,nMSu(4+ind3)**2,nMSu(4+ind4)**2,Qs**2)*
     -   dMstfin(ind4,ind3))*g2u2Hp(ind4,ind3))+(
     -   B0(0d0,nMSd(4+ind3)**2,nMSu(4+ind4)**2,Qs**2)*
     -   CONJG(dgudHpfin(ind4,ind3)))*gudHp(ind4,ind3)+
     -   Be(0d0,nMSd(4+ind3)**2,nMSu(4+ind4)**2)*
     -   (CONJG(gudHp(ind4,ind3))*dgudHpdiv1(ind4,ind3)+
     -   CONJG(dgudHpdiv1(ind4,ind3))*gudHp(ind4,ind3))))/(16.*Pi**2)
     
      do ind5=1,2
      if(ind3.eq.ind4) then
      ct4=ct4+(3*((CONJG(gudHp(ind4,ind5))*gudHp(ind4,ind5)*
     -   ((-(Ae(nMSu(4+ind4)**2)*dMstdiv1(ind4,ind4))+
     -   A0(nMSu(4+ind4)**2)*(dMstdiv1(ind4,ind4)-dMstfin(ind4,ind4)))*
     -   nMSd(4+ind5)**2+((-A0(nMSu(4+ind4)**2)+Ae(nMSd(4+ind5)**2))*
     -   dMstdiv1(ind4,ind4)+dMstfin(ind4,ind4)*(A0(nMSd(4+ind5)**2)+
     -   nMSd(4+ind5)**2))*nMSu(4+ind4)**2 - dMstfin(ind4,ind4)*
     -   nMSu(4+ind4)**4))/(-(nMSd(4+ind5)**2*nMSu(4+ind4))+
     -   nMSu(4+ind4)**3)**2+(CONJG(gudHp(ind5,ind4))*gudHp(ind5,ind4)*
     -   (nMSd(4+ind4)**2*((-A0(nMSd(4+ind4)**2)+Ae(nMSu(4+ind5)**2))*
     -   dMsbdiv1(ind4,ind4)+dMsbfin(ind4,ind4)*(A0(nMSu(4+ind5)**2)-
     -   nMSd(4+ind4)**2))+(-(Ae(nMSd(4+ind4)**2)*dMsbdiv1(ind4,ind4))+
     -   A0(nMSd(4+ind4)**2)*(dMsbdiv1(ind4,ind4)-dMsbfin(ind4,ind4))+
     -   dMsbfin(ind4,ind4)*nMSd(4+ind4)**2)*nMSu(4+ind5)**2))/
     -   (nMSd(4+ind4)**3 - nMSd(4+ind4)*nMSu(4+ind5)**2)**2))/
     -   (16.*Pi**2)
      else
      ct4=ct4+(-3*((CONJG(gudHp(ind4,ind5))*gudHp(ind3,ind5)*
     -   (Ae(nMSu(4+ind4)**2)*dMstdiv1(ind3,ind4)*
     -   (-nMSd(4+ind5)**2+nMSu(4+ind3)**2)+Ae(nMSu(4+ind3)**2)*
     -   dMstdiv1(ind3,ind4)*(nMSd(4+ind5)**2 - nMSu(4+ind4)**2)+
     -   Ae(nMSd(4+ind5)**2)*dMstdiv1(ind3,ind4)*(-nMSu(4+ind3)**2+
     -   nMSu(4+ind4)**2)+dMstfin(ind3,ind4)*
     -   (A0(nMSu(4+ind4)**2)*(-nMSd(4+ind5)**2+nMSu(4+ind3)**2)+
     -   A0(nMSu(4+ind3)**2)*(nMSd(4+ind5)**2 - nMSu(4+ind4)**2)+
     -   A0(nMSd(4+ind5)**2)*(-nMSu(4+ind3)**2+nMSu(4+ind4)**2))))/
     -   ((nMSd(4+ind5)**2 - nMSu(4+ind3)**2)*(nMSd(4+ind5)**2-
     -   nMSu(4+ind4)**2)*(nMSu(4+ind3)**2 - nMSu(4+ind4)**2))+
     -   (CONJG(gudHp(ind5,ind4))*gudHp(ind5,ind3)*
     -   (Ae(nMSu(4+ind5)**2)*dMsbdiv1(ind4,ind3)*
     -   (-nMSd(4+ind3)**2+nMSd(4+ind4)**2)+Ae(nMSd(4+ind4)**2)*
     -   dMsbdiv1(ind4,ind3)*(nMSd(4+ind3)**2 - nMSu(4+ind5)**2)+
     -   Ae(nMSd(4+ind3)**2)*dMsbdiv1(ind4,ind3)*(-nMSd(4+ind4)**2+
     -   nMSu(4+ind5)**2)+dMsbfin(ind4,ind3)*
     -   (A0(nMSu(4+ind5)**2)*(-nMSd(4+ind3)**2+nMSd(4+ind4)**2)+
     -   A0(nMSd(4+ind4)**2)*(nMSd(4+ind3)**2 - nMSu(4+ind5)**2)+
     -   A0(nMSd(4+ind3)**2)*(-nMSd(4+ind4)**2+nMSu(4+ind5)**2))))/
     -   ((nMSd(4+ind3)**2 - nMSd(4+ind4)**2)*(nMSd(4+ind3)**2-
     -   nMSu(4+ind5)**2)*(nMSd(4+ind4)**2 - nMSu(4+ind5)**2))))/
     -   (16.*Pi**2)
      end if
      end do
      end do
      end do
      
      CTdiafincha=ct1+ct2+ct3+ct4

      return
      end
      
c--------------------------------------------------------------------------------------------c

c calculates the tadpole CT of the order alpha_s alpha_t
      subroutine tadpoles2loop(dthu,dthd,dths,dtad,dtas)
      
      implicit none
      
      double precision dthu,dthd,dths,dtad,dtas
      
      double complex tadpole(5)
      
      double precision Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      
      double precision abslambda, abskappa, absAlambda, absAkappa,vs,
     &                 mueff,tanb,beta
     
      double precision me,mmu,mtau,mu,md,ms,mc,mt,mb,Ye(3),Yd(3),Yu(3)
      
      double precision ZH5(5,5),ZH(6,6)
     
      double complex UM(2,2),UP(2,2),NN(5,5),ZE(6,6),ZD(6,6),ZU(6,6)
      
      double complex nZU(6,6)
      
      double precision nMSu(6)
      
      double precision absM3,phiM3
      
      double complex g2thh(5),g2tha(5),g2uh(2,2,5),g2u2h(2,5,2,5),
     &               g4u(2,2,2,2),gLtFgSu(2),gRtFgSu(2),g4dL,
     &               g2u2dL(2,2),gLbFgdL
     
      double complex dg2thhfin(5),dg2thafin(5),dg2uhfin(2,2,5),
     &               dg2u2hfin(2,5,2,5),
     &               dg2thhdiv1(5),dg2thadiv1(5),dg2uhdiv1(2,2,5),
     &               dg2u2hdiv1(2,5,2,5)
     
      double complex dmtfin,dmtdiv1,dMstfin(2,2),dMstdiv1(2,2),
     &               dYtfin,dYtdiv1,dAtfin,dAtdiv1,dphiAtfin,dphiAtdiv1
      
      double precision Qs

      double precision lnnMSu(2),lnmt

      double precision F14a,F14b,F11a,F11b,F13ba,F13ab,F12ab,F12ba,F15
      
      double complex Ifin,TFI12,TFI6,TFI14,TFI5,TFI15,TFI11
      
      double complex Ifindif,TFI12a,TFI12b,TFI6a,TFI6b,TFI145,
     &               TFI1215a,TFI1215b,TFI1116ma,TFI1116mb,TFI1116pa,
     &               TFI1116pb
      
      integer i
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      common /SMinput/ Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      common /Higgsinput/ abslambda, abskappa, absAlambda, absAkappa,
     &                    vs,mueff,tanb,beta
      common /Yukawa/ me,mmu,mtau,mu,md,ms,mc,mt,mb,Ye,Yd,Yu
      common /rot/ ZH5,ZH,UM,UP,NN,ZE,ZD,ZU
      common /nSu/ nMSu,nZU
      common /gluino/ absM3,phiM3
      common/qcdcoup/ g2thh,g2tha,g2uh,g2u2h,g4u,gLtFgSu,gRtFgSu,g4dL,
     &                 g2u2dL,gLbFgdL
      common/dqcdcoup/ dg2thhfin,dg2thafin,dg2uhfin,dg2u2hfin,
     &                 dg2thhdiv1,dg2thadiv1,dg2uhdiv1,dg2u2hdiv1
      common/qcdcount/ dmtfin,dmtdiv1,dMstfin,dMstdiv1,dYtfin,dYtdiv1,
     &                 dAtfin,dAtdiv1,dphiAtfin,dphiAtdiv1
      common /scale/ Qs

c introducting some abbreviations
      do i=1,2
      lnnMSu(i)=Log(nMSu(i+4)**2/Qs**2)
      end do
      lnmt=Log(mt**2/Qs**2)
      
      F11a=(12d0*(lnnMSu(1)-1d0)*lnnMSu(1)+pi**2+6d0)*nMSu(5)**2
      F11b=(12d0*(lnnMSu(2)-1d0)*lnnMSu(2)+pi**2+6d0)*nMSu(6)**2
      F12ab=(6+Pi**2+3*lnnMSu(1)**2+6*lnnMSu(1)*(-1+lnnMSu(2)) 
     &       +3*(-2+lnnMSu(2))*lnnMSu(2))*nMSu(5)**2
      F12ba=(6+Pi**2+3*lnnMSu(1)**2+6*lnnMSu(1)*(-1+lnnMSu(2)) 
     &       +3*(-2+lnnMSu(2))*lnnMSu(2))*nMSu(6)**2
      F13ab=(nMSu(5)**2*(-12*(-2+lnnMSu(1))*lnnMSu(1)*nMSu(5)**2+
     -      3*(-4+lnnMSu(1)+lnnMSu(2))*(lnnMSu(1)+lnnMSu(2))*
     -      nMSu(6)**2-(18+Pi**2)*(nMSu(5)**2 - nMSu(6)**2)))/
     -      (nMSu(5)**2-nMSu(6)**2)
      F13ba=(nMSu(6)**2*(3*(-4+lnnMSu(1)+lnnMSu(2))*
     -      (lnnMSu(1)+lnnMSu(2))*nMSu(5)**2 - 
     -      12*(-2+lnnMSu(2))*lnnMSu(2)*nMSu(6)**2+(18+Pi**2)*
     -      (nMSu(5)**2 - nMSu(6)**2)))/(-nMSu(5)**2+nMSu(6)**2)
      F14a=nMSu(5)**2*(30+4*lnnMSu(1)*(-7+3*lnnMSu(1))+Pi**2)
      F14b=nMSu(6)**2*(30+4*lnnMSu(2)*(-7+3*lnnMSu(2))+Pi**2)
      F15=(4*lnmt*(3*lnmt - 5)+Pi**2+18)*mt**3
      Ifindif=absM3*mt*(Ifin(nMSu(5)**2,mt**2,absM3**2)-
     -        Ifin(nMSu(6)**2,mt**2,absM3**2))/(nMSu(5)**2-nMSu(6)**2)
      TFI12a=mt*TFI12(mt**2,absM3**2,nMSu(5)**2)
      TFI12b=mt*TFI12(mt**2,absM3**2,nMSu(6)**2)
      TFI6a=mt*absM3*TFI6(absM3**2,mt**2,nMSu(5)**2)
      TFI6b=mt*absM3*TFI6(absM3**2,mt**2,nMSu(6)**2)
      TFI145=(TFI14(nMSu(5)**2,mt**2,absM3**2) - 
     -         TFI14(nMSu(6)**2,mt**2,absM3**2) - 
     -         TFI5(nMSu(5)**2,mt**2,absM3**2)+
     -         TFI5(nMSu(6)**2,mt**2,absM3**2))/
     -         (nMSu(5)**2 - nMSu(6)**2)
      TFI1215a=TFI12(nMSu(5)**2,mt**2,absM3**2)-
     -         TFI15(nMSu(5)**2,mt**2,absM3**2)
      TFI1215b=TFI12(nMSu(6)**2,mt**2,absM3**2)-
     -         TFI15(nMSu(6)**2,mt**2,absM3**2)
      TFI1116ma=absM3*(TFI11(mt**2,absM3**2,nMSu(5)**2) - 
     -          mt**2*TFI6(nMSu(5)**2,absM3**2,mt**2))
      TFI1116mb=absM3*(TFI11(mt**2,absM3**2,nMSu(6)**2) - 
     -          mt**2*TFI6(nMSu(6)**2,absM3**2,mt**2))
      TFI1116pa=absM3*(TFI11(mt**2,absM3**2,nMSu(5)**2)+
     -          mt**2*TFI6(nMSu(5)**2,absM3**2,mt**2))
      TFI1116pb=absM3*(TFI11(mt**2,absM3**2,nMSu(6)**2)+
     -          mt**2*TFI6(nMSu(6)**2,absM3**2,mt**2))
      
      
      do i=1,5
c genuine two-loop contributions
      tadpole(i)=(g2uh(1,1,i)*((0,64)*TFI1215a*(Abs(gLtFgSu(1))**2+
     -        Abs(gRtFgSu(1))**2)+Pi**4*((0,4)*F14a*g3**2+
     -        4*F11a*g4u(1,1,1,1)+F12ba*(3*g4u(1,2,1,2)+g4u(1,2,2,1)))- 
     -        (0,64)*TFI6a*(CONJG(gRtFgSu(1))*gLtFgSu(1)+
     -        CONJG(gLtFgSu(1))*gRtFgSu(1)))-4*g2uh(2,1,i)*(Pi**4*
     -        (F13ab*g4u(1,1,1,2)+F13ba*g4u(1,2,2,2))+
     -        (0,16)*(CONJG(gLtFgSu(2))*(TFI145*gLtFgSu(1)+
     -        Ifindif*gRtFgSu(1))+CONJG(gRtFgSu(2))*
     -        (Ifindif*gLtFgSu(1)+TFI145*gRtFgSu(1))))+
     -        (0,64)*g2tha(i)*(TFI1116ma*CONJG(gRtFgSu(1))*gLtFgSu(1)+
     -        TFI1116mb*CONJG(gRtFgSu(2))*gLtFgSu(2) - 
     -        TFI1116ma*CONJG(gLtFgSu(1))*gRtFgSu(1) - 
     -        TFI1116mb*CONJG(gLtFgSu(2))*gRtFgSu(2)) - 
     -        (0,32)*g2thh(i)*(F15*g3**2*Pi**4+
     -        4*TFI12a*(Abs(gLtFgSu(1))**2+Abs(gRtFgSu(1))**2)+
     -        4*TFI12b*(Abs(gLtFgSu(2))**2+Abs(gRtFgSu(2))**2)+
     -        2*TFI1116pa*CONJG(gRtFgSu(1))*gLtFgSu(1)+
     -        2*TFI1116pb*CONJG(gRtFgSu(2))*gLtFgSu(2)+
     -        2*TFI1116pa*CONJG(gLtFgSu(1))*gRtFgSu(1)+
     -        2*TFI1116pb*CONJG(gLtFgSu(2))*gRtFgSu(2))+
     -        g2uh(2,2,i)*((0,64)*TFI1215b*(Abs(gLtFgSu(2))**2+
     -        Abs(gRtFgSu(2))**2)+Pi**4*((0,4)*F14b*g3**2+F12ab*
     -        (3*g4u(1,2,1,2)+g4u(2,1,1,2))+4*F11b*g4u(2,2,2,2)) - 
     -        (0,64)*TFI6b*(CONJG(gRtFgSu(2))*gLtFgSu(2)+
     -        CONJG(gLtFgSu(2))*gRtFgSu(2)))-4*g2uh(1,2,i)*(Pi**4*
     -        (F13ab*g4u(1,2,1,1)+F13ba*g4u(2,2,1,2))+
     -        (0,16)*(CONJG(gLtFgSu(1))*(TFI145*gLtFgSu(2)+
     -        Ifindif*gRtFgSu(2))+CONJG(gRtFgSu(1))*
     -        (Ifindif*gLtFgSu(2)+TFI145*gRtFgSu(2)))))/(512.*Pi**8)

c adding counterterm inserted diagrams
      tadpole(i)=tadpole(i)+
     -      ((0,1)/64.*(96*dmtfin*lnmt*mt**2*g2thh(i)-48*dmtdiv1*
     -      lnmt**2*mt**2*g2thh(i)-8*dmtdiv1*mt**2*Pi**2*g2thh(i)- 
     -      4*mt**2*((12+6*(-2+lnmt)*lnmt+Pi**2)*(mt*dg2thhdiv1(i)+
     -      dmtdiv1*g2thh(i))-12*(-1+lnmt)*(mt*dg2thhfin(i)+
     -      dmtfin*g2thh(i)))+Pi**2*dMstdiv1(1,1)*g2uh(1,1,i)+Pi**2*
     -      dMstdiv1(2,2)*g2uh(2,2,i)-12*dMstfin(1,1)*g2uh(1,1,i)*
     -      lnnMSu(1)+6*dMstdiv1(1,1)*g2uh(1,1,i)*lnnMSu(1)**2 - 
     -      12*dMstfin(2,2)*g2uh(2,2,i)*lnnMSu(2)+6*dMstdiv1(2,2)*
     -      g2uh(2,2,i)*lnnMSu(2)**2+(-12*dg2uhfin(1,1,i)*(-1+
     -      lnnMSu(1))+dg2uhdiv1(1,1,i)*(Pi**2+6*(2+(-2+lnnMSu(1))*
     -      lnnMSu(1))))*nMSu(5)**2+(-12*dg2uhfin(2,2,i)*(-1+
     -      lnnMSu(2))+dg2uhdiv1(2,2,i)*(Pi**2+6*(2+(-2+lnnMSu(2))*
     -      lnnMSu(2))))*nMSu(6)**2+(-12*(dMstfin(1,2)*g2uh(1,2,i)+
     -      dMstfin(2,1)*g2uh(2,1,i))*((-1+lnnMSu(1))*nMSu(5)**2 - 
     -      (-1+lnnMSu(2))*nMSu(6)**2)+(dMstdiv1(1,2)*g2uh(1,2,i)+
     -      dMstdiv1(2,1)*g2uh(2,1,i))*((Pi**2+6*(2+(-2+lnnMSu(1))*
     -      lnnMSu(1)))*nMSu(5)**2 -(Pi**2+6*(2+(-2+lnnMSu(2))*
     -      lnnMSu(2)))*nMSu(6)**2))/(nMSu(5)**2 - nMSu(6)**2)))/Pi**2
      end do
      
c rotating back to interaction eigenstates
      dtad=0d0
      dtas=0d0
      dthd=0d0
      dthu=0d0
      dths=0d0
      do i=1,5
      dtad=dtad+ZH5(i,1)*sin(beta)*Realpart(tadpole(i))
      dtas=dtas+ZH5(i,2)*Realpart(tadpole(i))
      dthd=dthd+ZH5(i,3)*Realpart(tadpole(i))
      dthu=dthu+ZH5(i,4)*Realpart(tadpole(i))
      dths=dths+ZH5(i,5)*Realpart(tadpole(i))
      end do
      
      return
      end
      
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cc  This function calucates the two-loop corrections of the order    cc
cc  alpha_s alpha_t to the VEV                                       cc
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      double complex function dvfunc()
      
      implicit none
      
      double precision Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      
      double precision me,mmu,mtau,mu,md,ms,mc,mt,mb,Ye(3),Yd(3),Yu(3)
      
      double precision phiu, phis, phil, phik, phiAk, phiAl, phiM1,
     &                phiM2, phiAu(3), phiAd(3), phiAe(3)
     
      double precision nMSu(6),nMSd(6)
      double complex nZU(6,6),nZD(6,6)
      
      double precision absM3,phiM3
      
      double precision CThetaW,SThetaW
      
      double precision absM1, absM2, absAu(3), absAd(3), absAe(3),  
     &                 ml2(3), mer2(3), mq2(3), mdr2(3), mur2(3)
      
      double complex gZ2u(2,2),g2Z2u(2,2),gZ2ugluon(2,2),gWudgluon(2,2)
     &              ,gZ2tL,gZ2tR,gWtb,g2W2u(2,2),gWud(2,2),g2W2d(2,2)
      
      double complex g2thh(5),g2tha(5),g2uh(2,2,5),g2u2h(2,5,2,5),
     &               g4u(2,2,2,2),gLtFgSu(2),gRtFgSu(2),g4dL,
     &               g2u2dL(2,2),gLbFgdL
     
      double complex dmtfin,dmtdiv1,dMstfin(2,2),dMstdiv1(2,2),
     &               dYtfin,dYtdiv1,dAtfin,dAtdiv1,dphiAtfin,dphiAtdiv1
     
      double complex CtZ,dMZ2,CtW,dMW2
      
      double complex stops,tops,Zsquarkonly1,Zsquarksonly6,Ztgluon,
     &               ZtFg1,ZtFg2,ZtFg3,ZuFg1,ZuFg2,ZuFg3,Zugluona,
     &               Zugluonb,Zugluonc,Zugluond,Zugluone
     
      double complex CTWfermion,CTWsquark1,CTWsquark2,CTWsquark3,
     &               CTWsquark4,Wsquarksonly1a,Wsquarksonly1b,
     &               Wsquarksonly6a,Wsquarksonly6b,WtFg1,WtFg2,WtFg3,
     &               WtFg4,Wtgluon,WudFg1,WudFg2,WudFg3,WudFg4,
     &               Wudgluona,Wudgluonb1,Wudgluonb2,Wudgluonc1,
     &               Wudgluonc2,Wudgluond,Wudgluone
     
      double complex gLbFgSd(2),gRbFgSd(2),g4d(2,2,2,2)
      
      double complex dmtep,dMstep(2,2),dMsbfin(2,2),dMsbdiv1(2,2),
     &                 dMsbep(2,2),dmbLfin,dmbLdiv1
      
      double precision A0,Ae,Idiv1,Idiv2
      double complex Ifin
      
      double complex TFIdv1fin,TFIdv1div1,TFIdv1div2,TFIdv2fin,
     &               TFIdv2div1,TFIdv2div2,TFIdv3fin,TFIdv3div1,
     &               TFIdv3div2,TFIdv4fin,TFIdv4div1,TFIdv4div2,
     &               TFIdv5fin,TFIdv6fin,TFIdv7fin,TFIdv7div1,
     &               TFIdv7div2,TFIdv8fin,TFIdv8div1,TFIdv8div2,
     &               TFIdv9fin,TFIdv9div1,TFIdv9div2,TFIdv10fin,
     &               TFIdv11fin,TFIdv11div1,
     &               TFIdv11div2,TFIdv12fin,TFIdv12div1,TFIdv12div2,
     &               TFIdv13fin,TFIdv13div1,TFIdv13div2,TFIdv14fin,
     &               TFIdv15fin,TFIdv15div1,
     &               TFIdv15div2,TFIdv16fin,TFIdv16div1,TFIdv16div2,
     &               TFIdv17fin,TFIdv17div1,TFIdv17div2,TFIdv18fin,
     &               TFIdv18div1,TFIdv18div2,TFIdv19fin,TFIdv19div1,
     &               TFIdv19div2,TFIdv20fin,TFIdv20div1,TFIdv20div2,
     &               TFIdv21fin,TFIdv21div1,TFIdv21div2,TFIdv22fin,
     &               TFIdv22div1,TFIdv22div2
     
      double complex TFIdv1finnum(2),TFIdv1div1num(2),TFIdv1div2num(2),
     &               TFIdv2finnum(2),TFIdv2div1num(2),TFIdv2div2num(2),
     &               TFIdv3finnum(2),TFIdv3div1num(2),TFIdv3div2num(2),
     &               TFIdv4finnum(2),TFIdv4div1num(2),TFIdv4div2num(2),
     &               TFIdv7finnum(2),TFIdv7div1num(2),TFIdv7div2num(2),
     &               TFIdv8finnum(2),TFIdv8div1num(2),TFIdv8div2num(2),
     &               TFIdv9finnum(2),TFIdv9div1num(2),TFIdv9div2num(2),
     &            TFIdv11finnum(2),TFIdv11div1num(2),TFIdv11div2num(2),
     &            TFIdv13finnum(2),TFIdv13div1num(2),TFIdv13div2num(2),
     &            TFIdv15finnum(2),TFIdv15div1num(2),TFIdv15div2num(2),
     &            TFIdv16finnum(2),TFIdv16div1num(2),TFIdv16div2num(2),
     &            TFIdv17finnum(2),TFIdv17div1num(2),TFIdv17div2num(2),
     &            TFIdv18finnum(2),TFIdv18div1num(2),TFIdv18div2num(2),
     &            TFIdv19finnum(2),TFIdv19div1num(2),TFIdv19div2num(2),
     &            TFIdv20finnum(2),TFIdv20div1num(2),TFIdv20div2num(2),
     &            TFIdv21finnum(2),TFIdv21div1num(2),TFIdv21div2num(2),
     &            TFIdv22finnum(2),TFIdv22div1num(2),TFIdv22div2num(2),
     &            TFIdv2finSu(2),TFIdv2div1Su(2),TFIdv2div2Su(2),
     &            TFIdv7finSu(2),TFIdv7div1Su(2),TFIdv7div2Su(2),
     &            TFIdv7finSd(2),TFIdv7div1Sd(2),TFIdv7div2Sd(2),
     &            TFIdv8finSu(2),TFIdv8div1Su(2),TFIdv8div2Su(2),
     &            TFIdv8finSd(2),TFIdv8div1Sd(2),TFIdv8div2Sd(2),
     &            TFIdv9finSu(2),TFIdv9div1Su(2),TFIdv9div2Su(2),
     &            TFIdv9finSd(2),TFIdv9div1Sd(2),TFIdv9div2Sd(2),
     &            TFIdv15finSu(2),TFIdv15div1Su(2),TFIdv15div2Su(2),
     &            TFIdv15finSd(2),TFIdv15div1Sd(2),TFIdv15div2Sd(2),
     &            TFIdv16finSu(2),TFIdv16div1Su(2),TFIdv16div2Su(2),
     &            TFIdv16finSd(2),TFIdv16div1Sd(2),TFIdv16div2Sd(2)
      
      integer i,j,k,l,ind3,ind4,ind5,ind3d,ind4d,ind5d
      
      double complex Ca,EPhiu
      
      double precision Qs
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
     
      common /SMinput/ Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      common /Yukawa/ me,mmu,mtau,mu,md,ms,mc,mt,mb,Ye,Yd,Yu
      common /phases/ phiu, phis, phil, phik, phiAk, phiAl, phiM1,
     &                phiM2, phiAu, phiAd, phiAe
      common /nSu/ nMSu,nZU
      common /gluino/ absM3,phiM3
      common /softbreak/ absM1, absM2, absAu, absAd, absAe,  
     &                   ml2, mer2, mq2, mdr2, mur2
      common/qcdcoup/ g2thh,g2tha,g2uh,g2u2h,g4u,gLtFgSu,gRtFgSu,g4dL,
     &                 g2u2dL,gLbFgdL
      common/qcdcount/ dmtfin,dmtdiv1,dMstfin,dMstdiv1,dYtfin,dYtdiv1,
     &                 dAtfin,dAtdiv1,dphiAtfin,dphiAtdiv1
     
      common /scale/ Qs
      
      
      
      CThetaW=Mw/Mz
      SThetaW=Sqrt(1-CThetaW**2)
      
      nZD=0d0
      nZD(5,5)=1d0
      nZD(6,6)=1d0
      nMSd(5)=Sqrt(mq2(3))
c set the right-handed bottom to some unlikely high number, because it does not 
c contribute to avoid problems with mass degeneracy
      nMSd(6)=123.45678d0

c Couplings of Z and W needed in the following calcualtion
      do i=1,2
      do j=1,2
      gZ2u(i,j)=-(0,1)/6d0*(((3*CThetaW)/SThetaW - SThetaW/CThetaW)*
     -   CONJG(nZU(4+i,5))*nZU(4+j,5)-(4*SThetaW*CONJG(nZU(4+i,6))*
     -    nZU(4+j,6))/CThetaW)
      g2Z2u(i,j)=((0,1)/18d0*((-3*CThetaW**2+SThetaW**2)**2*
     -   CONJG(nZU(4+i,5))*nZU(4+j,5)+16*SThetaW**4*CONJG(nZU(4+i,6))*
     -   nZU(4+j,6)))/(CThetaW**2*SThetaW**2)
      gZ2ugluon(i,j)=((0,1)/6d0*((3*CThetaW**2 - SThetaW**2)*
     -   CONJG(nZU(4+i,5))*nZU(4+j,5)-4*SThetaW**2*CONJG(nZU(4+i,6))*
     -   nZU(4+j,6)))/(CThetaW*SThetaW)
      gWudgluon(i,j)=((0,1)*CONJG(nZD(i+4,5))*nZU(4+j,5))/
     -   (Sqrt(2d0)*SThetaW)
      end do
      end do
      
      gZ2tL=-(0,1)/6d0*((3*CThetaW)/SThetaW - (SThetaW)/CThetaW)
      gZ2tR=((0,2)/3d0*SThetaW)/CThetaW
      g4d=0d0
      g4d(1,1,1,1)=g4dL
      g4d(1,2,1,2)=g4dL/2d0
      g4d(1,2,2,1)=-3d0*g4dL/2d0
      g4d(2,1,1,2)=-3d0*g4dL/2d0
      g4d(2,1,2,1)=g4dL/2d0
      g4d(2,2,2,2)=g4dL
      
      EPhiu=Cos(phiu)+(0d0,1d0)*Sin(phiu)
c MMM changed 21 Aug 2018      
c      gWtb=((0,-1)*CONJG(Sqrt(EPhiu)))/(Sqrt(2d0)*SThetaW)
      gWtb=((0,-1)*CONJG(exp((0,1) *phiu/2d0)))/(Sqrt(2d0)*SThetaW)
c end MMM changed 21 Aug 2018      
      
      do i=1,2
      do j=1,2
      g2W2u(i,j)=((0,0.5)*CONJG(nZU(4+i,5))*nZU(4+j,5))/SThetaW**2
      g2W2d(i,j)=((0,0.5)*CONJG(nZD(4+i,5))*nZD(4+j,5))/SThetaW**2
      gWud(i,j)=((0,-1)*CONJG(nZU(4+i,5))*nZD(4+j,5))/
     -          (Sqrt(2d0)*SThetaW)
      end do
      end do
      
      gLbFgSd(1)=gLbFgdL
      gLbFgSd(2)=0d0
      gRbFgSd=0d0
      
      Ca=(0d0,1d0)*Pi**2
      

c Contributions to the Z-self-energy
      Ztgluon=-(g3**2*(gZ2tL - gZ2tR)**2*mt**2*
     -     (-9+2*Pi**2+20*Log(mt**2/Qs**2)+24*Log(mt**2/Qs**2)**2))/
     -     (128.*Pi**4)
      
      do ind3=1,2
      TFIdv1finnum(ind3)=TFIdv1fin(mt,nMSu(4+ind3),absM3)
      TFIdv1div1num(ind3)=TFIdv1div1(mt,nMSu(4+ind3),absM3)
      TFIdv1div2num(ind3)=TFIdv1div2(mt,nMSu(4+ind3),absM3)
      
      TFIdv3finnum(ind3)=TFIdv3fin(mt,nMSu(4+ind3),absM3)
      TFIdv3div1num(ind3)=TFIdv3div1(mt,nMSu(4+ind3),absM3)
      TFIdv3div2num(ind3)=TFIdv3div2(mt,nMSu(4+ind3),absM3)
      
      TFIdv8finnum(ind3)=TFIdv8fin(mt,nMSu(4+ind3),absM3)
      TFIdv8div1num(ind3)=TFIdv8div1(mt,nMSu(4+ind3),absM3)
      TFIdv8div2num(ind3)=TFIdv8div2(mt,nMSu(4+ind3),absM3)
      
      TFIdv16finnum(ind3)=TFIdv16fin(mt,nMSu(4+ind3),absM3)
      TFIdv16div1num(ind3)=TFIdv16div1(mt,nMSu(4+ind3),absM3)
      TFIdv16div2num(ind3)=TFIdv16div2(mt,nMSu(4+ind3),absM3)
      
      TFIdv17finnum(ind3)=TFIdv17fin(mt,nMSu(4+ind3),absM3)
      TFIdv17div1num(ind3)=TFIdv17div1(mt,nMSu(4+ind3),absM3)
      TFIdv17div2num(ind3)=TFIdv17div2(mt,nMSu(4+ind3),absM3)
      
      TFIdv18finnum(ind3)=TFIdv18fin(mt,nMSu(4+ind3),absM3)
      TFIdv18div1num(ind3)=TFIdv18div1(mt,nMSu(4+ind3),absM3)
      TFIdv18div2num(ind3)=TFIdv18div2(mt,nMSu(4+ind3),absM3)
      
      TFIdv19finnum(ind3)=TFIdv19fin(mt,nMSu(4+ind3),absM3)
      TFIdv19div1num(ind3)=TFIdv19div1(mt,nMSu(4+ind3),absM3)
      TFIdv19div2num(ind3)=TFIdv19div2(mt,nMSu(4+ind3),absM3)
      
      TFIdv20finnum(ind3)=TFIdv20fin(mt,nMSu(4+ind3),absM3)
      TFIdv20div1num(ind3)=TFIdv20div1(mt,nMSu(4+ind3),absM3)
      TFIdv20div2num(ind3)=TFIdv20div2(mt,nMSu(4+ind3),absM3)
      
      TFIdv21finnum(ind3)=TFIdv21fin(mt,nMSu(4+ind3),absM3)
      TFIdv21div1num(ind3)=TFIdv21div1(mt,nMSu(4+ind3),absM3)
      TFIdv21div2num(ind3)=TFIdv21div2(mt,nMSu(4+ind3),absM3)
      
      TFIdv22finnum(ind3)=TFIdv22fin(mt,nMSu(4+ind3),absM3)
      TFIdv22div1num(ind3)=TFIdv22div1(mt,nMSu(4+ind3),absM3)
      TFIdv22div2num(ind3)=TFIdv22div2(mt,nMSu(4+ind3),absM3)
      end do
      
      
      ZtFg1=0d0
      do ind3=1,2
      do ind4=1,2
      if(ind3.eq.ind4) then
      ZtFg1=ZtFg1+(gZ2u(ind3,ind3)*(CONJG(gLtFgSu(ind3))*(absM3*
     -     (gZ2tL+gZ2tR)*mt*gRtFgSu(ind3)*(2*TFIdv3div1num(ind3)+
     -     TFIdv3div2num(ind3)+4*TFIdv3finnum(ind3))+gLtFgSu(ind3)*
     -     (-2*gZ2tL*mt**2*TFIdv22div1num(ind3)-gZ2tL*mt**2*
     -     TFIdv22div2num(ind3)-4*gZ2tL*mt**2*TFIdv22finnum(ind3)+
     -     2*gZ2tR*TFIdv21div1num(ind3)+gZ2tR*TFIdv21div2num(ind3)+
     -     4*gZ2tR*TFIdv21finnum(ind3)+2*gZ2tR*TFIdv20div1num(ind3)+
     -     gZ2tR*TFIdv20div2num(ind3)+4*gZ2tR*TFIdv20finnum(ind3) - 
     -     4*gZ2tR*TFIdv17div1num(ind3)-2*gZ2tR*TFIdv17div2num(ind3) - 
     -     8*gZ2tR*TFIdv17finnum(ind3)+2*gZ2tL*mt**2*
     -     TFIdv3div1num(ind3)+gZ2tL*mt**2*TFIdv3div2num(ind3)+
     -     4*gZ2tL*mt**2*TFIdv3finnum(ind3)))+CONJG(gRtFgSu(ind3))*
     -     (absM3*(gZ2tL+gZ2tR)*mt*gLtFgSu(ind3)*(2*
     -     TFIdv3div1num(ind3)+TFIdv3div2num(ind3)+4*
     -     TFIdv3finnum(ind3))+gRtFgSu(ind3)*(-2*gZ2tR*mt**2*
     -     TFIdv22div1num(ind3)-gZ2tR*mt**2*TFIdv22div2num(ind3) - 
     -     4*gZ2tR*mt**2*TFIdv22finnum(ind3)+2*gZ2tL*
     -     TFIdv21div1num(ind3)+gZ2tL*TFIdv21div2num(ind3)+
     -     4*gZ2tL*TFIdv21finnum(ind3)+2*gZ2tL*TFIdv20div1num(ind3)+
     -     gZ2tL*TFIdv20div2num(ind3)+4*gZ2tL*TFIdv20finnum(ind3) - 
     -     4*gZ2tL*TFIdv17div1num(ind3)-2*gZ2tL*TFIdv17div2num(ind3) - 
     -     8*gZ2tL*TFIdv17finnum(ind3)+2*gZ2tR*mt**2*
     -     TFIdv3div1num(ind3)+gZ2tR*mt**2*TFIdv3div2num(ind3)+
     -     4*gZ2tR*mt**2*TFIdv3finnum(ind3)))))/(64.*Pi**8)
      else
      ZtFg1=ZtFg1+(gZ2u(ind3,ind4)*(CONJG(gLtFgSu(ind3))*(absM3*
     -     (gZ2tL+gZ2tR)*mt*gRtFgSu(ind4)*(2*TFIdv1div1num(ind3) - 
     -      2*TFIdv1div1num(ind4)+TFIdv1div2num(ind3)-
     -      TFIdv1div2num(ind4)+4*TFIdv1finnum(ind3)-
     -      4*TFIdv1finnum(ind4))+gLtFgSu(ind4)*(-2*gZ2tL*mt**2*
     -      TFIdv19div1num(ind3)+2*gZ2tL*mt**2*TFIdv19div1num(ind4) - 
     -      gZ2tL*mt**2*TFIdv19div2num(ind3)+gZ2tL*mt**2*
     -      TFIdv19div2num(ind4)-4*gZ2tL*mt**2*TFIdv19finnum(ind3)+
     -      4*gZ2tL*mt**2*TFIdv19finnum(ind4)+2*gZ2tR*
     -      TFIdv8div1num(ind3)-2*gZ2tR*TFIdv8div1num(ind4)+
     -      gZ2tR*TFIdv8div2num(ind3)-gZ2tR*TFIdv8div2num(ind4)+
     -      4*gZ2tR*TFIdv8finnum(ind3)-4*gZ2tR*TFIdv8finnum(ind4)+
     -      2*gZ2tR*TFIdv16div1num(ind3)-2*gZ2tR*TFIdv16div1num(ind4)+
     -      gZ2tR*TFIdv16div2num(ind3)-gZ2tR*TFIdv16div2num(ind4)+
     -      4*gZ2tR*TFIdv16finnum(ind3)-4*gZ2tR*TFIdv16finnum(ind4) - 
     -      4*gZ2tR*TFIdv18div1num(ind3)+4*gZ2tR*TFIdv18div1num(ind4) - 
     -      2*gZ2tR*TFIdv18div2num(ind3)+2*gZ2tR*TFIdv18div2num(ind4) - 
     -      8*gZ2tR*TFIdv18finnum(ind3)+8*gZ2tR*TFIdv18finnum(ind4)+
     -      2*gZ2tL*mt**2*TFIdv1div1num(ind3)-2*gZ2tL*mt**2*
     -      TFIdv1div1num(ind4)+gZ2tL*mt**2*TFIdv1div2num(ind3) - 
     -      gZ2tL*mt**2*TFIdv1div2num(ind4)+4*gZ2tL*mt**2*
     -      TFIdv1finnum(ind3)-4*gZ2tL*mt**2*TFIdv1finnum(ind4)))+
     -      CONJG(gRtFgSu(ind3))*(absM3*(gZ2tL+gZ2tR)*mt*gLtFgSu(ind4)*
     -      (2*TFIdv1div1num(ind3)-2*TFIdv1div1num(ind4)+
     -      TFIdv1div2num(ind3)-TFIdv1div2num(ind4)+4*
     -      TFIdv1finnum(ind3)- 4*TFIdv1finnum(ind4))+gRtFgSu(ind4)*
     -      (-2*gZ2tR*mt**2*TFIdv19div1num(ind3)+2*gZ2tR*mt**2*
     -      TFIdv19div1num(ind4)-gZ2tR*mt**2*TFIdv19div2num(ind3)+
     -      gZ2tR*mt**2*TFIdv19div2num(ind4)-4*gZ2tR*mt**2*
     -      TFIdv19finnum(ind3)+4*gZ2tR*mt**2*TFIdv19finnum(ind4)+
     -      2*gZ2tL*TFIdv8div1num(ind3)-2*gZ2tL*TFIdv8div1num(ind4)+
     -      gZ2tL*TFIdv8div2num(ind3)-gZ2tL*TFIdv8div2num(ind4)+
     -      4*gZ2tL*TFIdv8finnum(ind3) -4*gZ2tL*TFIdv8finnum(ind4)+
     -      2*gZ2tL*TFIdv16div1num(ind3)-2*gZ2tL*TFIdv16div1num(ind4)+
     -      gZ2tL*TFIdv16div2num(ind3)-gZ2tL*TFIdv16div2num(ind4)+
     -      4*gZ2tL*TFIdv16finnum(ind3)-4*gZ2tL*TFIdv16finnum(ind4) - 
     -      4*gZ2tL*TFIdv18div1num(ind3)+4*gZ2tL*TFIdv18div1num(ind4)- 
     -      2*gZ2tL*TFIdv18div2num(ind3)+2*gZ2tL*TFIdv18div2num(ind4)- 
     -      8*gZ2tL*TFIdv18finnum(ind3)+8*gZ2tL*TFIdv18finnum(ind4)+
     -      2*gZ2tR*mt**2*TFIdv1div1num(ind3)-2*gZ2tR*mt**2*
     -      TFIdv1div1num(ind4)+gZ2tR*mt**2*TFIdv1div2num(ind3) - 
     -      gZ2tR*mt**2*TFIdv1div2num(ind4)+4*gZ2tR*mt**2*
     -      TFIdv1finnum(ind3)-4*gZ2tR*mt**2*TFIdv1finnum(ind4)))))/
     -      (64.*Pi**8*(nMSu(4+ind3)**2 - nMSu(4+ind4)**2))
      end if
      end do
      end do
      
      ZtFg2=ZtFg1
      
      ZtFg3=0d0
      do ind3=1,2
      ZtFg3=ZtFg3+(CONJG(gRtFgSu(ind3))*(gRtFgSu(ind3)*
     -        (2*gZ2tL**2*TFIdv13div1(mt,absM3,nMSu(4+ind3))+
     -        gZ2tL**2*TFIdv13div2(mt,absM3,nMSu(4+ind3)) - 
     -        4*gZ2tL**2*TFIdv13fin(mt,absM3,nMSu(4+ind3))+
     -        2*gZ2tR**2*mt**2*TFIdv12div1(mt,absM3,nMSu(4+ind3))+
     -        gZ2tR**2*mt**2*TFIdv12div2(mt,absM3,nMSu(4+ind3))+
     -        16*gZ2tL*gZ2tR*mt**2*TFIdv12fin(mt,absM3,nMSu(4+ind3)) - 
     -        4*gZ2tR**2*mt**2*TFIdv12fin(mt,absM3,nMSu(4+ind3)))+
     -        absM3*mt*gLtFgSu(ind3)*
     -        (2*(gZ2tL**2+gZ2tR**2)*TFIdv4div1(mt,absM3,nMSu(4+ind3))+
     -        (gZ2tL**2+gZ2tR**2)*TFIdv4div2(mt,absM3,nMSu(4+ind3)) - 
     -        4*gZ2tL**2*TFIdv4fin(mt,absM3,nMSu(4+ind3))+
     -        8*gZ2tL*gZ2tR*TFIdv4fin(mt,absM3,nMSu(4+ind3)) - 
     -        4*gZ2tR**2*TFIdv4fin(mt,absM3,nMSu(4+ind3))+
     -        8*gZ2tL*gZ2tR*mt**2*TFIdv5fin(mt,absM3,nMSu(4+ind3))))+
     -        CONJG(gLtFgSu(ind3))*(gLtFgSu(ind3)*
     -        (2*gZ2tR**2*TFIdv13div1(mt,absM3,nMSu(4+ind3))+
     -        gZ2tR**2*TFIdv13div2(mt,absM3,nMSu(4+ind3)) - 
     -        4*gZ2tR**2*TFIdv13fin(mt,absM3,nMSu(4+ind3))+
     -        2*gZ2tL**2*mt**2*TFIdv12div1(mt,absM3,nMSu(4+ind3))+
     -        gZ2tL**2*mt**2*TFIdv12div2(mt,absM3,nMSu(4+ind3)) - 
     -        4*gZ2tL**2*mt**2*TFIdv12fin(mt,absM3,nMSu(4+ind3))+
     -        16*gZ2tL*gZ2tR*mt**2*TFIdv12fin(mt,absM3,nMSu(4+ind3)))+
     -        absM3*mt*gRtFgSu(ind3)*
     -        (2*(gZ2tL**2+gZ2tR**2)*TFIdv4div1(mt,absM3,nMSu(4+ind3))+
     -        (gZ2tL**2+gZ2tR**2)*TFIdv4div2(mt,absM3,nMSu(4+ind3)) - 
     -        4*gZ2tL**2*TFIdv4fin(mt,absM3,nMSu(4+ind3))+
     -        8*gZ2tL*gZ2tR*TFIdv4fin(mt,absM3,nMSu(4+ind3)) - 
     -        4*gZ2tR**2*TFIdv4fin(mt,absM3,nMSu(4+ind3))+
     -        8*gZ2tL*gZ2tR*mt**2*TFIdv5fin(mt,absM3,nMSu(4+ind3)))))/
     -        (64.*Pi**8)
      end do
      ZtFg3=2d0*ZtFg3
      
      
      Zsquarkonly1=((0,0.0029296875)*Ca**2*((-16*g4u(1,1,1,1)*
     -      gZ2u(1,1)**2*(A0(nMSu(5)**2)**2 - 2*A0(nMSu(5)**2)*
     -      nMSu(5)**2+2*Ae(nMSu(5)**2)*nMSu(5)**2))/nMSu(5)**2+
     -      (4*(3*g4u(1,2,1,2)+g4u(1,2,2,1))*gZ2u(1,1)**2*
     -      (-(Ae(nMSu(6)**2)*nMSu(5)**2)+A0(nMSu(6)**2)*
     -      (-A0(nMSu(5)**2)+nMSu(5)**2)+(A0(nMSu(5)**2)-
     -      Ae(nMSu(5)**2))*nMSu(6)**2))/nMSu(5)**2+(4*(g4u(2,1,1,2)+
     -      3*g4u(2,1,2,1))*gZ2u(2,2)**2*(-(Ae(nMSu(6)**2)*nMSu(5)**2)+
     -      A0(nMSu(6)**2)*(-A0(nMSu(5)**2)+nMSu(5)**2)+
     -      (A0(nMSu(5)**2)- Ae(nMSu(5)**2))*nMSu(6)**2))/nMSu(6)**2 - 
     -      (16*g4u(2,2,2,2)*gZ2u(2,2)**2*(A0(nMSu(6)**2)**2 - 
     -      2*A0(nMSu(6)**2)*nMSu(6)**2+2*Ae(nMSu(6)**2)*nMSu(6)**2))/
     -      nMSu(6)**2-(4*g4u(2,2,2,2)*gZ2u(1,2)*gZ2u(2,1)*
     -      (4*Ae(nMSu(6)**2)*nMSu(5)**4+4*Ae(nMSu(5)**2)*nMSu(5)**2*
     -      nMSu(6)**2 -16*Ae(nMSu(6)**2)*nMSu(5)**2*nMSu(6)**2+
     -      nMSu(5)**4*nMSu(6)**2+8*Ae(nMSu(6)**2)*nMSu(6)**4 - 
     -      nMSu(6)**6+2*A0(nMSu(5)**2)*nMSu(5)**2*(2*A0(nMSu(6)**2)+
     -      nMSu(6)**2)+A0(nMSu(6)**2)**2*(-8*nMSu(5)**2+4*nMSu(6)**2)+
     -      2*A0(nMSu(6)**2)*(nMSu(5)**4 - 2*nMSu(6)**4)))/
     -      (nMSu(5)**2-nMSu(6)**2)**2-((3*g4u(2,2,1,2)+g4u(2,2,2,1))*
     -      gZ2u(1,2)*gZ2u(2,2)*(4*Ae(nMSu(6)**2)*nMSu(5)**4+
     -      4*Ae(nMSu(5)**2)*nMSu(5)**2*nMSu(6)**2-16*Ae(nMSu(6)**2)*
     -      nMSu(5)**2*nMSu(6)**2+nMSu(5)**4*nMSu(6)**2+
     -      8*Ae(nMSu(6)**2)*nMSu(6)**4 - nMSu(6)**6+2*A0(nMSu(5)**2)*
     -      nMSu(5)**2*(2*A0(nMSu(6)**2)+nMSu(6)**2)+A0(nMSu(6)**2)**2*
     -      (-8*nMSu(5)**2+4*nMSu(6)**2)+2*A0(nMSu(6)**2)*
     -      (nMSu(5)**4-2*nMSu(6)**4)))/(nMSu(5)**2 - nMSu(6)**2)**2 - 
     -      (4*g4u(1,2,2,2)*gZ2u(2,1)*gZ2u(2,2)*(4*Ae(nMSu(6)**2)*
     -      nMSu(5)**4+4*Ae(nMSu(5)**2)*nMSu(5)**2*nMSu(6)**2 - 
     -      16*Ae(nMSu(6)**2)*nMSu(5)**2*nMSu(6)**2+nMSu(5)**4*
     -      nMSu(6)**2+8*Ae(nMSu(6)**2)*nMSu(6)**4 - nMSu(6)**6+
     -      2*A0(nMSu(5)**2)*nMSu(5)**2*(2*A0(nMSu(6)**2)+nMSu(6)**2)+
     -      A0(nMSu(6)**2)**2*(-8*nMSu(5)**2+4*nMSu(6)**2)+
     -      2*A0(nMSu(6)**2)*(nMSu(5)**4 - 2*nMSu(6)**4)))/(nMSu(5)**2- 
     -      nMSu(6)**2)**2-(4*g4u(2,1,1,1)*gZ2u(1,1)*gZ2u(1,2)*
     -      (4*A0(nMSu(5)**2)**2*(nMSu(5)**2 - 2*nMSu(6)**2)+
     -      2*A0(nMSu(5)**2)*(-2*nMSu(5)**4+2*A0(nMSu(6)**2)*
     -      nMSu(6)**2+nMSu(6)**4)+nMSu(5)**2*(-nMSu(5)**4+2*
     -      A0(nMSu(6)**2)*nMSu(6)**2+4*Ae(nMSu(6)**2)*nMSu(6)**2+
     -      nMSu(6)**4)+4*Ae(nMSu(5)**2)*(2*nMSu(5)**4-4*nMSu(5)**2*
     -      nMSu(6)**2+nMSu(6)**4)))/(nMSu(5)**2 - nMSu(6)**2)**2- 
     -      ((g4u(1,1,1,2)+3*g4u(1,1,2,1))*gZ2u(1,1)*gZ2u(2,1)*
     -      (4*A0(nMSu(5)**2)**2*(nMSu(5)**2 - 2*nMSu(6)**2)+
     -      2*A0(nMSu(5)**2)*(-2*nMSu(5)**4+2*A0(nMSu(6)**2)*
     -      nMSu(6)**2+nMSu(6)**4)+nMSu(5)**2*(-nMSu(5)**4+
     -      2*A0(nMSu(6)**2)*nMSu(6)**2+4*Ae(nMSu(6)**2)*nMSu(6)**2+
     -      nMSu(6)**4)+4*Ae(nMSu(5)**2)*(2*nMSu(5)**4-4*nMSu(5)**2*
     -      nMSu(6)**2+nMSu(6)**4)))/(nMSu(5)**2 - nMSu(6)**2)**2 - 
     -      (4*g4u(1,1,1,1)*gZ2u(1,2)*gZ2u(2,1)*(4*A0(nMSu(5)**2)**2*
     -      (nMSu(5)**2-2*nMSu(6)**2)+2*A0(nMSu(5)**2)*(-2*nMSu(5)**4+
     -      2*A0(nMSu(6)**2)*nMSu(6)**2+nMSu(6)**4)+nMSu(5)**2*
     -      (-nMSu(5)**4+2*A0(nMSu(6)**2)*nMSu(6)**2+4*Ae(nMSu(6)**2)*
     -      nMSu(6)**2+nMSu(6)**4)+4*Ae(nMSu(5)**2)*(2*nMSu(5)**4 - 
     -      4*nMSu(5)**2*nMSu(6)**2+nMSu(6)**4)))/(nMSu(5)**2-
     -      nMSu(6)**2)**2-((3*g4u(2,2,1,2)+g4u(2,2,2,1))*gZ2u(1,1)*
     -      gZ2u(1,2)*(4*Ae(nMSu(6)**2)*nMSu(5)**4+4*A0(nMSu(6)**2)**2*
     -      nMSu(6)**2+4*Ae(nMSu(5)**2)*nMSu(5)**2*nMSu(6)**2 - 
     -      8*Ae(nMSu(6)**2)*nMSu(5)**2*nMSu(6)**2-nMSu(5)**4*
     -      nMSu(6)**2-8*Ae(nMSu(5)**2)*nMSu(6)**4+8*Ae(nMSu(6)**2)*
     -      nMSu(6)**4+nMSu(6)**6 - 2*A0(nMSu(6)**2)*(nMSu(5)**4 - 
     -      2*nMSu(6)**4)+A0(nMSu(5)**2)*(-2*nMSu(5)**2*nMSu(6)**2+
     -      4*A0(nMSu(6)**2)*(nMSu(5)**2 - 2*nMSu(6)**2))))/
     -      (nMSu(5)**2 - nMSu(6)**2)**2-(4*g4u(1,2,2,2)*gZ2u(1,1)*
     -      gZ2u(2,1)*(4*Ae(nMSu(6)**2)*nMSu(5)**4+4*A0(nMSu(6)**2)**2*
     -      nMSu(6)**2+4*Ae(nMSu(5)**2)*nMSu(5)**2*nMSu(6)**2 - 
     -      8*Ae(nMSu(6)**2)*nMSu(5)**2*nMSu(6)**2 - nMSu(5)**4*
     -      nMSu(6)**2-8*Ae(nMSu(5)**2)*nMSu(6)**4+8*Ae(nMSu(6)**2)*
     -      nMSu(6)**4+nMSu(6)**6 - 2*A0(nMSu(6)**2)*(nMSu(5)**4 - 
     -      2*nMSu(6)**4)+A0(nMSu(5)**2)*(-2*nMSu(5)**2*nMSu(6)**2+
     -      4*A0(nMSu(6)**2)*(nMSu(5)**2-2*nMSu(6)**2))))/(nMSu(5)**2-
     -      nMSu(6)**2)**2-((3*g4u(1,2,1,2)+g4u(1,2,2,1))*gZ2u(1,2)*
     -      gZ2u(2,1)*(4*Ae(nMSu(6)**2)*nMSu(5)**4+4*A0(nMSu(6)**2)**2*
     -      nMSu(6)**2+4*Ae(nMSu(5)**2)*nMSu(5)**2*nMSu(6)**2 - 
     -      8*Ae(nMSu(6)**2)*nMSu(5)**2*nMSu(6)**2 - nMSu(5)**4*
     -      nMSu(6)**2-8*Ae(nMSu(5)**2)*nMSu(6)**4+8*Ae(nMSu(6)**2)*
     -      nMSu(6)**4+nMSu(6)**6 - 2*A0(nMSu(6)**2)*(nMSu(5)**4 - 
     -      2*nMSu(6)**4)+A0(nMSu(5)**2)*(-2*nMSu(5)**2*nMSu(6)**2+
     -      4*A0(nMSu(6)**2)*(nMSu(5)**2 - 2*nMSu(6)**2))))/
     -      (nMSu(5)**2-nMSu(6)**2)**2-((g4u(2,1,1,2)+3*g4u(2,1,2,1))*
     -      gZ2u(1,2)*gZ2u(2,1)*(4*A0(nMSu(5)**2)**2*nMSu(5)**2+
     -      4*Ae(nMSu(5)**2)*(2*nMSu(5)**4 - 2*nMSu(5)**2*nMSu(6)**2+
     -      nMSu(6)**4) - 2*A0(nMSu(5)**2)*(-2*nMSu(5)**4+nMSu(6)**4+
     -      A0(nMSu(6)**2)*(4*nMSu(5)**2 - 2*nMSu(6)**2))+nMSu(5)**2*
     -      (nMSu(5)**4 - 2*A0(nMSu(6)**2)*nMSu(6)**2-nMSu(6)**4+
     -      Ae(nMSu(6)**2)*(-8*nMSu(5)**2+4*nMSu(6)**2))))/(nMSu(5)**2-
     -      nMSu(6)**2)**2-(4*g4u(2,1,1,1)*gZ2u(1,2)*gZ2u(2,2)*
     -      (4*A0(nMSu(5)**2)**2*nMSu(5)**2+4*Ae(nMSu(5)**2)*
     -      (2*nMSu(5)**4 - 2*nMSu(5)**2*nMSu(6)**2+nMSu(6)**4)-
     -      2*A0(nMSu(5)**2)*(-2*nMSu(5)**4+nMSu(6)**4+A0(nMSu(6)**2)*
     -      (4*nMSu(5)**2 - 2*nMSu(6)**2))+nMSu(5)**2*(nMSu(5)**4 - 
     -      2*A0(nMSu(6)**2)*nMSu(6)**2 - nMSu(6)**4+Ae(nMSu(6)**2)*
     -      (-8*nMSu(5)**2+4*nMSu(6)**2))))/(nMSu(5)**2-nMSu(6)**2)**2- 
     -      ((g4u(1,1,1,2)+3*g4u(1,1,2,1))*gZ2u(2,1)*gZ2u(2,2)*
     -      (4*A0(nMSu(5)**2)**2*nMSu(5)**2+4*Ae(nMSu(5)**2)*
     -      (2*nMSu(5)**4 - 2*nMSu(5)**2*nMSu(6)**2+nMSu(6)**4)-
     -      2*A0(nMSu(5)**2)*(-2*nMSu(5)**4+nMSu(6)**4+A0(nMSu(6)**2)*
     -      (4*nMSu(5)**2 - 2*nMSu(6)**2))+nMSu(5)**2*(nMSu(5)**4-
     -      2*A0(nMSu(6)**2)*nMSu(6)**2 - nMSu(6)**4+Ae(nMSu(6)**2)*
     -      (-8*nMSu(5)**2+4*nMSu(6)**2))))/
     -      (nMSu(5)**2 - nMSu(6)**2)**2))/Pi**8
      Zsquarkonly1=2d0*Zsquarkonly1
      
      
      Zsquarksonly6=0d0
      do ind3=1,2
      do ind4=1,2
      do ind5=1,2
      if(ind3.eq.ind4) then
      Zsquarksonly6=Zsquarksonly6+
     -      (3*Ca**2*g2Z2u(ind3,ind3)*(3*g4u(ind3,ind5,ind3,ind5)+
     -      g4u(ind3,ind5,ind5,ind3))*(-(Ae(nMSu(4+ind5)**2)*
     -      nMSu(4+ind3)**2)+A0(nMSu(4+ind5)**2)*(-A0(nMSu(4+ind3)**2)+
     -      nMSu(4+ind3)**2)+(A0(nMSu(4+ind3)**2)-Ae(nMSu(4+ind3)**2))*
     -      nMSu(4+ind5)**2))/(256.*Pi**8*nMSu(4+ind3)**2)
      else
      Zsquarksonly6=Zsquarksonly6+
     -     (-3*Ca**2*g2Z2u(ind3,ind4)*(3*g4u(ind4,ind5,ind3,ind5)+
     -     g4u(ind4,ind5,ind5,ind3))*((A0(nMSu(4+ind3)**2)-
     -     A0(nMSu(4+ind4)**2))*A0(nMSu(4+ind5)**2)+
     -     Ae(nMSu(4+ind5)**2)*(nMSu(4+ind3)**2 - nMSu(4+ind4)**2)+
     -     (Ae(nMSu(4+ind3)**2)-Ae(nMSu(4+ind4)**2))*nMSu(4+ind5)**2))/
     -     (256.*Pi**8*(nMSu(4+ind3)**2 - nMSu(4+ind4)**2))
      end if
      end do
      end do
      end do
      
      Zugluona=0d0
      do ind3=1,2
      do ind4=1,2
      if(ind3.eq.ind4) then
      Zugluona=Zugluona+
     -      (Ca**2*g3**2*gZ2u(ind3,ind3)**2*(4*A0(nMSu(4+ind3)**2)**2+
     -      20*A0(nMSu(4+ind3)**2)*nMSu(4+ind3)**2+
     -      8*Ae(nMSu(4+ind3)**2)*nMSu(4+ind3)**2+nMSu(4+ind3)**4))/
     -      (128.*Pi**8*nMSu(4+ind3)**2)
      else
      Zugluona=Zugluona+
     -      (g3**2*gZ2u(ind3,ind4)*gZ2u(ind4,ind3)*(Ca**2*
     -      nMSu(4+ind3)**6+Ca**2*nMSu(4+ind4)**6+Ca**2*
     -      nMSu(4+ind3)**2*nMSu(4+ind4)**2*(nMSu(4+ind3)**2+
     -      nMSu(4+ind4)**2)-
     -      Idiv2(nMSu(4+ind4)**2,nMSu(4+ind3)**2,0d0)*
     -      (nMSu(4+ind3)**2+nMSu(4+ind4)**2)**2+2*(2*Ca**2*
     -      nMSu(4+ind3)**4*(A0(nMSu(4+ind3)**2)+nMSu(4+ind3)**2)+
     -      2*Ca**2*nMSu(4+ind4)**4*(A0(nMSu(4+ind4)**2)+
     -      nMSu(4+ind4)**2)-
     -      Idiv1(nMSu(4+ind4)**2,nMSu(4+ind3)**2,0d0)*
     -      (nMSu(4+ind3)**2+nMSu(4+ind4)**2)**2+Ca**2*
     -      (nMSu(4+ind3)**2+nMSu(4+ind4)**2)*(A0(nMSu(4+ind4)**2)*
     -      nMSu(4+ind3)**2+A0(nMSu(4+ind3)**2)*nMSu(4+ind4)**2))+
     -      4*(Ca**2*nMSu(4+ind3)**2*(A0(nMSu(4+ind3)**2)**2+
     -      4*A0(nMSu(4+ind3)**2)*nMSu(4+ind3)**2+2*Ae(nMSu(4+ind3)**2)
     -      *nMSu(4+ind3)**2+4*nMSu(4+ind3)**4) - 
     -      Ifin(nMSu(4+ind4)**2,nMSu(4+ind3)**2,0d0)*(nMSu(4+ind3)**2+
     -      nMSu(4+ind4)**2)**2+Ca**2*(nMSu(4+ind3)**2+nMSu(4+ind4)**2)
     -      *(A0(nMSu(4+ind3)**2)*A0(nMSu(4+ind4)**2)+
     -      Ae(nMSu(4+ind4)**2)*nMSu(4+ind3)**2+Ae(nMSu(4+ind3)**2)*
     -      nMSu(4+ind4)**2)+Ca**2*nMSu(4+ind4)**2*
     -      (A0(nMSu(4+ind4)**2)**2+4*A0(nMSu(4+ind4)**2)*
     -      nMSu(4+ind4)**2+2*Ae(nMSu(4+ind4)**2)*nMSu(4+ind4)**2+
     -      4*nMSu(4+ind4)**4))))/
     -      (128.*Pi**8*(nMSu(4+ind3)**2-nMSu(4+ind4)**2)**2)
      end if
      end do
      end do
      
      Zugluonb=0d0
      do ind3=1,2
      do ind4=1,2
      if(ind3.eq.ind4) then
      Zugluonb=Zugluonb+
     -     (Ca**2*g3**2*gZ2u(ind3,ind3)**2*(16*A0(nMSu(4+ind3)**2)**2+
     -     4*A0(nMSu(4+ind3)**2)*nMSu(4+ind3)**2+
     -     32*Ae(nMSu(4+ind3)**2)*nMSu(4+ind3)**2+31*nMSu(4+ind3)**4))/
     -     (256.*Pi**8*nMSu(4+ind3)**2)
      else
      Zugluonb=Zugluonb-
     -     (g3**2*gZ2u(ind3,ind4)*gZ2u(ind4,ind3)*(4*Ca**2*
     -     Ae(nMSu(4+ind4)**2)*nMSu(4+ind3)**4 - 
     -     4*Idiv1(nMSu(4+ind4)**2,nMSu(4+ind3)**2,0d0)*
     -     nMSu(4+ind3)**4 - nMSu(4+ind3)**4*
     -     2*Idiv2(nMSu(4+ind4)**2,nMSu(4+ind3)**2,0d0)- 
     -     8*Ifin(nMSu(4+ind4)**2,nMSu(4+ind3)**2,0d0)*nMSu(4+ind3)**4+
     -     4*Ca**2*Ae(nMSu(4+ind3)**2)*nMSu(4+ind3)**2*nMSu(4+ind4)**2+
     -     48*Ca**2*Ae(nMSu(4+ind4)**2)*nMSu(4+ind3)**2*
     -     nMSu(4+ind4)**2-4*Idiv1(nMSu(4+ind4)**2,nMSu(4+ind3)**2,0d0)
     -     *nMSu(4+ind3)**2*nMSu(4+ind4)**2 - 
     -     2*Idiv2(nMSu(4+ind4)**2,nMSu(4+ind3)**2,0d0)*
     -     nMSu(4+ind3)**2*nMSu(4+ind4)**2 - 
     -     8*Ifin(nMSu(4+ind4)**2,nMSu(4+ind3)**2,0d0)*nMSu(4+ind3)**2*
     -     nMSu(4+ind4)**2+Ca**2*nMSu(4+ind3)**4*nMSu(4+ind4)**2 - 
     -     24*Ca**2*Ae(nMSu(4+ind4)**2)*nMSu(4+ind4)**4+64*Ca**2*
     -     nMSu(4+ind3)**2*nMSu(4+ind4)**4-21*Ca**2*nMSu(4+ind4)**6+
     -     12*Ca**2*A0(nMSu(4+ind4)**2)**2*(2*nMSu(4+ind3)**2-
     -     nMSu(4+ind4)**2)+2*Ca**2*A0(nMSu(4+ind3)**2)*
     -     nMSu(4+ind3)**2*(2*A0(nMSu(4+ind4)**2)+nMSu(4+ind4)**2)+
     -     2*Ca**2*A0(nMSu(4+ind4)**2)*(nMSu(4+ind3)**4+32*
     -     nMSu(4+ind3)**2*nMSu(4+ind4)**2-10*nMSu(4+ind4)**4)))/
     -     (256.*Pi**8*(nMSu(4+ind3)**2 - nMSu(4+ind4)**2)**2)
      end if
      end do
      end do
      Zugluonb=2d0*Zugluonb
      
      Zugluonc=0d0
      do ind3=1,2
      Zugluonc=Zugluonc-
     -     ((0,0.015625)*Ca**2*g3**2*g2Z2u(ind3,ind3)*(3*
     -     A0(nMSu(4+ind3)**2)**2+2*A0(nMSu(4+ind3)**2)*
     -     nMSu(4+ind3)**2+6*Ae(nMSu(4+ind3)**2)*nMSu(4+ind3)**2+
     -     4*nMSu(4+ind3)**4))/(Pi**8*nMSu(4+ind3)**2)
      end do
      
      
      do ind3=1,2
      TFIdv2finnum(ind3)=TFIdv2fin(nMSu(4+ind3),mt,absM3)
      TFIdv2div1num(ind3)=TFIdv2div1(nMSu(4+ind3),mt,absM3)
      TFIdv2div2num(ind3)=TFIdv2div2(nMSu(4+ind3),mt,absM3)
      
      TFIdv4finnum(ind3)=TFIdv4fin(nMSu(4+ind3),mt,absM3)
      TFIdv4div1num(ind3)=TFIdv4div1(nMSu(4+ind3),mt,absM3)
      TFIdv4div2num(ind3)=TFIdv4div2(nMSu(4+ind3),mt,absM3)
      
      TFIdv7finnum(ind3)=TFIdv7fin(nMSu(4+ind3),mt,absM3)
      TFIdv7div1num(ind3)=TFIdv7div1(nMSu(4+ind3),mt,absM3)
      TFIdv7div2num(ind3)=TFIdv7div2(nMSu(4+ind3),mt,absM3)
      
      TFIdv8finnum(ind3)=TFIdv8fin(nMSu(4+ind3),mt,absM3)
      TFIdv8div1num(ind3)=TFIdv8div1(nMSu(4+ind3),mt,absM3)
      TFIdv8div2num(ind3)=TFIdv8div2(nMSu(4+ind3),mt,absM3)
      
      TFIdv9finnum(ind3)=TFIdv9fin(nMSu(4+ind3),mt,absM3)
      TFIdv9div1num(ind3)=TFIdv9div1(nMSu(4+ind3),mt,absM3)
      TFIdv9div2num(ind3)=TFIdv9div1(nMSu(4+ind3),mt,absM3)
      
      TFIdv11finnum(ind3)=TFIdv11fin(nMSu(4+ind3),mt,absM3)
      TFIdv11div1num(ind3)=TFIdv11div1(nMSu(4+ind3),mt,absM3)
      TFIdv11div2num(ind3)=TFIdv11div2(nMSu(4+ind3),mt,absM3)
      
      TFIdv13finnum(ind3)=TFIdv13fin(nMSu(4+ind3),mt,absM3)
      TFIdv13div1num(ind3)=TFIdv13div1(nMSu(4+ind3),mt,absM3)
      TFIdv13div2num(ind3)=TFIdv13div2(nMSu(4+ind3),mt,absM3)
      
      TFIdv15finnum(ind3)=TFIdv15fin(nMSu(4+ind3),mt,absM3)
      TFIdv15div1num(ind3)=TFIdv15div1(nMSu(4+ind3),mt,absM3)
      TFIdv15div2num(ind3)=TFIdv15div2(nMSu(4+ind3),mt,absM3)
      
      TFIdv16finnum(ind3)=TFIdv16fin(nMSu(4+ind3),mt,absM3)
      TFIdv16div1num(ind3)=TFIdv16div1(nMSu(4+ind3),mt,absM3)
      TFIdv16div2num(ind3)=TFIdv16div2(nMSu(4+ind3),mt,absM3)
      end do
      
      ZuFg1=0d0
      do ind3=1,2
      do ind4=1,2
      do ind5=1,2
      if((ind3.eq.ind4) .AND. (ind3.eq.ind5)) then
      ZuFg1=ZuFg1-(gZ2u(ind3,ind3)**2*(CONJG(gRtFgSu(ind3))*
     -     (gRtFgSu(ind3)*(2*TFIdv13div1num(ind3)+TFIdv13div2num(ind3)+
     -     4*TFIdv13finnum(ind3)-2*TFIdv11div1num(ind3)-
     -     TFIdv11div2num(ind3)-4*TFIdv11finnum(ind3))-absM3*mt*
     -     gLtFgSu(ind3)*(2*TFIdv4div1num(ind3)+TFIdv4div2num(ind3)+
     -     4*TFIdv4finnum(ind3)))+CONJG(gLtFgSu(ind3))*(gLtFgSu(ind3)*
     -     (2*TFIdv13div1num(ind3)+TFIdv13div2num(ind3)+
     -     4*TFIdv13finnum(ind3)-2*TFIdv11div1num(ind3)- 
     -     TFIdv11div2num(ind3)-4*TFIdv11finnum(ind3))-absM3*mt*
     -     gRtFgSu(ind3)*(2*TFIdv4div1num(ind3)+
     -     TFIdv4div2num(ind3)+4*TFIdv4finnum(ind3)))))/(32.*Pi**8)
      elseif((ind3.eq.ind4) .AND. (ind3.ne.ind5)) then
      ZuFg1=ZuFg1-(gZ2u(ind3,ind3)*gZ2u(ind3,ind5)*
     -    (CONJG(gRtFgSu(ind3))*(gRtFgSu(ind5)*(-2*TFIdv9div1num(ind3)+
     -    2*TFIdv9div1num(ind5)-TFIdv9div2num(ind3)+
     -    TFIdv9div2num(ind5)-4*TFIdv9finnum(ind3)+
     -    4*TFIdv9finnum(ind5)+nMSu(4+ind3)**2*(2*TFIdv8div1num(ind3)+
     -    TFIdv8div2num(ind3)+4*TFIdv8finnum(ind3) - 
     -    2*TFIdv16div1num(ind3)-TFIdv16div2num(ind3) - 
     -    4*TFIdv16finnum(ind3))+nMSu(4+ind5)**2*(-2*
     -    TFIdv8div1num(ind3)-TFIdv8div2num(ind3)-4*TFIdv8finnum(ind3)+
     -    2*TFIdv16div1num(ind3)+TFIdv16div2num(ind3)+
     -    4*TFIdv16finnum(ind3))+2*TFIdv15div1num(ind3)- 
     -    2*TFIdv15div1num(ind5)+TFIdv15div2num(ind3) - 
     -    TFIdv15div2num(ind5)+4*TFIdv15finnum(ind3) - 
     -    4*TFIdv15finnum(ind5))+absM3*mt*gLtFgSu(ind5)*
     -    (2*TFIdv7div1num(ind3)-2*TFIdv7div1num(ind5)+
     -    TFIdv7div2num(ind3)-TFIdv7div2num(ind5)+
     -    4*TFIdv7finnum(ind3)-4*TFIdv7finnum(ind5) - 
     -    2*nMSu(4+ind3)**2*TFIdv2div1num(ind3)+2*nMSu(4+ind5)**2*
     -    TFIdv2div1num(ind3)-nMSu(4+ind3)**2*TFIdv2div2num(ind3)+
     -    nMSu(4+ind5)**2*TFIdv2div2num(ind3)-4*nMSu(4+ind3)**2*
     -    TFIdv2finnum(ind3)+4*nMSu(4+ind5)**2*TFIdv2finnum(ind3)))+
     -    CONJG(gLtFgSu(ind3))*(gLtFgSu(ind5)*(-2*TFIdv9div1num(ind3)+
     -    2*TFIdv9div1num(ind5) -TFIdv9div2num(ind3)+
     -    TFIdv9div2num(ind5)-4*TFIdv9finnum(ind3)+
     -    4*TFIdv9finnum(ind5)+nMSu(4+ind3)**2*(2*TFIdv8div1num(ind3)+
     -    TFIdv8div2num(ind3)+4*TFIdv8finnum(ind3) - 
     -    2*TFIdv16div1num(ind3)-TFIdv16div2num(ind3) - 
     -    4*TFIdv16finnum(ind3))+nMSu(4+ind5)**2*(-2*
     -    TFIdv8div1num(ind3)-TFIdv8div2num(ind3) - 
     -    4*TFIdv8finnum(ind3)+2*TFIdv16div1num(ind3)+
     -    TFIdv16div2num(ind3)+4*TFIdv16finnum(ind3))+
     -    2*TFIdv15div1num(ind3)-2*TFIdv15div1num(ind5)+
     -    TFIdv15div2num(ind3)-TFIdv15div2num(ind5)+
     -    4*TFIdv15finnum(ind3)-4*TFIdv15finnum(ind5))+
     -    absM3*mt*gRtFgSu(ind5)*(2*TFIdv7div1num(ind3) - 
     -    2*TFIdv7div1num(ind5)+TFIdv7div2num(ind3) - 
     -    TFIdv7div2num(ind5)+4*TFIdv7finnum(ind3) - 
     -    4*TFIdv7finnum(ind5)-2*nMSu(4+ind3)**2*TFIdv2div1num(ind3)+
     -    2*nMSu(4+ind5)**2*TFIdv2div1num(ind3)-nMSu(4+ind3)**2*
     -    TFIdv2div2num(ind3)+nMSu(4+ind5)**2*TFIdv2div2num(ind3)- 
     -    4*nMSu(4+ind3)**2*TFIdv2finnum(ind3)+4*nMSu(4+ind5)**2*
     -    TFIdv2finnum(ind3)))))/
     -    (32.*Pi**8*(nMSu(4+ind3)**2 - nMSu(4+ind5)**2)**2)
      elseif((ind3.eq.ind5) .AND. (ind3.ne.ind4)) then
      ZuFg1=ZuFg1-(gZ2u(ind3,ind3)*gZ2u(ind4,ind3)*
     -    (CONJG(gRtFgSu(ind4))*(gRtFgSu(ind3)*(-2*TFIdv9div1num(ind3)+
     -    2*TFIdv9div1num(ind4)-TFIdv9div2num(ind3)+
     -    TFIdv9div2num(ind4)-4*TFIdv9finnum(ind3)+
     -    4*TFIdv9finnum(ind4)+nMSu(4+ind3)**2*(2*TFIdv8div1num(ind3)+
     -    TFIdv8div2num(ind3)+4*TFIdv8finnum(ind3) - 
     -    2*TFIdv16div1num(ind3)-TFIdv16div2num(ind3)- 
     -    4*TFIdv16finnum(ind3))+nMSu(4+ind4)**2*(-2*
     -    TFIdv8div1num(ind3)-TFIdv8div2num(ind3)-4*TFIdv8finnum(ind3)+
     -    2*TFIdv16div1num(ind3)+TFIdv16div2num(ind3)+
     -    4*TFIdv16finnum(ind3))+2*TFIdv15div1num(ind3) - 
     -    2*TFIdv15div1num(ind4)+TFIdv15div2num(ind3)-
     -    TFIdv15div2num(ind4)+4*TFIdv15finnum(ind3) - 
     -    4*TFIdv15finnum(ind4))+absM3*mt*gLtFgSu(ind3)*
     -    (2*TFIdv7div1num(ind3)-2*TFIdv7div1num(ind4)+
     -    TFIdv7div2num(ind3)-TFIdv7div2num(ind4)+4*TFIdv7finnum(ind3)- 
     -    4*TFIdv7finnum(ind4)-2*nMSu(4+ind3)**2*TFIdv2div1num(ind3)+
     -    2*nMSu(4+ind4)**2*TFIdv2div1num(ind3)-nMSu(4+ind3)**2*
     -    TFIdv2div2num(ind3)+nMSu(4+ind4)**2*TFIdv2div2num(ind3)- 
     -    4*nMSu(4+ind3)**2*TFIdv2finnum(ind3)+4*nMSu(4+ind4)**2*
     -    TFIdv2finnum(ind3)))+CONJG(gLtFgSu(ind4))*(gLtFgSu(ind3)*
     -    (-2*TFIdv9div1num(ind3)+2*TFIdv9div1num(ind4) - 
     -    TFIdv9div2num(ind3)+TFIdv9div2num(ind4)-4*TFIdv9finnum(ind3)+
     -    4*TFIdv9finnum(ind4)+nMSu(4+ind3)**2*(2*TFIdv8div1num(ind3)+
     -    TFIdv8div2num(ind3)+4*TFIdv8finnum(ind3)- 
     -    2*TFIdv16div1num(ind3)-TFIdv16div2num(ind3)- 
     -    4*TFIdv16finnum(ind3))+nMSu(4+ind4)**2*
     -    (-2*TFIdv8div1num(ind3)-TFIdv8div2num(ind3)- 
     -    4*TFIdv8finnum(ind3)+2*TFIdv16div1num(ind3)+
     -    TFIdv16div2num(ind3)+4*TFIdv16finnum(ind3))+
     -    2*TFIdv15div1num(ind3)-2*TFIdv15div1num(ind4)+
     -    TFIdv15div2num(ind3)-TFIdv15div2num(ind4)+
     -    4*TFIdv15finnum(ind3)-4*TFIdv15finnum(ind4))+absM3*mt*
     -    gRtFgSu(ind3)*(2*TFIdv7div1num(ind3)-2*TFIdv7div1num(ind4)+
     -    TFIdv7div2num(ind3)-TFIdv7div2num(ind4)+
     -    4*TFIdv7finnum(ind3)-4*TFIdv7finnum(ind4) - 
     -    2*nMSu(4+ind3)**2*TFIdv2div1num(ind3)+2*nMSu(4+ind4)**2*
     -    TFIdv2div1num(ind3)-nMSu(4+ind3)**2*TFIdv2div2num(ind3)+
     -    nMSu(4+ind4)**2*TFIdv2div2num(ind3)-4*nMSu(4+ind3)**2*
     -    TFIdv2finnum(ind3)+4*nMSu(4+ind4)**2*TFIdv2finnum(ind3)))))/
     -    (32.*Pi**8*(nMSu(4+ind3)**2 - nMSu(4+ind4)**2)**2)
      elseif((ind3.ne.ind4) .AND. (ind4.eq.ind5)) then
      ZuFg1=ZuFg1+(gZ2u(ind3,ind4)*gZ2u(ind4,ind3)*
     -    (CONJG(gRtFgSu(ind4))*(gRtFgSu(ind4)*(-2*TFIdv9div1num(ind3)+
     -    2*TFIdv9div1num(ind4)-TFIdv9div2num(ind3)+
     -    TFIdv9div2num(ind4)-4*TFIdv9finnum(ind3)+
     -    4*TFIdv9finnum(ind4)+nMSu(4+ind3)**2*(2*TFIdv8div1num(ind4)+
     -    TFIdv8div2num(ind4)+4*TFIdv8finnum(ind4)-2*
     -    TFIdv16div1num(ind4)-TFIdv16div2num(ind4) - 
     -    4*TFIdv16finnum(ind4))+nMSu(4+ind4)**2*
     -    (-2*TFIdv8div1num(ind4)-TFIdv8div2num(ind4) - 
     -    4*TFIdv8finnum(ind4)+2*TFIdv16div1num(ind4)+
     -    TFIdv16div2num(ind4)+4*TFIdv16finnum(ind4))+
     -    2*TFIdv15div1num(ind3)-2*TFIdv15div1num(ind4)+
     -    TFIdv15div2num(ind3)-TFIdv15div2num(ind4)+
     -    4*TFIdv15finnum(ind3)-4*TFIdv15finnum(ind4))+absM3*mt*
     -    gLtFgSu(ind4)*(2*TFIdv7div1num(ind3)-2*TFIdv7div1num(ind4)+
     -    TFIdv7div2num(ind3)-TFIdv7div2num(ind4)+
     -    4*TFIdv7finnum(ind3)-4*TFIdv7finnum(ind4)-2*nMSu(4+ind3)**2*
     -    TFIdv2div1num(ind4)+2*nMSu(4+ind4)**2*TFIdv2div1num(ind4)- 
     -    nMSu(4+ind3)**2*TFIdv2div2num(ind4)+nMSu(4+ind4)**2*
     -    TFIdv2div2num(ind4)-4*nMSu(4+ind3)**2*TFIdv2finnum(ind4)+
     -    4*nMSu(4+ind4)**2*TFIdv2finnum(ind4)))+CONJG(gLtFgSu(ind4))*
     -    (gLtFgSu(ind4)*(-2*TFIdv9div1num(ind3)+2*TFIdv9div1num(ind4)- 
     -    TFIdv9div2num(ind3)+TFIdv9div2num(ind4)-4*TFIdv9finnum(ind3)+
     -    4*TFIdv9finnum(ind4)+nMSu(4+ind3)**2*(2*TFIdv8div1num(ind4)+
     -    TFIdv8div2num(ind4)+4*TFIdv8finnum(ind4)-
     -    2*TFIdv16div1num(ind4)-TFIdv16div2num(ind4) - 
     -    4*TFIdv16finnum(ind4))+nMSu(4+ind4)**2*(-2*
     -    TFIdv8div1num(ind4)-TFIdv8div2num(ind4) - 
     -    4*TFIdv8finnum(ind4)+2*TFIdv16div1num(ind4)+
     -    TFIdv16div2num(ind4)+4*TFIdv16finnum(ind4))+
     -    2*TFIdv15div1num(ind3)-2*TFIdv15div1num(ind4)+
     -    TFIdv15div2num(ind3) -TFIdv15div2num(ind4)+4*
     -    TFIdv15finnum(ind3)-4*TFIdv15finnum(ind4))+absM3*mt*
     -    gRtFgSu(ind4)*(2*TFIdv7div1num(ind3)-2*TFIdv7div1num(ind4)+
     -    TFIdv7div2num(ind3) - TFIdv7div2num(ind4)+4*
     -    TFIdv7finnum(ind3)-4*TFIdv7finnum(ind4)-2*nMSu(4+ind3)**2*
     -    TFIdv2div1num(ind4)+2*nMSu(4+ind4)**2*TFIdv2div1num(ind4)- 
     -    nMSu(4+ind3)**2*TFIdv2div2num(ind4)+nMSu(4+ind4)**2*
     -    TFIdv2div2num(ind4)-4*nMSu(4+ind3)**2*TFIdv2finnum(ind4)+
     -    4*nMSu(4+ind4)**2*TFIdv2finnum(ind4)))))/
     -    (32.*Pi**8*(nMSu(4+ind3)**2 - nMSu(4+ind4)**2)**2)
      
      else
      ZuFg1=ZuFg1+
     -    (gZ2u(ind3,ind5)*gZ2u(ind4,ind3)*
     -    (CONJG(gLtFgSu(ind4))*(absM3*mt*gRtFgSu(ind5)*
     -    (nMSu(4+ind5)**2*(-2*TFIdv7div1num(ind3)+
     -    2*TFIdv7div1num(ind4) -TFIdv7div2num(ind3)+
     -    TFIdv7div2num(ind4) -4*TFIdv7finnum(ind3)+
     -    4*TFIdv7finnum(ind4))+nMSu(4+ind4)**2*(2*TFIdv7div1num(ind3)- 
     -    2*TFIdv7div1num(ind5)+TFIdv7div2num(ind3) - 
     -    TFIdv7div2num(ind5)+4*TFIdv7finnum(ind3) - 
     -    4*TFIdv7finnum(ind5))+nMSu(4+ind3)**2*(-2*
     -    TFIdv7div1num(ind4)+2*TFIdv7div1num(ind5) - 
     -    TFIdv7div2num(ind4)+TFIdv7div2num(ind5)-4*TFIdv7finnum(ind4)+
     -    4*TFIdv7finnum(ind5)))+gLtFgSu(ind5)*(nMSu(4+ind5)**2*
     -    (2*TFIdv9div1num(ind3)-2*TFIdv9div1num(ind4)+
     -    TFIdv9div2num(ind3)-TFIdv9div2num(ind4)+4*TFIdv9finnum(ind3)- 
     -    4*TFIdv9finnum(ind4) -2*TFIdv15div1num(ind3)+
     -    2*TFIdv15div1num(ind4) -TFIdv15div2num(ind3)+
     -    TFIdv15div2num(ind4)-4*TFIdv15finnum(ind3)+4*
     -    TFIdv15finnum(ind4))+nMSu(4+ind4)**2*(-2*TFIdv9div1num(ind3)+
     -    2*TFIdv9div1num(ind5)-TFIdv9div2num(ind3)+
     -    TFIdv9div2num(ind5)-4*TFIdv9finnum(ind3)+
     -    4*TFIdv9finnum(ind5)+2*TFIdv15div1num(ind3) - 
     -    2*TFIdv15div1num(ind5)+TFIdv15div2num(ind3) - 
     -    TFIdv15div2num(ind5)+4*TFIdv15finnum(ind3) - 
     -    4*TFIdv15finnum(ind5))+nMSu(4+ind3)**2*(2*
     -    TFIdv9div1num(ind4)-2*TFIdv9div1num(ind5)+
     -    TFIdv9div2num(ind4)-TFIdv9div2num(ind5)+4*TFIdv9finnum(ind4)- 
     -    4*TFIdv9finnum(ind5) -2*TFIdv15div1num(ind4)+
     -    2*TFIdv15div1num(ind5)-TFIdv15div2num(ind4)+
     -    TFIdv15div2num(ind5) -4*TFIdv15finnum(ind4)+
     -    4*TFIdv15finnum(ind5))))+CONJG(gRtFgSu(ind4))*(absM3*mt*
     -    gLtFgSu(ind5)*(nMSu(4+ind5)**2*(-2*TFIdv7div1num(ind3)+
     -    2*TFIdv7div1num(ind4) -TFIdv7div2num(ind3)+
     -    TFIdv7div2num(ind4) -4*TFIdv7finnum(ind3)+
     -    4*TFIdv7finnum(ind4))+nMSu(4+ind4)**2*(2*TFIdv7div1num(ind3)- 
     -    2*TFIdv7div1num(ind5)+TFIdv7div2num(ind3) - 
     -    TFIdv7div2num(ind5)+4*TFIdv7finnum(ind3) - 4*
     -    TFIdv7finnum(ind5))+nMSu(4+ind3)**2*(-2*TFIdv7div1num(ind4)+
     -    2*TFIdv7div1num(ind5) -TFIdv7div2num(ind4)+
     -    TFIdv7div2num(ind5) -4*TFIdv7finnum(ind4)+
     -    4*TFIdv7finnum(ind5)))+gRtFgSu(ind5)*(nMSu(4+ind5)**2*
     -    (2*TFIdv9div1num(ind3) -2*TFIdv9div1num(ind4)+
     -    TFIdv9div2num(ind3)-TFIdv9div2num(ind4)+4*TFIdv9finnum(ind3)- 
     -    4*TFIdv9finnum(ind4) -2*TFIdv15div1num(ind3)+2*
     -    TFIdv15div1num(ind4)-TFIdv15div2num(ind3)+
     -    TFIdv15div2num(ind4)-4*TFIdv15finnum(ind3)+4*
     -    TFIdv15finnum(ind4))+nMSu(4+ind4)**2*(-2*TFIdv9div1num(ind3)+
     -    2*TFIdv9div1num(ind5) -TFIdv9div2num(ind3)+
     -    TFIdv9div2num(ind5) -4*TFIdv9finnum(ind3)+
     -    4*TFIdv9finnum(ind5)+2*TFIdv15div1num(ind3) - 
     -    2*TFIdv15div1num(ind5)+TFIdv15div2num(ind3) - 
     -    TFIdv15div2num(ind5)+4*TFIdv15finnum(ind3) - 4*
     -    TFIdv15finnum(ind5))+nMSu(4+ind3)**2*(2*TFIdv9div1num(ind4)- 
     -    2*TFIdv9div1num(ind5)+TFIdv9div2num(ind4) - 
     -    TFIdv9div2num(ind5)+4*TFIdv9finnum(ind4) - 
     -    4*TFIdv9finnum(ind5) -2*TFIdv15div1num(ind4)+
     -    2*TFIdv15div1num(ind5) -TFIdv15div2num(ind4)+
     -    TFIdv15div2num(ind5) -4*TFIdv15finnum(ind4)+
     -    4*TFIdv15finnum(ind5))))))/(32.*Pi**8*(nMSu(4+ind3)**2-
     -    nMSu(4+ind4)**2)*(nMSu(4+ind3)**2 - nMSu(4+ind5)**2)*
     -    (nMSu(4+ind4)**2 - nMSu(4+ind5)**2))
      end if
      end do
      end do
      end do
      
      ZuFg2=CONJG(ZuFg1)
      
      ZuFg3=0d0
      do ind3=1,2
      do ind4=1,2
      if(ind3.eq.ind4) then
      ZuFg3=ZuFg3+((0,-0.125)*g2Z2u(ind3,ind3)*(CONJG(gLtFgSu(ind3))*
     -    (absM3*mt*gRtFgSu(ind3)*TFIdv14fin(mt,absM3,nMSu(4+ind3))+
     -    gLtFgSu(ind3)*TFIdv10fin(mt,absM3,nMSu(4+ind3)))+
     -    CONJG(gRtFgSu(ind3))*(absM3*mt*gLtFgSu(ind3)*
     -    TFIdv14fin(mt,absM3,nMSu(4+ind3))+
     -    gRtFgSu(ind3)*TFIdv10fin(mt,absM3,nMSu(4+ind3)))))/Pi**8
      else
      ZuFg3=ZuFg3+((0,-0.125)*g2Z2u(ind3,ind4)*(CONJG(gLtFgSu(ind3))*
     -   (absM3*mt*gRtFgSu(ind4)*(Ifin(mt**2,absM3**2,nMSu(4+ind3)**2)- 
     -    Ifin(mt**2,absM3**2,nMSu(4+ind4)**2))+gLtFgSu(ind4)*
     -    (TFIdv6fin(mt,absM3,nMSu(4+ind3))-
     -    TFIdv6fin(mt,absM3,nMSu(4+ind4))))+CONJG(gRtFgSu(ind3))*(
     -    absM3*mt*gLtFgSu(ind4)*(Ifin(mt**2,absM3**2,nMSu(4+ind3)**2)- 
     -    Ifin(mt**2,absM3**2,nMSu(4+ind4)**2))+gRtFgSu(ind4)*
     -    (TFIdv6fin(mt,absM3,nMSu(4+ind3)) - 
     -    TFIdv6fin(mt,absM3,nMSu(4+ind4))))))/
     -    (Pi**8*(nMSu(4+ind3)**2 - nMSu(4+ind4)**2))
      end if
      end do
      end do
      
      Zugluond=(g3**2*(gZ2u(2,1)*gZ2ugluon(1,2)+gZ2u(1,2)*
     -    gZ2ugluon(2,1))*(12*Ca**2*A0(nMSu(5)**2)**2 - 12*Ca**2*
     -    A0(nMSu(6)**2)**2+44*Ca**2*A0(nMSu(5)**2)*nMSu(5)**2+
     -    nMSu(5)**2*(24*Ca**2*Ae(nMSu(5)**2)+4*
     -    Idiv1(nMSu(5)**2,nMSu(6)**2,0d0)+2*
     -    Idiv2(nMSu(5)**2,nMSu(6)**2,0d0)+8*
     -    Ifin(nMSu(5)**2,nMSu(6)**2,0d0)+43*Ca**2*nMSu(5)**2) - 
     -    44*Ca**2*A0(nMSu(6)**2)*nMSu(6)**2-2*(12*Ca**2*Ae(nMSu(6)**2)
     -    +2*Idiv1(nMSu(5)**2,nMSu(6)**2,0d0)+
     -    Idiv2(nMSu(5)**2,nMSu(6)**2,0d0)+4*
     -    Ifin(nMSu(5)**2,nMSu(6)**2,0d0))*nMSu(6)**2 - 43*Ca**2*
     -    nMSu(6)**4))/(256.*Pi**8*(nMSu(5)**2 - nMSu(6)**2))
     -    +(Ca**2*g3**2*((gZ2u(1,1)*gZ2ugluon(1,1)*(A0(nMSu(5)**2)**2+
     -    2*(A0(nMSu(5)**2)+Ae(nMSu(5)**2))*nMSu(5)**2+2*nMSu(5)**4))/
     -    nMSu(5)**2+(gZ2u(2,2)*gZ2ugluon(2,2)*(A0(nMSu(6)**2)**2+
     -    2*(A0(nMSu(6)**2)+Ae(nMSu(6)**2))*nMSu(6)**2+2*nMSu(6)**4))
     -    /nMSu(6)**2))/(8.*Pi**8)
     
      Zugluone=(g3**2*(gZ2ugluon(1,1)**2*
     -    Ifin(nMSu(5)**2,nMSu(5)**2,0d0)+2*gZ2ugluon(1,2)*
     -    gZ2ugluon(2,1)*Ifin(nMSu(5)**2,nMSu(6)**2,0d0)+
     -    gZ2ugluon(2,2)**2*Ifin(nMSu(6)**2,nMSu(6)**2,0d0)))/
     -    (16.*Pi**8)

c maggie changed 8 Oct 2017
      dmtep = 0.D0
c end maggie changed 8 Oct 2017
      
c Counterterm-inserted diagrams for Z-self-energy
      CTZ=(3*Ca*(((0,8)*(gZ2tL-gZ2tR)**2*((dmtdiv1+dmtep-2*dmtfin)*
     -   mt**2+(-2*dmtdiv1+dmtfin)*A0(mt**2)+dmtdiv1*Ae(mt**2)))/mt+
     -   ((0,2)*Ae(nMSu(6)**2)*nMSu(5)**2*(dMstdiv1(2,2)*((0,-1)*
     -   g2Z2u(2,2)*(nMSu(5)**2 - nMSu(6)**2)**2+2*(gZ2u(2,2)**2*
     -   (nMSu(5)**2 - nMSu(6)**2)**2+gZ2u(1,2)*gZ2u(2,1)*nMSu(6)**2*
     -   (-2*nMSu(5)**2+nMSu(6)**2)))+nMSu(6)**2*(2*dMstdiv1(1,1)*
     -   gZ2u(1,2)*gZ2u(2,1)*nMSu(6)**2+dMstdiv1(1,2)*((0,1)*
     -   g2Z2u(1,2)*(nMSu(5)**2 - nMSu(6)**2)+2*gZ2u(1,2)*(gZ2u(1,1)*
     -   nMSu(6)**2+gZ2u(2,2)*(-2*nMSu(5)**2+nMSu(6)**2)))+
     -   dMstdiv1(2,1)*((0,1)*g2Z2u(2,1)*(nMSu(5)**2 - nMSu(6)**2)+
     -   2*gZ2u(2,1)*(gZ2u(1,1)*nMSu(6)**2+gZ2u(2,2)*(-2*nMSu(5)**2+
     -   nMSu(6)**2)))))+(0,2)*A0(nMSu(6)**2)*nMSu(5)**2*
     -   (dMstdiv1(2,2)*(-(gZ2u(1,2)*gZ2u(2,1)*nMSu(6)**4)+
     -   (0,1)*g2Z2u(2,2)*(nMSu(5)**2 - nMSu(6)**2)**2 -2*gZ2u(2,2)**2*
     -   (nMSu(5)**2 - nMSu(6)**2)**2)+dMstfin(2,2)*((0,-1)*g2Z2u(2,2)*
     -   (nMSu(5)**2 - nMSu(6)**2)**2+2*(gZ2u(2,2)**2*(nMSu(5)**2 - 
     -   nMSu(6)**2)**2+gZ2u(1,2)*gZ2u(2,1)*nMSu(6)**2*(-2*nMSu(5)**2+
     -   nMSu(6)**2)))+nMSu(6)**2*((gZ2u(2,1)*((dMstdiv1(1,1)+
     -   2*dMstfin(1,1))*gZ2u(1,2)+dMstdiv1(2,1)*(gZ2u(1,1) - gZ2u(2,2)))+
     -   dMstdiv1(1,2)*gZ2u(1,2)*(gZ2u(1,1) - gZ2u(2,2)))*nMSu(6)**2+
     -   dMstfin(1,2)*((0,1)*g2Z2u(1,2)*(nMSu(5)**2 - nMSu(6)**2)+
     -   2*gZ2u(1,2)*(gZ2u(1,1)*nMSu(6)**2+gZ2u(2,2)*(-2*nMSu(5)**2+
     -   nMSu(6)**2)))+dMstfin(2,1)*((0,1)*g2Z2u(2,1)*(nMSu(5)**2 - 
     -   nMSu(6)**2)+2*gZ2u(2,1)*(gZ2u(1,1)*nMSu(6)**2+gZ2u(2,2)*
     -   (-2*nMSu(5)**2+nMSu(6)**2)))))+nMSu(6)**2*((0,-1)*nMSu(5)**2*
     -   (nMSu(5)**2 - nMSu(6)**2)*((0,2)*dMstep(1,2)*g2Z2u(1,2)*
     -   nMSu(5)**2+(0,2)*dMstep(2,1)*g2Z2u(2,1)*nMSu(5)**2+(0,2)*
     -   dMstep(2,2)*g2Z2u(2,2)*nMSu(5)**2 -(0,2)*dMstfin(2,2)*
     -   g2Z2u(2,2)*nMSu(5)**2+dMstdiv1(1,2)*gZ2u(1,1)*gZ2u(1,2)*
     -   nMSu(5)**2 -4*dMstep(1,2)*gZ2u(1,1)*gZ2u(1,2)*nMSu(5)**2+
     -   2*dMstfin(1,2)*gZ2u(1,1)*gZ2u(1,2)*nMSu(5)**2+dMstdiv1(2,1)*
     -   gZ2u(1,1)*gZ2u(2,1)*nMSu(5)**2 -4*dMstep(2,1)*gZ2u(1,1)*
     -   gZ2u(2,1)*nMSu(5)**2+2*dMstfin(2,1)*gZ2u(1,1)*gZ2u(2,1)*
     -   nMSu(5)**2+dMstdiv1(1,1)*gZ2u(1,2)*gZ2u(2,1)*nMSu(5)**2 - 
     -   dMstdiv1(2,2)*gZ2u(1,2)*gZ2u(2,1)*nMSu(5)**2 -4*dMstep(2,2)*
     -   gZ2u(1,2)*gZ2u(2,1)*nMSu(5)**2 -2*dMstfin(2,2)*gZ2u(1,2)*
     -   gZ2u(2,1)*nMSu(5)**2 -dMstdiv1(1,2)*gZ2u(1,2)*gZ2u(2,2)*
     -   nMSu(5)**2 -4*dMstep(1,2)*gZ2u(1,2)*gZ2u(2,2)*nMSu(5)**2 - 
     -   2*dMstfin(1,2)*gZ2u(1,2)*gZ2u(2,2)*nMSu(5)**2 -dMstdiv1(2,1)*
     -   gZ2u(2,1)*gZ2u(2,2)*nMSu(5)**2 -4*dMstep(2,1)*gZ2u(2,1)*
     -   gZ2u(2,2)*nMSu(5)**2 -2*dMstfin(2,1)*gZ2u(2,1)*gZ2u(2,2)*
     -   nMSu(5)**2 -4*dMstep(2,2)*gZ2u(2,2)**2*nMSu(5)**2+
     -   4*dMstfin(2,2)*gZ2u(2,2)**2*nMSu(5)**2 -(0,2)*dMstep(1,2)*
     -   g2Z2u(1,2)*nMSu(6)**2 -(0,2)*dMstep(2,1)*g2Z2u(2,1)*
     -   nMSu(6)**2 -(0,2)*dMstep(2,2)*g2Z2u(2,2)*nMSu(6)**2+(0,2)*
     -   dMstfin(2,2)*g2Z2u(2,2)*nMSu(6)**2+dMstdiv1(1,2)*gZ2u(1,1)*
     -   gZ2u(1,2)*nMSu(6)**2+4*dMstep(1,2)*gZ2u(1,1)*gZ2u(1,2)*
     -   nMSu(6)**2+2*dMstfin(1,2)*gZ2u(1,1)*gZ2u(1,2)*nMSu(6)**2+
     -   dMstdiv1(2,1)*gZ2u(1,1)*gZ2u(2,1)*nMSu(6)**2+4*dMstep(2,1)*
     -   gZ2u(1,1)*gZ2u(2,1)*nMSu(6)**2+2*dMstfin(2,1)*gZ2u(1,1)*
     -   gZ2u(2,1)*nMSu(6)**2+dMstdiv1(1,1)*gZ2u(1,2)*gZ2u(2,1)*
     -   nMSu(6)**2 -dMstdiv1(2,2)*gZ2u(1,2)*gZ2u(2,1)*nMSu(6)**2+
     -   4*dMstep(2,2)*gZ2u(1,2)*gZ2u(2,1)*nMSu(6)**2 -2*dMstfin(2,2)*
     -   gZ2u(1,2)*gZ2u(2,1)*nMSu(6)**2 -dMstdiv1(1,2)*gZ2u(1,2)*
     -   gZ2u(2,2)*nMSu(6)**2+4*dMstep(1,2)*gZ2u(1,2)*gZ2u(2,2)*
     -   nMSu(6)**2 -2*dMstfin(1,2)*gZ2u(1,2)*gZ2u(2,2)*nMSu(6)**2 - 
     -   dMstdiv1(2,1)*gZ2u(2,1)*gZ2u(2,2)*nMSu(6)**2+4*dMstep(2,1)*
     -   gZ2u(2,1)*gZ2u(2,2)*nMSu(6)**2-2*dMstfin(2,1)*gZ2u(2,1)*
     -   gZ2u(2,2)*nMSu(6)**2+4*dMstep(2,2)*gZ2u(2,2)**2*nMSu(6)**2- 
     -   4*dMstfin(2,2)*gZ2u(2,2)**2*nMSu(6)**2+(0,2)*dMstep(1,1)*
     -   (g2Z2u(1,1)+(0,2)*(gZ2u(1,1)**2+gZ2u(1,2)*gZ2u(2,1)))*
     -   (nMSu(5)**2 - nMSu(6)**2)+2*dMstfin(1,1)*((0,-1)*g2Z2u(1,1)*
     -   (nMSu(5)**2 - nMSu(6)**2)+2*gZ2u(1,1)**2*
     -   (nMSu(5)**2 - nMSu(6)**2)+gZ2u(1,2)*gZ2u(2,1)*(nMSu(5)**2+
     -   nMSu(6)**2)))+2*Ae(nMSu(5)**2)*(dMstdiv1(1,1)*(g2Z2u(1,1)*
     -   (nMSu(5)**2 - nMSu(6)**2)**2+(0,2)*(gZ2u(1,2)*gZ2u(2,1)*
     -   nMSu(5)**2*(nMSu(5)**2-2*nMSu(6)**2)+gZ2u(1,1)**2*(nMSu(5)**2-
     -   nMSu(6)**2)**2))+nMSu(5)**2*((0,2)*dMstdiv1(2,2)*gZ2u(1,2)*
     -   gZ2u(2,1)*nMSu(5)**2+dMstdiv1(1,2)*(g2Z2u(1,2)*(nMSu(5)**2- 
     -   nMSu(6)**2)+(0,2)*gZ2u(1,2)*(gZ2u(2,2)*nMSu(5)**2+
     -   gZ2u(1,1)*(nMSu(5)**2 - 2*nMSu(6)**2)))+dMstdiv1(2,1)*
     -   (g2Z2u(2,1)*(nMSu(5)**2 - nMSu(6)**2)+(0,2)*gZ2u(2,1)*
     -   (gZ2u(2,2)*nMSu(5)**2+gZ2u(1,1)*(nMSu(5)**2-2*nMSu(6)**2)))))+
     -   2*A0(nMSu(5)**2)*(-(dMstdiv1(1,1)*(g2Z2u(1,1)*(nMSu(5)**2 - 
     -   nMSu(6)**2)**2+(0,1)*(gZ2u(1,2)*gZ2u(2,1)*nMSu(5)**4+2*
     -   gZ2u(1,1)**2*(nMSu(5)**2 - nMSu(6)**2)**2)))-(0,1)*
     -   (dMstfin(1,1)*((0,1)*g2Z2u(1,1)*(nMSu(5)**2 - nMSu(6)**2)**2 - 
     -   2*(gZ2u(1,2)*gZ2u(2,1)*nMSu(5)**2*(nMSu(5)**2 - 2*nMSu(6)**2)+
     -   gZ2u(1,1)**2*(nMSu(5)**2 - nMSu(6)**2)**2))+nMSu(5)**2*
     -   ((gZ2u(2,1)*(-((dMstdiv1(2,2)+2*dMstfin(2,2))*gZ2u(1,2))+
     -   dMstdiv1(2,1)*(gZ2u(1,1)-gZ2u(2,2)))+dMstdiv1(1,2)*gZ2u(1,2)*
     -   (gZ2u(1,1) - gZ2u(2,2)))*nMSu(5)**2+dMstfin(1,2)*((0,1)*
     -   g2Z2u(1,2)*(nMSu(5)**2 - nMSu(6)**2) -2*gZ2u(1,2)*(gZ2u(2,2)*
     -   nMSu(5)**2+gZ2u(1,1)*(nMSu(5)**2-2*nMSu(6)**2)))+dMstfin(2,1)*
     -   ((0,1)*g2Z2u(2,1)*(nMSu(5)**2 - nMSu(6)**2)-2*gZ2u(2,1)*
     -   (gZ2u(2,2)*nMSu(5)**2+gZ2u(1,1)*(nMSu(5)**2 - 
     -   2*nMSu(6)**2))))))))/(nMSu(5)**2*nMSu(6)**2*
     -   (nMSu(5)**2 - nMSu(6)**2)**2)))/(32.*Pi**4)
    
     
      stops=ZtFg1+ZtFg2+ZtFg3+Zsquarkonly1+Zsquarksonly6+Zugluona+Zugluonb+
     .     Zugluonc+ZuFg1+ZuFg2+ZuFg3+Zugluond+Zugluone
     
      tops=Ztgluon
      
      dMZ2=-stops-tops-CTZ
      

c contributions to W-self-energy
      Wtgluon= (g3**2*gWtb*mt**2*CONJG(gWtb)*(3+2*Pi**2-
     -  12*Log(mt**2/Qs**2)+72*Log(mt**2/Qs**2)**2))/(768.*Pi**4)
     
      
      WtFg1=0d0
      do ind3=1,2
      do ind4=1,2
      WtFg1=WtFg1+(gWtb*CONJG(gWud(ind4,ind3))*gLbFgSd(ind3)*
     -  (-(Ca**2*(absM3**2*mt**2+2*absM3**2*A0(mt**2)+
     -  2*A0(absM3**2)*(mt**2+2*A0(mt**2))+4*mt**2*Ae(absM3**2)+
     -  4*absM3**2*Ae(mt**2))*gRtFgSu(ind4))+
     -  ((2*Idiv1(nMSd(4+ind3)**2,absM3**2,0d0)+
     -  Idiv2(nMSd(4+ind3)**2,absM3**2,0d0)+
     -  4*Ifin(nMSd(4+ind3)**2,absM3**2,0d0))*(absM3-nMSd(4+ind3))*
     -  (absM3+nMSd(4+ind3))*(absM3*mt*gLtFgSu(ind4)+gRtFgSu(ind4)*
     -  (absM3**2-nMSd(4+ind3)**2)))/(nMSd(4+ind3)**2-nMSu(4+ind4)**2)-
     -  (Ca**2*(2*(mt**2+2*A0(mt**2))*A0(nMSd(4+ind3)**2)+4*mt**2*
     -  Ae(nMSd(4+ind3)**2)+(mt**2+2*A0(mt**2)+4*Ae(mt**2))*
     -  nMSd(4+ind3)**2)*(absM3*mt*gLtFgSu(ind4)+gRtFgSu(ind4)*
     -  (absM3**2-nMSd(4+ind3)**2)))/(nMSd(4+ind3)**2-nMSu(4+ind4)**2)-
     -  ((2*Idiv1(nMSd(4+ind3)**2,mt**2,absM3**2)+
     -  Idiv2(nMSd(4+ind3)**2,mt**2,absM3**2)+
     -  4*Ifin(nMSd(4+ind3)**2,mt**2,absM3**2))*
     -  (absM3*mt*gLtFgSu(ind4)*(absM3**2-mt**2-nMSd(4+ind3)**2)+
     -  gRtFgSu(ind4)*(absM3**4-absM3**2*mt**2-
     -  (2*absM3**2+mt**2)*nMSd(4+ind3)**2+nMSd(4+ind3)**4)))/
     -  (nMSd(4+ind3)**2-nMSu(4+ind4)**2)+
     -  ((2*Idiv1(nMSu(4+ind4)**2,absM3**2,0d0)+
     -  Idiv2(nMSu(4+ind4)**2,absM3**2,0d0)+
     -  4*Ifin(nMSu(4+ind4)**2,absM3**2,0d0))*(absM3-nMSu(4+ind4))*
     -  (absM3+nMSu(4+ind4))*(absM3*mt*gLtFgSu(ind4)+gRtFgSu(ind4)*
     -  (absM3**2-nMSu(4+ind4)**2)))/(-nMSd(4+ind3)**2+nMSu(4+ind4)**2)
     -   +(Ca**2*(2*(mt**2+2*A0(mt**2))*A0(nMSu(4+ind4)**2)+
     -  4*mt**2*Ae(nMSu(4+ind4)**2)+(mt**2+2*A0(mt**2)+4*Ae(mt**2))*
     -  nMSu(4+ind4)**2)*(absM3*mt*gLtFgSu(ind4)+gRtFgSu(ind4)*
     -  (absM3**2-nMSu(4+ind4)**2)))/(nMSd(4+ind3)**2-nMSu(4+ind4)**2)+
     -  ((2*Idiv1(nMSu(4+ind4)**2,mt**2,absM3**2)+
     -  Idiv2(nMSu(4+ind4)**2,mt**2,absM3**2)+
     -  4*Ifin(nMSu(4+ind4)**2,mt**2,absM3**2))*
     -  (absM3*mt*gLtFgSu(ind4)*(absM3**2-mt**2-nMSu(4+ind4)**2)+
     -  gRtFgSu(ind4)*(absM3**4-absM3**2*mt**2-
     -  (2*absM3**2+mt**2)*nMSu(4+ind4)**2+nMSu(4+ind4)**4)))/
     -  (nMSd(4+ind3)**2-nMSu(4+ind4)**2)))/(128.*mt**2*Pi**8)

      end do
      end do
      
      WtFg2=CONJG(WtFg1)
      
      WtFg3=0d0
      WtFg4=0d0
      do ind3=1,2
      WtFg3=WtFg3-(gWtb*CONJG(gWtb)*(-2*mt*CONJG(gRtFgSu(ind3))*
     -    (mt*gRtFgSu(ind3)*(absM3**4*Ca**2*mt**2-absM3**2*Ca**2*mt**4-
     -    6*absM3**4*Ca**2*A0(nMSu(4+ind3)**2)+
     -    6*absM3**2*Ca**2*mt**2*A0(nMSu(4+ind3)**2)+4*absM3**2*
     -    Ca**2*mt**2*Ae(absM3**2)-4*Ca**2*mt**4*Ae(absM3**2)+
     -    4*absM3**4*Ca**2*Ae(mt**2)-4*absM3**2*Ca**2*mt**2*Ae(mt**2)+
     -    4*absM3**4*Ca**2*Ae(nMSu(4+ind3)**2)-
     -    4*absM3**2*Ca**2*mt**2*Ae(nMSu(4+ind3)**2)+
     -    4*absM3**4*Idiv1(nMSu(4+ind3)**2,mt**2,absM3**2)+
     -    2*absM3**2*mt**2*Idiv1(nMSu(4+ind3)**2,mt**2,absM3**2)-
     -    6*mt**4*Idiv1(nMSu(4+ind3)**2,mt**2,absM3**2)-
     -    2*absM3**4*Idiv2(nMSu(4+ind3)**2,mt**2,absM3**2)+
     -    absM3**2*mt**2*Idiv2(nMSu(4+ind3)**2,mt**2,absM3**2)+
     -    mt**4*Idiv2(nMSu(4+ind3)**2,mt**2,absM3**2)-
     -    4*absM3**2*mt**2*Ifin(nMSu(4+ind3)**2,mt**2,absM3**2)+
     -    4*mt**4*Ifin(nMSu(4+ind3)**2,mt**2,absM3**2)+
     -    absM3**4*Ca**2*nMSu(4+ind3)**2+
     -    6*absM3**2*Ca**2*A0(nMSu(4+ind3)**2)*nMSu(4+ind3)**2+
     -    4*absM3**2*Ca**2*Ae(absM3**2)*nMSu(4+ind3)**2+
     -    8*Ca**2*mt**2*Ae(absM3**2)*nMSu(4+ind3)**2-
     -    4*absM3**2*Ca**2*Ae(mt**2)*nMSu(4+ind3)**2-
     -    4*absM3**2*Ca**2*Ae(nMSu(4+ind3)**2)*nMSu(4+ind3)**2+
     -    2*absM3**2*Idiv1(nMSu(4+ind3)**2,mt**2,absM3**2)*
     -    nMSu(4+ind3)**2+12*mt**2*
     -    Idiv1(nMSu(4+ind3)**2,mt**2,absM3**2)*nMSu(4+ind3)**2+
     -    absM3**2*Idiv2(nMSu(4+ind3)**2,mt**2,absM3**2)*
     -    nMSu(4+ind3)**2-2*mt**2*Idiv2(nMSu(4+ind3)**2,mt**2,absM3**2)
     -    *nMSu(4+ind3)**2-4*absM3**2*
     -    Ifin(nMSu(4+ind3)**2,mt**2,absM3**2)*nMSu(4+ind3)**2-
     -    8*mt**2*Ifin(nMSu(4+ind3)**2,mt**2,absM3**2)*nMSu(4+ind3)**2-
     -    absM3**2*Ca**2*nMSu(4+ind3)**4-
     -    4*Ca**2*Ae(absM3**2)*nMSu(4+ind3)**4-
     -    6*Idiv1(nMSu(4+ind3)**2,mt**2,absM3**2)*nMSu(4+ind3)**4+
     -    Idiv2(nMSu(4+ind3)**2,mt**2,absM3**2)*nMSu(4+ind3)**4+
     -    4*Ifin(nMSu(4+ind3)**2,mt**2,absM3**2)*nMSu(4+ind3)**4+
     -    2*absM3**2*Ca**2*A0(mt**2)*(-4*A0(nMSu(4+ind3)**2)+
     -    3*(-absM3**2+mt**2+nMSu(4+ind3)**2))+2*Ca**2*A0(absM3**2)*
     -    (2*A0(nMSu(4+ind3)**2)*(absM3**2+mt**2-nMSu(4+ind3)**2)+
     -    2*A0(mt**2)*(absM3**2-mt**2+nMSu(4+ind3)**2)+
     -    3*(-(absM3**2*mt**2)+mt**4-(absM3**2+2*mt**2)*nMSu(4+ind3)**2
     -    +nMSu(4+ind3)**4)))+absM3*gLtFgSu(ind3)*(absM3**4*Ca**2*mt**2
     -    -absM3**2*Ca**2*mt**4-6*absM3**2*Ca**2*mt**2*
     -    A0(nMSu(4+ind3)**2)+6*Ca**2*mt**4*A0(nMSu(4+ind3)**2)+
     -    4*absM3**2*Ca**2*mt**2*Ae(absM3**2)-4*Ca**2*mt**4*
     -    Ae(absM3**2)+4*absM3**4*Ca**2*Ae(mt**2)-4*absM3**2*Ca**2*
     -    mt**2*Ae(mt**2)+4*absM3**2*Ca**2*mt**2*Ae(nMSu(4+ind3)**2)-
     -    4*Ca**2*mt**4*Ae(nMSu(4+ind3)**2)+
     -    10*absM3**2*mt**2*Idiv1(nMSu(4+ind3)**2,mt**2,absM3**2)-
     -    10*mt**4*Idiv1(nMSu(4+ind3)**2,mt**2,absM3**2)-
     -    3*absM3**2*mt**2*Idiv2(nMSu(4+ind3)**2,mt**2,absM3**2)+
     -    3*mt**4*Idiv2(nMSu(4+ind3)**2,mt**2,absM3**2)-
     -    4*absM3**2*mt**2*Ifin(nMSu(4+ind3)**2,mt**2,absM3**2)+
     -    4*mt**4*Ifin(nMSu(4+ind3)**2,mt**2,absM3**2)-
     -    Ca**2*mt**4*nMSu(4+ind3)**2-
     -    6*Ca**2*mt**2*A0(nMSu(4+ind3)**2)*nMSu(4+ind3)**2+
     -    4*Ca**2*mt**2*Ae(absM3**2)*nMSu(4+ind3)**2-
     -    8*absM3**2*Ca**2*Ae(mt**2)*nMSu(4+ind3)**2-
     -    4*Ca**2*mt**2*Ae(mt**2)*nMSu(4+ind3)**2+
     -    4*Ca**2*mt**2*Ae(nMSu(4+ind3)**2)*nMSu(4+ind3)**2+
     -    10*mt**2*Idiv1(nMSu(4+ind3)**2,mt**2,absM3**2)*
     -    nMSu(4+ind3)**2-3*mt**2*Idiv2(nMSu(4+ind3)**2,mt**2,absM3**2)
     -    *nMSu(4+ind3)**2-4*mt**2*Ifin(nMSu(4+ind3)**2,mt**2,absM3**2)
     -    *nMSu(4+ind3)**2+Ca**2*mt**2*nMSu(4+ind3)**4+4*Ca**2*
     -    Ae(mt**2)*nMSu(4+ind3)**4+2*Ca**2*A0(mt**2)*
     -    (-2*A0(nMSu(4+ind3)**2)*(absM3**2+mt**2-nMSu(4+ind3)**2)-
     -    3*(absM3**4-absM3**2*mt**2-(2*absM3**2+mt**2)*nMSu(4+ind3)**2
     -    +nMSu(4+ind3)**4))-2*Ca**2*A0(absM3**2)*
     -    (2*A0(mt**2)*(-absM3**2+mt**2+nMSu(4+ind3)**2)+mt**2*(-4*
     -    A0(nMSu(4+ind3)**2)+3*(absM3**2-mt**2+nMSu(4+ind3)**2)))))+
     -    CONJG(gLtFgSu(ind3))*(gLtFgSu(ind3)*(absM3**6*Ca**2*mt**2-
     -    4*absM3**4*Ca**2*mt**4+3*absM3**2*Ca**2*mt**6+
     -    10*absM3**4*Ca**2*mt**2*A0(nMSu(4+ind3)**2)-
     -    8*absM3**2*Ca**2*mt**4*A0(nMSu(4+ind3)**2)-
     -    2*Ca**2*mt**6*A0(nMSu(4+ind3)**2)-4*absM3**4*
     -    Ca**2*mt**2*Ae(absM3**2)+4*Ca**2*mt**6*Ae(absM3**2)-
     -    4*absM3**6*Ca**2*Ae(mt**2)+4*absM3**2*Ca**2*mt**4*Ae(mt**2)-
     -    4*absM3**4*Ca**2*mt**2*Ae(nMSu(4+ind3)**2)+
     -    4*Ca**2*mt**6*Ae(nMSu(4+ind3)**2)+
     -    2*absM3**6*Idiv1(nMSu(4+ind3)**2,absM3**2,0d0)-
     -    4*absM3**4*mt**2*Idiv1(nMSu(4+ind3)**2,absM3**2,0d0)+
     -    2*absM3**2*mt**4*Idiv1(nMSu(4+ind3)**2,absM3**2,0d0)-
     -    2*absM3**6*Idiv1(nMSu(4+ind3)**2,mt**2,absM3**2)-
     -    6*absM3**4*mt**2*Idiv1(nMSu(4+ind3)**2,mt**2,absM3**2)-
     -    2*absM3**2*mt**4*Idiv1(nMSu(4+ind3)**2,mt**2,absM3**2)+
     -    10*mt**6*Idiv1(nMSu(4+ind3)**2,mt**2,absM3**2)+
     -    absM3**6*Idiv2(nMSu(4+ind3)**2,absM3**2,0d0)-
     -    2*absM3**4*mt**2*Idiv2(nMSu(4+ind3)**2,absM3**2,0d0)+
     -    absM3**2*mt**4*Idiv2(nMSu(4+ind3)**2,absM3**2,0d0)-
     -    absM3**6*Idiv2(nMSu(4+ind3)**2,mt**2,absM3**2)+
     -    5*absM3**4*mt**2*Idiv2(nMSu(4+ind3)**2,mt**2,absM3**2)-
     -    absM3**2*mt**4*Idiv2(nMSu(4+ind3)**2,mt**2,absM3**2)-
     -    3*mt**6*Idiv2(nMSu(4+ind3)**2,mt**2,absM3**2)-
     -    4*absM3**6*Ifin(nMSu(4+ind3)**2,absM3**2,0d0)+
     -    8*absM3**4*mt**2*Ifin(nMSu(4+ind3)**2,absM3**2,0d0)-
     -    4*absM3**2*mt**4*Ifin(nMSu(4+ind3)**2,absM3**2,0d0)+
     -    4*absM3**6*Ifin(nMSu(4+ind3)**2,mt**2,absM3**2)-
     -    4*absM3**4*mt**2*Ifin(nMSu(4+ind3)**2,mt**2,absM3**2)+
     -    4*absM3**2*mt**4*Ifin(nMSu(4+ind3)**2,mt**2,absM3**2)-
     -    4*mt**6*Ifin(nMSu(4+ind3)**2,mt**2,absM3**2)-
     -    5*absM3**4*Ca**2*mt**2*nMSu(4+ind3)**2-
     -    Ca**2*mt**6*nMSu(4+ind3)**2-8*absM3**2*
     -    Ca**2*mt**2*A0(nMSu(4+ind3)**2)*nMSu(4+ind3)**2+
     -    4*Ca**2*mt**4*A0(nMSu(4+ind3)**2)*nMSu(4+ind3)**2-
     -    8*Ca**2*mt**4*Ae(absM3**2)*nMSu(4+ind3)**2+
     -    12*absM3**4*Ca**2*Ae(mt**2)*nMSu(4+ind3)**2+
     -    8*absM3**2*Ca**2*mt**2*Ae(mt**2)*nMSu(4+ind3)**2+
     -    4*Ca**2*mt**4*Ae(mt**2)*nMSu(4+ind3)**2-
     -    8*Ca**2*mt**4*Ae(nMSu(4+ind3)**2)*nMSu(4+ind3)**2-6*
     -    absM3**4*Idiv1(nMSu(4+ind3)**2,absM3**2,0d0)*nMSu(4+ind3)**2-
     -    2*mt**4*Idiv1(nMSu(4+ind3)**2,absM3**2,0d0)*nMSu(4+ind3)**2+
     -    6*absM3**4*Idiv1(nMSu(4+ind3)**2,mt**2,absM3**2)*
     -    nMSu(4+ind3)**2-18*mt**4*
     -    Idiv1(nMSu(4+ind3)**2,mt**2,absM3**2)*nMSu(4+ind3)**2-
     -    3*absM3**4*Idiv2(nMSu(4+ind3)**2,absM3**2,0d0)*
     -    nMSu(4+ind3)**2-mt**4*Idiv2(nMSu(4+ind3)**2,absM3**2,0d0)*
     -    nMSu(4+ind3)**2+3*absM3**4*
     -    Idiv2(nMSu(4+ind3)**2,mt**2,absM3**2)*nMSu(4+ind3)**2+7*
     -    mt**4*Idiv2(nMSu(4+ind3)**2,mt**2,absM3**2)*nMSu(4+ind3)**2+
     -    12*absM3**4*Ifin(nMSu(4+ind3)**2,absM3**2,0d0)*
     -    nMSu(4+ind3)**2+4*mt**4*Ifin(nMSu(4+ind3)**2,absM3**2,0d0)*
     -    nMSu(4+ind3)**2-12*absM3**4*
     -    Ifin(nMSu(4+ind3)**2,mt**2,absM3**2)*nMSu(4+ind3)**2+
     -    4*mt**4*Ifin(nMSu(4+ind3)**2,mt**2,absM3**2)*nMSu(4+ind3)**2+
     -    5*absM3**2*Ca**2*mt**2*nMSu(4+ind3)**4+
     -    2*Ca**2*mt**4*nMSu(4+ind3)**4-
     -    2*Ca**2*mt**2*A0(nMSu(4+ind3)**2)*nMSu(4+ind3)**4+
     -    4*Ca**2*mt**2*Ae(absM3**2)*nMSu(4+ind3)**4-
     -    12*absM3**2*Ca**2*Ae(mt**2)*nMSu(4+ind3)**4-
     -    8*Ca**2*mt**2*Ae(mt**2)*nMSu(4+ind3)**4+
     -    4*Ca**2*mt**2*Ae(nMSu(4+ind3)**2)*nMSu(4+ind3)**4+6*absM3**2*
     -    Idiv1(nMSu(4+ind3)**2,absM3**2,0d0)*nMSu(4+ind3)**4+
     -    4*mt**2*Idiv1(nMSu(4+ind3)**2,absM3**2,0d0)*nMSu(4+ind3)**4-
     -    6*absM3**2*Idiv1(nMSu(4+ind3)**2,mt**2,absM3**2)*
     -    nMSu(4+ind3)**4+6*mt**2*Idiv1(nMSu(4+ind3)**2,mt**2,absM3**2)
     -    *nMSu(4+ind3)**4+3*absM3**2*
     -    Idiv2(nMSu(4+ind3)**2,absM3**2,0d0)*nMSu(4+ind3)**4+
     -    2*mt**2*Idiv2(nMSu(4+ind3)**2,absM3**2,0d0)*nMSu(4+ind3)**4-
     -    3*absM3**2*Idiv2(nMSu(4+ind3)**2,mt**2,absM3**2)*
     -    nMSu(4+ind3)**4-5*mt**2*
     -    Idiv2(nMSu(4+ind3)**2,mt**2,absM3**2)*nMSu(4+ind3)**4-
     -    12*absM3**2*Ifin(nMSu(4+ind3)**2,absM3**2,0d0)*
     -    nMSu(4+ind3)**4-8*mt**2*Ifin(nMSu(4+ind3)**2,absM3**2,0d0)*
     -    nMSu(4+ind3)**4+12*absM3**2*
     -    Ifin(nMSu(4+ind3)**2,mt**2,absM3**2)*nMSu(4+ind3)**4+
     -    4*mt**2*Ifin(nMSu(4+ind3)**2,mt**2,absM3**2)*nMSu(4+ind3)**4-
     -    Ca**2*mt**2*nMSu(4+ind3)**6+4*Ca**2*Ae(mt**2)*
     -    nMSu(4+ind3)**6-2*Idiv1(nMSu(4+ind3)**2,absM3**2,0d0)*
     -    nMSu(4+ind3)**6+2*Idiv1(nMSu(4+ind3)**2,mt**2,absM3**2)*
     -    nMSu(4+ind3)**6-Idiv2(nMSu(4+ind3)**2,absM3**2,0d0)*
     -    nMSu(4+ind3)**6+Idiv2(nMSu(4+ind3)**2,mt**2,absM3**2)*
     -    nMSu(4+ind3)**6+4*Ifin(nMSu(4+ind3)**2,absM3**2,0d0)*
     -    nMSu(4+ind3)**6-4*Ifin(nMSu(4+ind3)**2,mt**2,absM3**2)*
     -    nMSu(4+ind3)**6+2*Ca**2*A0(mt**2)*(absM3**6+4*absM3**4*mt**2
     -    -5*absM3**2*mt**4-(3*absM3**4+6*absM3**2*mt**2+mt**4)*
     -    nMSu(4+ind3)**2+(3*absM3**2+2*mt**2)*nMSu(4+ind3)**4-
     -    nMSu(4+ind3)**6+2*A0(nMSu(4+ind3)**2)*(absM3**2+mt**2-
     -    nMSu(4+ind3)**2)**2)+2*Ca**2*A0(absM3**2)*
     -    (-2*A0(mt**2)*(absM3**4-mt**4-2*absM3**2*nMSu(4+ind3)**2+
     -    nMSu(4+ind3)**4)+mt**2*(absM3**4+4*absM3**2*mt**2-5*mt**4+
     -    2*(2*absM3**2+5*mt**2)*nMSu(4+ind3)**2-5*nMSu(4+ind3)**4-
     -    4*A0(nMSu(4+ind3)**2)*(absM3**2+mt**2-nMSu(4+ind3)**2))))-2*
     -    absM3*mt*gRtFgSu(ind3)*(absM3**4*Ca**2*mt**2-absM3**2*Ca**2*
     -    mt**4-6*absM3**2*Ca**2*mt**2*A0(nMSu(4+ind3)**2)+
     -    6*Ca**2*mt**4*A0(nMSu(4+ind3)**2)+4*absM3**2*Ca**2*
     -    mt**2*Ae(absM3**2)-4*Ca**2*mt**4*Ae(absM3**2)+
     -    4*absM3**4*Ca**2*Ae(mt**2)-4*absM3**2*Ca**2*mt**2*Ae(mt**2)+
     -    4*absM3**2*Ca**2*mt**2*Ae(nMSu(4+ind3)**2)-
     -    4*Ca**2*mt**4*Ae(nMSu(4+ind3)**2)+
     -    10*absM3**2*mt**2*Idiv1(nMSu(4+ind3)**2,mt**2,absM3**2)-
     -    10*mt**4*Idiv1(nMSu(4+ind3)**2,mt**2,absM3**2)-
     -    3*absM3**2*mt**2*Idiv2(nMSu(4+ind3)**2,mt**2,absM3**2)+
     -    3*mt**4*Idiv2(nMSu(4+ind3)**2,mt**2,absM3**2)-
     -    4*absM3**2*mt**2*Ifin(nMSu(4+ind3)**2,mt**2,absM3**2)+
     -    4*mt**4*Ifin(nMSu(4+ind3)**2,mt**2,absM3**2)-
     -    Ca**2*mt**4*nMSu(4+ind3)**2-
     -    6*Ca**2*mt**2*A0(nMSu(4+ind3)**2)*nMSu(4+ind3)**2+
     -    4*Ca**2*mt**2*Ae(absM3**2)*nMSu(4+ind3)**2-
     -    8*absM3**2*Ca**2*Ae(mt**2)*nMSu(4+ind3)**2-
     -    4*Ca**2*mt**2*Ae(mt**2)*nMSu(4+ind3)**2+
     -    4*Ca**2*mt**2*Ae(nMSu(4+ind3)**2)*nMSu(4+ind3)**2+10*mt**2
     -    *Idiv1(nMSu(4+ind3)**2,mt**2,absM3**2)*nMSu(4+ind3)**2-3*
     -    mt**2*Idiv2(nMSu(4+ind3)**2,mt**2,absM3**2)*nMSu(4+ind3)**2-
     -    4*mt**2*Ifin(nMSu(4+ind3)**2,mt**2,absM3**2)*nMSu(4+ind3)**2+
     -    Ca**2*mt**2*nMSu(4+ind3)**4+4*Ca**2*Ae(mt**2)*
     -    nMSu(4+ind3)**4+2*Ca**2*A0(mt**2)*
     -    (-2*A0(nMSu(4+ind3)**2)*(absM3**2+mt**2-nMSu(4+ind3)**2)-
     -    3*(absM3**4-absM3**2*mt**2-(2*absM3**2+mt**2)*nMSu(4+ind3)**2
     -    +nMSu(4+ind3)**4))-2*Ca**2*A0(absM3**2)*(2*A0(mt**2)*
     -    (-absM3**2+mt**2+nMSu(4+ind3)**2)+mt**2*(-4*
     -    A0(nMSu(4+ind3)**2)+3*(absM3**2-mt**2+nMSu(4+ind3)**2)))))))/
     -    (128.*mt**2*Pi**8*(absM3-mt-nMSu(4+ind3))*(absM3+mt-
     -   nMSu(4+ind3))*(absM3-mt+nMSu(4+ind3))*(absM3+mt+nMSu(4+ind3)))
     
     
      WtFg4=WtFg4+(gWtb*CONJG(gWtb)*CONJG(gLbFgSd(ind3))*gLbFgSd(ind3)*
     -    (Ca**2*(absM3**2*mt**2+2*A0(absM3**2)*(mt**2-2*A0(mt**2))+
     -    2*absM3**2*A0(mt**2)-4*mt**2*Ae(absM3**2)-4*absM3**2*
     -    Ae(mt**2))+(2*Idiv1(nMSd(4+ind3)**2,absM3**2,0d0)+
     -    Idiv2(nMSd(4+ind3)**2,absM3**2,0d0)-4*
     -    Ifin(nMSd(4+ind3)**2,absM3**2,0d0))*(absM3-nMSd(4+ind3))*
     -    (absM3+nMSd(4+ind3))-(2*
     -    Idiv1(nMSd(4+ind3)**2,mt**2,absM3**2)+
     -    Idiv2(nMSd(4+ind3)**2,mt**2,absM3**2)-
     -    4*Ifin(nMSd(4+ind3)**2,mt**2,absM3**2))*
     -    (absM3**2+mt**2-nMSd(4+ind3)**2)+
     -    Ca**2*(-2*(mt**2-2*A0(mt**2))*A0(nMSd(4+ind3)**2)+
     -    4*mt**2*Ae(nMSd(4+ind3)**2)-(mt**2+2*A0(mt**2)-4*Ae(mt**2))
     -    *nMSd(4+ind3)**2)))/(128.*mt**2*Pi**8)
      end do
      
      Wsquarksonly1a=0d0
      do ind3d=1,2
      do ind3=1,2
      do ind4=1,2
      do ind5=1,2
      if(ind3.eq.ind4) then
      Wsquarksonly1a=Wsquarksonly1a+
     -    ((0,0.0029296875)*Ca**2*CONJG(gWud(ind3,ind3d))*
     -    (3*g4u(ind3,ind5,ind3,ind5)+g4u(ind3,ind5,ind5,ind3))*
     -    gWud(ind3,ind3d)*(2*A0(nMSu(4+ind5)**2)*nMSd(4+ind3d)**4+
     -    4*Ae(nMSu(4+ind5)**2)*nMSd(4+ind3d)**4-
     -    8*Ae(nMSu(4+ind5)**2)*nMSd(4+ind3d)**2*nMSu(4+ind3)**2-
     -    2*A0(nMSu(4+ind5)**2)*nMSu(4+ind3)**4+
     -    4*Ae(nMSu(4+ind5)**2)*nMSu(4+ind3)**4+
     -    4*Ae(nMSd(4+ind3d)**2)*nMSd(4+ind3d)**2*nMSu(4+ind5)**2-
     -    8*Ae(nMSu(4+ind3)**2)*nMSd(4+ind3d)**2*nMSu(4+ind5)**2+
     -    nMSd(4+ind3d)**4*nMSu(4+ind5)**2+
     -    4*Ae(nMSu(4+ind3)**2)*nMSu(4+ind3)**2*nMSu(4+ind5)**2-
     -    nMSu(4+ind3)**4*nMSu(4+ind5)**2+
     -    2*A0(nMSd(4+ind3d)**2)*nMSd(4+ind3d)**2*
     -    (2*A0(nMSu(4+ind5)**2)+nMSu(4+ind5)**2)-
     -    2*A0(nMSu(4+ind3)**2)*
     -    (A0(nMSu(4+ind5)**2)*(4*nMSd(4+ind3d)**2-2*nMSu(4+ind3)**2)+
     -    nMSu(4+ind3)**2*nMSu(4+ind5)**2)))/
     -    (Pi**8*(nMSd(4+ind3d)**2-nMSu(4+ind3)**2)**2)
      else
      Wsquarksonly1a=Wsquarksonly1a+
     -    ((0,0.046875)*Ca**2*CONJG(gWud(ind4,ind3d))*
     -    (3*g4u(ind4,ind5,ind3,ind5)+g4u(ind4,ind5,ind5,ind3))*
     -    gWud(ind3,ind3d)*(Ae(nMSu(4+ind5)**2)/4.+A0(nMSu(4+ind5)**2)*
     -    (0.125+(A0(nMSu(4+ind4)**2)*(nMSd(4+ind3d)**2-
     -    nMSu(4+ind3)**2)*nMSu(4+ind4)**2+A0(nMSd(4+ind3d)**2)*
     -    nMSd(4+ind3d)**2*(nMSu(4+ind3)**2-nMSu(4+ind4)**2)+
     -    A0(nMSu(4+ind3)**2)*nMSu(4+ind3)**2*(-nMSd(4+ind3d)**2+
     -    nMSu(4+ind4)**2))/(4.*(nMSd(4+ind3d)-nMSu(4+ind3))*
     -    (nMSd(4+ind3d)+nMSu(4+ind3))*(nMSd(4+ind3d)-nMSu(4+ind4))*
     -    (nMSu(4+ind3)-nMSu(4+ind4))*(nMSd(4+ind3d)+nMSu(4+ind4))*
     -    (nMSu(4+ind3)+nMSu(4+ind4))))+((4*Ae(nMSd(4+ind3d)**2)*
     -    nMSd(4+ind3d)**2*nMSu(4+ind3)**2-4*Ae(nMSu(4+ind3)**2)*
     -    nMSd(4+ind3d)**2*nMSu(4+ind3)**2+nMSd(4+ind3d)**4*
     -    nMSu(4+ind3)**2-nMSd(4+ind3d)**2*nMSu(4+ind3)**4+
     -    2*A0(nMSu(4+ind4)**2)*nMSd(4+ind3d)**2*nMSu(4+ind4)**2-
     -    4*Ae(nMSd(4+ind3d)**2)*nMSd(4+ind3d)**2*nMSu(4+ind4)**2+
     -    4*Ae(nMSu(4+ind4)**2)*nMSd(4+ind3d)**2*nMSu(4+ind4)**2-
     -    nMSd(4+ind3d)**4*nMSu(4+ind4)**2-
     -    2*A0(nMSu(4+ind4)**2)*nMSu(4+ind3)**2*nMSu(4+ind4)**2+
     -    4*Ae(nMSu(4+ind3)**2)*nMSu(4+ind3)**2*nMSu(4+ind4)**2-
     -    4*Ae(nMSu(4+ind4)**2)*nMSu(4+ind3)**2*nMSu(4+ind4)**2+
     -    nMSu(4+ind3)**4*nMSu(4+ind4)**2+nMSd(4+ind3d)**2*
     -    nMSu(4+ind4)**4-nMSu(4+ind3)**2*nMSu(4+ind4)**4+
     -    2*A0(nMSd(4+ind3d)**2)*nMSd(4+ind3d)**2*(nMSu(4+ind3)**2-
     -    nMSu(4+ind4)**2)+2*A0(nMSu(4+ind3)**2)*nMSu(4+ind3)**2*
     -    (-nMSd(4+ind3d)**2+nMSu(4+ind4)**2))*nMSu(4+ind5)**2)/
     -    (16.*(nMSd(4+ind3d)-nMSu(4+ind3))*(nMSd(4+ind3d)+
     -    nMSu(4+ind3))*(nMSd(4+ind3d)-nMSu(4+ind4))*(nMSu(4+ind3)-
     -    nMSu(4+ind4))*(nMSd(4+ind3d)+nMSu(4+ind4))*(nMSu(4+ind3)+
     -    nMSu(4+ind4)))))/Pi**8
      end if
      end do
      end do
      end do
      end do
      
      
      Wsquarksonly1b=0d0
      do ind3=1,2
      do ind3d=1,2
      do ind4d=1,2
      do ind5d=1,2
      if(ind3d.eq.ind4d) then
      Wsquarksonly1b=Wsquarksonly1b+
     -   ((0,0.0029296875)*Ca**2*CONJG(gWud(ind3,ind3d))*
     -   (3*g4d(ind3d,ind5d,ind3d,ind5d)+g4d(ind3d,ind5d,ind5d,ind3d))*
     -   gWud(ind3,ind3d)*((2*A0(nMSd(4+ind5d)**2)+4*
     -   Ae(nMSd(4+ind5d)**2)+nMSd(4+ind5d)**2)*(nMSd(4+ind3d)**2-
     -   nMSu(4+ind3)**2)**2+2*(2*A0(nMSd(4+ind5d)**2)+
     -   nMSd(4+ind5d)**2)*(-nMSd(4+ind3d)**4+A0(nMSu(4+ind3)**2)*
     -   nMSu(4+ind3)**2+nMSd(4+ind3d)**2*nMSu(4+ind3)**2+
     -   A0(nMSd(4+ind3d)**2)*(nMSd(4+ind3d)**2-2*nMSu(4+ind3)**2))+
     -   4*nMSd(4+ind5d)**2*(Ae(nMSu(4+ind3)**2)*nMSu(4+ind3)**2+
     -   Ae(nMSd(4+ind3d)**2)*(nMSd(4+ind3d)**2-2*nMSu(4+ind3)**2)+
     -   A0(nMSd(4+ind3d)**2)*(-nMSd(4+ind3d)**2+nMSu(4+ind3)**2))))/
     -   (Pi**8*(nMSd(4+ind3d)**2-nMSu(4+ind3)**2)**2)
      else
      Wsquarksonly1b=Wsquarksonly1b+
     -   ((0,0.0029296875)*Ca**2*CONJG(gWud(ind3,ind4d))*
     -   (3*g4d(ind3d,ind5d,ind4d,ind5d)+g4d(ind3d,ind5d,ind5d,ind4d))*
     -   gWud(ind3,ind3d)*(2*A0(nMSd(4+ind5d)**2)+4*
     -   Ae(nMSd(4+ind5d)**2)+nMSd(4+ind5d)**2+(2*(2*
     -   A0(nMSd(4+ind5d)**2)+nMSd(4+ind5d)**2)*(A0(nMSu(4+ind3)**2)*
     -   (nMSd(4+ind3d)**2-nMSd(4+ind4d)**2)*nMSu(4+ind3)**2+
     -   A0(nMSd(4+ind3d)**2)*nMSd(4+ind3d)**2*(nMSd(4+ind4d)**2-
     -   nMSu(4+ind3)**2)+A0(nMSd(4+ind4d)**2)*nMSd(4+ind4d)**2*
     -   (-nMSd(4+ind3d)**2+nMSu(4+ind3)**2)))/((nMSd(4+ind3d)-
     -   nMSd(4+ind4d))*(nMSd(4+ind3d)+nMSd(4+ind4d))*(nMSd(4+ind3d)-
     -   nMSu(4+ind3))*(nMSd(4+ind4d)-nMSu(4+ind3))*(nMSd(4+ind3d)+
     -   nMSu(4+ind3))*(nMSd(4+ind4d)+nMSu(4+ind3)))+(4*
     -   nMSd(4+ind5d)**2*(Ae(nMSu(4+ind3)**2)*(nMSd(4+ind3d)**2-
     -   nMSd(4+ind4d)**2)*nMSu(4+ind3)**2+Ae(nMSd(4+ind3d)**2)*
     -   nMSd(4+ind3d)**2*(nMSd(4+ind4d)**2-nMSu(4+ind3)**2)+
     -   Ae(nMSd(4+ind4d)**2)*nMSd(4+ind4d)**2*(-nMSd(4+ind3d)**2+
     -   nMSu(4+ind3)**2)))/((nMSd(4+ind3d)-nMSd(4+ind4d))*
     -   (nMSd(4+ind3d)+nMSd(4+ind4d))*(nMSd(4+ind3d)-nMSu(4+ind3))*
     -   (nMSd(4+ind4d)-nMSu(4+ind3))*(nMSd(4+ind3d)+nMSu(4+ind3))*
     -   (nMSd(4+ind4d)+nMSu(4+ind3)))))/Pi**8
      end if
      end do
      end do
      end do
      end do
      
      Wsquarksonly6a=0d0
      do ind3=1,2
      do ind4=1,2
      do ind5=1,2
      if(ind3.eq.ind4) then
      Wsquarksonly6a=Wsquarksonly6a+
     -    (3*Ca**2*g2W2u(ind3,ind3)*(3*g4u(ind3,ind5,ind3,ind5)+
     -    g4u(ind3,ind5,ind5,ind3))*(-(Ae(nMSu(4+ind5)**2)*
     -    nMSu(4+ind3)**2)+A0(nMSu(4+ind5)**2)*(-A0(nMSu(4+ind3)**2)+
     -    nMSu(4+ind3)**2)+(A0(nMSu(4+ind3)**2)-Ae(nMSu(4+ind3)**2))*
     -    nMSu(4+ind5)**2))/(256.*Pi**8*nMSu(4+ind3)**2)
      else
      Wsquarksonly6a=Wsquarksonly6a+
     -    (-3*Ca**2*g2W2u(ind3,ind4)*(3*g4u(ind4,ind5,ind3,ind5)+
     -    g4u(ind4,ind5,ind5,ind3))*((A0(nMSu(4+ind3)**2)-
     -    A0(nMSu(4+ind4)**2))*A0(nMSu(4+ind5)**2)+Ae(nMSu(4+ind5)**2)*
     -    (nMSu(4+ind3)**2-nMSu(4+ind4)**2)+(Ae(nMSu(4+ind3)**2)-
     -    Ae(nMSu(4+ind4)**2))*nMSu(4+ind5)**2))/
     -    (256.*Pi**8*(nMSu(4+ind3)**2-nMSu(4+ind4)**2))
      end if
      end do
      end do
      end do
      
      Wsquarksonly6b=0d0
      do ind3d=1,2
      do ind4d=1,2
      do ind5d=1,2
      if(ind3d.eq.ind4d) then
      Wsquarksonly6b=Wsquarksonly6b+
     -   (3*Ca**2*g2W2d(ind3d,ind3d)*
     -   (3*g4d(ind3d,ind5d,ind3d,ind5d)+g4d(ind3d,ind5d,ind5d,ind3d))*
     -   (-(Ae(nMSd(4+ind5d)**2)*nMSd(4+ind3d)**2)+A0(nMSd(4+ind5d)**2)
     -   *(-A0(nMSd(4+ind3d)**2)+nMSd(4+ind3d)**2)+
     -   (A0(nMSd(4+ind3d)**2)-Ae(nMSd(4+ind3d)**2))*nMSd(4+ind5d)**2))
     -   /(256.*Pi**8*nMSd(4+ind3d)**2)
      else
      Wsquarksonly6b=Wsquarksonly6b+
     -   (-3*Ca**2*g2W2d(ind3d,ind4d)*
     -   (3*g4d(ind4d,ind5d,ind3d,ind5d)+g4d(ind4d,ind5d,ind5d,ind3d))*
     -   ((A0(nMSd(4+ind3d)**2)-A0(nMSd(4+ind4d)**2))*
     -   A0(nMSd(4+ind5d)**2)+Ae(nMSd(4+ind5d)**2)*(nMSd(4+ind3d)**2-
     -   nMSd(4+ind4d)**2)+(Ae(nMSd(4+ind3d)**2)-Ae(nMSd(4+ind4d)**2))*
     -   nMSd(4+ind5d)**2))/
     -   (256.*Pi**8*(nMSd(4+ind3d)**2-nMSd(4+ind4d)**2))
      end if
      end do
      end do
      end do
      
      Wudgluona=0d0
      do ind3=1,2
      do ind3d=1,2
      Wudgluona=Wudgluona+(g3**2*CONJG(gWud(ind3,ind3d))*
     -   gWud(ind3,ind3d)*(-(Ca**2*nMSd(4+ind3d)**6)-Ca**2*
     -   nMSu(4+ind3)**6-Ca**2*nMSd(4+ind3d)**2*nMSu(4+ind3)**2*
     -   (nMSd(4+ind3d)**2+nMSu(4+ind3)**2)+
     -   Idiv2(nMSu(4+ind3)**2,nMSd(4+ind3d)**2,0d0)*(nMSd(4+ind3d)**2+
     -   nMSu(4+ind3)**2)**2+2*(-2*Ca**2*nMSd(4+ind3d)**4*
     -   (A0(nMSd(4+ind3d)**2)+nMSd(4+ind3d)**2)-2*Ca**2*
     -   nMSu(4+ind3)**4*(A0(nMSu(4+ind3)**2)+nMSu(4+ind3)**2)+
     -   Idiv1(nMSu(4+ind3)**2,nMSd(4+ind3d)**2,0d0)*(nMSd(4+ind3d)**2+
     -   nMSu(4+ind3)**2)**2-Ca**2*(nMSd(4+ind3d)**2+nMSu(4+ind3)**2)*
     -   (A0(nMSu(4+ind3)**2)*nMSd(4+ind3d)**2+A0(nMSd(4+ind3d)**2)*
     -   nMSu(4+ind3)**2))+4*(-(Ca**2*nMSd(4+ind3d)**2*(
     -   A0(nMSd(4+ind3d)**2)**2+4*A0(nMSd(4+ind3d)**2)*
     -   nMSd(4+ind3d)**2+2*Ae(nMSd(4+ind3d)**2)*nMSd(4+ind3d)**2+4*
     -   nMSd(4+ind3d)**4))+Ifin(nMSu(4+ind3)**2,nMSd(4+ind3d)**2,0d0)*
     -   (nMSd(4+ind3d)**2+nMSu(4+ind3)**2)**2-Ca**2*(nMSd(4+ind3d)**2+
     -   nMSu(4+ind3)**2)*(A0(nMSd(4+ind3d)**2)*A0(nMSu(4+ind3)**2)+
     -   Ae(nMSu(4+ind3)**2)*nMSd(4+ind3d)**2+Ae(nMSd(4+ind3d)**2)*
     -   nMSu(4+ind3)**2)-Ca**2*nMSu(4+ind3)**2*(A0(nMSu(4+ind3)**2)**2
     -   +4*A0(nMSu(4+ind3)**2)*nMSu(4+ind3)**2+2*Ae(nMSu(4+ind3)**2)*
     -   nMSu(4+ind3)**2+4*nMSu(4+ind3)**4))))/
     -   (128.*Pi**8*(nMSd(4+ind3d)**2-nMSu(4+ind3)**2)**2)
      end do
      end do
      
      Wudgluonb1=0d0
      do ind3=1,2
      do ind3d=1,2
      Wudgluonb1=Wudgluonb1+
     -  (g3**2*CONJG(gWud(ind3,ind3d))*gWud(ind3,ind3d)*
     -  (4*Ca**2*Ae(nMSu(4+ind3)**2)*nMSd(4+ind3d)**4-
     -  4*Idiv1(nMSu(4+ind3)**2,nMSd(4+ind3d)**2,0d0)*nMSd(4+ind3d)**4-
     -  2*Idiv2(nMSu(4+ind3)**2,nMSd(4+ind3d)**2,0d0)*nMSd(4+ind3d)**4-
     -  8*Ifin(nMSu(4+ind3)**2,nMSd(4+ind3d)**2,0d0)*nMSd(4+ind3d)**4+
     -  4*Ca**2*Ae(nMSd(4+ind3d)**2)*nMSd(4+ind3d)**2*nMSu(4+ind3)**2+
     -  48*Ca**2*Ae(nMSu(4+ind3)**2)*nMSd(4+ind3d)**2*nMSu(4+ind3)**2-
     -  4*Idiv1(nMSu(4+ind3)**2,nMSd(4+ind3d)**2,0d0)*nMSd(4+ind3d)**2*
     -  nMSu(4+ind3)**2-2*Idiv2(nMSu(4+ind3)**2,nMSd(4+ind3d)**2,0d0)*
     -  nMSd(4+ind3d)**2*nMSu(4+ind3)**2-
     -  8*Ifin(nMSu(4+ind3)**2,nMSd(4+ind3d)**2,0d0)*nMSd(4+ind3d)**2*
     -  nMSu(4+ind3)**2+Ca**2*nMSd(4+ind3d)**4*nMSu(4+ind3)**2-
     -  24*Ca**2*Ae(nMSu(4+ind3)**2)*nMSu(4+ind3)**4+
     -  64*Ca**2*nMSd(4+ind3d)**2*nMSu(4+ind3)**4-21*Ca**2*
     -  nMSu(4+ind3)**6+12*Ca**2*A0(nMSu(4+ind3)**2)**2*
     -  (2*nMSd(4+ind3d)**2-nMSu(4+ind3)**2)+2*Ca**2*
     -  A0(nMSd(4+ind3d)**2)*nMSd(4+ind3d)**2*(2*A0(nMSu(4+ind3)**2)+
     -  nMSu(4+ind3)**2)+2*Ca**2*A0(nMSu(4+ind3)**2)*(nMSd(4+ind3d)**4+
     -  32*nMSd(4+ind3d)**2*nMSu(4+ind3)**2-10*nMSu(4+ind3)**4)))/
     -  (256.*Pi**8*(nMSd(4+ind3d)**2-nMSu(4+ind3)**2)**2)
      end do
      end do
      
      Wudgluonb2=0d0
      do ind3=1,2
      do ind3d=1,2
      Wudgluonb2=Wudgluonb2-(g3**2*CONJG(gWud(ind3,ind3d))*
     -  gWud(ind3,ind3d)*(21*Ca**2*nMSd(4+ind3d)**6-
     -  2*Ca**2*A0(nMSu(4+ind3)**2)*nMSd(4+ind3d)**2*nMSu(4+ind3)**2-
     -  4*Ca**2*Ae(nMSu(4+ind3)**2)*nMSd(4+ind3d)**2*nMSu(4+ind3)**2+
     -  4*Idiv1(nMSu(4+ind3)**2,nMSd(4+ind3d)**2,0d0)*nMSd(4+ind3d)**2*
     -  nMSu(4+ind3)**2+2*Idiv2(nMSu(4+ind3)**2,nMSd(4+ind3d)**2,0d0)*
     -  nMSd(4+ind3d)**2*nMSu(4+ind3)**2+
     -  8*Ifin(nMSu(4+ind3)**2,nMSd(4+ind3d)**2,0d0)*nMSd(4+ind3d)**2*
     -  nMSu(4+ind3)**2-64*Ca**2*nMSd(4+ind3d)**4*nMSu(4+ind3)**2+
     -  4*Idiv1(nMSu(4+ind3)**2,nMSd(4+ind3d)**2,0d0)*nMSu(4+ind3)**4+
     -  2*Idiv2(nMSu(4+ind3)**2,nMSd(4+ind3d)**2,0d0)*nMSu(4+ind3)**4+
     -  8*Ifin(nMSu(4+ind3)**2,nMSd(4+ind3d)**2,0d0)*nMSu(4+ind3)**4-
     -  Ca**2*nMSd(4+ind3d)**2*nMSu(4+ind3)**4+12*Ca**2*
     -  A0(nMSd(4+ind3d)**2)**2*(nMSd(4+ind3d)**2-2*nMSu(4+ind3)**2)+
     -  4*Ca**2*Ae(nMSd(4+ind3d)**2)*(6*nMSd(4+ind3d)**4-
     -  12*nMSd(4+ind3d)**2*nMSu(4+ind3)**2-nMSu(4+ind3)**4)+
     -  2*Ca**2*A0(nMSd(4+ind3d)**2)*(10*nMSd(4+ind3d)**4-32*
     -  nMSd(4+ind3d)**2*nMSu(4+ind3)**2-nMSu(4+ind3)**2*(2*
     -  A0(nMSu(4+ind3)**2)+nMSu(4+ind3)**2))))/
     -  (256.*Pi**8*(nMSd(4+ind3d)**2-nMSu(4+ind3)**2)**2)
      end do
      end do
      
      Wudgluonc1=0d0
      do ind3=1,2
      Wudgluonc1=Wudgluonc1
     -  +((0,-0.015625)*Ca**2*g3**2*g2W2u(ind3,ind3)*(3*
     -  A0(nMSu(4+ind3)**2)**2+2*A0(nMSu(4+ind3)**2)*nMSu(4+ind3)**2+
     -  6*Ae(nMSu(4+ind3)**2)*nMSu(4+ind3)**2+4*nMSu(4+ind3)**4))/
     -  (Pi**8*nMSu(4+ind3)**2)
      end do
      
      Wudgluonc2=0d0
      do ind3d=1,2
      Wudgluonc2=Wudgluonc2
     -  +((0,-0.015625)*Ca**2*g3**2*g2W2d(ind3d,ind3d)*(3*
     -  A0(nMSd(4+ind3d)**2)**2+2*A0(nMSd(4+ind3d)**2)*
     -  nMSd(4+ind3d)**2+6*Ae(nMSd(4+ind3d)**2)*nMSd(4+ind3d)**2+
     -  4*nMSd(4+ind3d)**4))/(Pi**8*nMSd(4+ind3d)**2)
      end do
      
      
      do ind3=1,2
      TFIdv2finSu(ind3)=TFIdv2fin(nMSu(4+ind3),mt,absM3)
      TFIdv7finSu(ind3)=TFIdv7fin(nMSu(4+ind3),mt,absM3)
      TFIdv7finSd(ind3)=TFIdv7fin(nMSd(4+ind3),mt,absM3)
      TFIdv8finSu(ind3)=TFIdv8fin(nMSu(4+ind3),mt,absM3)
      TFIdv9finSu(ind3)=TFIdv9fin(nMSu(4+ind3),mt,absM3)
      TFIdv9finSd(ind3)=TFIdv9fin(nMSd(4+ind3),mt,absM3)
      TFIdv15finSu(ind3)=TFIdv15fin(nMSu(4+ind3),mt,absM3)
      TFIdv15finSd(ind3)=TFIdv15fin(nMSd(4+ind3),mt,absM3)
      TFIdv16finSu(ind3)=TFIdv16fin(nMSu(4+ind3),mt,absM3)
      
      TFIdv2div1Su(ind3)=TFIdv2div1(nMSu(4+ind3),mt,absM3)
      TFIdv7div1Su(ind3)=TFIdv7div1(nMSu(4+ind3),mt,absM3)
      TFIdv7div1Sd(ind3)=TFIdv7div1(nMSd(4+ind3),mt,absM3)
      TFIdv8div1Su(ind3)=TFIdv8div1(nMSu(4+ind3),mt,absM3)
      TFIdv9div1Su(ind3)=TFIdv9div1(nMSu(4+ind3),mt,absM3)
      TFIdv9div1Sd(ind3)=TFIdv9div1(nMSd(4+ind3),mt,absM3)
      TFIdv15div1Su(ind3)=TFIdv15div1(nMSu(4+ind3),mt,absM3)
      TFIdv15div1Sd(ind3)=TFIdv15div1(nMSd(4+ind3),mt,absM3)
      TFIdv16div1Su(ind3)=TFIdv16div1(nMSu(4+ind3),mt,absM3)
      
      TFIdv2div2Su(ind3)=TFIdv2div2(nMSu(4+ind3),mt,absM3)
      TFIdv7div2Su(ind3)=TFIdv7div2(nMSu(4+ind3),mt,absM3)
      TFIdv7div2Sd(ind3)=TFIdv7div2(nMSd(4+ind3),mt,absM3)
      TFIdv8div2Su(ind3)=TFIdv8div2(nMSu(4+ind3),mt,absM3)
      TFIdv9div2Su(ind3)=TFIdv9div2(nMSu(4+ind3),mt,absM3)
      TFIdv9div2Sd(ind3)=TFIdv9div2(nMSd(4+ind3),mt,absM3)
      TFIdv15div2Su(ind3)=TFIdv15div2(nMSu(4+ind3),mt,absM3)
      TFIdv15div2Sd(ind3)=TFIdv15div2(nMSd(4+ind3),mt,absM3)
      TFIdv16div2Su(ind3)=TFIdv16div2(nMSu(4+ind3),mt,absM3)
      end do
      
      WudFg1=0d0
      do ind3d=1,2
      do ind3=1,2
      do ind4=1,2
      if(ind3.eq.ind4) then
      WudFg1=WudFg1-(CONJG(gWud(ind3,ind3d))*gWud(ind3,ind3d)*
     -   (CONJG(gRtFgSu(ind3))*(gRtFgSu(ind3)*(-2*TFIdv9div1Sd(ind3d)+
     -   2*TFIdv9div1Su(ind3)-TFIdv9div2Sd(ind3d)+TFIdv9div2Su(ind3)-
     -   4*TFIdv9finSd(ind3d)+4*TFIdv9finSu(ind3)+nMSd(4+ind3d)**2*
     -   (2*TFIdv8div1Su(ind3)+TFIdv8div2Su(ind3)+4*TFIdv8finSu(ind3)-
     -   2*TFIdv16div1Su(ind3)-TFIdv16div2Su(ind3)-
     -   4*TFIdv16finSu(ind3))+nMSu(4+ind3)**2*(-2*TFIdv8div1Su(ind3)-
     -   TFIdv8div2Su(ind3)-4*TFIdv8finSu(ind3)+2*TFIdv16div1Su(ind3)+
     -   TFIdv16div2Su(ind3)+4*TFIdv16finSu(ind3))+
     -   2*TFIdv15div1Sd(ind3d)-2*TFIdv15div1Su(ind3)+
     -   TFIdv15div2Sd(ind3d)-TFIdv15div2Su(ind3)+
     -   4*TFIdv15finSd(ind3d)-4*TFIdv15finSu(ind3))+absM3*mt*
     -   gLtFgSu(ind3)*(2*TFIdv7div1Sd(ind3d)-2*TFIdv7div1Su(ind3)+
     -   TFIdv7div2Sd(ind3d)-TFIdv7div2Su(ind3)+4*TFIdv7finSd(ind3d)-
     -   4*TFIdv7finSu(ind3)-2*nMSd(4+ind3d)**2*TFIdv2div1Su(ind3)+
     -   2*nMSu(4+ind3)**2*TFIdv2div1Su(ind3)-nMSd(4+ind3d)**2*
     -   TFIdv2div2Su(ind3)+nMSu(4+ind3)**2*TFIdv2div2Su(ind3)-
     -   4*nMSd(4+ind3d)**2*TFIdv2finSu(ind3)+4*nMSu(4+ind3)**2*
     -   TFIdv2finSu(ind3)))+CONJG(gLtFgSu(ind3))*(gLtFgSu(ind3)*
     -   (-2*TFIdv9div1Sd(ind3d)+2*TFIdv9div1Su(ind3)-
     -   TFIdv9div2Sd(ind3d)+TFIdv9div2Su(ind3)-4*TFIdv9finSd(ind3d)+
     -   4*TFIdv9finSu(ind3)+nMSd(4+ind3d)**2*(2*TFIdv8div1Su(ind3)+
     -   TFIdv8div2Su(ind3)+4*TFIdv8finSu(ind3)-2*TFIdv16div1Su(ind3)-
     -   TFIdv16div2Su(ind3)-4*TFIdv16finSu(ind3))+nMSu(4+ind3)**2*
     -   (-2*TFIdv8div1Su(ind3)-TFIdv8div2Su(ind3)-4*TFIdv8finSu(ind3)+
     -   2*TFIdv16div1Su(ind3)+TFIdv16div2Su(ind3)+
     -   4*TFIdv16finSu(ind3))+2*TFIdv15div1Sd(ind3d)-
     -   2*TFIdv15div1Su(ind3)+TFIdv15div2Sd(ind3d)-
     -   TFIdv15div2Su(ind3)+4*TFIdv15finSd(ind3d)-
     -   4*TFIdv15finSu(ind3))+absM3*mt*gRtFgSu(ind3)*
     -   (2*TFIdv7div1Sd(ind3d)-2*TFIdv7div1Su(ind3)+
     -   TFIdv7div2Sd(ind3d)-TFIdv7div2Su(ind3)+4*TFIdv7finSd(ind3d)-
     -   4*TFIdv7finSu(ind3)-2*nMSd(4+ind3d)**2*TFIdv2div1Su(ind3)+
     -   2*nMSu(4+ind3)**2*TFIdv2div1Su(ind3)-nMSd(4+ind3d)**2*
     -   TFIdv2div2Su(ind3)+nMSu(4+ind3)**2*TFIdv2div2Su(ind3)-
     -   4*nMSd(4+ind3d)**2*TFIdv2finSu(ind3)+4*nMSu(4+ind3)**2*
     -   TFIdv2finSu(ind3)))))/
     -   (32.*Pi**8*(nMSd(4+ind3d)**2-nMSu(4+ind3)**2)**2)
      else
      WudFg1=WudFg1+(CONJG(gWud(ind4,ind3d))*gWud(ind3,ind3d)*
     -   (CONJG(gLtFgSu(ind3))*(absM3*mt*gRtFgSu(ind4)*
     -   (nMSu(4+ind4)**2*(2*TFIdv7div1Sd(ind3d)-
     -    2*TFIdv7div1Su(ind3)+TFIdv7div2Sd(ind3d)-TFIdv7div2Su(ind3)+
     -    4*TFIdv7finSd(ind3d)-4*TFIdv7finSu(ind3))+nMSd(4+ind3d)**2*
     -    (2*TFIdv7div1Su(ind3)-2*TFIdv7div1Su(ind4)+
     -    TFIdv7div2Su(ind3)-TFIdv7div2Su(ind4)+4*TFIdv7finSu(ind3)-
     -    4*TFIdv7finSu(ind4))+nMSu(4+ind3)**2*(-2*TFIdv7div1Sd(ind3d)+
     -    2*TFIdv7div1Su(ind4)-TFIdv7div2Sd(ind3d)+TFIdv7div2Su(ind4)-
     -    4*TFIdv7finSd(ind3d)+4*TFIdv7finSu(ind4)))+gLtFgSu(ind4)*
     -    (nMSu(4+ind4)**2*(-2*TFIdv9div1Sd(ind3d)+2*
     -    TFIdv9div1Su(ind3)-TFIdv9div2Sd(ind3d)+TFIdv9div2Su(ind3)-
     -    4*TFIdv9finSd(ind3d)+4*TFIdv9finSu(ind3)+
     -    2*TFIdv15div1Sd(ind3d)-2*TFIdv15div1Su(ind3)+
     -    TFIdv15div2Sd(ind3d)-TFIdv15div2Su(ind3)+
     -    4*TFIdv15finSd(ind3d)-4*TFIdv15finSu(ind3))+nMSd(4+ind3d)**2*
     -    (-2*TFIdv9div1Su(ind3)+2*TFIdv9div1Su(ind4)-
     -    TFIdv9div2Su(ind3)+TFIdv9div2Su(ind4)-4*TFIdv9finSu(ind3)+
     -    4*TFIdv9finSu(ind4)+2*TFIdv15div1Su(ind3)-
     -    2*TFIdv15div1Su(ind4)+TFIdv15div2Su(ind3)-
     -    TFIdv15div2Su(ind4)+4*TFIdv15finSu(ind3)-
     -    4*TFIdv15finSu(ind4))+nMSu(4+ind3)**2*(2*TFIdv9div1Sd(ind3d)-
     -    2*TFIdv9div1Su(ind4)+ TFIdv9div2Sd(ind3d)-TFIdv9div2Su(ind4)+
     -    4*TFIdv9finSd(ind3d)-4*TFIdv9finSu(ind4)-
     -    2*TFIdv15div1Sd(ind3d)+2*TFIdv15div1Su(ind4)-
     -    TFIdv15div2Sd(ind3d)+TFIdv15div2Su(ind4)-
     -    4*TFIdv15finSd(ind3d)+4*TFIdv15finSu(ind4))))+
     -    CONJG(gRtFgSu(ind3))*(absM3*mt*gLtFgSu(ind4)*(nMSu(4+ind4)**2
     -    *(2*TFIdv7div1Sd(ind3d)-2*TFIdv7div1Su(ind3)+
     -    TFIdv7div2Sd(ind3d)-TFIdv7div2Su(ind3)+4*TFIdv7finSd(ind3d)-
     -    4*TFIdv7finSu(ind3))+nMSd(4+ind3d)**2*
     -    (2*TFIdv7div1Su(ind3)-2*TFIdv7div1Su(ind4)+
     -    TFIdv7div2Su(ind3)-TFIdv7div2Su(ind4)+4*TFIdv7finSu(ind3)-
     -    4*TFIdv7finSu(ind4))+nMSu(4+ind3)**2*(-2*TFIdv7div1Sd(ind3d)+
     -    2*TFIdv7div1Su(ind4)-TFIdv7div2Sd(ind3d)+TFIdv7div2Su(ind4)-
     -    4*TFIdv7finSd(ind3d)+4*TFIdv7finSu(ind4)))+gRtFgSu(ind4)*
     -    (nMSu(4+ind4)**2*(-2*TFIdv9div1Sd(ind3d)+2*TFIdv9div1Su(ind3)
     -    -TFIdv9div2Sd(ind3d)+TFIdv9div2Su(ind3)-4*TFIdv9finSd(ind3d)+
     -    4*TFIdv9finSu(ind3)+2*TFIdv15div1Sd(ind3d)-
     -    2*TFIdv15div1Su(ind3)+TFIdv15div2Sd(ind3d)-
     -    TFIdv15div2Su(ind3)+4*TFIdv15finSd(ind3d)-
     -    4*TFIdv15finSu(ind3))+nMSd(4+ind3d)**2*(-2*
     -    TFIdv9div1Su(ind3)+2*TFIdv9div1Su(ind4)-TFIdv9div2Su(ind3)+
     -    TFIdv9div2Su(ind4)-4*TFIdv9finSu(ind3)+4*TFIdv9finSu(ind4)+
     -    2*TFIdv15div1Su(ind3)-2*TFIdv15div1Su(ind4)+
     -    TFIdv15div2Su(ind3)-TFIdv15div2Su(ind4)+
     -    4*TFIdv15finSu(ind3)-4*TFIdv15finSu(ind4))+nMSu(4+ind3)**2*
     -    (2*TFIdv9div1Sd(ind3d)-2*TFIdv9div1Su(ind4)+
     -    TFIdv9div2Sd(ind3d)-TFIdv9div2Su(ind4)+4*TFIdv9finSd(ind3d)-
     -    4*TFIdv9finSu(ind4)-2*TFIdv15div1Sd(ind3d)+
     -    2*TFIdv15div1Su(ind4)-TFIdv15div2Sd(ind3d)+
     -    TFIdv15div2Su(ind4)-4*TFIdv15finSd(ind3d)+
     -    4*TFIdv15finSu(ind4))))))/(32.*Pi**8*(nMSd(4+ind3d)**2-
     -    nMSu(4+ind3)**2)*(nMSd(4+ind3d)**2-nMSu(4+ind4)**2)*
     -    (nMSu(4+ind3)**2-nMSu(4+ind4)**2))
      end if
      end do
      end do
      end do
!       
      
      do ind3=1,2
      TFIdv8finSd(ind3)=TFIdv8fin(nMSd(4+ind3),0d0,absM3)
      TFIdv9finSu(ind3)=TFIdv9fin(nMSu(4+ind3),0d0,absM3)
      TFIdv9finSd(ind3)=TFIdv9fin(nMSd(4+ind3),0d0,absM3)
      TFIdv15finSu(ind3)=TFIdv15fin(nMSu(4+ind3),0d0,absM3)
      TFIdv15finSd(ind3)=TFIdv15fin(nMSd(4+ind3),0d0,absM3)
      TFIdv16finSd(ind3)=TFIdv16fin(nMSd(4+ind3),0d0,absM3)
      
      TFIdv8div1Sd(ind3)=TFIdv8div1(nMSd(4+ind3),0d0,absM3)
      TFIdv9div1Su(ind3)=TFIdv9div1(nMSu(4+ind3),0d0,absM3)
      TFIdv9div1Sd(ind3)=TFIdv9div1(nMSd(4+ind3),0d0,absM3)
      TFIdv15div1Su(ind3)=TFIdv15div1(nMSu(4+ind3),0d0,absM3)
      TFIdv15div1Sd(ind3)=TFIdv15div1(nMSd(4+ind3),0d0,absM3)
      TFIdv16div1Sd(ind3)=TFIdv16div1(nMSd(4+ind3),0d0,absM3)
      
      TFIdv8div2Sd(ind3)=TFIdv8div2(nMSd(4+ind3),0d0,absM3)
      TFIdv9div2Su(ind3)=TFIdv9div2(nMSu(4+ind3),0d0,absM3)
      TFIdv9div2Sd(ind3)=TFIdv9div2(nMSd(4+ind3),0d0,absM3)
      TFIdv15div2Su(ind3)=TFIdv15div2(nMSu(4+ind3),0d0,absM3)
      TFIdv15div2Sd(ind3)=TFIdv15div2(nMSd(4+ind3),0d0,absM3)
      TFIdv16div2Sd(ind3)=TFIdv16div2(nMSd(4+ind3),0d0,absM3)
      end do
      
      WudFg2=0d0
      do ind3=1,2
      do ind3d=1,2
      do ind4d=1,2
      if(ind3d.eq.ind4d) then
      WudFg2=WudFg2+(CONJG(gWud(ind3,ind3d))*(CONJG(gLbFgSd(ind3d))*
     -    gLbFgSd(ind3d)+CONJG(gRbFgSd(ind3d))*gRbFgSd(ind3d))*
     -    gWud(ind3,ind3d)*(-2*TFIdv9div1Sd(ind3d)+2*
     -    TFIdv9div1Su(ind3)-TFIdv9div2Sd(ind3d)+TFIdv9div2Su(ind3)-
     -    4*TFIdv9finSd(ind3d)+4*TFIdv9finSu(ind3)+nMSd(4+ind3d)**2*
     -    (2*TFIdv8div1Sd(ind3d)+TFIdv8div2Sd(ind3d)+
     -    4*TFIdv8finSd(ind3d)-2*TFIdv16div1Sd(ind3d)-
     -    TFIdv16div2Sd(ind3d)-4*TFIdv16finSd(ind3d))+nMSu(4+ind3)**2*
     -    (-2*TFIdv8div1Sd(ind3d)-TFIdv8div2Sd(ind3d)-
     -    4*TFIdv8finSd(ind3d)+2*TFIdv16div1Sd(ind3d)+
     -    TFIdv16div2Sd(ind3d)+4*TFIdv16finSd(ind3d))
     -    +2*TFIdv15div1Sd(ind3d)-2*TFIdv15div1Su(ind3)+
     -    TFIdv15div2Sd(ind3d)-TFIdv15div2Su(ind3)+4*
     -    TFIdv15finSd(ind3d)-4*TFIdv15finSu(ind3)))/
     -    (32.*Pi**8*(nMSd(4+ind3d)**2-nMSu(4+ind3)**2)**2)
      else
      WudFg2=WudFg2+(CONJG(gWud(ind3,ind4d))*(CONJG(gLbFgSd(ind3d))*
     -   gLbFgSd(ind4d)+CONJG(gRbFgSd(ind3d))*gRbFgSd(ind4d))*
     -   gWud(ind3,ind3d)*(nMSu(4+ind3)**2*(-2*TFIdv9div1Sd(ind3d)+
     -   2*TFIdv9div1Sd(ind4d)-TFIdv9div2Sd(ind3d)+TFIdv9div2Sd(ind4d)-
     -   4*TFIdv9finSd(ind3d)+4*TFIdv9finSd(ind4d)+
     -   2*TFIdv15div1Sd(ind3d)-2*TFIdv15div1Sd(ind4d)+
     -   TFIdv15div2Sd(ind3d)-TFIdv15div2Sd(ind4d)+
     -   4*TFIdv15finSd(ind3d)-4*TFIdv15finSd(ind4d))+nMSd(4+ind3d)**2*
     -   (-2*TFIdv9div1Sd(ind4d)+2*TFIdv9div1Su(ind3)-
     -   TFIdv9div2Sd(ind4d)+TFIdv9div2Su(ind3)-4*TFIdv9finSd(ind4d)+
     -   4*TFIdv9finSu(ind3)+2*TFIdv15div1Sd(ind4d)-
     -   2*TFIdv15div1Su(ind3)+
     -   TFIdv15div2Sd(ind4d)-TFIdv15div2Su(ind3)+
     -   4*TFIdv15finSd(ind4d)-4*TFIdv15finSu(ind3))+nMSd(4+ind4d)**2*
     -   (2*TFIdv9div1Sd(ind3d)-2*TFIdv9div1Su(ind3)+
     -   TFIdv9div2Sd(ind3d)-TFIdv9div2Su(ind3)+4*TFIdv9finSd(ind3d)-
     -   4*TFIdv9finSu(ind3)-2*TFIdv15div1Sd(ind3d)+
     -   2*TFIdv15div1Su(ind3)-TFIdv15div2Sd(ind3d)+
     -   TFIdv15div2Su(ind3)-4*TFIdv15finSd(ind3d)+
     -   4*TFIdv15finSu(ind3))))/(32.*Pi**8*(nMSd(4+ind3d)**2-
     -   nMSd(4+ind4d)**2)*(nMSd(4+ind3d)**2-nMSu(4+ind3)**2)*
     -   (nMSd(4+ind4d)**2-nMSu(4+ind3)**2))
      end if
      end do
      end do
      end do
      
      WudFg3=0d0
      do ind3=1,2
      do ind4=1,2
      if(ind3.eq.ind4) then
      WudFg3=WudFg3+((0,-0.125)*g2W2u(ind3,ind3)*
     -  (CONJG(gLtFgSu(ind3))*(absM3*mt*gRtFgSu(ind3)*
     -  TFIdv14fin(mt,absM3,nMSu(4+ind3))+
     -  gLtFgSu(ind3)*TFIdv10fin(mt,absM3,nMSu(4+ind3)))+
     -  CONJG(gRtFgSu(ind3))*(absM3*mt*gLtFgSu(ind3)*
     -  TFIdv14fin(mt,absM3,nMSu(4+ind3))+
     -  gRtFgSu(ind3)*TFIdv10fin(mt,absM3,nMSu(4+ind3)))))/Pi**8
      else
      WudFg3=WudFg3+((0,-0.125)*g2W2u(ind3,ind4)*
     -   (CONJG(gLtFgSu(ind3))*(absM3*mt*gRtFgSu(ind4)*
     -   (Ifin(mt**2,absM3**2,nMSu(4+ind3)**2)-
     -   Ifin(mt**2,absM3**2,nMSu(4+ind4)**2))+
     -   gLtFgSu(ind4)*(TFIdv6fin(mt,absM3,nMSu(4+ind3))-
     -   TFIdv6fin(mt,absM3,nMSu(4+ind4))))+
     -   CONJG(gRtFgSu(ind3))*(absM3*mt*gLtFgSu(ind4)*
     -   (Ifin(mt**2,absM3**2,nMSu(4+ind3)**2)-
     -   Ifin(mt**2,absM3**2,nMSu(4+ind4)**2))+
     -   gRtFgSu(ind4)*(TFIdv6fin(mt,absM3,nMSu(4+ind3))-
     -   TFIdv6fin(mt,absM3,nMSu(4+ind4))))))/
     -   (Pi**8*(nMSu(4+ind3)**2-nMSu(4+ind4)**2))
      end if
      end do
      end do
      
      WudFg4=0d0
      do ind3d=1,2
      do ind4d=1,2
      if(ind3d.eq.ind4d) then
      WudFg4=WudFg4+((0d0,-0.125)*g2W2d(ind3d,ind3d)*
     -   (CONJG(gLbFgSd(ind3d))*gLbFgSd(ind3d)+CONJG(gRbFgSd(ind3d))*
     -   gRbFgSd(ind3d))*TFIdv10fin(0d0,absM3,nMSd(4+ind3d)))/Pi**8
      else
      WudFg4=WudFg4+((0d0,-0.125)*g2W2d(ind3d,ind4d)*
     -   (CONJG(gLbFgSd(ind4d))*gLbFgSd(ind3d)+CONJG(gRbFgSd(ind4d))*
     -   gRbFgSd(ind3d))*(TFIdv6fin(0d0,absM3,nMSd(4+ind3d))-
     -   TFIdv6fin(0d0,absM3,nMSd(4+ind4d))))/
     -   (Pi**8*(nMSd(4+ind3d)**2-nMSd(4+ind4d)**2))
      end if
      end do
      end do
      
      Wudgluond=0d0
      Wudgluone=0d0
      do ind3=1,2
      do ind4=1,2
      Wudgluond=Wudgluond+(g3**2*(CONJG(gWud(ind3,ind4))*
     -   CONJG(gWudgluon(ind4,ind3))+gWud(ind3,ind4)*
     -   gWudgluon(ind4,ind3))*(12*Ca**2*A0(nMSd(4+ind4)**2)**2-
     -   12*Ca**2*A0(nMSu(4+ind3)**2)**2+
     -   44*Ca**2*A0(nMSd(4+ind4)**2)*nMSd(4+ind4)**2+
     -   24*Ca**2*Ae(nMSd(4+ind4)**2)*nMSd(4+ind4)**2+
     -   4*Idiv1(nMSu(4+ind3)**2,nMSd(4+ind4)**2,0d0)*nMSd(4+ind4)**2+
     -   2*Idiv2(nMSu(4+ind3)**2,nMSd(4+ind4)**2,0d0)*nMSd(4+ind4)**2+
     -   8*Ifin(nMSu(4+ind3)**2,nMSd(4+ind4)**2,0d0)*nMSd(4+ind4)**2+
     -   43*Ca**2*nMSd(4+ind4)**4-44*Ca**2*A0(nMSu(4+ind3)**2)*
     -   nMSu(4+ind3)**2-24*Ca**2*Ae(nMSu(4+ind3)**2)*nMSu(4+ind3)**2-
     -   4*Idiv1(nMSu(4+ind3)**2,nMSd(4+ind4)**2,0d0)*nMSu(4+ind3)**2-
     -   2*Idiv2(nMSu(4+ind3)**2,nMSd(4+ind4)**2,0d0)*nMSu(4+ind3)**2-
     -   8*Ifin(nMSu(4+ind3)**2,nMSd(4+ind4)**2,0d0)*nMSu(4+ind3)**2-
     -   43*Ca**2*nMSu(4+ind3)**4))/(512.*Pi**8*(nMSd(4+ind4)**2-
     -   nMSu(4+ind3)**2))
      Wudgluone=Wudgluone-(g3**2*CONJG(gWudgluon(ind4,ind3))*
     -    gWudgluon(ind4,ind3)*
     -    Ifin(nMSu(4+ind3)**2,nMSd(4+ind4)**2,0d0))/(16.*Pi**8)
      end do
      end do
      
      dmbLfin=Abs(nZU(5,5))**2*dMstfin(1,1)+Abs(nZU(5,6))**2
     -       *dMstfin(2,2)-CONJG(nZU(5,6))*nZU(6,6)*dMstfin(1,2)
     -       -nZU(5,6)*CONJG(nZU(6,6))*dMstfin(2,1)
     -       -2*mt*dmtfin
      
     
      dmbLdiv1=Abs(nZU(5,5))**2*dMstdiv1(1,1)+Abs(nZU(5,6))**2
     -       *dMstdiv1(2,2)-CONJG(nZU(5,6))*nZU(6,6)*dMstdiv1(1,2)
     -       -nZU(5,6)*CONJG(nZU(6,6))*dMstdiv1(2,1)
     -       -2*mt*dmtdiv1
      
      
      do k=1,2
      do l=1,2
      dMsbfin(k,l)=nZD(k+4,5)*CONJG(nZD(l+4,5))*dmbLfin
      dMsbdiv1(k,l)=nZD(k+4,5)*CONJG(nZD(l+4,5))*dmbLdiv1
      dMsbep(k,l)=0d0
      end do
      end do

      
c Counterterm-inserted Diagrams
      CTWfermion=((0,-0.09375)*Ca*gWtb*((dmtdiv1+4*dmtep-6*dmtfin)*
     -   mt**2+(-6*dmtdiv1+4*dmtfin)*A0(mt**2)+4*dmtdiv1*Ae(mt**2))*
     -   CONJG(gWtb))/(mt*Pi**4)
     
      CTWsquark1=0d0
      do ind3=1,2
      do ind4=1,2
      do ind3d=1,2
      if(ind3.eq.ind4) then
      CTWsquark1=CTWsquark1+
     -    ((0,-0.046875)*Ca*CONJG(gWud(ind3,ind3d))*gWud(ind3,ind3d)*
     -    (4*Ae(nMSd(4+ind3d)**2)*dMstdiv1(ind3,ind3)*nMSd(4+ind3d)**2-
     -    8*Ae(nMSu(4+ind3)**2)*dMstdiv1(ind3,ind3)*nMSd(4+ind3d)**2-
     -    8*A0(nMSu(4+ind3)**2)*dMstfin(ind3,ind3)*nMSd(4+ind3d)**2+
     -    2*A0(nMSd(4+ind3d)**2)*(dMstdiv1(ind3,ind3)+2*
     -    dMstfin(ind3,ind3))*nMSd(4+ind3d)**2+dMstdiv1(ind3,ind3)*
     -    nMSd(4+ind3d)**4+4*dMstep(ind3,ind3)*nMSd(4+ind3d)**4+
     -    2*dMstfin(ind3,ind3)*nMSd(4+ind3d)**4-
     -    2*A0(nMSu(4+ind3)**2)*dMstdiv1(ind3,ind3)*nMSu(4+ind3)**2+
     -    4*Ae(nMSu(4+ind3)**2)*dMstdiv1(ind3,ind3)*nMSu(4+ind3)**2+
     -    4*A0(nMSu(4+ind3)**2)*dMstfin(ind3,ind3)*nMSu(4+ind3)**2-
     -    8*dMstep(ind3,ind3)*nMSd(4+ind3d)**2*nMSu(4+ind3)**2-
     -    dMstdiv1(ind3,ind3)*nMSu(4+ind3)**4+
     -    4*dMstep(ind3,ind3)*nMSu(4+ind3)**4-
     -    2*dMstfin(ind3,ind3)*nMSu(4+ind3)**4))/
     -    (Pi**4*(nMSd(4+ind3d)**2-nMSu(4+ind3)**2)**2)
      else
      CTWsquark1=CTWsquark1+
     -    ((0,0.046875)*Ca*CONJG(gWud(ind4,ind3d))*gWud(ind3,ind3d)*
     -    (-((dMstdiv1(ind3,ind4)+4*dMstep(ind3,ind4)+2*
     -    dMstfin(ind3,ind4))*(nMSd(4+ind3d)**2-nMSu(4+ind3)**2)*
     -    (nMSd(4+ind3d)**2-nMSu(4+ind4)**2)*(nMSu(4+ind3)**2-
     -    nMSu(4+ind4)**2))-2*(dMstdiv1(ind3,ind4)+2*
     -    dMstfin(ind3,ind4))*(A0(nMSu(4+ind4)**2)*(nMSd(4+ind3d)**2-
     -    nMSu(4+ind3)**2)*nMSu(4+ind4)**2+A0(nMSd(4+ind3d)**2)*
     -    nMSd(4+ind3d)**2*(nMSu(4+ind3)**2-nMSu(4+ind4)**2)+
     -    A0(nMSu(4+ind3)**2)*nMSu(4+ind3)**2*(-nMSd(4+ind3d)**2+
     -    nMSu(4+ind4)**2))-4*dMstdiv1(ind3,ind4)*(Ae(nMSu(4+ind4)**2)*
     -    (nMSd(4+ind3d)**2-nMSu(4+ind3)**2)*nMSu(4+ind4)**2+
     -    Ae(nMSd(4+ind3d)**2)*nMSd(4+ind3d)**2*(nMSu(4+ind3)**2-
     -    nMSu(4+ind4)**2)+Ae(nMSu(4+ind3)**2)*nMSu(4+ind3)**2*
     -    (-nMSd(4+ind3d)**2+nMSu(4+ind4)**2))))/(Pi**4*(nMSd(4+ind3d)-
     -    nMSu(4+ind3))*(nMSd(4+ind3d)+nMSu(4+ind3))*
     -    (nMSd(4+ind3d)-nMSu(4+ind4))*(nMSu(4+ind3)-nMSu(4+ind4))*
     -    (nMSd(4+ind3d)+nMSu(4+ind4))*(nMSu(4+ind3)+nMSu(4+ind4)))
      end if
      end do
      end do
      end do
      
      
      CTWsquark2=0d0
      do ind3=1,2
      ind3d=1
      ind4d=1
      CTWsquark2=CTWsquark2+
     -    ((0,0.046875)*Ca*CONJG(gWud(ind3,ind3d))*gWud(ind3,ind3d)*
     -    (dMsbdiv1(ind3d,ind3d)*nMSd(4+ind3d)**4-
     -    4*dMsbep(ind3d,ind3d)*nMSd(4+ind3d)**4+
     -    2*dMsbfin(ind3d,ind3d)*nMSd(4+ind3d)**4-
     -    2*A0(nMSu(4+ind3)**2)*dMsbdiv1(ind3d,ind3d)*nMSu(4+ind3)**2-
     -    4*Ae(nMSu(4+ind3)**2)*dMsbdiv1(ind3d,ind3d)*nMSu(4+ind3)**2-
     -    4*A0(nMSu(4+ind3)**2)*dMsbfin(ind3d,ind3d)*nMSu(4+ind3)**2+
     -    8*dMsbep(ind3d,ind3d)*nMSd(4+ind3d)**2*nMSu(4+ind3)**2-
     -    dMsbdiv1(ind3d,ind3d)*nMSu(4+ind3)**4-
     -    4*dMsbep(ind3d,ind3d)*nMSu(4+ind3)**4-
     -    2*dMsbfin(ind3d,ind3d)*nMSu(4+ind3)**4-
     -    4*Ae(nMSd(4+ind3d)**2)*dMsbdiv1(ind3d,ind3d)*
     -    (nMSd(4+ind3d)**2-2*nMSu(4+ind3)**2)+2*A0(nMSd(4+ind3d)**2)*
     -    (dMsbdiv1(ind3d,ind3d)*nMSd(4+ind3d)**2-2*
     -    dMsbfin(ind3d,ind3d)*(nMSd(4+ind3d)**2-2*nMSu(4+ind3)**2))))/
     -    (Pi**4*(nMSd(4+ind3d)**2-nMSu(4+ind3)**2)**2)
      end do
      
      
      CTWsquark3=0d0
      do ind3=1,2
      do ind4=1,2
      if(ind3.eq.ind4) then
      CTWsquark3=CTWsquark3+
     -    (-3*Ca*g2W2u(ind3,ind3)*(-(Ae(nMSu(4+ind3)**2)*
     -    dMstdiv1(ind3,ind3))+A0(nMSu(4+ind3)**2)*
     -    (dMstdiv1(ind3,ind3)-dMstfin(ind3,ind3))+
     -    (-dMstep(ind3,ind3)+dMstfin(ind3,ind3))*nMSu(4+ind3)**2))/
     -    (16.*Pi**4*nMSu(4+ind3)**2)
      else
      CTWsquark3=CTWsquark3+
     -    (3*Ca*g2W2u(ind4,ind3)*(Ae(nMSu(4+ind3)**2)*
     -    dMstdiv1(ind4,ind3)-Ae(nMSu(4+ind4)**2)*dMstdiv1(ind4,ind3)+
     -    A0(nMSu(4+ind3)**2)*dMstfin(ind4,ind3)-A0(nMSu(4+ind4)**2)*
     -    dMstfin(ind4,ind3)+dMstep(ind4,ind3)*nMSu(4+ind3)**2-
     -    dMstep(ind4,ind3)*nMSu(4+ind4)**2))/
     -    (16.*Pi**4*(nMSu(4+ind3)**2-nMSu(4+ind4)**2))
      end if
      end do
      end do
      
      CTWsquark4=0d0
      ind3d=1
      ind4d=1
      CTWsquark4=(-3*Ca*g2W2d(ind3d,ind3d)*(-(Ae(nMSd(4+ind3d)**2)*
     -    dMsbdiv1(ind3d,ind3d))+A0(nMSd(4+ind3d)**2)*
     -    (dMsbdiv1(ind3d,ind3d)-dMsbfin(ind3d,ind3d))+
     -    (-dMsbep(ind3d,ind3d)+dMsbfin(ind3d,ind3d))*
     -    nMSd(4+ind3d)**2))/(16.*Pi**4*nMSd(4+ind3d)**2)
      
      
      CTW=CTWfermion+CTWsquark1+CTWsquark2+CTWsquark3+CTWsquark4
      
      dMW2=-(Wtgluon+WtFg1+WtFg2+WtFg3+WtFg4+Wsquarksonly1a+
     .     Wsquarksonly1b+Wsquarksonly6a+Wsquarksonly6b+Wudgluona+
     .     Wudgluonb1+Wudgluonc1+WudFg1+WudFg2+WudFg3+Wudgluonb2+
     .     Wudgluonc2+WudFg4+Wudgluond+Wudgluone+CTW)
       
      dvfunc=Realpart(2*(CThetaW**4*dMZ2-2*CThetaW**2*dMW2+dMW2)/v)
      
      return
      end
      
c--------------------------------------------------------------------------------------------c

c counterterm mass matrix in the gaugeless limit      
      subroutine counterM2loop(dMHpm2,dthu,dthd,dths,dtad,dtas,dv,dM)

      implicit none

      double precision Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      
      double precision abslambda, abskappa, absAlambda, absAkappa,vs,
     &                 mueff,tanb,beta
      
      double precision phiu, phis, phil, phik, phiAk, phiAl, phiM1,
     &                phiM2, phiAu(3), phiAd(3), phiAe(3)
     
      double precision Mh(5),MHpm,Mcha(2),Mchi(5), MSe(6),MSd(6),
     &                 MSu(6),MSv(3)
     
      double precision phi,phiIl,phiIk
   
      double precision dv,dMHpm2,dthu,dthd,dths,dtad,dtas

      double precision dM(5,5)

      double precision ImAkappaDR,ImAlambdaDR,ReAkappaDR,ReAlambdaDR 
      
      logical MHpmInput,CPcon,mQrun,onelswitch
      double precision selfcha2loop,dMHpm22l
     
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
     
      
      common /SMinput/ Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      common /Higgsinput/ abslambda, abskappa, absAlambda, absAkappa,
     &                    vs,mueff,tanb,beta
      common /phases/ phiu, phis, phil, phik, phiAk, phiAl, phiM1,
     &                phiM2, phiAu, phiAd, phiAe
      common /treemasses/ Mh,MHpm,Mcha,Mchi, MSe,MSd,MSu,MSv
      common /switch/ MHpmInput,CPcon,mQrun,onelswitch
      common /MHpm2l/ dMHpm22l,selfcha2loop
      common /DRpara/ ReAlambdaDR,ReAkappaDR,ImAlambdaDR,ImAkappaDR
      
     
      
      phi=-phik+phil- 2*phis+phiu
      phiIl=phis+phiAl+phil+phiu
      phiIk=3*phis+phiAk+phik 
    
      if(MHpmInput) then
c Maggie changed 14/1/2019
c Mistake found by Nhung                  
c      ReAlambdaDR=ReAlambdaDR+
c     -   (Cos(phiAl - phiIl)*(-2*dthu*Cos(beta)**3+2*dtad*1/sin(beta) - 
c     -   2*dthd*Sin(beta)**3+v*(dMHpm2+abslambda**2*dv*v)*Sin(2*beta))+
c     -   (-2*(dthu*Cos(beta)**3+dtad*1/sin(beta)+dthd*Sin(beta)**3)+
c     -   v*(dMHpm2+abslambda**2*dv*v)*Sin(2*beta))*Sin(phiAl - phiIl))/
c     -   (2.*Sqrt(2d0)*abslambda*v*vS)
c      ImAlambdaDR=ImAlambdaDR+
c     -   (Cos(phiAl - phiIl)*(2*dthu*Cos(beta)**3+2*dtad*1/sin(beta)+
c     -   2*dthd*Sin(beta)**3 - v*(dMHpm2+abslambda**2*dv*v)*
c     -   Sin(2*beta))+(-2*dthu*Cos(beta)**3+2*dtad*1/sin(beta) - 
c     -   2*dthd*Sin(beta)**3+v*(dMHpm2+abslambda**2*dv*v)*Sin(2*beta))*
c     -   Sin(phiAl - phiIl))/(2.*Sqrt(2d0)*abslambda*v*vS)

      ReAlambdaDR=ReAlambdaDR -((Sqrt(2d0)*((Cos(phiAl - phiIl)*
     -    sin(beta)*(dthu*Cos(3*beta) + 4*dthd*Sin(beta)**3 + 
     -    Cos(beta)*(3*dthu - 4*v*(dMHpm2 + abslambda**2*dv*v)*
     -    Sin(beta))))/4.+ dtad*Sin(phiAl - phiIl)))/(abslambda*v*vS*
     -    sin(beta)))

      ImAlambdaDR=ImAlambdaDR+(Sqrt(2d0)*dtad)/(abslambda*v*vS*
     -    cos(phiAl - phiIl)*sin(beta)) + (-((Sqrt(2d0)*((Cos(phiAl 
     -    - phiIl)*sin(beta)*(dthu*Cos(3*beta) + 4*dthd*Sin(beta)**3 + 
     -    Cos(beta)*(3*dthu - 4*v*(dMHpm2 + abslambda**2*dv*v)*
     -    Sin(beta))))/4.+ dtad*Sin(phiAl - phiIl)))/(abslambda*v*vS*
     -    sin(beta))))*Tan(phiAl-phiIl)      
c end Maggie changed 14/1/2019
      else
      selfcha2loop=dMHpm2
C      dMHpm22l=(-(abslambda**2*dv*v**2)+Cos(beta)*(-dthd+
C     -   dthu/tan(beta))+1/cos(beta)*(dthd+dtad/sin(beta)**2*
C     -   (1 - 2/(1+Tan(phiAl - phiIl)))))/v
C      dMHpm2=dMHpm22l
C      ImAlambdaDR=ImAlambdaDR+
C     -   (Sqrt(2d0)*dtad/sin(beta))/(abslambda*v*vS*Cos(phiAl - phiIl)+
C     -   abslambda*v*vS*Sin(phiAl - phiIl))
c----- ramona/nhung bug fixed 20/4/16   
       dMHpm22l=(-(abslambda**2*dv*v**2)+Cos(beta)*(-dthd+ 
     -   dthu/tan(beta))+1/cos(beta)*(dthd+dtad/sin(beta)**2* 
     -    Tan(phiAl - phiIl)   ))/v 
      dMHpm2=dMHpm22l 
      ImAlambdaDR=ImAlambdaDR+ 
     -   (Sqrt(2d0)*dtad/sin(beta))/(abslambda*v*vS*Cos(phiAl - phiIl) )
c----- end ramona/nhung bug fixed 20/4/16        
      end if
      
      
      ImAkappaDR=ImAkappaDR+(Sqrt(2d0)/cos(phik+3*Phis)*(-(dtas*vS)+v*
     -   Cos(beta)*(dtad - 3*abskappa*abslambda*dv*vS**2*Sin(beta)*
     -   Sin(phi))))/(abskappa*vS**3)

      
      
      dM(1,1)=dMHpm2+abslambda**2*dv*v
      dM(1,2)=(-4*dthu*Cos(beta)**3 - 6*abskappa*abslambda*dv*vS**2*
     -    Cos(phi)-4*dthd*Sin(beta)**3+2*dv*MHpm**2*Sin(2*beta)+2*
     -    dMHpm2*v*Sin(2*beta)+3*abslambda**2*dv*v**2*Sin(2*beta))/
     -    (4.*vS)
      dM(1,3)=(Cos(beta)*(2*dtad/sin(beta) ))/(2.*v)
      dM(1,4)=(2*dtad - dv*vS*Sin(beta)*(-(abskappa*abslambda*vs*
     -    Sin(phi))+abskappa*abslambda*vS*Sin(phi)))/(2.*v)
      dM(1,5)=(dtad/sin(beta))/vS+(abskappa*abslambda*dv*vS*Sin(phi))/2.
      dM(2,1)=dM(1,2)
      dM(2,2)=(-4*dthu*v*Cos(beta)**4*Sin(beta)+
     -    v*(dMHpm2*v+2*dv*(MHpm**2+abslambda**2*v**2))*Sin(2*beta)**2- 
     -    4*v*Cos(beta)*(dthd*Sin(beta)**4 - 3*dtad*Tan(phik+3*Phis))+
     -    4*vS*(dths - 3*dtas*Tan(phik+3*Phis))+
     -    6*dv*v*vS**2*Sin(2*beta)*(abskappa*abslambda*Cos(phi) - 
     -    3*abskappa*abslambda*Sin(phi)*Tan(phik+3*Phis)))/(4.*vS**2)
      dM(2,3)=dtad/vS-(3*abskappa*abslambda*dv*vS*Sin(beta)*
     -    Sin(phi))/2.
      dM(2,4)=(dtad/tan(beta))/vS - (3*abskappa*abslambda*dv*vS*
     -    Cos(beta)*Sin(phi))/2.
      dM(2,5)=(2*dtas*vS - v*Cos(beta)*(2*dtad+dv*vS*Sin(beta)*
     -    (-(abskappa*abslambda*vs*Sin(phi)) - 
     -    3*abskappa*abslambda*vS*Sin(phi))))/vS**2
      dM(3,1)=dM(1,3)
      dM(3,2)=dM(2,3)
      dM(3,3)=(dthd/cos(beta)+Sin(beta)*(-(dthu*Cos(beta)**2)+
     -    Sin(beta)*(v*(dMHpm2+abslambda**2*dv*v) - 
     -    dthd*Sin(beta)*Tan(beta))))/v
      dM(3,4)=(2*dthu*Cos(beta)**3+2*dthd*Sin(beta)**3+
     -    v*(-dMHpm2+abslambda**2*dv*v)*Sin(2*beta))/(2.*v)
      dM(3,5)=(2*dthu*Cos(beta)**3*Sin(beta)+
     -    abslambda**2*dv*Cos(beta)*(2*vS**2 - 3*v**2*Sin(beta)**2)+
     -    Sin(beta)*(-(abskappa*abslambda*dv*vS**2*Cos(phi))+
     -    2*dthd*Sin(beta)**3-(dv*MHpm**2+dMHpm2*v)*Sin(2*beta)))/
     -    (2.*vS)
      dM(4,1)=dM(1,4)
      dM(4,2)=dM(2,4)
      dM(4,3)=dM(3,4)
      dM(4,4)=(v*(dMHpm2+abslambda**2*dv*v)*Cos(beta)**2-dthu*
     -    Cos(beta)**3/tan(beta)+dthu/sin(beta)-dthd*Cos(beta)*
     -    Sin(beta)**2)/v
      dM(4,5)=(4*dthu*Cos(beta)**4+4*abslambda**2*dv*vS**2*Sin(beta) - 
     -    Cos(beta)*(2*abskappa*abslambda*dv*vS**2*Cos(phi) - 
     -    4*dthd*Sin(beta)**3+(2*dv*MHpm**2+2*dMHpm2*v+
     -    3*abslambda**2*dv*v**2)*Sin(2*beta)))/(4.*vS)
      dM(5,1)=dM(1,5)
      dM(5,2)=dM(2,5)
      dM(5,3)=dM(3,5)
      dM(5,4)=dM(4,5)
      dM(5,5)=(-4*dthu*v*Cos(beta)**4*Sin(beta)+
     -    v*(dMHpm2*v+2*dv*(MHpm**2+abslambda**2*v**2))*Sin(2*beta)**2+
     -    6*abskappa*abslambda*dv*v*vS**2*Sin(2*beta)*Sin(phi)*
     -    Tan(phik+3*Phis)-4*v*Cos(beta)*(abskappa*abslambda*dv*vS**2*
     -    Cos(phi)*Sin(beta)+dthd*Sin(beta)**4+dtad*Tan(phik+3*Phis))+
     -    4*vS*(dths+dtas*Tan(phik+3*Phis)))/(4.*vS**2)
   
      
      return
      end
      
c------------------------------------------------------------------------------------------------c
      
c calculates all couplings of the neutral and charged Higgs bosons to top/stop/sbottoms 
c in the gaugeless limit and further qcdcouplings
      subroutine qcdcouplings
      
      implicit none
      
      double precision Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      
      double precision abslambda, abskappa, absAlambda, absAkappa,vs,
     &                 mueff,tanb,beta
     
      double precision absM1, absM2, absAu(3), absAd(3), absAe(3),  
     &                 ml2(3), mer2(3), mq2(3), mdr2(3), mur2(3)
      
      double precision phiu, phis, phil, phik, phiAk, phiAl, phiM1,
     &                phiM2, phiAu(3), phiAd(3), phiAe(3)
     
      double precision me,mmu,mtau,mu,md,ms,mc,mt,mb,Ye(3),Yd(3),Yu(3)
      
      double precision ZH5(5,5),ZH(6,6)
     
      double complex UM(2,2),UP(2,2),NN(5,5),ZE(6,6),ZD(6,6),ZU(6,6)
      
      double complex nZU(6,6),nZD(6,6)
      
      double precision nMSu(6),nMSd(6)
      
      double precision absM3,phiM3
     

      double complex g2thh(5),g2tha(5),g2uh(2,2,5),g2u2h(2,5,2,5),
     &               g4u(2,2,2,2),gLtFgSu(2),gRtFgSu(2),g4dL,
     &               g2u2dL(2,2),gLbFgdL
     
      double complex gudLHp(2),g2u2Hp(2,2),g2dL2Hp
      
      integer i,gt1,gt2,gt3,gt4
      
      common /SMinput/ Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      common /Higgsinput/ abslambda, abskappa, absAlambda, absAkappa,
     &                    vs,mueff,tanb,beta
      common /softbreak/ absM1, absM2, absAu, absAd, absAe,  
     &                   ml2, mer2, mq2, mdr2, mur2
      common /phases/ phiu, phis, phil, phik, phiAk, phiAl, phiM1,
     &                phiM2, phiAu, phiAd, phiAe
     
      common /Yukawa/ me,mmu,mtau,mu,md,ms,mc,mt,mb,Ye,Yd,Yu
      common /rot/ ZH5,ZH,UM,UP,NN,ZE,ZD,ZU
      common /nSu/ nMSu,nZU
      common /nSd/ nMSd,nZD
      common /gluino/ absM3,phiM3
      
      common/qcdcoup/ g2thh,g2tha,g2uh,g2u2h,g4u,gLtFgSu,gRtFgSu,g4dL,
     &                 g2u2dL,gLbFgdL
      common/ qcdchacoup / gudLHp,g2u2Hp,g2dL2Hp
      
      do i=1,5
c coupling of Higgs to two tops, CP-odd component
      g2tha(i)=Yu(3)*Cos(beta)*ZH5(i,1)/sqrt(2d0)
c coupling of Higgs to two tops, CP-even component
      g2thh(i)=-(0d0,1d0)*Yu(3)*ZH5(i,4)/sqrt(2d0)
      end do
      
      
      do gt1=1,2
      do gt2=1,2
      do gt3=1,5
c coupling of Higgs to two stops
      g2uh(gt1,gt2,gt3)=
     -        (absAu(3)*Yu(3)*(exp(2*(0,1)*(phiu+phiAu(3)))*
     -        CONJG(nZU(4+gt1,5))*nZU(4+gt2,6)*
     -        (Cos(beta)*ZH5(gt3,1)-(0,1)*ZH5(gt3,4))-
     -        CONJG(nZU(4+gt1,6))*nZU(4+gt2,5)*
     -        (Cos(beta)*ZH5(gt3,1)+(0,1)*ZH5(gt3,4))))/
     -        (sqrt(2d0)*exp((0,1)*(phiu+phiAu(3))))-
     -        (0,1)*v*(CONJG(nZU(4+gt1,5))*nZU(4+gt2,5)+
     -        CONJG(nZU(4+gt1,6))*nZU(4+gt2,6))*
     -        Sin(beta)*Yu(3)**2*ZH5(gt3,4)+abslambda*Yu(3)*
     -        ((vS*(-(exp(2*(0,1)*(phil+phis))*
     -        CONJG(nZU(4+gt1,6))*nZU(4+gt2,5)*
     -        (Sin(beta)*ZH5(gt3,1)-(0,1)*ZH5(gt3,3)))+
     -        CONJG(nZU(4+gt1,5))*nZU(4+gt2,6)*
     -        (Sin(beta)*ZH5(gt3,1)+(0,1)*ZH5(gt3,3))))/
     -        (2.*exp((0,1)*(phil+phis)))-
     -        (v*Cos(beta)*(exp(2*(0,1)*(phil+phis))*
     -        CONJG(nZU(4+gt1,6))*nZU(4+gt2,5)*
     -        (ZH5(gt3,2)-(0,1)*ZH5(gt3,5))-
     -        CONJG(nZU(4+gt1,5))*nZU(4+gt2,6)*
     -        (ZH5(gt3,2)+(0,1)*ZH5(gt3,5))))/
     -        (2.*exp((0,1)*(phil+phis))))
      end do
      end do
      end do

c coupling of two Higgs to two stops
      do gt1=1,2
      do gt2=1,5
      do gt3=1,2
      do gt4=1,5
      g2u2h(gt1,gt2,gt3,gt4)=
     -        (0,-1)*(CONJG(nZU(4+gt1,5))*nZU(4+gt3,5)+
     -        CONJG(nZU(4+gt1,6))*nZU(4+gt3,6))*Yu(3)**2*
     -        (Cos(beta)**2*ZH5(gt2,1)*ZH5(gt4,1)+
     -        ZH5(gt2,4)*ZH5(gt4,4))+abslambda*Yu(3)*
     -        ((exp((0,1)*(phil+phis))*CONJG(nZU(4+gt1,6))*
     -        nZU(4+gt3,5)*(ZH5(gt2,2)*((0,-1)*Sin(beta)*ZH5(gt4,1) -
     -        ZH5(gt4,3))+ZH5(gt2,5)*(-(Sin(beta)*ZH5(gt4,1))+
     -        (0,1)*ZH5(gt4,3))+((0,-1)*Sin(beta)*ZH5(gt2,1) - 
     -        ZH5(gt2,3))*(ZH5(gt4,2)-(0,1)*ZH5(gt4,5))))/2.+
     -        (CONJG(nZU(4+gt1,5))*nZU(4+gt3,6)*
     -        (ZH5(gt2,5)*(Sin(beta)*ZH5(gt4,1)+(0,1)*ZH5(gt4,3))+
     -        ZH5(gt2,2)*((0,-1)*Sin(beta)*ZH5(gt4,1)+ZH5(gt4,3))+
     -        ((0,-1)*Sin(beta)*ZH5(gt2,1)+ZH5(gt2,3))*
     -        (ZH5(gt4,2)+(0,1)*ZH5(gt4,5))))/
     -        (2.*exp((0,1)*(phil+phis))))
      end do
      end do
      end do
      end do
      
c coupling of four stops
      do gt1=1,2
      do gt2=1,2
      do gt3=1,2
      do gt4=1,2
      g4u(gt1,gt2,gt3,gt4)= -(0d0,1d0)/6d0*g3**2*
     -     (CONJG(nZU(4+gt1,6))*(2d0*CONJG(nZU(4+gt2,6))*nZU(4+gt3,6)*
     -     nZU(4+gt4,6)+CONJG(nZU(4+gt2,5))*(nZU(4+gt3,6)*nZU(4+gt4,5)-
     -     3d0*nZU(4+gt3,5)*nZU(4+gt4,6)))+CONJG(nZU(4+gt1,5))*
     -     (2d0*CONJG(nZU(4+gt2,5))*nZU(4+gt3,5)*nZU(4+gt4,5)+
     -     CONJG(nZU(4+gt2,6))*(-3d0*nZU(4+gt3,6)*nZU(4+gt4,5)+
     -     nZU(4+gt3,5)*nZU(4+gt4,6))))
      end do
      end do
      end do
      end do
      

c coupling of four left-handed sbottoms
      g4dL=-(0d0,1d0)*g3**2/3d0
      
c coupling of two stops to two left-handed sbottoms
      do gt2=1,2
      do gt4=1,2
      g2u2dL(gt2,gt4)=-(0,1)/6d0*g3**2*(CONJG(nZU(4+gt2,5))*
     -     nZU(4+gt4,5) - CONJG(nZU(4+gt2,6))*nZU(4+gt4,6))
      end do
      end do
      
c coupling of top, Gluino, Stop
      do gt3=1,2
      gLtFgSu(gt3)=((0,1)*g3*CONJG(nZU(gt3+4,6)))/
     -             (Sqrt(2d0)*exp(((0,1)*(phiu+phiM3))/2d0))
      gRtFgSu(gt3)=(-(0,1)*exp(((0,1)*(phiu+phiM3))/2d0)*
     -              g3*CONJG(nZU(gt3+4,5)))/Sqrt(2d0)
      end do
      
c coupling bottom, Gluino, left-handed sbottom
      gLbFgdL=((0,-1)*g3*exp(-(0d0,1d0)/2d0*(phiM3)))/Sqrt(2d0)
      
      
c charged Higgs boson coupling to stop and left-handed sbottom
      do gt1=1,2
      gudLHp(gt1)=(0,1)*Yu(3)*(CONJG(nZU(4+gt1,6))*
     -      (Sqrt(2d0)*abslambda*vs*Sin(beta)*exp((0,1)*(phil+phis))+ 
     -      2*absAu(3)*Cos(beta)*exp(-(0,1)*(phiu+phiAu(3))))+ 
     -      Sqrt(2d0)*v*CONJG(nZU(4+gt1,5))*Cos(beta)*Sin(beta)*
     -      Yu(3))/2.
      end do
      

c two charged Higgs bosons coupling to two stops
      do gt1=1,2
      do gt3=1,2
      g2u2Hp(gt1,gt3)=(0,-1)*CONJG(nZU(4+gt1,6))*
     -                 Cos(beta)**2*nZU(4+gt3,6)*Yu(3)**2
      end do
      end do
      
c two charged Higgs bosons coupling to two left-handed-sbottoms
      g2dL2Hp=(0,-1)*Cos(beta)**2*Yu(3)**2
      
      return
      end
      
c---------------------------------------------------------------------------------------- c

c Calculates the counterterm of the top sector

      subroutine topsector(dmtfin,dmtdiv1,dYtfin,dYtdiv1)
      
      implicit none
      
      double complex dmtfin,dmtdiv1,dYtfin,dYtdiv1
      
      double precision nMSu(6)
      double complex nZU(6,6)
     
      double precision absM3,phiM3
      
      double precision me,mmu,mtau,mu,md,ms,mc,mt,mb,Ye(3),Yd(3),Yu(3)
      
      double precision Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      
      double precision abslambda, abskappa, absAlambda, absAkappa,
     &                    vs,mueff,tanb,beta
      
      double precision phiu, phis, phil, phik, phiAk, phiAl, phiM1,
     &                phiM2, phiAu(3), phiAd(3), phiAe(3)
      
      double precision Qs
      
      double complex B0
      double precision A0
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      common /nSu/ nMSu,nZU
      common /gluino/ absM3,phiM3
      common /SMinput/ Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      common /Yukawa/ me,mmu,mtau,mu,md,ms,mc,mt,mb,Ye,Yd,Yu
      common /Higgsinput/ abslambda, abskappa, absAlambda, absAkappa,
     &                    vs,mueff,tanb,beta
      common /phases/ phiu, phis, phil, phik, phiAk, phiAl, phiM1,
     &                phiM2, phiAu, phiAd, phiAe
      common /scale/ Qs
      
      dmtfin=(g3**2*(-4*mt**2-2*A0(absM3**2)-6*A0(mt**2)+
     -       A0(nMSu(5)**2)+A0(nMSu(6)**2)+((2*absM3*mt*
     -       CONJG(nZU(6,6))*nZU(6,5)+exp((0,1)*(phiM3+phiu))*
     -       (absM3**2+mt**2-nMSu(5)**2+2*absM3*exp((0,1)*(phiM3+phiu))
     -       *mt*CONJG(nZU(6,5))*nZU(6,6)))*
     -       Realpart(B0(mt**2,absM3**2,nMSu(5)**2,Qs**2)))/
     -       exp((0,1)*(phiM3+phiu))-(-absM3**2-mt**2+nMSu(6)**2+
     -       (2*absM3*mt*CONJG(nZU(6,6))*nZU(6,5))/
     -       exp((0,1)*(phiM3+phiu))+2*absM3*exp((0,1)*(phiM3+phiu))*
     -       mt*CONJG(nZU(6,5))*nZU(6,6))*
     -       Realpart(B0(mt**2,absM3**2,nMSu(6)**2,Qs**2))))/
     -       (24.*mt*Pi**2)
     
      dmtdiv1=-(g3**2*mt)/(6.*Pi**2)
      
      dYtfin=dmtfin/v*Sqrt(2d0)/Sin(beta)
      dYtdiv1=dmtdiv1/v*Sqrt(2d0)/Sin(beta)
      
      return
      end
      
      
c calculates the counterterm of the stop sector
      subroutine stopsector(dmtfin,dmtdiv1,dMstfin,dMstdiv1,
     &                      dAtfin,dAtdiv1,dphiAtfin,dphiAtdiv1)
     
      implicit none
      
      double complex dmtfin,dmtdiv1,dMstfin(2,2),dMstdiv1(2,2),
     &               dAtfin,dAtdiv1,dphiAtfin,dphiAtdiv1
      
      double precision nMSu(6)
      double complex nZU(6,6)
     
      double precision absM3,phiM3
      
      double precision me,mmu,mtau,mu,md,ms,mc,mt,mb,Ye(3),Yd(3),Yu(3)
      
      double precision absM1, absM2, absAu(3), absAd(3), absAe(3),  
     &                 ml2(3), mer2(3), mq2(3), mdr2(3), mur2(3)
     
      double precision abslambda, abskappa, absAlambda, absAkappa,
     &                    vs,mueff,tanb,beta
      
      double precision phiu, phis, phil, phik, phiAk, phiAl, phiM1,
     &                phiM2, phiAu(3), phiAd(3), phiAe(3)

      double precision a_sDR,Qs,g3
      
      double complex g2thh(5),g2tha(5),g2uh(2,2,5),g2u2h(2,5,2,5),
     &               g4u(2,2,2,2),gLtFgSu(2),gRtFgSu(2),g4dL,
     &               g2u2dL(2,2),gLbFgdL
     
      double complex Ktfin,Ktdiv1
      
      double complex B0
      double precision A0
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      common /nSu/ nMSu,nZU
      common /gluino/ absM3,phiM3
      common /Higgsinput/ abslambda, abskappa, absAlambda, absAkappa,
     &                    vs,mueff,tanb,beta
      common /Yukawa/ me,mmu,mtau,mu,md,ms,mc,mt,mb,Ye,Yd,Yu
      common /softbreak/ absM1, absM2, absAu, absAd, absAe,  
     &                   ml2, mer2, mq2, mdr2, mur2
      common /phases/ phiu, phis, phil, phik, phiAk, phiAl, phiM1,
     &                phiM2, phiAu, phiAd, phiAe
      common/qcdcoup/ g2thh,g2tha,g2uh,g2u2h,g4u,gLtFgSu,gRtFgSu,g4dL,
     &                 g2u2dL,gLbFgdL
      common /scale/ Qs
      
      g3=Sqrt(a_sDR(Qs)*4d0*pi)
      dMstfin(1,1)=(-12*A0(nMSu(5)**2)*(g3**2 - (0,1)*g4u(1,1,1,1))+
     -    (0,3)*A0(nMSu(6)**2)*(3*g4u(1,2,1,2)+g4u(1,2,2,1))+
     -    16*(-((A0(absM3**2)+A0(mt**2))*(CONJG(gLtFgSu(1))*gLtFgSu(1)+
     -    CONJG(gRtFgSu(1))*gRtFgSu(1)))-g3**2*nMSu(5)**2 - 2*mt*
     -    (CONJG(gLtFgSu(1))*(mt*gLtFgSu(1)+absM3*gRtFgSu(1))+
     -    CONJG(gRtFgSu(1))*(absM3*gLtFgSu(1)+mt*gRtFgSu(1)))*
     -    Realpart(B0(nMSu(5)**2,absM3**2,mt**2,Qs**2))+
     -    (CONJG(gLtFgSu(1))*gLtFgSu(1)+CONJG(gRtFgSu(1))*gRtFgSu(1))*
     -    (-absM3**2+mt**2+nMSu(5)**2)*
     -    Realpart(B0(nMSu(5)**2,mt**2,absM3**2,Qs**2))))/(48.*Pi**2)
     
      dMstdiv1(1,1)=(4*(-8*CONJG(gLtFgSu(1))*((absM3**2+mt**2)*
     -    gLtFgSu(1)+absM3*mt*gRtFgSu(1))-8*CONJG(gRtFgSu(1))*(absM3*
     -    mt*gLtFgSu(1)+(absM3**2+mt**2)*gRtFgSu(1))+
     -    (-3*g3**2+(0,3)*g4u(1,1,1,1)+4*CONJG(gLtFgSu(1))*gLtFgSu(1)+
     -    4*CONJG(gRtFgSu(1))*gRtFgSu(1))*nMSu(5)**2)+
     -    (0,3)*(3*g4u(1,2,1,2)+g4u(1,2,2,1))*nMSu(6)**2)/(48.*Pi**2)
     
      dMstfin(2,2)=((0,3)*A0(nMSu(5)**2)*(g4u(2,1,1,2)+3*g4u(2,1,2,1))- 
     -    12*A0(nMSu(6)**2)*(g3**2-(0,1)*g4u(2,2,2,2))+
     -    16*(-((A0(absM3**2)+A0(mt**2))*(CONJG(gLtFgSu(2))*gLtFgSu(2)+
     -    CONJG(gRtFgSu(2))*gRtFgSu(2)))-g3**2*nMSu(6)**2 - 2*mt*
     -    (CONJG(gLtFgSu(2))*(mt*gLtFgSu(2)+absM3*gRtFgSu(2))+
     -    CONJG(gRtFgSu(2))*(absM3*gLtFgSu(2)+mt*gRtFgSu(2)))*
     -    Realpart(B0(nMSu(6)**2,absM3**2,mt**2,Qs**2))+
     -    (CONJG(gLtFgSu(2))*gLtFgSu(2)+CONJG(gRtFgSu(2))*gRtFgSu(2))*
     -    (-absM3**2+mt**2+nMSu(6)**2)*
     -    Realpart(B0(nMSu(6)**2,mt**2,absM3**2,Qs**2))))/(48.*Pi**2)
      
      dMstdiv1(2,2)=((0,3)*(g4u(2,1,1,2)+3*g4u(2,1,2,1))*nMSu(5)**2-
     -    12*(g3**2 - (0,1)*g4u(2,2,2,2))*nMSu(6)**2+
     -    16*(CONJG(gLtFgSu(2))*(-2*(absM3**2+mt**2)*gLtFgSu(2)-
     -    2*absM3*mt*gRtFgSu(2)+gLtFgSu(2)*nMSu(6)**2)+
     -    CONJG(gRtFgSu(2))*(-2*absM3*mt*gLtFgSu(2)-2*(absM3**2+mt**2)*
     -    gRtFgSu(2)+gRtFgSu(2)*nMSu(6)**2)))/(48.*Pi**2)
     
      dMstfin(2,1)=((0,3)*A0(nMSu(5)**2)*(g4u(1,1,1,2)+3*g4u(1,1,2,1))+
     -    4*((0,3)*A0(nMSu(6)**2)*g4u(1,2,2,2)-4*(A0(absM3**2)+
     -    A0(mt**2))*(CONJG(gLtFgSu(2))*gLtFgSu(1)+CONJG(gRtFgSu(2))*
     -    gRtFgSu(1))-4*mt*(CONJG(gLtFgSu(2))*(mt*gLtFgSu(1)+absM3*
     -    gRtFgSu(1))+CONJG(gRtFgSu(2))*(absM3*gLtFgSu(1)+mt*
     -    gRtFgSu(1)))*Realpart(B0(nMSu(5)**2,absM3**2,mt**2,Qs**2))+
     -    2*(CONJG(gLtFgSu(2))*gLtFgSu(1)+CONJG(gRtFgSu(2))*
     -    gRtFgSu(1))*(-absM3**2+mt**2+nMSu(5)**2)*
     -    Realpart(B0(nMSu(5)**2,mt**2,absM3**2,Qs**2)) - 
     -    4*mt*(CONJG(gLtFgSu(2))*(mt*gLtFgSu(1)+absM3*gRtFgSu(1))+
     -    CONJG(gRtFgSu(2))*(absM3*gLtFgSu(1)+mt*gRtFgSu(1)))*
     -    Realpart(B0(nMSu(6)**2,absM3**2,mt**2,Qs**2))+
     -    2*(CONJG(gLtFgSu(2))*gLtFgSu(1)+CONJG(gRtFgSu(2))*
     -    gRtFgSu(1))*(-absM3**2+mt**2+nMSu(6)**2)*
     -    Realpart(B0(nMSu(6)**2,mt**2,absM3**2,Qs**2))))/(48.*Pi**2)
     
      dMstdiv1(2,1)=((0,3)*((g4u(1,1,1,2)+3*g4u(1,1,2,1))*nMSu(5)**2+
     -    4*g4u(1,2,2,2)*nMSu(6)**2)+8*(CONJG(gLtFgSu(2))*(-4*absM3**2*
     -    gLtFgSu(1) - 4*absM3*mt*gRtFgSu(1)+gLtFgSu(1)*(-4*mt**2+
     -    nMSu(5)**2+nMSu(6)**2))+CONJG(gRtFgSu(2))*(-4*absM3*mt*
     -    gLtFgSu(1) - 4*absM3**2*gRtFgSu(1)+gRtFgSu(1)*
     -    (-4*mt**2+nMSu(5)**2+nMSu(6)**2))))/(48.*Pi**2)
     
      dMstfin(1,2)=CONJG(dMstfin(2,1))
      dMstdiv1(1,2)=CONJG(dMstdiv1(2,1))
      
      Ktfin=-(absAu(3)*exp(-(0,1)*(phiAu(3)+phiu))-
     -     mueff*exp((0,1)*(phil+phis))/tanb)*dmtfin+
     -     CONJG(nZU(5,5))*nZU(5,6)*(dmstfin(1,1)-dmstfin(2,2))+
     -     CONJG(nZU(5,5))*nZU(6,6)*dmstfin(1,2)
     -     +nZU(5,6)*CONJG(nZU(6,5))*dmstfin(2,1)
     
      Ktdiv1=-(absAu(3)*exp(-(0,1)*(phiAu(3)+phiu))-
     -     mueff*exp((0,1)*(phil+phis))/tanb)*dmtdiv1+
     -     CONJG(nZU(5,5))*nZU(5,6)*(dmstdiv1(1,1)-dmstdiv1(2,2))+
     -     CONJG(nZU(5,5))*nZU(6,6)*dmstdiv1(1,2)
     -     +nZU(5,6)*CONJG(nZU(6,5))*dmstdiv1(2,1)

     
      dAtfin=RealPart(exp((0,1)*(phiAu(3)+phiu))*Ktfin)/mt
      dAtdiv1=RealPart(exp((0,1)*(phiAu(3)+phiu))*Ktdiv1)/mt
      
      if(Abs(absAu(3)).le.1d-14) then
      dphiAtfin=0d0
      dphiAtdiv1=0d0
      else
      dphiAtfin=-AIMAG(exp((0,1)*(phiAu(3)+phiu))*Ktfin)/mt/absAu(3)
      dphiAtdiv1=-AIMAG(exp((0,1)*(phiAu(3)+phiu))*Ktdiv1)/mt/absAu(3)
      end if
      
      return
      end
      
c-------------------------------------------------------------------------c
      
      
c couplings needed in the calculation of counterterm inserted diagrams
      subroutine dqcdcouplings
      
      implicit none
      
      double precision Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      
      double precision abslambda, abskappa, absAlambda, absAkappa,vs,
     &                 mueff,tanb,beta
     
      double precision absM1, absM2, absAu(3), absAd(3), absAe(3),  
     &                 ml2(3), mer2(3), mq2(3), mdr2(3), mur2(3)
      
      double precision phiu, phis, phil, phik, phiAk, phiAl, phiM1,
     &                phiM2, phiAu(3), phiAd(3), phiAe(3)
     
      double precision me,mmu,mtau,mu,md,ms,mc,mt,mb,Ye(3),Yd(3),Yu(3)
      
      double precision ZH5(5,5),ZH(6,6)
     
      double complex UM(2,2),UP(2,2),NN(5,5),ZE(6,6),ZD(6,6),ZU(6,6)
      
      double complex nZU(6,6)
      
      double precision nMSu(6)
      
      double precision absM3,phiM3

      double complex dmtfin,dmtdiv1,dMstfin(2,2),dMstdiv1(2,2),
     &               dYtfin,dYtdiv1,dAtfin,dAtdiv1,dphiAtfin,dphiAtdiv1

      double complex dg2thhfin(5),dg2thafin(5),dg2uhfin(2,2,5),
     &               dg2u2hfin(2,5,2,5),
     &               dg2thhdiv1(5),dg2thadiv1(5),dg2uhdiv1(2,2,5),
     &               dg2u2hdiv1(2,5,2,5)
     
      double complex dgudLHpfin(2),dg2u2Hpfin(2,2),dg2dL2Hpfin,
     &               dgudLHpdiv1(2),dg2u2Hpdiv1(2,2),dg2dL2Hpdiv1
      
      integer i,gt1,gt2,gt3,gt4
      
      logical MHpmInput,CPcon,mQrun,onelswitch
      
      common /SMinput/ Mz,Gf,alpha,Mw,e,v,g1,g2,alphaS,g3
      common /Higgsinput/ abslambda, abskappa, absAlambda, absAkappa,
     &                    vs,mueff,tanb,beta
      common /softbreak/ absM1, absM2, absAu, absAd, absAe,  
     &                   ml2, mer2, mq2, mdr2, mur2
      common /phases/ phiu, phis, phil, phik, phiAk, phiAl, phiM1,
     &                phiM2, phiAu, phiAd, phiAe
     
      common /Yukawa/ me,mmu,mtau,mu,md,ms,mc,mt,mb,Ye,Yd,Yu
      common /rot/ ZH5,ZH,UM,UP,NN,ZE,ZD,ZU
      common /nSu/ nMSu,nZU
      common /gluino/ absM3,phiM3
      common/qcdcount/ dmtfin,dmtdiv1,dMstfin,dMstdiv1,dYtfin,dYtdiv1,
     &                 dAtfin,dAtdiv1,dphiAtfin,dphiAtdiv1
      common/dqcdcoup/ dg2thhfin,dg2thafin,dg2uhfin,dg2u2hfin,
     &                 dg2thhdiv1,dg2thadiv1,dg2uhdiv1,dg2u2hdiv1
      common/ dqcdchacoup / dgudLHpfin,dg2u2Hpfin,dg2dL2Hpfin,
     &                      dgudLHpdiv1,dg2u2Hpdiv1,dg2dL2Hpdiv1
      common /switch/ MHpmInput,CPcon,mQrun,onelswitch
     
c if DR scheme is used in top/stop sector finite parts of CT are set to zero
      if(mQrun) then
      dmtfin=0d0
      dYtfin=0d0
      dphiAtfin=0d0
      dAtfin=0d0
      dMstfin=0d0
      end if
      
      do i=1,5
      dg2thafin(i)=dYtfin*Cos(beta)*ZH5(i,1)/sqrt(2d0)
      dg2thhfin(i)=-(0d0,1d0)*dYtfin*ZH5(i,4)/sqrt(2d0)
      dg2thadiv1(i)=dYtdiv1*Cos(beta)*ZH5(i,1)/sqrt(2d0)
      dg2thhdiv1(i)=-(0d0,1d0)*dYtdiv1*ZH5(i,4)/sqrt(2d0)
      end do
      
      do gt1=1,2
      do gt2=1,2
      do gt3=1,5
      dg2uhfin(gt1,gt2,gt3)=(dAtfin*Yu(3)*(CONJG(nZU(4+gt1,5))*
     -    nZU(4+gt2,6)*(Sqrt(2d0)*Cos(beta)*
     -    (exp((0,1)*(phiu+phiAu(3))))*ZH5(gt3,1)-(0,1)*Sqrt(2d0)*
     -    Cos(phiu+phiAu(3))*ZH5(gt3,4)+Sqrt(2d0)*Sin(phiu+phiAu(3))*
     -    ZH5(gt3,4))-CONJG(nZU(4+gt1,6))*nZU(4+gt2,5)*(Sqrt(2d0)*
     -    Cos(beta)*(exp(-(0,1)*(phiu+phiAu(3))))*ZH5(gt3,1)+(0,1)*
     -    Sqrt(2d0)*ZH5(gt3,4)*exp(-(0,1)*(phiu+phiAu(3))))))/2.
     -    +(dphiAtfin*Yu(3)*(-(CONJG(nZU(4+gt1,6))*nZU(4+gt2,5)*
     -    (Sqrt(2d0)*absAu(3)*Cos(beta)*((0,-1)*
     -    exp(-(0,1)*(phiu+phiAu(3))))*ZH5(gt3,1)+Sqrt(2d0)*absAu(3)*
     -    Cos(phiu+phiAu(3))*ZH5(gt3,4) -(0,1)*Sqrt(2d0)*absAu(3)*
     -    Sin(phiu+phiAu(3))*ZH5(gt3,4)))+CONJG(nZU(4+gt1,5))*
     -    nZU(4+gt2,6)*(Sqrt(2d0)*absAu(3)*Cos(beta)*((0,1)*
     -    exp((0,1)*(phiu+phiAu(3))))*ZH5(gt3,1)+Sqrt(2d0)*absAu(3)*
     -    Cos(phiu+phiAu(3))*ZH5(gt3,4)+(0,1)*Sqrt(2d0)*absAu(3)*
     -    Sin(phiu+phiAu(3))*ZH5(gt3,4))))/2.+dYtfin*((Yu(3)*((0,-2)*v*
     -    CONJG(nZU(4+gt1,5))*nZU(4+gt2,5)*Sin(beta)*ZH5(gt3,4) - 
     -    (0,2)*v*CONJG(nZU(4+gt1,6))*nZU(4+gt2,6)*Sin(beta)*
     -    ZH5(gt3,4)))/2.+(-(CONJG(nZU(4+gt1,6))*((0,2)*v*nZU(4+gt2,6)*
     -    Sin(beta)*Yu(3)*ZH5(gt3,4)+nZU(4+gt2,5)*((abslambda*vs*
     -    Sin(beta)*(Cos(phil+phis)+(0,1)*Sin(phil+phis))+Sqrt(2d0)*
     -    absAu(3)*Cos(beta)*(exp(-(0,1)*(phiu+phiAu(3)))))*ZH5(gt3,1)+
     -    abslambda*v*Cos(beta)*(Cos(phil+phis)+(0,1)*Sin(phil+phis))*
     -    ZH5(gt3,2)-(0,1)*abslambda*vs*Cos(phil+phis)*ZH5(gt3,3)+
     -    abslambda*vs*Sin(phil+phis)*ZH5(gt3,3)+(0,1)*Sqrt(2d0)*
     -    absAu(3)*Cos(phiu+phiAu(3))*ZH5(gt3,4)+Sqrt(2d0)*absAu(3)*
     -    Sin(phiu+phiAu(3))*ZH5(gt3,4)-(0,1)*abslambda*v*Cos(beta)*
     -    Cos(phil+phis)*ZH5(gt3,5)+abslambda*v*Cos(beta)*
     -    Sin(phil+phis)*ZH5(gt3,5))))+CONJG(nZU(4+gt1,5))*((0,-2)*v*
     -    nZU(4+gt2,5)*Sin(beta)*Yu(3)*ZH5(gt3,4)+nZU(4+gt2,6)*
     -    ((abslambda*vs*Sin(beta)*(Cos(phil+phis) - (0,1)*
     -    Sin(phil+phis))+Sqrt(2d0)*absAu(3)*Cos(beta)*
     -    (exp((0,1)*(phiu+phiAu(3)))))*ZH5(gt3,1)+abslambda*v*
     -    Cos(beta)*(Cos(phil+phis) - (0,1)*Sin(phil+phis))*ZH5(gt3,2)
     -    +(0,1)*abslambda*vs*Cos(phil+phis)*ZH5(gt3,3)+abslambda*vs*
     -    Sin(phil+phis)*ZH5(gt3,3) -(0,1)*Sqrt(2d0)*absAu(3)*
     -    Cos(phiu+phiAu(3))*ZH5(gt3,4)+Sqrt(2d0)*absAu(3)*
     -    Sin(phiu+phiAu(3))*ZH5(gt3,4)+(0,1)*abslambda*v*Cos(beta)*
     -    Cos(phil+phis)*ZH5(gt3,5)+abslambda*v*Cos(beta)*
     -    Sin(phil+phis)*ZH5(gt3,5))))/2.)
     
      dg2uhdiv1(gt1,gt2,gt3)=(dAtdiv1*Yu(3)*(CONJG(nZU(4+gt1,5))*
     -    nZU(4+gt2,6)*(Sqrt(2d0)*Cos(beta)*
     -    (exp((0,1)*(phiu+phiAu(3))))*ZH5(gt3,1)-(0,1)*Sqrt(2d0)*
     -    Cos(phiu+phiAu(3))*ZH5(gt3,4)+Sqrt(2d0)*Sin(phiu+phiAu(3))*
     -    ZH5(gt3,4))-CONJG(nZU(4+gt1,6))*nZU(4+gt2,5)*(Sqrt(2d0)*
     -    Cos(beta)*(exp(-(0,1)*(phiu+phiAu(3))))*ZH5(gt3,1)+(0,1)*
     -    Sqrt(2d0)*ZH5(gt3,4)*exp(-(0,1)*(phiu+phiAu(3))))))/2.
     -    +(dphiAtdiv1*Yu(3)*(-(CONJG(nZU(4+gt1,6))*nZU(4+gt2,5)*
     -    (Sqrt(2d0)*absAu(3)*Cos(beta)*((0,-1)*
     -    exp(-(0,1)*(phiu+phiAu(3))))*ZH5(gt3,1)+Sqrt(2d0)*absAu(3)*
     -    Cos(phiu+phiAu(3))*ZH5(gt3,4) -(0,1)*Sqrt(2d0)*absAu(3)*
     -    Sin(phiu+phiAu(3))*ZH5(gt3,4)))+CONJG(nZU(4+gt1,5))*
     -    nZU(4+gt2,6)*(Sqrt(2d0)*absAu(3)*Cos(beta)*((0,1)*
     -    exp((0,1)*(phiu+phiAu(3))))*ZH5(gt3,1)+Sqrt(2d0)*absAu(3)*
     -    Cos(phiu+phiAu(3))*ZH5(gt3,4)+(0,1)*Sqrt(2d0)*absAu(3)*
     -    Sin(phiu+phiAu(3))*ZH5(gt3,4))))/2.+dYtdiv1*((Yu(3)*((0,-2)*v*
     -    CONJG(nZU(4+gt1,5))*nZU(4+gt2,5)*Sin(beta)*ZH5(gt3,4) - 
     -    (0,2)*v*CONJG(nZU(4+gt1,6))*nZU(4+gt2,6)*Sin(beta)*
     -    ZH5(gt3,4)))/2.+(-(CONJG(nZU(4+gt1,6))*((0,2)*v*nZU(4+gt2,6)*
     -    Sin(beta)*Yu(3)*ZH5(gt3,4)+nZU(4+gt2,5)*((abslambda*vs*
     -    Sin(beta)*(Cos(phil+phis)+(0,1)*Sin(phil+phis))+Sqrt(2d0)*
     -    absAu(3)*Cos(beta)*(exp(-(0,1)*(phiu+phiAu(3)))))*ZH5(gt3,1)+
     -    abslambda*v*Cos(beta)*(Cos(phil+phis)+(0,1)*Sin(phil+phis))*
     -    ZH5(gt3,2)-(0,1)*abslambda*vs*Cos(phil+phis)*ZH5(gt3,3)+
     -    abslambda*vs*Sin(phil+phis)*ZH5(gt3,3)+(0,1)*Sqrt(2d0)*
     -    absAu(3)*Cos(phiu+phiAu(3))*ZH5(gt3,4)+Sqrt(2d0)*absAu(3)*
     -    Sin(phiu+phiAu(3))*ZH5(gt3,4)-(0,1)*abslambda*v*Cos(beta)*
     -    Cos(phil+phis)*ZH5(gt3,5)+abslambda*v*Cos(beta)*
     -    Sin(phil+phis)*ZH5(gt3,5))))+CONJG(nZU(4+gt1,5))*((0,-2)*v*
     -    nZU(4+gt2,5)*Sin(beta)*Yu(3)*ZH5(gt3,4)+nZU(4+gt2,6)*
     -    ((abslambda*vs*Sin(beta)*(Cos(phil+phis) - (0,1)*
     -    Sin(phil+phis))+Sqrt(2d0)*absAu(3)*Cos(beta)*
     -    (exp((0,1)*(phiu+phiAu(3)))))*ZH5(gt3,1)+abslambda*v*
     -    Cos(beta)*(Cos(phil+phis) - (0,1)*Sin(phil+phis))*ZH5(gt3,2)
     -    +(0,1)*abslambda*vs*Cos(phil+phis)*ZH5(gt3,3)+abslambda*vs*
     -    Sin(phil+phis)*ZH5(gt3,3) -(0,1)*Sqrt(2d0)*absAu(3)*
     -    Cos(phiu+phiAu(3))*ZH5(gt3,4)+Sqrt(2d0)*absAu(3)*
     -    Sin(phiu+phiAu(3))*ZH5(gt3,4)+(0,1)*abslambda*v*Cos(beta)*
     -    Cos(phil+phis)*ZH5(gt3,5)+abslambda*v*Cos(beta)*
     -    Sin(phil+phis)*ZH5(gt3,5))))/2.)
      end do
      end do
      end do

      
      do gt1=1,2
      do gt2=1,5
      do gt3=1,2
      do gt4=1,5
      dg2u2hfin(gt1,gt2,gt3,gt4)=(0,-1)*(CONJG(nZU(4+gt1,5))*
     -    nZU(4+gt3,5)+CONJG(nZU(4+gt1,6))*nZU(4+gt3,6))*2*dYtfin*
     -    Yu(3)*(Cos(beta)**2*ZH5(gt2,1)*ZH5(gt4,1)+ZH5(gt2,4)*
     -    ZH5(gt4,4))+((0,-0.5)*abslambda*dYtfin*(CONJG(nZU(4+gt1,6))*
     -    ((0,-1)*Cos(phil+phis)+Sin(phil+phis))*(ZH5(gt2,5)*
     -    (Sin(beta)*ZH5(gt4,1) - (0,1)*ZH5(gt4,3))+ZH5(gt2,2)*
     -    ((0,1)*Sin(beta)*ZH5(gt4,1)+ZH5(gt4,3))+((0,1)*Sin(beta)*
     -    ZH5(gt2,1)+ZH5(gt2,3))*(ZH5(gt4,2)-(0,1)*ZH5(gt4,5)))*
     -    nZU(4+gt3,5)+CONJG(nZU(4+gt1,5))*((0,1)*Cos(phil+phis)+
     -    Sin(phil+phis))*(ZH5(gt2,5)*(Sin(beta)*ZH5(gt4,1)+
     -    (0,1)*ZH5(gt4,3))+ZH5(gt2,2)*((0,-1)*Sin(beta)*ZH5(gt4,1)+
     -    ZH5(gt4,3))+((0,-1)*Sin(beta)*ZH5(gt2,1)+ZH5(gt2,3))*
     -    (ZH5(gt4,2)+(0,1)*ZH5(gt4,5)))*nZU(4+gt3,6)))
      
      dg2u2hdiv1(gt1,gt2,gt3,gt4)=(0,-1)*(CONJG(nZU(4+gt1,5))*
     -    nZU(4+gt3,5)+CONJG(nZU(4+gt1,6))*nZU(4+gt3,6))*2*dYtdiv1*
     -    Yu(3)*(Cos(beta)**2*ZH5(gt2,1)*ZH5(gt4,1)+ZH5(gt2,4)*
     -    ZH5(gt4,4))+((0,-0.5)*abslambda*dYtdiv1*(CONJG(nZU(4+gt1,6))*
     -    ((0,-1)*Cos(phil+phis)+Sin(phil+phis))*(ZH5(gt2,5)*
     -    (Sin(beta)*ZH5(gt4,1) - (0,1)*ZH5(gt4,3))+ZH5(gt2,2)*
     -    ((0,1)*Sin(beta)*ZH5(gt4,1)+ZH5(gt4,3))+((0,1)*Sin(beta)*
     -    ZH5(gt2,1)+ZH5(gt2,3))*(ZH5(gt4,2)-(0,1)*ZH5(gt4,5)))*
     -    nZU(4+gt3,5)+CONJG(nZU(4+gt1,5))*((0,1)*Cos(phil+phis)+
     -    Sin(phil+phis))*(ZH5(gt2,5)*(Sin(beta)*ZH5(gt4,1)+
     -    (0,1)*ZH5(gt4,3))+ZH5(gt2,2)*((0,-1)*Sin(beta)*ZH5(gt4,1)+
     -    ZH5(gt4,3))+((0,-1)*Sin(beta)*ZH5(gt2,1)+ZH5(gt2,3))*
     -    (ZH5(gt4,2)+(0,1)*ZH5(gt4,5)))*nZU(4+gt3,6)))
      end do
      end do
      end do
      end do
      

      do gt1=1,2
      dgudLHpfin(gt1)=(0,1)*dYtfin*(CONJG(nZU(4+gt1,6))*
     -      (Sqrt(2d0)*abslambda*vs*Sin(beta)*exp((0,1)*(phil+phis))+ 
     -      2*absAu(3)*Cos(beta)*exp(-(0,1)*(phiu+phiAu(3))))+ 
     -      Sqrt(2d0)*v*CONJG(nZU(4+gt1,5))*Cos(beta)*Sin(beta)*
     -      2*Yu(3))/2.+
     -      (0,1)*Yu(3)*(CONJG(nZU(4+gt1,6))*
     -      (2*dAtfin*Cos(beta)*exp(-(0,1)*(phiu+phiAu(3)))))/2.+
     -      (0,1)*Yu(3)*(CONJG(nZU(4+gt1,6))*
     -      (2*absAu(3)*Cos(beta)*
     -      (-(0,1)*dphiAtfin)*exp(-(0,1)*(phiu+phiAu(3)))))/2.
      dgudLHpdiv1(gt1)=(0,1)*dYtdiv1*(CONJG(nZU(4+gt1,6))*
     -      (Sqrt(2d0)*abslambda*vs*Sin(beta)*exp((0,1)*(phil+phis))+ 
     -      2*absAu(3)*Cos(beta)*exp(-(0,1)*(phiu+phiAu(3))))+ 
     -      Sqrt(2d0)*v*CONJG(nZU(4+gt1,5))*Cos(beta)*Sin(beta)*
     -      2*Yu(3))/2.+
     -      (0,1)*Yu(3)*(CONJG(nZU(4+gt1,6))*
     -      (2*dAtdiv1*Cos(beta)*exp(-(0,1)*(phiu+phiAu(3)))))/2.+
     -      (0,1)*Yu(3)*(CONJG(nZU(4+gt1,6))*
     -      (2*absAu(3)*Cos(beta)*
     -      (-(0,1)*dphiAtdiv1)*exp(-(0,1)*(phiu+phiAu(3)))))/2.
      end do
     
      
      
      do gt1=1,2
      do gt3=1,2
      dg2u2Hpfin(gt1,gt3)=(0,-1)*CONJG(nZU(4+gt1,6))*
     -            Cos(beta)**2*nZU(4+gt3,6)*2*dYtfin*Yu(3)
      dg2u2Hpdiv1(gt1,gt3)=(0,-1)*CONJG(nZU(4+gt1,6))*
     -            Cos(beta)**2*nZU(4+gt3,6)*2*dYtdiv1*Yu(3)
      end do
      end do
      
      dg2dL2Hpfin=(0,-1)*Cos(beta)**2*Yu(3)*2*dYtfin
      dg2dL2Hpdiv1=(0,-1)*Cos(beta)**2*Yu(3)*2*dYtdiv1
      
      return
      end
      
c----------------------------------------------------------------------c
c The following functions are necessary to caluclate the two-loop
c vacuum integrals:
c I(x,y,z)=1/Pi^4*Integral dq*dk/(k^2-x)/(q^2-y)/((k-q)^2-z)
c I is symmetric under exchange of any two arugments
c I=Ifin+1/epsilon*Idiv1+1/epsilon^2*Idiv2
c
c taken from hep-ph/0111209 and hep-ph/0501132
c----------------------------------------------------------------------c

      double complex function Ifin(xi,yi,zi)
      
      implicit none
      
      double precision x,y,z,xi,yi,zi,temp
      double precision lnx,lny,lnz
      double complex fxi,DiLi
      double precision Qs, Ae
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      common /scale/ Qs
     
      x=xi
      y=yi
      z=zi
      
      if(x.le.1d-8) then
      x=z
      z=0d0
      end if
      if(y.le.1d-8) then
      y=z
      z=0d0
      end if
      if(x.le.1d-8) then
      x=y
      y=0d0
      end if

      if(y.gt.x) then
      temp=x
      x=y
      y=temp
      end if
      
      if(x.gt.1d-8) lnx=Log(x/Qs**2)
      if(y.gt.1d-8) lny=Log(y/Qs**2)
      if(z.gt.1d-8) lnz=Log(z/Qs**2)
     
      if(x.gt.1d-8 .AND. y.gt.1d-8 .AND. z.gt.1d-8) then
      Ifin=(1d0/2d0*(x-y-z)*lny*lnz+1d0/2d0*(y-x-z)*lnx*lnz+
     .     1d0/2d0*(z-y-x)*lny*lnx+2d0*x*lnx+2d0*y*lny+2d0*z*lnz
     .     -5d0/2d0*(x+y+z)-1d0/2d0*fxi(x,y,z))

      elseif(x.gt.1d-8 .AND. y.gt.1d-8 .AND. z.le.1d-8) then
      if(abs(x-y).ge.1d-8) then
      Ifin=((x-y)*(DiLi(dcmplx(y/x))-Log(x/y)*Log(dcmplx((x-y)/Qs**2))+
     .      lnx**2/2d0-pi**2/6d0)-5d0/2d0*(x+y)+2d0*x*lnx+2d0*y*lny-
     .      x*lnx*lny)
      else
      Ifin=(-x*lnx**2+4d0*x*lnx-5d0*x)
      end if
      
      elseif(x.gt.1d-8 .AND. y.le.1d-8 .AND. z.le.1d-8) then
      Ifin=(-x*lnx**2/2d0+2d0*x*lnx-5d0/2d0*x-pi**2/6d0*x)
      
      else
      Ifin=0d0
      end if
      
      Ifin=Ifin-dcmplx(Ae(x)+Ae(y)+Ae(z))
      
      Ifin=pi**4*Ifin
      
      return
      end
      
      double complex function fxi(x,y,z)
      
      implicit none
      
      double precision x,y,z
      double complex R,arg1,arg2
      double complex DiLi
      
      double precision pi,eps
      parameter (pi = 2d0*Asin(1d0),eps=1d-42)
      
      R=sqrt(dcmplx(x**2+y**2+z**2-2d0*(x*y+x*z+y*z)))
      arg1=(z+x-y-R)/2d0/z-eps*(0,1)
      arg2=(z+y-x-R)/2d0/z-eps*(0,1)
      
      fxi=R*(2d0*Log(arg1)*Log(arg2)-
     .   Log(x/z)*Log(y/z)-2d0*DiLi(dcmplx(arg1))-
     .   2d0*DiLi(dcmplx(arg2))+pi**2/3d0)
      
      return
      end
      
      double precision function Idiv2(xi,yi,zi)
      
      implicit none
      
      double precision xi,yi,zi
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      Idiv2=-(xi+yi+zi)/2d0
      
      Idiv2=pi**4*Idiv2
      
      return
      end
      
      
      double precision function Idiv1(x,y,z)
      
      double precision x,y,z
      double precision lnx,lny,lnz
      double precision Qs
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      common /scale/ Qs
    
      if(x.gt.1d-8) lnx=Log(x/Qs**2)
      if(y.gt.1d-8) lny=Log(y/Qs**2)
      if(z.gt.1d-8) lnz=Log(z/Qs**2)
     
      Idiv1=(-x-y-z)*3d0/2d0
      if(x.gt.1d-8) Idiv1=Idiv1+x*lnx
      if(y.gt.1d-8) Idiv1=Idiv1+y*lny
      if(z.gt.1d-8) Idiv1=Idiv1+z*lnz
      
      Idiv1=pi**4*Idiv1
      
      return
      end
      
      double precision function Ae(x)
      
      double precision x,lnx
      double precision Qs
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      common /scale/ Qs
      
      if(x.le.1d-8) then
      Ae=0d0
      else
      lnx=Log(x/Qs**2)
      
      Ae=-x*(-1d0-pi**2/12d0+lnx-lnx**2/2d0)
      end if
      
      return
      end
      
      
      
      double complex function Be(p2,m12,m22)
      
      implicit none
      
!       double precision A0
     
      double precision p2,m12,m22
      double precision Qs
      double precision e,s
      double complex x1,x2
      double complex DiLi
      double precision Ae,A0
      double complex B0
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      common /scale/ Qs
      
      if(p2.le.1d-12) then
      
         if(abs(m12-m22).le.1d-12) then
         
         Be=(Ae(m12)-A0(m12))/m12
      
         else
         
         Be=(Ae(m12)-Ae(m22))/(m12-m22)
         
         end if
         
      else !(i.e. p2.ne.0d0)
      
      
      e=1d-40
      
      s=p2-m22+m12
      x1=(s-Sqrt(s**2-4d0*p2*(m12-(0,1)*e)))/2d0/p2
      x2=(s+Sqrt(s**2-4d0*p2*(m12-(0,1)*e)))/2d0/p2
     
      
      Be=pi**2/12d0+log(p2/Qs**2-(0,1)*e)**2/2d0
     -   -(log(p2/Qs**2-(0,1)*e)-2d0)*(B0(p2,m12,m22,Qs**2)
     -   +log(p2/Qs**2-(0,1)*e))
     -   +(1d0-x1)*log(1d0-x1)**2/2d0+x1*log(-x1)**2/2d0
     -   +(1d0-x2)*log(1d0-x2)**2/2d0+x2*log(-x2)**2/2d0
     -   +(1d0-x1)*log(1d0-x1)*log(1d0-x2)+x1*log(-x1)*log(-x2)
     -   +(x1-x2)*(DiLi(x2/(x2-x1))-DiLi((x2-1d0)/(x2-x1))
     -   +log(x2-x1)*log((x2-1d0)/x2))
     
      end if
      
      
      return
      end


c ---------------------------------------------------------------------c
c DiLi calculates the dilogarithm of any complex argument z
c The subroutines called within this function are taken from 
c LoopTools (see Comput. Phys. Commun. 118 (1999) 153)
c----------------------------------------------------------------------c

      double complex function DiLi(z)
      
      implicit none

      double complex z
      double complex zdilog,zlog,zxdilo
      integer ieps,ipi12
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
     
      if(imag(z).eq.0d0) then
      ieps=-1
      call ffzxdl(zxdilo,ipi12,zlog,realpart(z),ieps)
      DiLi=zxdilo+ipi12*pi**2/12d0
      else
      call ffzzdl(zdilog,ipi12,zlog,z)
      DiLi=zdilog+ipi12*pi**2/12d0
      end if

      return
      end
      
      
      subroutine ffzxdl(zxdilo,ipi12,zlog,x,ieps)
***#[*comment:***************************************************
*         Computes the dilogarithm (Li2, Sp) for any (real) x         *
*         to precision precx. If an error message is given add         *
*         more bf's. For x > 1 the imaginary part is                  *
*          -/+i*pi*log(x), corresponding to x+ieps.                  *
*         The number of factors pi^2/12 is passed separately in         *
*         ipi12 for accuracy.  We also calculate log(1-x)                  *
*         which is likely to be needed.                                    *
*                                                                        *
*         Input:         x         (real)                                             *
*                  ieps         (integer,+/-1)                                    *
*                                                                        *
*         Output: zxdilo         (complex) the dilog mod factors pi2/12         *
*                  ipi12         (integer) these factors                           *
*                  zlog         (complex) log(1-x)                           *
*                                                                        *
*         Calls:         log,dfflo1                                             *
*                                                                        *
***#]*comment:***************************************************
*  #[ declarations:
         implicit none
*
*         arguments
*
         integer ipi12,ieps
         DOUBLE PRECISION x
         DOUBLE COMPLEX zxdilo,zlog
*
*         local variables
*
         integer jsgn
         DOUBLE PRECISION fact,u,u2,dfflo1,ffbnd,a,xdilo,
     +                  xprec,bdn02,bdn05,bdn10,bdn15,bf(20)
         DOUBLE COMPLEX cy,cfact
!          external ffbnd,dfflo1
         save xprec,bdn02,bdn05,bdn10,bdn15
         
      double precision pi,precx,xalogm,xalog2,xlg2,xloss
      parameter (pi = 2d0*Asin(1d0),precx=4.d-16,xalogm=2.d-308,
     +           xalog2=4.d-154,xlg2=Log(2d0),xloss=1d0/8d0)
*
*         common blocks
*
! #include "ff.h"
*  #] declarations:
*  #[ initialisations:
         data xprec /-1D0/
        bf(1) = - 1.D+0/4.D+0
        bf(2) = + 1.D+0/36.D+0
        bf(3) = - 1.D+0/36.D+2
        bf(4) = + 1.D+0/21168.D+1
        bf(5) = - 1.D+0/108864.D+2
        bf(6) = + 1.D+0/52690176.D+1
        bf(7) = - 691.D+0/16999766784.D+3
        bf(8) = + 1.D+0/1120863744.D+3
        bf(9) = - 3617.D+0/18140058832896.D+4
        bf(10) = + 43867.D+0/97072790126247936.D+3
        bf(11) = - 174611.D+0/168600109166641152.D+5
        bf(12) = + 77683.D+0/32432530090601152512.D+4
        bf(13) = - 236364091.D+0/4234560341829359173632.D+7
        bf(14) = + 657931.D+0/5025632054039239458816.D+6
        bf(15) = - 3392780147.D+0/109890470493622010006470656.D+7
        bf(16)=+172.3168255201D+0/2355349904102724211909.3102313472D+6
        bf(17)=-770.9321041217D+0/4428491985594062112714.2791446528D+8
        bf(18)=( 0.4157635644614046176D-28)
        bf(19)=(-0.9962148488284986022D-30)
        bf(20)=( 0.2394034424896265390D-31)
         
         
         if ( xprec .ne. precx ) then
             xprec = precx
             bdn02 = ffbnd(1,2,bf)
             bdn05 = ffbnd(1,5,bf)
             bdn10 = ffbnd(1,10,bf)
             bdn15 = ffbnd(1,15,bf)
         endif
*  #] initialisations:
*  #[ exceptional cases:
         if ( x .eq. 1) then
             zxdilo = 0
             zlog = -99999
             ipi12 = 2
             return
         elseif (x .eq. -1) then
             zxdilo = 0
             zlog = xlg2
             ipi12 = -1
             return
         elseif (x .eq. .5D0) then
             zxdilo = - xlg2**2/2
             zlog = -xlg2
             ipi12 = 1
             return
         elseif ( abs(x) .lt. precx ) then
             zxdilo = x
             zlog = -x
             ipi12 = 0
             return
         endif
*  #] exceptional cases:
*  #[ transform to (-1,.5):
         if (x .lt. -1) then
             fact = log(-x)
             cy = - fact**2/2
             ipi12 = -2
             if ( -x*xloss .gt. 1 ) then
                  u = -dfflo1(1/x)
             else
                  u = -log(1-1/x)
             endif
             zlog = log(1-x)
             jsgn = -1
         elseif ( x .lt. .5D0) then
             cy = 0
             ipi12 = 0
             if ( abs(x) .lt. xloss ) then
                  zlog = dfflo1(x)
             else
                  zlog = log(1-x)
             endif
             u = -DBLE(zlog)
             jsgn = 1
         elseif ( x .le. 2 ) then
             u = -log(x)
             if ( abs(1-x) .lt. xalogm ) then
                  cy = 0
             elseif ( x .lt. 1 ) then
                  zlog = log(1-x)
                  cy = DBLE(u)*zlog
             elseif ( ieps .gt. 0 ) then
                  zlog = DCMPLX(log(x-1),-pi)
                  cy = DBLE(u)*zlog
             else
                  zlog = DCMPLX(log(x-1),+pi)
                  cy = DBLE(u)*zlog
             endif
             ipi12 = 2
             jsgn = -1
         else
             if ( ieps .gt. 0 ) then
                  cfact = DCMPLX(log(x),-pi)
                  zlog = DCMPLX(log(x-1),-pi)
             else
                  cfact = DCMPLX(log(x),+pi)
                  zlog = DCMPLX(log(x-1),+pi)
             endif
             cy = - cfact**2/2
             ipi12 = -2
             if ( x*xloss .gt. 1 ) then
                  u = -dfflo1(1/x)
             else
                  u = -log(1-1/x)
             endif
             jsgn = -1
         endif
*  #] transform to (-1,.5):
*  #[ calculate dilog:
         if ( abs(u) .lt. xalog2 ) then
             xdilo = u
         else
         u2 = u**2
         a = abs(u2)
         if ( a .gt. bdn15 ) then
             xdilo = u2*(bf(16) + u2*(bf(17) + u2*(bf(18) +
     +                      u2*(bf(19) + u2*bf(20) ))))
         else
             xdilo = 0
         endif
         if ( a .gt. bdn10 ) then
             xdilo = u2*(bf(11) + u2*(bf(12) + u2*(bf(13) +
     +                      u2*(bf(14) + u2*(bf(15) + xdilo)))))
         endif
         if ( a .gt. bdn05 ) then
             xdilo = u2*(bf(6) + u2*(bf(7) + u2*(bf(8) +
     +                      u2*(bf(9) + u2*(bf(10) + xdilo)))))
         endif
         if ( a .gt. bdn02 ) then
             xdilo = u2*(bf(3) + u2*(bf(4) + u2*(bf(5) + xdilo)))
         endif
*         watch the powers of u.
         xdilo = u + u2*(bf(1) + u*(bf(2) + xdilo))
         endif
         if(jsgn.eq.1)then
             zxdilo =  DBLE(xdilo) + cy
         else
             zxdilo = -DBLE(xdilo) + cy
         endif
*  #] calculate dilog:
*###] ffzxdl:
         end
         
         
         
*###[ ffzzdl:
        subroutine ffzzdl(zdilog,ipi12,zlog,cx)
***#[*comment:***************************************************
*                                                                *
*        Computes the dilogarithm (Li2, Sp) for any (complex) cx *
*        to about 15 significant figures. This can be improved        *
*        by adding more of the bf's. For real cx > 1 an error is        *
*        generated as the imaginary part is undefined then.        *
*        For use in ffcdbd zlog = log(1-cx) is also calculated        *
*                                                                *
*        Input:        cx        (complex)                                *
*                                                                *
*        Output:        zdilog        (complex) Li2(cx) mod factors pi^2/12        *
*                ipi12        (integer) these factors                        *
*                zlog        (complex) log(1-cx)                        *
*                                                                *
*        Calls:        log,zfflo1,(d/a)imag,real/dble                        *
*                                                                *
***#]*comment:*************************************************** 
*  #[ declarations:
        implicit none
*
*        arguments
*
        integer ipi12
        DOUBLE COMPLEX zdilog,zlog,cx
*
*        local variables
*
        integer jsgn
        DOUBLE PRECISION xprec,bdn02,bdn05,bdn10,bdn15,
     +                xi,xr,s1,s2,xa,a,absc,ffbnd,bf(20)
        DOUBLE COMPLEX cfact,cx1,cy,cz,cz2,zfflo1,c
!         external ffbnd,zfflo1
        save xprec,bdn02,bdn05,bdn10,bdn15
        
        double precision pi,precc,xalogm,xalog2,xclogm,xlg2,xloss
        parameter (pi = 2d0*Asin(1d0),precc=4.d-16,xalogm=2.d-308,
     +           xalog2=4.d-154,xclogm=2.d-308,xlg2=Log(2d0),
     +           xloss=1d0/8d0)
*
*         common blocks
*
! #include "ff.h"
*  #] declarations:
*  #[ initialisations:
*        statement function
*
        absc(c) = abs(DBLE(c)) + abs(DIMAG(c))
*  #] declarations: 
*  #[ initialisations:
        data xprec /-1D0/
        bf(1) = - 1.D+0/4.D+0
        bf(2) = + 1.D+0/36.D+0
        bf(3) = - 1.D+0/36.D+2
        bf(4) = + 1.D+0/21168.D+1
        bf(5) = - 1.D+0/108864.D+2
        bf(6) = + 1.D+0/52690176.D+1
        bf(7) = - 691.D+0/16999766784.D+3
        bf(8) = + 1.D+0/1120863744.D+3
        bf(9) = - 3617.D+0/18140058832896.D+4
        bf(10) = + 43867.D+0/97072790126247936.D+3
        bf(11) = - 174611.D+0/168600109166641152.D+5
        bf(12) = + 77683.D+0/32432530090601152512.D+4
        bf(13) = - 236364091.D+0/4234560341829359173632.D+7
        bf(14) = + 657931.D+0/5025632054039239458816.D+6
        bf(15) = - 3392780147.D+0/109890470493622010006470656.D+7
        bf(16)=+172.3168255201D+0/2355349904102724211909.3102313472D+6
        bf(17)=-770.9321041217D+0/4428491985594062112714.2791446528D+8
        bf(18)=( 0.4157635644614046176D-28)
        bf(19)=(-0.9962148488284986022D-30)
        bf(20)=( 0.2394034424896265390D-31)
*
*        common blocks
*
! #include "ff.h"
*

        if ( xprec .ne. precc ) then
            xprec = precc
            bdn02 = ffbnd(1,2,bf)
            bdn05 = ffbnd(1,5,bf)
            bdn10 = ffbnd(1,10,bf)
            bdn15 = ffbnd(1,15,bf)
        endif
*  #] initialisations: 
*  #[ exceptional cases:
        xi  = DIMAG(cx)
        xr  = DBLE(cx)
        if ( xi .eq. 0 ) then
            if ( xr .gt. 1 ) write(*,*) 'suppressed error message'!call fferr(31,ier)
            call ffzxdl(zdilog,ipi12,zlog,xr,1)
            return
        endif
        if ( abs(xi) .lt. xalog2 ) then
            s1 = 0
        else
            s1 = xi**2
        endif
        if ( abs(xr) .lt. xalog2 ) then
            s2 = 0
        else
            s2 = xr**2
        endif
        xa = sqrt(s1 + s2)
        if ( xa .lt. precc ) then
            zdilog = cx
            zlog = -cx
            ipi12 = 0
            return
        endif
*  #] exceptional cases: 
*  #[ transform to |x|<1, Re(x) < 0.5:
        if ( xr .le. .5D0) then
            if (xa .gt. 1) then
                if ( 1/xa .lt. xalogm ) then
                    cfact = 0
                elseif ( 1/xa .lt. xclogm ) then
                    cx1 = cx*DBLE(1/xa)
                    cfact = log(-cx1) + log(DBLE(xa))
                else
                    cfact = log(-cx)
                endif
                cy = - cfact**2/2
                ipi12 = -2
                if ( xa*xloss**2 .gt. 1) then
                    if ( 1/xa .lt. xclogm ) then
                        cx1 = cx*DBLE(1/xa)
                        cx1 = 1/cx1
                        cx1 = cx1*DBLE(1/xa)
                    else
                        cx1 = 1/cx
                    endif
                    cz = -zfflo1(cx1)
                else
                    cz = -log(1-1/cx)
                endif
                zlog = log(1-cx)
                jsgn = -1
            else
                cy = 0
                ipi12 = 0
                if ( xa .lt. xloss**2 ) then
                    zlog = zfflo1(cx)
                else
                    zlog = log(1-cx)
                endif
                cz = -zlog
                jsgn = 1
            endif
        else
            if (xa .le. sqrt(2*xr)) then
                cz = -log(cx)
                if ( abs(xr-1) + abs(xi) .lt. xclogm ) then
                    cy = 0
                else
                    zlog = log(1-cx)
                    cy = cz*zlog
                endif
                ipi12 = 2
                jsgn = -1
            else
                if ( 1/xa .lt. xalogm ) then
                    cfact = 0
                elseif ( 1/xa .lt. xclogm ) then
                    cx1 = cx*DBLE(1/xa)
                    cfact = log(-cx1) + log(DBLE(xa))
                else
                    cfact = log(-cx)
                endif
                cy = - cfact**2/2
                ipi12 = -2
                if ( xa*xloss .gt. 1) then
                    if ( 1/xa .lt. xclogm ) then
                        cx1 = cx*DBLE(1/xa)
                        cx1 = 1/cx1
                        cx1 = cx1*DBLE(1/xa)
                    else
                        cx1 = 1/cx
                    endif
                    cz = -zfflo1(cx1)
                else
                    cz = -log(1-1/cx)
                endif
                zlog = log(1-cx)
                jsgn = -1
            endif
        endif
*  #] transform to |x|<1, Re(x) < 0.5: 
*  #[ get dilog:
        if ( absc(cz) .lt. xclogm ) then
            zdilog = cz
        else
        cz2 = cz*cz
        a = DBLE(cz)**2 + DIMAG(cz)**2
        if ( a .gt. bdn15 ) then
            zdilog = cz2*(DBLE(bf(16)) + cz2*(DBLE(bf(17))
     +                   + cz2*(DBLE(bf(18)) + cz2*(DBLE(bf(19))
     +                   + cz2*(DBLE(bf(20)))))))
        else
            zdilog = 0
        endif
        if ( a .gt. bdn10 ) then
            zdilog = cz2*(DBLE(bf(11)) + cz2*(DBLE(bf(12))
     +                   + cz2*(DBLE(bf(13)) + cz2*(DBLE(bf(14))
     +                   + cz2*(DBLE(bf(15)) + zdilog)))))
        endif
        if ( a .gt. bdn05 ) then
            zdilog = cz2*(DBLE(bf(6)) + cz2*(DBLE(bf(7))
     +                   + cz2*(DBLE(bf(8)) + cz2*(DBLE(bf(9))
     +                   + cz2*(DBLE(bf(10)) + zdilog)))))
        endif
        if ( a .gt. bdn02 ) then
            zdilog = cz2*(DBLE(bf(3)) + cz2*(DBLE(bf(4))
     +                   + cz2*(DBLE(bf(5)) + zdilog)))
        endif
*        watch the powers of z.
        zdilog = cz + cz2*(DBLE(bf(1)) + cz*(DBLE(bf(2)) + zdilog))
        endif
        if(jsgn.eq.1)then
            zdilog =  zdilog + cy
        else
            zdilog = -zdilog + cy
        endif
*  #] get dilog: 
*###] ffzzdl: 
        end
         

*###[ ffbnd:
        DOUBLE PRECISION function ffbnd(n1,n2,array)
*************************************************************************
*                                                                        *
*        calculate bound = (precx*|a(n1)/a(n1+n2)|^(1/n2) which is the        *
*        maximum value of x in a series expansion sum_(i=n1)^(n1+n2)        *
*        a(i)*x(i) to give a result of accuracy precx (actually of |next        *
*        term| < prec                                                        *
*                                                                        *
*************************************************************************
        implicit none
        integer n1,n2
        DOUBLE PRECISION array(n1+n2)
        double precision precx
        parameter (precx=4.d-16)
! #include "ff.h"
        if ( array(n1+n2) .eq. 0 ) then
           print *,'ffbnd: fatal: array not initialized; did you call ',
     +                'ltini?'
           stop
        endif
        ffbnd = (precx*abs(array(n1)/array(n1+n2)))**(1/DBLE(n2))
* added 22 Mar 11: be a bit more conservative:
        ffbnd = .8D0*ffbnd
*###] ffbnd:
        end
        
        
        
        
*###[ dfflo1:
        DOUBLE PRECISION function dfflo1(x)
***#[*comment:***************************************************
*        calculates log(1-x) for |x|<.14 in a faster way to ~15        *
*        significant figures.                                        *
***#]*comment:***************************************************
*  #[ declarations:
        implicit none
        DOUBLE PRECISION x,bdn01,bdn05,bdn10,bdn15,bdn19,xprec,
     +                xa,ffbnd,xninv(30)
        DOUBLE COMPLEX zxfflg
!         external ffbnd,zxfflg
        save xprec,bdn01,bdn05,bdn10,bdn15,bdn19
        double precision precx
        integer i
        parameter (precx=4.d-16)
! #include "ff.h"
*  #] declarations:
*  #[ initialisation:
        data xprec /-1D0/
        
        do i=1,30
        xninv(i)=1d0/i
        end do
        
        if ( xprec .ne. precx ) then
            xprec = precx
*            determine the boundaries for 1,5,10,15 terms
            bdn01 = ffbnd(1,1,xninv)
            bdn05 = ffbnd(1,5,xninv)
            bdn10 = ffbnd(1,10,xninv)
            bdn15 = ffbnd(1,15,xninv)
            bdn19 = ffbnd(1,19,xninv)
        endif
*  #] initialisation:
*  #[ calculations:
        xa = abs(x)
        if ( xa .gt. bdn19 ) then
            dfflo1 = DBLE(zxfflg(1-x,0,0D0))
            return
        endif
        if ( xa .gt. bdn15 ) then
            dfflo1 = x*( xninv(16) + x*( xninv(17) + x*( xninv(18) +
     +                x*( xninv(19) + x*xninv(20) ))))
        else
            dfflo1 = 0
        endif
        if ( xa .gt. bdn10 ) then
            dfflo1 = x*( xninv(11) + x*( xninv(12) + x*( xninv(13) +
     +                x*( xninv(14) + x*( xninv(15) + dfflo1 )))))
        endif
        if ( xa .gt. bdn05 ) then
            dfflo1 = x*( xninv(6) + x*( xninv(7) + x*( xninv(8) +
     +                x*( xninv(9) + x*( xninv(10) + dfflo1 )))))
        endif
        if ( xa .gt. bdn01 ) then
            dfflo1 = x*( xninv(2) + x*( xninv(3) + x*( xninv(4) +
     +                x*( xninv(5) + dfflo1 ))))
        endif
        dfflo1 = - x*( xninv(1) + dfflo1 )
*  #] calculations:
*###] dfflo1:
        end
        
        
        
*###[ zxfflg:
        DOUBLE COMPLEX function zxfflg(x,ieps,y)
***#[*comment:***********************************************************
*                                                                        *
*        Calculate the complex logarithm of x.  The following cases        *
*        are treted separately:                                                *
*                |x| too small:                give warning and return 0        *
*                                        (for Absoft, Apollo DN300)        *
*                |x| < 0:                take sign according to ieps        *
*                                                                        *
***#]*comment:***********************************************************
*  #[ declarations:
*
*        arguments
*
        implicit none
        integer ieps
        DOUBLE PRECISION x,y
*
*        local variables
*
        DOUBLE PRECISION xlog
        
      double precision pi,xalogm
      parameter (pi = 2d0*Asin(1d0),xalogm=2.d-308)
*
*        common blocks
*
! #include "ff.h"
*  #] declarations:
*  #[ calculations:
        if ( abs(x) .lt. xalogm ) then
            zxfflg = 0
        elseif ( x .gt. 0 ) then
            zxfflg = log(x)
        else
            xlog = log(-x)
*            checked imaginary parts 19-May-1988
            if ( abs(ieps) .eq. 1 ) then
                if ( y*ieps .lt. 0 ) then
                    zxfflg = DCMPLX(xlog,-pi)
                else
                    zxfflg = DCMPLX(xlog,pi)
                endif
            elseif ( ieps .eq. 2 ) then
                zxfflg = DCMPLX(xlog,-pi)
            elseif ( ieps .eq. -2 ) then
                zxfflg = DCMPLX(xlog,+pi)
            else
!                 call fferr(52,ier)
                zxfflg = DCMPLX(xlog,pi)
            endif
        endif
*  #] calculations:
*###] zxfflg:
        end
        
        
*###[ zfflo1:
        DOUBLE COMPLEX function zfflo1(cx)
***#[*comment:***************************************************
*        calculates log(1-x) for |x|<.14 in a faster way to ~15        *
*        significant figures.                                        *
***#]*comment:*************************************************** 
*  #[ declarations:
        implicit none
        DOUBLE COMPLEX cx,c,zfflog
        DOUBLE PRECISION xprec,bdn01,bdn05,bdn10,bdn15,bdn19,
     +                absc,xa,ffbnd,xninv(30)
!         external zfflog,ffbnd
        double precision precc
        integer i
        double complex czero
        parameter (precc=4.d-16,czero=(0d0,0d0))
        save xprec,bdn01,bdn05,bdn10,bdn15,bdn19
! #include "ff.h"
        absc(c) = abs(DBLE(c)) + abs(DIMAG(c))
*  #] declarations: 
*  #[ initialisations:
        data xprec /-1D0/
        
        do i=1,30
        xninv(i)=1d0/i
        end do
        
        if ( precc .ne. xprec ) then
            xprec = precc
*            determine the boundaries for 1,5,10,15 terms
            bdn01 = ffbnd(1,1,xninv)
            bdn05 = ffbnd(1,5,xninv)
            bdn10 = ffbnd(1,10,xninv)
            bdn15 = ffbnd(1,15,xninv)
            bdn19 = ffbnd(1,19,xninv)
        endif
*  #] initialisations: 
*  #[ calculations:
        xa = absc(cx)
        if ( xa .gt. bdn19 ) then
            c = cx-1
            xa = absc(c)
            zfflo1 = zfflog(1-cx,0,czero)
            return
        endif
        if ( xa .gt. bdn15 ) then
            zfflo1 = cx*( DBLE(xninv(16)) + cx*( DBLE(xninv(17))
     +                   + cx*( DBLE(xninv(18)) + cx*( DBLE(xninv(19))
     +                   + cx*( DBLE(xninv(20)) )))))
        else
            zfflo1 = 0
        endif
        if ( xa .gt. bdn10 ) then
            zfflo1 = cx*( DBLE(xninv(11)) + cx*( DBLE(xninv(12))
     +                   + cx*( DBLE(xninv(13)) + cx*( DBLE(xninv(14))
     +                   + cx*( DBLE(xninv(15)) + zfflo1 )))))
        endif
        if ( xa .gt. bdn05 ) then
            zfflo1 = cx*( DBLE(xninv(6)) + cx*( DBLE(xninv(7))
     +                   + cx*( DBLE(xninv(8)) + cx*( DBLE(xninv(9))
     +                   + cx*( DBLE(xninv(10)) + zfflo1 )))))
        endif
        if ( xa .gt. bdn01 ) then
            zfflo1 = cx*( DBLE(xninv(2)) + cx*( DBLE(xninv(3))
     +                   + cx*( DBLE(xninv(4)) + cx*( DBLE(xninv(5))
     +                   + zfflo1 ))))
        endif
        zfflo1 = - cx*( DBLE(xninv(1)) + zfflo1 )
*  #] calculations: 
*###] zfflo1: 
        end
        
        
*###[ zfflog:
        DOUBLE COMPLEX function zfflog(cx,ieps,cy)
***#[*comment:***********************************************************
*                                                                        *
*        Calculate the complex logarithm of cx.  The following cases        *
*        are treted separately:                                                *
*                |cx| too small:                give warning and return 0        *
*                                        (for Absoft, Apollo DN300)        *
*                Im(cx) = 0, Re(cx) < 0:        take sign according to ieps        *
*                                                                        *
***#]*comment:*********************************************************** 
*  #[ declarations:
*
*        arguments
*
        implicit none
        integer ieps
        DOUBLE COMPLEX cx,cy
*
*        local variables
*
        DOUBLE COMPLEX c,ctroep
        DOUBLE PRECISION absc,xa,xlog1p
        
        double precision pi,xalogm,xclogm
        parameter (pi = 2d0*Asin(1d0),xalogm=2.d-308,xclogm=2.d-308)
*
*        common blocks, statement function
*
! #include "ff.h"
        absc(c) = abs(DBLE(c)) + abs(DIMAG(c))
*  #] declarations: 
*  #[ calculations:
        xa = absc(cx)
        if ( xa .lt. xalogm ) then
            if ( cx .ne. 0 ) write(*,*) 'suppressed error message'!call fferr(23,ier)
            zfflog = 0
        elseif ( DBLE(cx) .lt. 0 .and. DIMAG(cx) .eq. 0 ) then
*     +                 abs(DIMAG(cx)) .lt. precc*abs(DBLE(cx)) ) then
            xlog1p = log(-DBLE(cx))
*            checked imaginary parts 19-May-1988
            if ( abs(ieps) .eq. 1 ) then
                if ( ieps*DBLE(cy) .lt. 0 ) then
                    zfflog = DCMPLX(xlog1p,-pi)
                elseif ( ieps*DBLE(cy) .gt. 0 ) then
                    zfflog = DCMPLX(xlog1p,pi)
                else
                    write(*,*) 'suppressed error message'!call fferr(51,ier)
                    zfflog = DCMPLX(xlog1p,pi)
                endif
            elseif ( ieps .ge. 2 .and. ieps .le. 3 ) then
                zfflog = DCMPLX(xlog1p,-pi)
            elseif ( ieps .le. -2 .and. ieps .ge. -3 ) then
                zfflog = DCMPLX(xlog1p,pi)
            else
                write(*,*) 'suppressed error message'!call fferr(51,ier)
                zfflog = DCMPLX(xlog1p,pi)
            endif
        elseif ( xa .lt. xclogm .or. 1/xa .lt. xclogm ) then
            ctroep = cx*DBLE(1/xa)
            zfflog = log(ctroep) + DBLE(log(xa))
        else
*            print *,'zfflog: neem log van ',cx
            zfflog = log(cx)
        endif
*  #] calculations: 
*###] zfflog: 
        end
      
c--------------------------------------------------------------------------------------------c

c auxillary function (combination of Ae and A0)
      double precision function F1(x)
      
      implicit none
      
      double precision x
      double precision lnx,Qs
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      common /scale/ Qs
      
      lnx=log(x/Qs**2)
      F1=x*(4*lnx*(3*lnx-7)+pi**2+30)
      
      return
      end
      
      
      
      double precision function F2(x)
      
      implicit none
      
      double precision x
      double precision lnx,Qs
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      common /scale/ Qs
      
      lnx=log(x/Qs**2)
      F2=4*lnx*(3*lnx-1)+pi**2+2
      
      return
      end
      
      
      double precision function F3(x,y)
      
      implicit none
      
      double precision x,y
      double precision lnx,lny,Qs
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      common /scale/ Qs
      
      lnx=log(x/Qs**2)
      lny=log(y/Qs**2)
      F3=((-1+lnx+lny)*x)/y
      
      return
      end
      
      
      
      double precision function F4(x,y)
      
      implicit none
      
      double precision x,y
      double precision lnx,lny,Qs
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      common /scale/ Qs
      
      lnx=log(x/Qs**2)
      lny=log(y/Qs**2)
      F4=x*((4+3*lnx**2+2*lny-lny**2-
     .   2*lnx*(3+lny))*x+2*(-2+lnx+lny)*y)
      
      return
      end
      
      
      double precision function F5(x,y)
      
      implicit none
      
      double precision x,y
      double precision lnx,lny,Qs
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      common /scale/ Qs
      
      lnx=log(x/Qs**2)
      lny=log(y/Qs**2)
      F5=y*(((-2+lnx)**2+2*lnx*lny-3*lny**2)*x+
     .        4*(-1+lny)*y)
      
      return
      end
      
      
      
      double precision function F6(x,y)
      
      implicit none
      
      double precision x,y
      double precision lnx,lny,Qs
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      common /scale/ Qs
      
      lnx=log(x/Qs**2)
      lny=log(y/Qs**2)
      F6=(6+3*lnx**2+6*lnx*(-1+lny)+
     .    3*(-2+lny)*lny+Pi**2)*x
      
      return
      end
      
      
      
      double precision function F7(x,y)
      
      implicit none
      
      double precision x,y
      double precision lnx,lny,Qs
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      common /scale/ Qs
      
      lnx=log(x/Qs**2)
      lny=log(y/Qs**2)
      F7=y*(3*(-4+lnx+lny)*(lnx+lny)*x+
     .     (18+Pi**2)*(x-y)-12*(-2+lny)*lny*y)
      
      return
      end
      
      
      double precision function F8(x,y)
      
      implicit none
      
      double precision x,y
      double precision lnx,lny,Qs
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      common /scale/ Qs
      
      lnx=log(x/Qs**2)
      lny=log(y/Qs**2)
      F8=12*(-1+lnx)*lnx*x+(6+Pi**2)*(x-y)-
     .   3*(-2+lnx+lny)*(lnx+lny)*y
      
      return
      end
      
      
      double precision function F9(x,y)
      
      implicit none
      
      double precision x,y
      double precision lnx,lny,Qs
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      common /scale/ Qs
      
      lnx=log(x/Qs**2)
      lny=log(y/Qs**2)
      F9=(18+Pi**2)*(x-y)**2+6*lnx**2*x*(2*x-y)-
     .    6*lny**2*(x-2*y)*y+
     .    24*lny*(x-y)*y-12*lnx*x*(2*x+(-2+lny)*y)
      
      return
      end
      
      
      double precision function F10(x,y)
      
      implicit none
      
      double precision x,y
      double precision lnx,lny,Qs
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      common /scale/ Qs
      
      lnx=log(x/Qs**2)
      lny=log(y/Qs**2)
      F10=3*(lnx+lny)**2+Pi**2
      
      return
      end
      
      double precision function F17(x,y,z)
      
      implicit none
      
      double precision x,y,z
      double precision lnx,lny,lnz,Qs
      
      common /scale/ Qs
      
      lnx=log(x/Qs**2)
      lny=log(y/Qs**2)
      lnz=log(z/Qs**2)
      F17=(z*((lnx-lny)*(-4+lnx+lny+2*lnz)*x*y+
     -    (-((lnx-lnz)*(-4+lnx+3*lnz)*x)+(lny-lnz)*
     -    (-4+lny+3*lnz)*y)*z))/((x-y)*(x-z)*(y-z))
      
      return
      end
      
      
      double precision function F18(x,y,z)
      
      implicit none
      
      double precision x,y,z
      double precision lnx,lny,lnz,Qs
      
      common /scale/ Qs
      
      lnx=log(x/Qs**2)
      lny=log(y/Qs**2)
      lnz=log(z/Qs**2)
      F18=(y*((4+lnx**2+2*lnx*(-2+lny)-(-2+lnz)*lnz-2*lny*(1+lnz))*x+
     -     2*(-2+lny+lnz)*z))/(x-z)**2
      
      return
      end
      
      double precision function F19(x,y,z)
      
      implicit none
      
      double precision x,y,z
      double precision lnx,lny,lnz,Qs
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      common /scale/ Qs
      
      lnx=log(x/Qs**2)
      lny=log(y/Qs**2)
      lnz=log(z/Qs**2)
      F19=(-y+z)*(3*lnx**2*x*(y - z)+6*lnx*x*((-2+lny)*y+2*z)+
     -    3*lnz**2*z*(-x - y+4*z) - 6*lnz*z*(-2*x+lnx*x +
     -    (-2+lny)*y+4*z)+(-x+z)*(-((18+3*(-4+lny)*lny+
     -    Pi**2)*y)+(18+Pi**2)*z))
      
      return
      end
      
      double precision function F20(x,y)
      
      implicit none
      
      double precision x,y
      double precision lnx,lny,Qs
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      common /scale/ Qs
      
      lnx=log(x/Qs**2)
      lny=log(y/Qs**2)
      F20=-(y*(-(Pi**4*(18+Pi**2)*x) - 
     -      3*Pi**4*x*(-4+lnx+lny)*(lnx+lny)))
      
      return
      end
      
      
      double precision function F21(x,y)
      
      implicit none
      
      double precision x,y
      double precision lnx,lny,Qs
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      common /scale/ Qs
      
      lnx=log(x/Qs**2)
      lny=log(y/Qs**2)
      F21=18+3*lnx**2+6*lnx*(-2+lny) - 12*lny+3*lny**2+Pi**2
      
      return
      end
      
      
      double precision function F22(x,y)
      
      implicit none
      
      double precision x,y
      double precision lnx,lny,Qs
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      common /scale/ Qs
      
      lnx=log(x/Qs**2)
      lny=log(y/Qs**2)
      F22=Pi**4*(9*x+15*y+3*lnx**2*y+6*(-2+lnx)*lny*y+
     -    3*lny**2*y+Pi**2*y - 6*lnx*(x+y))
      
      return
      end
      
c----------------------------------------------------------------------c
c The following functions TFI are abbreviations for two-loop integrals c
c The Mathematica Package Tarcer (arXiv:9801383) was used to perform   c
c the reduction to the two-loop vacuum integral and products of        c
c one-loop integrals.                                                  c
c----------------------------------------------------------------------c

c TFI1 corresponds to
c TFI[D, 0, {{3, Sqrt[x]}, {1, Sqrt[y]}, {0, 0}, {0, 0}, {1, Sqrt[z]}}] 
c in Tarcer notation
      double complex function TFI1(x,y,z)
      
      implicit none
      
      double precision x,y,z
      
      double precision lnx,lny,lnz
      double complex Ifin
      double precision Qs
     
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      common /scale/ Qs
      
      lnx=log(x/Qs**2)
      lny=log(y/Qs**2)
      lnz=log(z/Qs**2)

      TFI1=-(Pi**4*((-3+6*lnx)*x**4+3*(y - z)**3*
     -      ((-1+lnx+lny)*y+z - (lnx+lnz)*z) - 
     -      3*x**3*((-4+7*lnx+lny)*y+(-4+7*lnx+lnz)*z)+
     -      x**2*(9*(-2+3*lnx+lny)*y**2+
     -      (3*(10+lny+2*lnx*(-3+lnx+lny)+lnz+
     -      2*(lnx - lny)*lnz)+Pi**2)*y*z+
     -      9*(-2+3*lnx+lnz)*z**2)+
     -      x*(-3*(-4+5*lnx+3*lny)*y**3+
     -      (30+3*lnx*(5+2*lny - 2*lnz)+3*lnz+
     -      6*lny*(-5+lny+lnz)+Pi**2)*y**2*z+
     -      (30+3*lny - 30*lnz+6*lnz*(lny+lnz)+
     -      lnx*(15 - 6*lny+6*lnz)+Pi**2)*y*z**2 - 
     -      3*(-4+5*lnx+3*lnz)*z**3))+12*x*y*z*Ifin(z,y,x))
     -      /(6.*x*(x**2+(y - z)**2 - 2*x*(y+z))**2)

      return
      end
      

c TFI[D, 0, {0, 0, 0, 0, 1}, {{3, Sqrt[x]}, {1, Sqrt[y]}, {0, 0}, {0, 0}, {1, Sqrt[z]}}]
      double complex function TFI2(x,y,z)
      
      implicit none
      
      double precision x,y,z
      
      double precision lnx,lny,lnz
      double complex Ifin
      double precision Qs
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      common /scale/ Qs
      
      lnx=log(x/Qs**2)
      lny=log(y/Qs**2)
      lnz=log(z/Qs**2)
      
      TFI2=(Pi**4*(3*(7 - 6*lnx)*x**4+
     -     x**3*((-42+3*lnx**2+3*(-4+lny)*lny+
     -     6*lnx*(8+lny)+Pi**2)*y+(-42+3*lnx**2+
     -     3*(-6+lnz)*lnz+6*lnx*(9+lnz)+Pi**2)*z)+
     -     x**2*(-3*(3*lnx**2+3*(-4+lny)*lny+6*lnx*(2+lny)+
     -     Pi**2)*y**2 - (126+12*lnx**2+9*(-4+lny)*lny - 
     -     66*lnz+6*lny*lnz+9*lnz**2+6*lnx*(-1+2*lny+2*lnz) +
     -     5*Pi**2)*y*z - 3*(3*lnx**2+3*(-6+lnz)*lnz+
     -     6*lnx*(3+lnz)+Pi**2)*z**2)+x*(3*(14 - 12*lny+
     -     3*(lnx+lny)**2+Pi**2)*y**3 - (15*lnx**2+
     -     6*lnx*(-5+4*lny+lnz) - 3*(14+lnz*(-26+3*lnz)+
     -     lny*(4+8*lnz))+Pi**2)*y**2*z+(126 - 3*lnx**2+
     -     6*lnx*(-8+lny - 2*lnz) - 24*lnz+
     -     3*lny*(-12+3*lny+4*lnz)+Pi**2)*y*z**2+3*(14 +
     -     3*lnx**2+3*(-6+lnz)*lnz+6*lnx*(1+lnz)+
     -     Pi**2)*z**3) - (y - z)*((3*(7+lnx**2+2*lnx*(-1+lny) +
     -     (-4+lny)*lny)+Pi**2)*y**3+3*(-3*lnx**2+2*lny**2+
     -     (-7+lnz)*(-3+lnz)+6*lny*(-2+lnz) - 
     -     2*lnx*(-2+lny+2*lnz))*y**2*z+(21+9*lnx**2 - 3*lny**2 - 
     -     6*lny*(-2+lnz)+6*(-4+lnz)*lnz+6*lnx*(-1+3*lnz)+
     -     2*Pi**2)*y*z**2 - (3*(7 - 6*lnz+(lnx+lnz)**2)+
     -     Pi**2)*z**3))+6*((x - y)**3 - (3*x**2+y**2)*z+
     -     3*(x+y)*z**2 - z**3)*Ifin(z,y,x))/
     -     (12.*(x**2+(y - z)**2 - 2*x*(y+z))**2)
      
      return
      end
      

c TFI[D, 0, {1, 0, 0, 0, 0}, {{3, Sqrt[x]}, {1, Sqrt[y]}, {0, 0},{0, 0}, {1, Sqrt[z]}}]
      double complex function TFI3(x,y,z)
      
      implicit none
      
      double precision x,y,z
      
      double precision lnx,lny,lnz
      double complex Ifin
      double precision Qs
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      common /scale/ Qs
      
      lnx=log(x/Qs**2)
      lny=log(y/Qs**2)
      lnz=log(z/Qs**2)
      
      TFI3=(Pi**4*(3*(7 - 6*lnx)*x**4+x**3*((3*(lnx**2+(-7+lny)*
     -     (2+lny)+lnx*(17+2*lny))+Pi**2)*y+(3*(lnx**2+
     -     (-7+lnz)*(2+lnz)+lnx*(17+2*lnz))+Pi**2)*z) - 
     -     (y - z)**2*((3*(7+lnx**2+(-5+lny)*lny+
     -     lnx*(-1+2*lny))+Pi**2)*y**2+3*(-2*lnx**2+lny**2+
     -     4*lny*lnz+lnz**2 - 7*(-2+lny+lnz) - 2*lnx*
     -     (-1+lny+lnz))*y*z+(3*(7+lnx**2+(-5+lnz)*lnz +
     -     lnx*(-1+2*lnz))+Pi**2)*z**2)+x**2*(-3*(3*(lnx**2+
     -     (-5+lny)*lny+lnx*(5+2*lny))+Pi**2)*y**2 - 
     -     (3*(42+4*lnx**2+lny*(-17+3*lny) - 17*lnz+2*lny*lnz+
     -     3*lnz**2+lnx*(-2+4*lny+4*lnz))+5*Pi**2)*y*z - 
     -     3*(3*(lnx**2+(-5+lnz)*lnz+lnx*(5+2*lnz))+Pi**2)*
     -     z**2)+3*x*((14 - 15*lny+3*(lnx+lnx**2+2*lnx*lny+
     -     lny**2)+Pi**2)*y**3 - (-28+2*lny+3*lnx*(1+lnx+
     -     2*lny)+19*lnz - 6*lny*lnz - 3*lnz**2)*y**2*z - (-28+
     -     19*lny - 3*lny**2+2*lnz+3*(lnx+lnx**2+2*lnx*lnz - 
     -     2*lny*lnz))*y*z**2+(14 - 15*lnz+3*(lnx+lnx**2+
     -     2*lnx*lnz+lnz**2)+Pi**2)*z**3))+6*((x - y)**3+
     -     (-3*x**2+y**2)*z+(3*x+y)*z**2 - z**3)*Ifin(z,y,x))/
     -     (6.*(x**2+(y - z)**2 - 2*x*(y+z))**2)
      
      return
      end
      
      
c TFI[D, 0, {1, 0, 0, 0, 1}, {{3, Sqrt[x]}, {1, Sqrt[y]}, {0, 0},{0, 0}, {1, Sqrt[z]}}]
      double complex function TFI4(x,y,z)
      
      implicit none
      
      double precision x,y,z
      
      double precision lnx,lny,lnz
      double complex Ifin
      double precision Qs
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      common /scale/ Qs
      
      lnx=log(x/Qs**2)
      lny=log(y/Qs**2)
      lnz=log(z/Qs**2)
      
      TFI4=(Pi**4*((39 - 30*lnx)*x**5+x**4*(3*(-16+3*lnx**2+
     -     3*(-4+lny)*lny+lnx*(20+6*lny)+Pi**2)*y+
     -     (-96+3*lnx**2+3*(-10+lnz)*lnz+6*lnx*(17+lnz) +
     -     Pi**2)*z)+2*(y - z)**3*((-9+6*lny)*y**2 - (3*(8+
     -     lny**2+2*lny*(-2+lnz)+(-4+lnz)*lnz)+Pi**2)*y*z+
     -     3*(-3+2*lnz)*z**2) - 3*x**3*((-32*lny+9*(lnx+lny)**2 +
     -     3*(8+Pi**2))*y**2+(74+6*lnx**2+11*(-4+lny)*lny +
     -     2*lnx*(1+8*lny - 2*lnz) - 26*lnz+6*lny*lnz+lnz**2+
     -     3*Pi**2)*y*z+(-12+3*lnx**2+6*lnx*(7+lnz)+
     -     lnz*(-34+3*lnz)+Pi**2)*z**2) - x*(y - z)*(3*(5+
     -     lnx*(-10+3*lnx)+6*lnx*lny+3*lny**2+Pi**2)*y**3 - 
     -     3*(21+7*lnx**2+lnz*(-2*(5+lny)+5*lnz)+2*lnx*
     -     (-8+lny+6*lnz)+2*Pi**2)*y**2*z+(3*(-43+5*lnx**2+
     -     lny*(36 - 9*lny - 14*lnz) - 2*lnx*(1+2*lny - 7*lnz)+
     -     4*lnz) - 2*Pi**2)*y*z**2 - (75+3*lnx**2+3*(-22+lnz)*
     -     lnz+6*lnx*(2+lnz)+Pi**2)*z**3)+x**2*(3*(38+
     -     9*lnx**2+3*lny*(-8+3*lny)+2*lnx*(-10+9*lny)+
     -     3*Pi**2)*y**3 - 3*(-22+2*(8 - 3*lny)*lny+14*lnz+
     -     3*(3*lnx**2+6*lnx*(-1+lnz)+lnz*(-4*lny+lnz))+
     -     Pi**2)*y**2*z+(330 - 9*lnx**2+45*(-4+lny)*lny -
     -     36*lnz+48*lny*lnz - 6*lnz**2+6*lnx*(7*lny - 
     -     10*(1+lnz))+5*Pi**2)*y*z**2+3*(26+3*lnx**2+
     -     3*(-14+lnz)*lnz+lnx*(22+6*lnz)+Pi**2)*z**3))+
     -     6*(3*x**4 - 2*(y - z)**3*z - x**3*(9*y+11*z) - 
     -     3*x*(y - z)*(y**2 - 3*z**2)+3*x**2*
     -     (3*y**2+2*y*z+5*z**2))*Ifin(z,y,x))
     -     /(12.*(x**2+(y - z)**2 - 2*x*(y+z))**2)
      
      return
      end
      
      
c TFI[D, 0, {0, 0, 0, 0, 1}, {{1, Sqrt[x]}, {1, Sqrt[y]}, {0, 0}, {0, 0},{1, Sqrt[z]}}]
      double complex function TFI5(x,y,z)
      
      implicit none
      
      double precision x,y,z
      
      double precision lnx,lny,lnz
      double complex Ifin
      double precision Qs
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      common /scale/ Qs
      
      lnx=log(x/Qs**2)
      lny=log(y/Qs**2)
      lnz=log(z/Qs**2)
      
      TFI5=(Pi**4*(-((18+3*lny**2+6*lny*(-2+lnz) - 12*lnz+
     -     3*lnz**2+Pi**2)*y*z)+x*((18+3*lnx**2+6*lnx*
     -     (-2+lny) - 12*lny+3*lny**2+Pi**2)*y - (18+3*lnx**2 +
     -     6*lnx*(-2+lnz) - 12*lnz+3*lnz**2+Pi**2)*z))+
     -     6*(x+y - z)*Ifin(z,y,x))/12.
      
      return
      end
      
      
c TFI[D, 0, {{1, Sqrt[x]}, {1, Sqrt[y]}, {0, 0}, {0, 0}, {2, Sqrt[z]}}]
      double complex function TFI6(x,y,z)
      
      implicit none
      
      double precision x,y,z
      
      double precision lnx,lny,lnz
      double complex Ifin
      double precision Qs
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      common /scale/ Qs
      
      lnx=log(x/Qs**2)
      lny=log(y/Qs**2)
      lnz=log(z/Qs**2)
      
      TFI6=(Pi**4*(-((24+3*lnx**2+6*lnx*(-3+lnz) - 6*lnz+
     -     3*lnz**2+Pi**2)*x**2) - (y - z)*((24+3*lny**2+
     -     6*lny*(-3+lnz) - 6*lnz+3*lnz**2+Pi**2)*y+
     -     6*(3 - 2*lnz)*z)+x*(-3*(12+lnx**2+lny**2+4*lnz - 
     -     2*lnz**2 - 2*lny*(3+lnz)+lnx*(4*lny - 2*(3+lnz)))*y+
     -     (6+3*lnx**2+6*lnx*(-3+lnz)+6*lnz+3*lnz**2+
     -     Pi**2)*z)) - 6*(x+y - z)*Ifin(z,y,x))/
     -     (6.*(x**2+(y - z)**2 - 2*x*(y+z)))
      
      return
      end
      
      
c TFI[D, 0, {0, 0, 0, 0, 1}, {{1, Sqrt[x]}, {1, Sqrt[y]}, {0, 0}, {0, 0}, {2, Sqrt[z]}}]
      double complex function TFI7(x,y,z)
      
      implicit none
      
      double precision x,y,z
      
      double precision lnx,lny,lnz
      double complex Ifin
      double precision Qs
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      common /scale/ Qs
      
      lnx=log(x/Qs**2)
      lny=log(y/Qs**2)
      lnz=log(z/Qs**2)
      
      TFI7=(-(Pi**4*((15+3*lnx**2+6*lnx*(-2+lnz) - 6*lnz+
     -     3*lnz**2+Pi**2)*x**3+(y - z)**2*((15+3*lny**2+
     -     6*lny*(-2+lnz) - 6*lnz+3*lnz**2+Pi**2)*y+
     -     (9 - 6*lnz)*z)+x**2*(3*(9 - 4*lny+lny**2+2*lnx*
     -     (-2+lny - lnz)+2*lnz - lnz**2)*y - (21+6*lnx**2+
     -     12*lnx*(-2+lnz) - 6*lnz+6*lnz**2+2*Pi**2)*z)+
     -     x*(3*(9+lnx**2+2*lnx*(-2+lny)+2*lnz - lnz**2 - 
     -     2*lny*(2+lnz))*y**2 - 3*(12+2*lnx**2+2*lny**2+
     -     2*lny*(-4+lnz)+2*lnz**2+2*lnx*(-4+lny+lnz)+
     -     Pi**2)*y*z+(-3+3*lnx**2+6*lnx*(-2+lnz)+6*lnz+
     -     3*lnz**2+Pi**2)*z**2))) - 6*(x**2+(y - z)**2 - 2*x*z)*
     -     Ifin(z,y,x))/(6.*(x**2+(y - z)**2 - 2*x*(y+z)))
      
      return
      end
      
      
c TFI[D, 0, {{2, Sqrt[x]}, {2, Sqrt[y]}, {0, 0}, {0, 0}, {1, Sqrt[z]}}]
      double complex function TFI8(x,y,z)
      
      implicit none
      
      double precision x,y,z
      
      double precision lnx,lny,lnz
      double complex Ifin
      double precision Qs
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      common /scale/ Qs
      
      lnx=log(x/Qs**2)
      lny=log(y/Qs**2)
      lnz=log(z/Qs**2)
      
      TFI8=(Pi**4*(6*(lnx - lny)*(x - y)**3+((6*(7+lnx**2+
     -     3*lny - (2+lny)*lnz+lnx*(-7+lny+lnz))+Pi**2)*
     -     x**2+2*(3*(14+lnx**2+2*lnx*(-4+lny)+
     -     (-8+lny)*lny+4*lnz)+Pi**2)*x*y+(6*(7+lnx*
     -     (3+lny - lnz) - 2*lnz+lny*(-7+lny+lnz))+Pi**2)*
     -     y**2)*z - 6*((lnx**2+3*lny+lnx*(-5+2*lny)+2*lnz - 
     -     lnz*(2*lny+lnz))*x+((-5+lny)*lny+lnx*(3+2*lny - 
     -     2*lnz) - (-2+lnz)*lnz)*y)*z**2 - (-6*(-7+lnx+lny+
     -     lnx*lny)+6*(-4+lnx+lny)*lnz+6*lnz**2+Pi**2)*
     -     z**3)+12*(x+y - z)*z*Ifin(z,y,x))/
     -     (6.*(x**2+(y - z)**2 - 2*x*(y+z))**2)
      
      return
      end
      
      
c TFI[D, 0, {1, 0, 0, 0, 0}, {{2, Sqrt[x]}, {2, Sqrt[y]}, {0, 0}, {0, 0}, {1, Sqrt[z]}}]
      double complex function TFI9(x,y,z)
      
      implicit none
      
      double precision x,y,z
      
      double precision lnx,lny,lnz
      double complex Ifin
      double precision Qs
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      common /scale/ Qs
      
      lnx=log(x/Qs**2)
      lny=log(y/Qs**2)
      lnz=log(z/Qs**2)
      
      TFI9=(-(Pi**4*((3*(8+lnx**2+2*lnx*(-4+lny)+lny**2)+
     -     Pi**2)*x**4+(y - z)**3*(6*(-3+2*lny)*y - 
     -     (24+3*lny**2+6*lny*(-1+lnz)+3*(-6+lnz)*lnz+
     -     Pi**2)*z) - 3*x**3*((18+3*lnx**2+6*lnx*(-4+lny) +
     -     lny*(4+3*lny)+Pi**2)*y+(18+3*lnx**2+2*lny*
     -     (-1+2*lny)+lnx*(-20+8*lny - 2*lnz) - (-2+lnz)*lnz+
     -     Pi**2)*z) - x*(y - z)*((-30+3*lnx**2+6*lnx*(-4+lny)+
     -     3*lny*(12+lny)+Pi**2)*y**2 - (84+6*lnx**2 - 
     -     3*(-6+lny)*lny - 6*lnx*(8+lny - 3*lnz) - 42*lnz+
     -     9*lnz**2+2*Pi**2)*y*z - (-3*(10+lnx**2 - 4*lny**2+
     -     lny*(6 - 4*lnz)+(-2+lnz)*lnz+lnx*(-8 - 4*lny+ 
     -     6*lnz)) +Pi**2)*z**2)+x**2*(3*(6+3*lnx**2+6*lnx*
     -     (-4+lny)+3*lny*(4+lny)+Pi**2)*y**2 - 3*(66+
     -     4*(-7+lnx)*lnx - lny*(14+lny) - 10*lnz+8*lnx*lnz - 
     -     2*lny*lnz+3*lnz**2+Pi**2)*y*z+(3*(-8+lnx**2+
     -     6*(-1+lny)*lny+2*lnx*(-2+5*lny - 4*lnz)+
     -     10*lnz+2*lny*lnz - 3*lnz**2)+2*Pi**2)*z**2))) - 
     -     6*(x**3 - (y - z)**3 - 3*x**2*(y+z)+x*(3*y**2 - 4*y*z+
     -     z**2))*Ifin(z,y,x))/
     -     (6.*(x**2+(y - z)**2 - 2*x*(y+z))**2)
      
      return
      end
      
      
c  TFI[D, 0, {0, 0, 0, 0, 1}, {{2, Sqrt[x]}, {2, Sqrt[y]}, {0, 0}, {0, 0}, {1, Sqrt[z]}}]
      double complex function TFI10(x,y,z)
      
      implicit none
      
      double precision x,y,z
      
      double precision lnx,lny,lnz
      double complex Ifin
      double precision Qs
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      common /scale/ Qs
      
      lnx=log(x/Qs**2)
      lny=log(y/Qs**2)
      lnz=log(z/Qs**2)
      
      TFI10=(Pi**4*((-3+6*lnx)*x**4+3*(y - z)**3*((-1+2*lny)*y+
     -      z - 2*lnz*z) - 6*x**3*((-2+3*lnx+lny)*y+(-2+3*lnx +
     -      lnz)*z)+2*x**2*(9*(-1+lnx+lny)*y**2 +(3*(12+lny+
     -      2*lnx*(-5+lnx+lny)+lnz+2*(lnx - lny)*lnz)+Pi**2)*
     -      y*z+9*(-1+lnx+lnz)*z**2)+2*x*(-3*(-2+lnx+
     -      3*lny)*y**3+(36+lnx*(3+6*lny - 6*lnz)+3*lnz+
     -      6*lny*(-5+lny+lnz)+Pi**2)*y**2*z+(36+3*lny - 
     -      30*lnz+6*lnz*(lny+lnz)+lnx*(3 - 6*lny+6*lnz)+
     -      Pi**2)*y*z**2 - 3*(-2+lnx+3*lnz)*z**3))+ 24*x*y*z*
     -      Ifin(z,y,x))/(6.*(x**2+(y - z)**2 - 2*x*(y+z))**2)
      
      return
      end
      
      
c TFI[D, 0, {1, 0, 0, 0, 0}, {{2, Sqrt[x]}, {1, Sqrt[y]}, {0, 0}, {0, 0}, {1, Sqrt[z]}}]
      double complex function TFI11(x,y,z)
      
      implicit none
      
      double precision x,y,z
      
      double precision lnx,lny,lnz
      double complex Ifin
      double precision Qs
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      common /scale/ Qs
      
      lnx=log(x/Qs**2)
      lny=log(y/Qs**2)
      lnz=log(z/Qs**2)
      
      TFI11=(Pi**4*x*((x - y)*(6*(3 - 2*lnx)*x+(3*(8 - 2*lnx+
     -      lnx**2+2*(-3+lnx)*lny+lny**2)+Pi**2)*y)+
     -      ((6+3*lnx**2+3*(-6+lnz)*lnz+6*lnx*(1+lnz)+
     -      Pi**2)*x+3*(2*lnx**2 - lny**2 - 4*lny*lnz - lnz**2+
     -      6*(-2+lny+lnz)+2*lnx*(-2+lny+lnz))*y)*z - 
     -      (24+3*lnx**2+6*lnx*(-1+lnz)+3*(-6+lnz)*lnz+
     -      Pi**2)*z**2)+6*(2*x**2+(y - z)**2 - 3*x*(y+z))*
     -      Ifin(z,y,x))/(6.*(x**2+(y - z)**2 - 2*x*(y+z)))
      
      return
      end
      
      
c TFI[D, 0, {0, 0, 0, 0, 1}, {{2, Sqrt[y]}, {1, Sqrt[x]}, {0, 0}, {0, 0}, {1, Sqrt[z]}}]
      double complex function TFI12(x,y,z)
      
      implicit none
      
      double precision x,y,z
      
      double precision lnx,lny,lnz
      double complex Ifin
      double precision Qs
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      common /scale/ Qs
      
      lnx=log(x/Qs**2)
      lny=log(y/Qs**2)
      lnz=log(z/Qs**2)
      
      TFI12=(Pi**4*((9 - 6*lnx)*x**3+x**2*((3*(5+lnx**2+2*lnx*
     -      (-1+lny)+(-4+lny)*lny)+Pi**2)*y+3*(-3+4*lnx - 
     -      2*lnz)*z)+x*(-((3*(5+lnx**2+2*lnx*(-2+lny)+
     -      (-2+lny)*lny)+Pi**2)*y**2)+3*(lnx**2 - 2*(-3+lny)*
     -      (-1+lny) - 2*lnx*(1+lny - 2*lnz) - 2*(1+lny)*lnz+
     -      lnz**2)*y*z - 3*(3+2*lnx - 4*lnz)*z**2)+(y - z)*((-9+
     -      6*lny)*y**2 - (3*(8+lny**2+2*lny*(-2+lnz)+
     -      (-4+lnz)*lnz)+Pi**2)*y*z+3*(-3+2*lnz)*z**2))+
     -      6*(x**2+z*(-y+z) - x*(y+2*z))*Ifin(z,y,x))/
     -      (6.*(x**2+(y - z)**2 - 2*x*(y+z)))
      
      return
      end
      
      
c TFI[D, 0, {0, 1, 0, 0, 0}, {{3, Sqrt[x]}, {1, Sqrt[y]}, {0, 0}, {0, 0}, {1, Sqrt[z]}}]
      double complex function TFI13(x,y,z)
      
      implicit none
      
      double precision x,y,z
      
      double precision lnx,lny,lnz
      double complex Ifin
      double precision Qs
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      common /scale/ Qs
      
      lnx=log(x/Qs**2)
      lny=log(y/Qs**2)
      lnz=log(z/Qs**2)
      
      TFI13=(-(Pi**4*(3*x**4*((-1+2*lnx)*y+(-1+lnx+lnz)*z)+
     -      3*(y - z)**3*((-1+lnx+lny)*y**2 - (-1+lnx+lnz)*
     -      z**2) - 3*x**3*((-4+7*lnx+lny)*y**2+(-8+11*lnx+
     -      5*lnz)*y*z+4*(-1+lnx+lnz)*z**2)+x**2*(9*(-2+
     -      3*lnx+lny)*y**3+(3*(4+lny+2*lnx*(lnx+lny)+
     -      7*lnz+2*(lnx - lny)*lnz)+Pi**2)*y**2*z+3*(-10+
     -      13*lnx+7*lnz)*y*z**2+18*(-1+lnx+lnz)*z**3)+x*
     -      (-3*(-4+5*lnx+3*lny)*y**4+(42+lnx*(3+6*lny - 
     -      6*lnz) - 9*lnz+6*lny*(-5+lny+lnz)+Pi**2)*y**3*z+
     -      (3*(6+9*lnx+lny - 2*lnx*lny+2*(-3+lnx+lny)*lnz +
     -      2*lnz**2)+Pi**2)*y**2*z**2+3*(-lnx+lnz)*y*z**3 - 12*
     -      (-1+lnx+lnz)*z**4))) - 12*x*y**2*z*
     -      Ifin(z,y,x))/(6.*x*(x**2+(y - z)**2 - 2*x*(y+z))**2)
      
      return
      end
      
      
      
c TFI[D, 0, {0, 1, 0, 0, 0}, {{1, Sqrt[x]}, {1, Sqrt[y]}, {0, 0}, {0, 0}, {1, Sqrt[z]}}]
      double complex function TFI14(x,y,z)
      
      implicit none
      
      double precision x,y,z
      
      double precision lnx,lny,lnz
      double complex Ifin
      double precision Qs
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      common /scale/ Qs
      
      lnx=log(x/Qs**2)
      lny=log(y/Qs**2)
      lnz=log(z/Qs**2)
      
      TFI14=-(Pi**4*(3*(6+lnx**2+2*lnx*(-2+lnz)+
     -       (-4+lnz)*lnz)+Pi**2)*x*z)/6.+y*Ifin(z,y,x)
      
      return
      end
      
      
      
c TFI[D, 0, {0, 1, 0, 0, 0}, {{2, Sqrt[x]}, {1, Sqrt[y]}, {0, 0}, {0, 0}, {1, Sqrt[z]}}] 
      double complex function TFI15(x,y,z)
      
      implicit none
      
      double precision x,y,z
      
      double precision lnx,lny,lnz
      double complex Ifin
      double precision Qs
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      common /scale/ Qs
      
      lnx=log(x/Qs**2)
      lny=log(y/Qs**2)
      lnz=log(z/Qs**2)
      
      TFI15=(Pi**4*((x - y)*y*(6*(3 - 2*lnx)*x+(24+3*lnx**2+
     -      6*lnx*(-1+lny)+3*(-6+lny)*lny+Pi**2)*y) - 
     -      ((6+3*lnx**2+6*lnx*(-1+lnz)+3*(-2+lnz)*lnz+
     -      Pi**2)*x**2 - 3*(6+3*lnx**2+lnz*(-10+3*lnz)+lnx*
     -      (-2+6*lnz)+Pi**2)*x*y+(42 - 3*lnx**2 - 6*lnx*
     -      (-1+lny)+3*lny**2+6*(-4+lnz)*lnz+6*lny*
     -      (-3+2*lnz)+Pi**2)*y**2)*z+(2*(6+3*lnx**2+6*lnx*
     -      (-1+lnz)+3*(-2+lnz)*lnz+Pi**2)*x+(3*(-4+
     -      lnx**2+2*lnx*(-1+lnz)+lnz*(2+lnz))+Pi**2)*y)*
     -      z**2 - (6+3*lnx**2+6*lnx*(-1+lnz)+3*(-2+lnz)*
     -      lnz+Pi**2)*z**3) - 6*y*(-x+y+z)*Ifin(z,y,x))/
     -      (6.*(x**2+(y - z)**2 - 2*x*(y+z)))
      
      return
      end
      
      
      
c TFI[D, 0, {1, 0, 0, 0,1}, {{1, Sqrt[x]}, {1, Sqrt[y]}, {0, 0}, {0, 0}, {1, Sqrt[z]}}]
      double complex function TFI16(x,y,z)
      
      implicit none
      
      double precision x,y,z
      
      double precision lnx,lny,lnz
      double complex Ifin
      double precision Qs
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      common /scale/ Qs
      
      lnx=log(x/Qs**2)
      lny=log(y/Qs**2)
      lnz=log(z/Qs**2)
      
      TFI16=(Pi**4*(-((3*(6+lny**2+2*lny*(-2+lnz)+
     -      (-4+lnz)*lnz)+Pi**2)*x*y*z) - 2*(3*(6+lny**2 +
     -      2*lny*(-2+lnz)+(-4+lnz)*lnz)+Pi**2)*y**2*z+
     -      x**2*((3*(6+lnx**2+2*lnx*(-2+lny)+(-4+lny)*lny) +
     -      Pi**2)*y - 18*z - (3*(-4+lnx+lnz)*(lnx+lnz)+
     -      Pi**2)*z))+6*x*(x+y - z)*Ifin(z,y,x))/12.
      
      return
      end
      
      
      double complex function TFI17(x,y,z)
      
      implicit none
      
      double precision x,y,z
      
      double precision lnx,lny,lnz
      double complex Ifin
      double precision Idiv1
      double precision Qs
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      common /scale/ Qs
      
      lnx=log(x/Qs**2)
      lny=log(y/Qs**2)
      lnz=log(z/Qs**2)
      
      TFI17=(Pi**4*(-((3*(6+lny**2+2*lny*(-2+lnz)+
     -      (-4+lnz)*lnz)+Pi**2)*y*(y - z)**2*z)+x**3*
     -      (3*(10+3*lnx**2+lny*(-8+3*lny)+lnx*(-8+6*lny)+
     -      Pi**2)*y - 18*z - (3*(-4+lnx+lnz)*(lnx+lnz)+
     -      Pi**2)*z)+x**2*(-4*(3*(4+lnx**2+(-3+lny)*lny+
     -      lnx*(-3+2*lny))+Pi**2)*y**2 - (3*(6 - 2*lnx**2+lny*
     -      (-20+7*lny)+4*lnx*(1+2*lny - 3*lnz)+8*lnz+
     -      6*lny*lnz - 3*lnz**2)+Pi**2)*y*z+2*(3*(6+lnx**2+
     -      2*lnx*(-2+lnz)+(-4+lnz)*lnz)+Pi**2)*z**2)+
     -      x*((3*(6+lnx**2+2*lnx*(-2+lny)+(-4+lny)*lny) +
     -      Pi**2)*y**3 - 3*(10+3*lnx**2+2*(-2+lny)*lny+lnz**2+ 
     -      2*lnx*(-6+2*lny+lnz)+Pi**2)*y**2*z +(9*lnx**2+6*lnx*
     -      (-6+lny+2*lnz) +3*(34+5*lny**2+8*lny*(-2+lnz) - 
     -      20*lnz+6*lnz**2)+7*Pi**2)*y*z**2 - (3*(6+lnx**2+
     -      2*lnx*(-2+lnz)+(-4+lnz)*lnz)+Pi**2)*z**3)) - 
     -      12*x*(x - y - z)*(x+y - z)*Idiv1(z,y,x)+6*(3*x**3+
     -      (y - z)**3 - x*(y - z)*(y+5*z) - x**2*(3*y +7*z))*
     -      Ifin(z,y,x))/(12.*(x**2+(y - z)**2 - 2*x*(y+z)))
      
      return
      end
      
      double complex function TFI18(x,y,z)
      
      implicit none
      
      double precision x,y,z
      
      double precision lnx,lny,lnz
      double complex Ifin
      double precision Idiv1
      double precision Qs
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      common /scale/ Qs
      
      lnx=log(x/Qs**2)
      lny=log(y/Qs**2)
      lnz=log(z/Qs**2)
      
      TFI18=-6*Pi**4*x*y+3*lnx**2*Pi**4*x*y+6*lnx*lny*Pi**4*x*y+
     -      3*lny**2*Pi**4*x*y+Pi**6*x*y+6*Pi**4*y*z+12*lnx*Pi**4*y*z-
     -      6*lnx**2*Pi**4*y*z-12*lnx*lny*Pi**4*y*z - 
     -      3*lny**2*Pi**4*y*z - 12*lnz*Pi**4*y*z+6*lny*lnz*Pi**4*y*z+
     -      3*lnz**2*Pi**4*y*z - Pi**6*y*z - 12*(x-z)*Idiv1(x,y,0d0)+
     -      6*(x+y - 2*z)*Ifin(x,y,0d0) - 6*y*Ifin(z,y,0d0)+
     -      6*z*Ifin(z,y,0d0)
      
      return
      end
     
     
      
      
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
ccc Functions needed for the evaluation of dv at order alpha_s alpha_t                   c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

c 
c TFIdv1(m1,m2,m3)=TFI[D, 0, {0, 0, 0, 0, 1}, {{2, m1},{1, m2}, {0, 0}, {0, 0}, {1, m3}}]
c
      double complex function TFIdv1fin(m1,m2,m3)
      
      implicit none

      double precision m1,m2,m3
      
      double complex Ifin
      double precision A0,Ae,Idiv1
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      TFIdv1fin=(2*(m1**2 - m2**2 - m3**2)*Pi**4*
     -     (A0(m1**2)*(-m2**2+A0(m2**2))+m2**2*Ae(m1**2)+
     -     m1**2*(-A0(m2**2)+Ae(m2**2)))+2*(m1**2+m2**2 - m3**2)*Pi**4*
     -     (A0(m2**2)*(m3**2 - A0(m3**2)) - m3**2*Ae(m2**2)+m2**2*
     -     (A0(m3**2) - Ae(m3**2)))+4*m2**2*Pi**4*(A0(m1**2)*
     -     (-m3**2+A0(m3**2))+m3**2*Ae(m1**2)+m1**2*(-A0(m3**2)+
     -     Ae(m3**2)))-2*((-m2**4+(m1**2 - m3**2)**2)*
     -     Idiv1(m3**2,m2**2,m1**2)+(-m1**4+(m2 - m3)*m3**2*
     -     (m2+m3)+m1**2*(m2**2+2*m3**2))*Ifin(m3**2,m2**2,m1**2)))/
     -     (2.*(m1 - m2 - m3)*(m1+m2 - m3)*(m1 - m2+m3)*(m1+m2+m3))

      if(abs(m1 - m2 - m3).le.1d-12) write(*,*) '(m1 - m2 - m3)<0!!'
      if(abs(m1+m2 - m3).le.1d-12) write(*,*) '(m1+m2 - m3)<0!!'
      if(abs(m1+m2+m3).le.1d-12) write(*,*) '(m1+m2+m3)<0!!'
      if(abs(m1 - m2+m3).le.1d-12) write(*,*) '(m1 - m2+m3)<0!!'
      
      return
      end
      
      
      double complex function TFIdv1div1(m1,m2,m3)
      
      implicit none
     
      double precision m1,m2,m3
      
      double precision A0,Idiv1,Idiv2
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      TFIdv1div1=((m1**2 - m2**2 - m3**2)*Pi**4*
     -     (2*m2**2*A0(m1**2)+2*m1**2*(-m2**2+A0(m2**2)))+
     -     2*(m1**2+m2**2 - m3**2)*Pi**4*(-(m3**2*A0(m2**2))+m2**2*
     -     (m3**2 - A0(m3**2)))+4*m2**2*Pi**4*(m3**2*A0(m1**2)+m1**2*
     -     (-m3**2+A0(m3**2)))-2*((-m1**4+(m2 - m3)*m3**2*(m2+m3)+
     -     m1**2*(m2**2+2*m3**2))*Idiv1(m3**2,m2**2,m1**2)+
     -     (-m2**4+(m1**2 - m3**2)**2)*Idiv2(m3**2,m2**2,m1**2)))/
     -     (2.*(m1 - m2 - m3)*(m1+m2 - m3)*(m1 - m2+m3)*(m1+m2+m3))
      
      return
      end
      
      
      double complex function TFIdv1div2(m1,m2,m3)
      
      implicit none
     
      double precision m1,m2,m3
      
      double precision Idiv2
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      TFIdv1div2=(m2**2*(m1**4 - m1**2*m2**2 - m2**2*m3**2+m3**4)*
     -     Pi**4+(m1**4 - m2**2*m3**2+m3**4 - m1**2*(m2**2+2*m3**2))*
     -     Idiv2(m3**2,m2**2,m1**2))/((m1 - m2 - m3)*(m1+m2 - m3)*
     -     (m1 - m2+m3)*(m1+m2+m3))

      
      return
      end
      
      
      
      
c
c TFIdv2=TFI[D, 0, {1, 0, 0, 0, 0}, {{2, m1}, {1, m2}, {0, 0},{0, 0}, {1, m3}}]
c
      double complex function TFIdv2fin(m1,m2,m3)
      
      implicit none
      
      double precision m1,m2,m3
      
      double complex Ifin
      
      double precision A0,Ae,Idiv1
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      TFIdv2fin=((m1**2 - m2**2+m3**2)*Pi**4*(A0(m1**2)*(-m2**2+
     -     A0(m2**2))+m2**2*Ae(m1**2)+m1**2*(-A0(m2**2)+Ae(m2**2)))+
     -     2*m1**2*Pi**4*(A0(m2**2)*(m3**2 - A0(m3**2)) - m3**2*
     -     Ae(m2**2)+m2**2*(A0(m3**2) - Ae(m3**2)))+(m1**2+m2**2 - 
     -     m3**2)*Pi**4*(A0(m1**2)*(-m3**2+A0(m3**2))+m3**2*Ae(m1**2)+
     -     m1**2*(-A0(m3**2)+Ae(m3**2)))+2*m1**2*(-m1**2+m2**2+m3**2)*
     -     Idiv1(m3**2,m2**2,m1**2)+(2*m1**4+(m2**2 - m3**2)**2 - 
     -     3*m1**2*(m2**2+m3**2))*Ifin(m3**2,m2**2,m1**2))/
     -     ((m1 - m2 - m3)*(m1+m2 - m3)*(m1 - m2+m3)*(m1+m2+m3))
     
      if(abs(m1 - m2 - m3).le.1d-12) write(*,*) '(m1 - m2 - m3)<0!!'
      if(abs(m1+m2 - m3).le.1d-12) write(*,*) '(m1+m2 - m3)<0!!'
      if(abs(m1+m2+m3).le.1d-12) write(*,*) '(m1+m2+m3)<0!!'
      if(abs(m1 - m2+m3).le.1d-12) write(*,*) '(m1 - m2+m3)<0!!'
      
      return
      end
      
      
      double complex function TFIdv2div1(m1,m2,m3)
      
      implicit none
     
      double precision m1,m2,m3
      
      double precision Idiv1,Idiv2,A0
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      TFIdv2div1=(2*(-(m2**2 - m3**2)**2+m1**2*(m2**2+m3**2))*Pi**4*
     -     A0(m1**2)-2*m1**2*Pi**4*(-m2**4-m3**4+m1**2*(m2**2+m3**2)- 
     -     (m1**2-m2**2-m3**2)*(A0(m2**2)+A0(m3**2)))+2*(2*m1**4+
     -     (m2**2-m3**2)**2-3*m1**2*(m2**2+m3**2))*
     -     Idiv1(m3**2,m2**2,m1**2)+4*m1**2*(-m1**2+m2**2+m3**2)*
     -     Idiv2(m3**2,m2**2,m1**2))/(2.*(m1 - m2 - m3)*(m1+m2 - m3)*
     -     (m1 - m2+m3)*(m1+m2+m3))
     
      return
      end
      
      
      
      double complex function TFIdv2div2(m1,m2,m3)
      
      implicit none
     
      double precision m1,m2,m3
      
      double precision Idiv2
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      TFIdv2div2=(m1**2*(-m2**4 - m3**4+m1**2*(m2**2+m3**2))*Pi**4+
     -     (2*m1**4+(m2**2 - m3**2)**2 - 3*m1**2*(m2**2+m3**2))*
     -     Idiv2(m3**2,m2**2,m1**2))/
     -     ((m1 - m2 - m3)*(m1+m2 - m3)*(m1 - m2+m3)*(m1+m2+m3))
 
      return
      end
      
      
c
c TFIdv3=TFI[D, 0, {0, 0, 0, 0, 1},{{2, m1}, {2, m2}, {0, 0}, {0, 0}, {1, m3}}]
c      
      double complex function TFIdv3fin(m1,m2,m3)
      
      implicit none
      

      
      double precision m1,m2,m3
      
      double complex Ifin
      
      double precision Idiv1,Idiv2,Ae,A0
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      TFIdv3fin=-((2*Pi**4*(m1**2*m2**2*((m1**2 - m2**2)**2 - m3**4)+
     -     m1**2*(-(m1**2 - m2**2)**2+(m1**2+m2**2)*m3**2)*A0(m2**2)+
     -     A0(m1**2)*(m2**2*(-(m1**2 - m2**2)**2+(m1**2+m2**2)*m3**2)- 
     -     m3**2*(m1**2+m2**2 - m3**2)*A0(m2**2)) -m3**2*(m1**2+m2**2- 
     -     m3**2)*(m2**2*Ae(m1**2)+m1**2*Ae(m2**2)))+2*Pi**4*(m1**2*
     -     m3**2*(-m2**4+(m1**2 - m3**2)**2)+m1**2*(-m1**4+(m2 - m3)*
     -     m3**2*(m2+m3)+m1**2*(m2**2+2*m3**2))*A0(m3**2)+A0(m1**2)*
     -     (m3**2*(-m1**4+(m2-m3)*m3**2*(m2+m3)+m1**2*(m2**2+2*m3**2))- 
     -     m2**2*(m1**2 - m2**2+m3**2)*A0(m3**2))-m2**2*(m1**2 - m2**2+
     -     m3**2)*(m3**2*Ae(m1**2)+m1**2*Ae(m3**2)))-2*Pi**4*(-(m2**2*
     -     m3**2*(-m1**4+(m2**2-m3**2)**2))+m2**2*((m2**2-m3**2)**2- 
     -     m1**2*(m2**2+m3**2))*A0(m3**2)+A0(m2**2)*(m3**2*((m2**2 - 
     -     m3**2)**2 - m1**2*(m2**2+m3**2))+m1**2*(-m1**2+m2**2+m3**2)*
     -     A0(m3**2)) -m1**2*(m1**2 - m2**2 - m3**2)*(m3**2*Ae(m2**2)+
     -     m2**2*Ae(m3**2))) -(m1 - m2 - m3)*(m1+m2 - m3)*(m1 - m2+m3)*
     -     (m1+m2+m3)*(m1**2+m2**2+m3**2)*Idiv1(m3**2,m2**2,m1**2)+
     -     2*(m1**2 - m2**2 - m3**2)*(m1**2+m2**2 - m3**2)*(m1**2 - 
     -     m2**2+m3**2)*Idiv2(m3**2,m2**2,m1**2) - 4*m1**2*m2**2*m3**2*
     -     Ifin(m3**2,m2**2,m1**2))/((m1+m2 - m3)**2*(m1 - m2+m3)**2*
     -     (-m1+m2+m3)**2*(m1+m2+m3)**2))

     
      if(abs(m1 - m2 - m3).le.1d-12) write(*,*) '(m1 - m2 - m3)<0!!'
      if(abs(m1+m2 - m3).le.1d-12) write(*,*) '(m1+m2 - m3)<0!!'
      if(abs(m1+m2+m3).le.1d-12) write(*,*) '(m1+m2+m3)<0!!'
      if(abs(m1 - m2+m3).le.1d-12) write(*,*) '(m1 - m2+m3)<0!!'
      
      return
      end
      
      
      
      double complex function TFIdv3div1(m1,m2,m3)
      
      implicit none
     
      double precision m1,m2,m3
      
      double precision Idiv1,Idiv2,A0
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      TFIdv3div1= (2*(m2**2*m3**2*(m2**2 - m3**2)**2+m1**6*(m2**2+
     -     m3**2)-2*m1**4*(m2**4+m2**2*m3**2+m3**4)+m1**2*(m2**6 - 
     -     2*m2**4*m3**2 - 2*m2**2*m3**4+m3**6))*Pi**4+4*m1**2*m2**2*
     -     m3**2*(Pi**4*(A0(m1**2)+A0(m2**2)+A0(m3**2))+
     -     Idiv1(m3**2,m2**2,m1**2))+(m1 - m2 - m3)*(m1+m2 - m3)*(m1 - 
     -     m2+m3)*(m1+m2+m3)*(m1**2+m2**2+m3**2)*
     -     Idiv2(m3**2,m2**2,m1**2))/((m1+m2 - m3)**2*(m1 - m2+m3)**2*
     -     (-m1+m2+m3)**2*(m1+m2+m3)**2)
      return
      end
      
      
      
      double complex function TFIdv3div2(m1,m2,m3)
      
      implicit none
     
      double precision m1,m2,m3
      
      double precision Idiv2
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      TFIdv3div2=(2*m1**2*m2**2*m3**2*((m1**2+m2**2+m3**2)*Pi**4+
     -     2*Idiv2(m3**2,m2**2,m1**2)))/((m1+m2 - m3)**2*
     -     (m1 - m2+m3)**2*(-m1+m2+m3)**2*(m1+m2+m3)**2)
     
      return
      end
      
      
      
c      
c myTFI[D, 0, {1, 0, 0, 0,0}, {{3, m1_}, {1, m2_}, {0, 0}, {0, 0}, {1, m3_}}] := 
c TFIdv4fin[m1, m2, m3]+TFIdv4div1[m1, m2, m3]*pol+TFIdv4div2[m1, m2, m3]*pol^2     
c      
      double complex function TFIdv4fin(m1,m2,m3)
      
      implicit none
      

      
      double precision m1,m2,m3
      
      double complex Ifin
      
      double precision Idiv1,Idiv2,Ae,A0
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      TFIdv4fin=(m1**2*(-(m2**2 - m3**2)**4+3*m1**6*(m2**2+m3**2)+
     -     m1**2*(m2**2+m3**2)*(5*m2**4 - 6*m2**2*m3**2+5*m3**4) - 
     -     m1**4*(7*m2**4+6*m2**2*m3**2+7*m3**4))*Pi**4 - 
     -     5*m1**8*Pi**4*A0(m3**2)+11*m1**6*m2**2*Pi**4*A0(m3**2) - 
     -     7*m1**4*m2**4*Pi**4*A0(m3**2)+m1**2*m2**6*Pi**4*A0(m3**2)+
     -     13*m1**6*m3**2*Pi**4*A0(m3**2) - 2*m1**4*m2**2*m3**2*Pi**4*
     -     A0(m3**2)+m1**2*m2**4*m3**2*Pi**4*A0(m3**2)-11*m1**4*m3**4*
     -     Pi**4*A0(m3**2)-5*m1**2*m2**2*m3**4*Pi**4*A0(m3**2)+3*m1**2*
     -     m3**6*Pi**4*A0(m3**2)+m1**2*Pi**4*A0(m2**2)*(-((5*m1**2 - 
     -     3*m2**2)*(m1**2 - m2**2)**2)+(11*m1**4 - 2*m1**2*m2**2 - 5*
     -     m2**4)*m3**2+(-7*m1**2+m2**2)*m3**4+m3**6 - 2*(m1**4+2*
     -     (m2**2-m3**2)**2-3*m1**2*(m2**2+m3**2))*A0(m3**2))+Pi**4*
     -     A0(m1**2)*(3*(m2**2 - m3**2)**4 -5*m1**6*(m2**2+m3**2) - 
     -     11*m1**2*(m2**2-m3**2)**2*(m2**2+m3**2)+m1**4*(13*m2**4+10*
     -     m2**2*m3**2+13*m3**4)+2*((m1**2 - m2**2)**3+(-2*m1**4 - 
     -     3*m1**2*m2**2+3*m2**4)*m3**2-3*m2**2*m3**4+m3**6)*A0(m2**2)+
     -     2*(m1**6+(m2**2 - m3**2)**3+3*m1**2*m3**2*(-m2**2+m3**2) - 
     -     m1**4*(2*m2**2+3*m3**2))*A0(m3**2))+2*m1**6*m2**2*Pi**4*
     -     Ae(m1**2)-6*m1**4*m2**4*Pi**4*Ae(m1**2)+6*m1**2*m2**6*Pi**4*
     -     Ae(m1**2)-2*m2**8*Pi**4*Ae(m1**2)+2*m1**6*m3**2*Pi**4*
     -     Ae(m1**2)-8*m1**4*m2**2*m3**2*Pi**4*Ae(m1**2)-6*m1**2*m2**4*
     -     m3**2*Pi**4*Ae(m1**2)+8*m2**6*m3**2*Pi**4*Ae(m1**2) - 
     -     6*m1**4*m3**4*Pi**4*Ae(m1**2)-6*m1**2*m2**2*m3**4*Pi**4*
     -     Ae(m1**2)-12*m2**4*m3**4*Pi**4*Ae(m1**2)+6*m1**2*m3**6*
     -     Pi**4*Ae(m1**2)+8*m2**2*m3**6*Pi**4*Ae(m1**2)-2*m3**8*Pi**4*
     -     Ae(m1**2)+2*m1**8*Pi**4*Ae(m2**2)-6*m1**6*m2**2*Pi**4*
     -     Ae(m2**2)+6*m1**4*m2**4*Pi**4*Ae(m2**2)-2*m1**2*m2**6*Pi**4*
     -     Ae(m2**2) - 6*m1**6*m3**2*Pi**4*Ae(m2**2)+2*m1**2*m2**4*
     -     m3**2*Pi**4*Ae(m2**2)+6*m1**4*m3**4*Pi**4*Ae(m2**2)+
     -     2*m1**2*m2**2*m3**4*Pi**4*Ae(m2**2) - 2*m1**2*m3**6*Pi**4*
     -     Ae(m2**2)+2*m1**8*Pi**4*Ae(m3**2) - 6*m1**6*m2**2*Pi**4*
     -     Ae(m3**2)+6*m1**4*m2**4*Pi**4*Ae(m3**2)-2*m1**2*m2**6*Pi**4*
     -     Ae(m3**2)-6*m1**6*m3**2*Pi**4*Ae(m3**2)+2*m1**2*m2**4*m3**2*
     -     Pi**4*Ae(m3**2)+6*m1**4*m3**4*Pi**4*Ae(m3**2)+2*m1**2*m2**2*
     -     m3**4*Pi**4*Ae(m3**2)-2*m1**2*m3**6*Pi**4*Ae(m3**2)-6*m1**8*
     -     Idiv1(m3**2,m2**2,m1**2)+16*m1**6*m2**2*
     -     Idiv1(m3**2,m2**2,m1**2)-14*m1**4*m2**4*
     -     Idiv1(m3**2,m2**2,m1**2)+4*m1**2*m2**6*
     -     Idiv1(m3**2,m2**2,m1**2)+16*m1**6*m3**2*
     -     Idiv1(m3**2,m2**2,m1**2)-4*m1**4*m2**2*m3**2*
     -     Idiv1(m3**2,m2**2,m1**2)-4*m1**2*m2**4*m3**2*
     -     Idiv1(m3**2,m2**2,m1**2)-14*m1**4*m3**4*
     -     Idiv1(m3**2,m2**2,m1**2)-4*m1**2*m2**2*m3**4*
     -     Idiv1(m3**2,m2**2,m1**2)+4*m1**2*m3**6*
     -     Idiv1(m3**2,m2**2,m1**2)+4*m1**8*Idiv2(m3**2,m2**2,m1**2)- 
     -     8*m1**6*m2**2*Idiv2(m3**2,m2**2,m1**2)+4*m1**4*m2**4*
     -     Idiv2(m3**2,m2**2,m1**2)-8*m1**6*m3**2*
     -     Idiv2(m3**2,m2**2,m1**2)+8*m1**4*m2**2*m3**2*
     -     Idiv2(m3**2,m2**2,m1**2)+4*m1**4*m3**4*
     -     Idiv2(m3**2,m2**2,m1**2)+2*m1**2*((m1**2 - m2**2)**3+
     -     (-3*m1**4+m2**4)*m3**2+(3*m1**2+m2**2)*m3**4 - m3**6)*
     -     Ifin(m3**2,m2**2,m1**2))/(2.*m1**2*(m1+m2 - m3)**2*
     -     (m1 - m2+m3)**2*(-m1+m2+m3)**2*(m1+m2+m3)**2)

     
      if(abs(m1).le.1d-12) write(*,*) '(m1)<0!!'
      if(abs(m1 - m2 - m3).le.1d-12) write(*,*) '(m1 - m2 - m3)<0!!'
      if(abs(m1+m2 - m3).le.1d-12) write(*,*) '(m1+m2 - m3)<0!!'
      if(abs(m1+m2+m3).le.1d-12) write(*,*) '(m1+m2+m3)<0!!'
      if(abs(m1 - m2+m3).le.1d-12) write(*,*) '(m1 - m2+m3)<0!!'
     
      return
      end
      
      
      
      double complex function TFIdv4div1(m1,m2,m3)
      
      implicit none

      double precision m1,m2,m3
      
      double precision Idiv1,Idiv2,A0
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      TFIdv4div1=(2*(-(m2**2 - m3**2)**4+m1**6*(m2**2+m3**2)+
     -     3*m1**2*(m2**2 - m3**2)**2*(m2**2+m3**2) - 
     -     m1**4*(3*m2**4+4*m2**2*m3**2+3*m3**4))*Pi**4*A0(m1**2)+
     -     m1**2*((-5*m1**6*(m2**2+m3**2)+(m2**2 - m3**2)**2*(3*m2**4 -
     -     2*m2**2*m3**2+3*m3**4)+m1**4*(13*m2**4+16*m2**2*m3**2+
     -     13*m3**4)+m1**2*(-11*m2**6+m2**4*m3**2+m2**2*m3**4 - 
     -     11*m3**6))*Pi**4+2*(((m1**2 - m2**2)**3+(-3*m1**4+m2**4)*
     -     m3**2+(3*m1**2+m2**2)*m3**4 - m3**6)*(Pi**4*(A0(m2**2)+
     -     A0(m3**2))+Idiv1(m3**2,m2**2,m1**2)) -(m1 - m2 - m3)*
     -     (m1+m2 - m3)*(m1 - m2+m3)*(m1+m2+m3)*(3*m1**2 - 2*(m2**2+
     -     m3**2))*Idiv2(m3**2,m2**2,m1**2))))/(2.*m1**2*(m1+m2-m3)**2*
     -     (m1 - m2+m3)**2*(-m1+m2+m3)**2*(m1+m2+m3)**2)

      return
      end
      
      
      double complex function TFIdv4div2(m1,m2,m3)
      
      implicit none
     
      double precision m1,m2,m3
      
      double precision Idiv2
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      TFIdv4div2=((m1**6*(m2**2+m3**2)-(m2**2-m3**2)**2*(m2**4+m3**4)- 
     -     m1**4*(3*m2**4+5*m2**2*m3**2+3*m3**4)+3*m1**2*(m2**6+m3**6))
     -     *Pi**4+((m1**2 - m2**2)**3+(-3*m1**4+m2**4)*m3**2+(3*m1**2+
     -     m2**2)*m3**4 -m3**6)*Idiv2(m3**2,m2**2,m1**2))/((m1+m2 - 
     -     m3)**2*(m1 - m2+m3)**2*(-m1+m2+m3)**2*(m1+m2+m3)**2)

     
      return
      end
      
      
c      
c myTFI[D, 0, {{3, m1_}, {1, m2_}, {0, 0}, {0, 0}, {1, m3_}}] := 
c TFIdv5fin[m1, m2, m3]+TFIdv5div1[m1, m2, m3]*pol+TFIdv5div2[m1, m2, m3]*pol^2     
c      
      double complex function TFIdv5fin(m1,m2,m3)
      
      implicit none

      double precision m1,m2,m3
      
      double complex Ifin
      
      double precision Idiv1,Idiv2,Ae,A0
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      TFIdv5fin= (m1**2*(-(m2**2 - m3**2)**4+3*m1**6*(m2**2+m3**2)+
     -     m1**2*(m2**2+m3**2)*(5*m2**4 - 6*m2**2*m3**2+5*m3**4) - 
     -     m1**4*(7*m2**4+6*m2**2*m3**2+7*m3**4))*Pi**4+
     -     Pi**4*A0(m1**2)*((m2**2 - m3**2)**4 - 3*m1**6*(m2**2+m3**2)- 
     -     5*m1**2*(m2**2 - m3**2)**2*(m2**2+m3**2)+m1**4*(7*m2**4+
     -     6*m2**2*m3**2+7*m3**4)-2*m1**2*m3**2*(m1**2+m2**2 - m3**2)*
     -     A0(m2**2)-2*m1**2*m2**2*(m1**2 - m2**2+m3**2)*A0(m3**2)) - 
     -     m1**2*((3*m1**6+(m2**2 - m3**2)**3-m1**4*(5*m2**2+7*m3**2)+
     -     m1**2*(m2**4-2*m2**2*m3**2+5*m3**4))*Pi**4*A0(m3**2)+Pi**4*
     -     A0(m2**2)*(3*m1**6-(m2**2-m3**2)**3-m1**4*(7*m2**2+5*m3**2)+
     -     m1**2*(5*m2**4 - 2*m2**2*m3**2+m3**4)+2*m1**2*(-m1**2+m2**2+
     -     m3**2)*A0(m3**2))+2*m1**2*(2*m2**2*m3**2*Pi**4*Ae(m1**2)+
     -     2*m2**2*m3**2*Pi**4*Ae(m2**2)+2*m2**2*m3**2*Pi**4*Ae(m3**2)+
     -     m1**4*Idiv1(m3**2,m2**2,m1**2)-2*m1**2*m2**2*
     -     Idiv1(m3**2,m2**2,m1**2)+m2**4*Idiv1(m3**2,m2**2,m1**2) - 
     -     2*m1**2*m3**2*Idiv1(m3**2,m2**2,m1**2)-2*m2**2*m3**2*
     -     Idiv1(m3**2,m2**2,m1**2)+m3**4*Idiv1(m3**2,m2**2,m1**2) - 
     -     2*m1**4*Idiv2(m3**2,m2**2,m1**2)+4*m1**2*m2**2*
     -     Idiv2(m3**2,m2**2,m1**2)-2*m2**4*Idiv2(m3**2,m2**2,m1**2)+
     -     4*m1**2*m3**2*Idiv2(m3**2,m2**2,m1**2)-4*m2**2*m3**2*
     -     Idiv2(m3**2,m2**2,m1**2)-2*m3**4*Idiv2(m3**2,m2**2,m1**2)+
     -     2*m2**2*m3**2*Ifin(m3**2,m2**2,m1**2))))/(2.*m1**4*(m1+m2 -
     -     m3)**2*(m1 - m2+m3)**2*(-m1+m2+m3)**2*(m1+m2+m3)**2)
      
      if(abs(m1).le.1d-12) write(*,*) '(m1)<0!!'
      if(abs(m1 - m2 - m3).le.1d-12) write(*,*) '(m1 - m2 - m3)<0!!'
      if(abs(m1+m2 - m3).le.1d-12) write(*,*) '(m1+m2 - m3)<0!!'
      if(abs(m1+m2+m3).le.1d-12) write(*,*) '(m1+m2+m3)<0!!'
      if(abs(m1 - m2+m3).le.1d-12) write(*,*) '(m1 - m2+m3)<0!!'
     
      return
      end
     
      
c      
c myTFI[D, 0, {0, 0, 0, 0,1}, {{1, m1_}, {1, m2_}, {0, 0}, {0, 0}, {1, m3_}}] := 
c TFIdv6fin[m1, m2, m3]+TFIdv6div1[m1, m2, m3]*pol+TFIdv6div2[m1, m2, m3]*pol^2     
c      
      double complex function TFIdv6fin(m1,m2,m3)
      
      implicit none
     
      double precision m1,m2,m3
      
      double complex Ifin
      
      double precision Ae,A0
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      TFIdv6fin=(Pi**4*(A0(m1**2)*A0(m2**2)-(A0(m1**2)+A0(m2**2))*
     -     A0(m3**2)+(m2 - m3)*(m2+m3)*Ae(m1**2)+(m1 - m3)*(m1+m3)*
     -     Ae(m2**2)-(m1**2+m2**2)*Ae(m3**2))+(m1**2+m2**2 - m3**2)*
     -     Ifin(m3**2,m2**2,m1**2))/2.
     
      return
      end
      
      
      double complex function TFIdv7fin(m1,m2,m3)
      
      implicit none
      
      double precision A0
      
      double precision m1,m2,m3
      
      double complex Ifin
      
      double precision Ae
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      TFIdv7fin=-(Pi**4*(A0(m2**2)*A0(m3**2)+m3**2*Ae(m2**2)+
     -  m2**2*Ae(m3**2)))+m1**2*Ifin(m3**2,m2**2,m1**2)
      
      return
      end
      
      
      double complex function TFIdv7div1(m1,m2,m3)
      
      implicit none
      
      double precision A0
      
      double precision m1,m2,m3
      
      double precision Idiv1
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      TFIdv7div1=-(Pi**4*(m3**2*A0(m2**2)+m2**2*A0(m3**2)))+m1**2*
     -     Idiv1(m3**2,m2**2,m1**2)
      
      return
      end
      
      
      double complex function TFIdv7div2(m1,m2,m3)
      
      implicit none
      
      double precision m1,m2,m3
      
      double precision Idiv2
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      TFIdv7div2=-(m2**2*m3**2*Pi**4)+m1**2*Idiv2(m3**2,m2**2,m1**2)
      
      return
      end
      
      
      
      
      
      double complex function TFIdv8fin(m1,m2,m3)
      
      implicit none
      
      double precision A0
      
      double precision m1,m2,m3
      
      double complex Ifin
      
      double precision Ae,Idiv1
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      if(m1.eq.m3) then
      TFIdv8fin=((-8*m1**2 + m2**2)*Pi**4*A0(m1**2)**2 + 
     -    4*m1**2*Pi**4*A0(m1**2)*(2*m1**2 - m2**2 + A0(m2**2))+ 
     -    2*m1**2*(Pi**4*(-2*m1**2*A0(m2**2)+(-8*m1**2 + 3*m2**2)*
     -    Ae(m1**2)+2*m1**2*Ae(m2**2))-m2**2*Idiv1(m2**2,m1**2,m1**2))+
     -    (4*m1**4 + 4*m1**2*m2**2 - m2**4)*Ifin(m2**2,m1**2,m1**2))/
     -    (8*m1**2 - 2*m2**2)
      else
      TFIdv8fin=(Pi**4*(2*m1**4*(-m1**2+m2**2+m3**2)*A0(m2**2)+
     -     A0(m1**2)*(2*m1**2*m2**2*(-m1**2+m2**2+m3**2)+(3*m1**4+
     -     (m2**2 - m3**2)**2 -4*m1**2*(m2**2+m3**2))*A0(m2**2))+
     -     (3*m1**4+(m2**2 - m3**2)**2 - 4*m1**2*(m2**2+m3**2))*
     -     (m2**2*Ae(m1**2)+m1**2*Ae(m2**2)))+Pi**4*(-4*m1**4*m2**2*
     -     A0(m3**2) -A0(m1**2)*(4*m1**2*m2**2*m3**2+(m1**4+(m2**2 - 
     -     m3**2)**2 - 2*m1**2*(3*m2**2+m3**2))*A0(m3**2)) -(m1**4+
     -     (m2**2 - m3**2)**2 - 2*m1**2*(3*m2**2+m3**2))*(m3**2*
     -     Ae(m1**2)+m1**2*Ae(m3**2)))+Pi**4*(2*m1**2*m2**2*(m1**2+
     -     m2**2 - m3**2)*A0(m3**2)+A0(m2**2)*(2*m1**2*m3**2*(m1**2+
     -     m2**2 - m3**2)-(3*m1**4+m2**4 - 2*(2*m1**2+m2**2)*m3**2+
     -     m3**4)*A0(m3**2))-(3*m1**4+m2**4 - 2*(2*m1**2+m2**2)*m3**2+
     -     m3**4)*(m3**2*Ae(m2**2)+m2**2*Ae(m3**2))) -2*m1**2*(-m2**4+
     -     (m1**2 - m3**2)**2)*Idiv1(m3**2,m2**2,m1**2)+(3*m1**6+
     -     (m2**2 - m3**2)**3 - m1**4*(3*m2**2+7*m3**2) -m1**2*(m2**4+
     -     4*m2**2*m3**2 -5*m3**4))*Ifin(m3**2,m2**2,m1**2))/
     -     (2.*(m1 - m2 - m3)*(m1+m2 - m3)*(m1 - m2+m3)*(m1+m2+m3))
      end if
      
      return
      end
      
      
      double complex function TFIdv8div1(m1,m2,m3)
      
      implicit none
      
      double precision A0
      
      double precision m1,m2,m3
      
      double precision Idiv1,Idiv2
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      if(m1.eq.m3) then
      TFIdv8div1=(4*m1**4*(m1-m2)*(m1+m2)*Pi**4+2*m1**2*(-8*m1**2+
     -    3*m2**2)*Pi**4*A0(m1**2) + 4*m1**4*Pi**4*A0(m2**2)+ 
     -    (4*m1**4 + 4*m1**2*m2**2 - m2**4)*Idiv1(m2**2,m1**2,m1**2) - 
     -    2*m1**2*m2**2*Idiv2(m2**2,m1**2,m1**2))/(8*m1**2 - 2*m2**2)
      else
      TFIdv8div1=-(2*m1**2*m2**2*(m1**4-m1**2*m2**2-m2**2*m3**2+m3**4)*
     -     Pi**4+(-(m2**2 - m3**2)**3+m1**4*(-3*m2**2+m3**2)+
     -     2*m1**2*(m2 - m3)*(m2+m3)*(2*m2**2+m3**2))*Pi**4*A0(m1**2)+
     -     (-3*m1**6+m1**4*(4*m2**2+7*m3**2)+(-(m2**2*m3)+m3**3)**2 - 
     -     m1**2*(m2**4 - 2*m2**2*m3**2+5*m3**4))*Pi**4*A0(m2**2)+
     -     m1**6*Pi**4*A0(m3**2) - 3*m1**4*m2**2*Pi**4*A0(m3**2)+
     -     m1**2*m2**4*Pi**4*A0(m3**2)+m2**6*Pi**4*A0(m3**2) - 
     -     2*m1**4*m3**2*Pi**4*A0(m3**2)-6*m1**2*m2**2*m3**2*Pi**4*
     -     A0(m3**2)-2*m2**4*m3**2*Pi**4*A0(m3**2)+m1**2*m3**4*Pi**4*
     -     A0(m3**2)+m2**2*m3**4*Pi**4*A0(m3**2) - 3*m1**6*
     -     Idiv1(m3**2,m2**2,m1**2)+3*m1**4*m2**2*
     -     Idiv1(m3**2,m2**2,m1**2)+m1**2*m2**4*
     -     Idiv1(m3**2,m2**2,m1**2)- m2**6*Idiv1(m3**2,m2**2,m1**2)+
     -     7*m1**4*m3**2*Idiv1(m3**2,m2**2,m1**2)+4*m1**2*m2**2*m3**2*
     -     Idiv1(m3**2,m2**2,m1**2)+3*m2**4*m3**2*
     -     Idiv1(m3**2,m2**2,m1**2) -5*m1**2*m3**4*
     -     Idiv1(m3**2,m2**2,m1**2)-3*m2**2*m3**4*
     -     Idiv1(m3**2,m2**2,m1**2)+m3**6*Idiv1(m3**2,m2**2,m1**2)+
     -     2*m1**2*(-m2**4+(m1**2-m3**2)**2)*Idiv2(m3**2,m2**2,m1**2))/
     -     (2.*(m1 - m2 - m3)*(m1+m2 - m3)*(m1 - m2+m3)*(m1+m2+m3))
      end if
      
      return
      end
      
      
      double complex function TFIdv8div2(m1,m2,m3)
      
      implicit none
      
      double precision m1,m2,m3
      
      double precision Idiv2
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      if(m1.eq.m3) then
      TFIdv8div2=(m1**4*(-8*m1**2 + 5*m2**2)*Pi**4+
     -     (4*m1**4+4*m1**2*m2**2-m2**4)*Idiv2(m2**2,m1**2,m1**2))/
     -     (8*m1**2-2*m2**2)
      else
      TFIdv8div2=((m1**2*m2**2*(3*m1**4 - 4*m1**2*m2**2+m2**4) - 
     -     (m1**6+m1**4*m2**2+3*m1**2*m2**4+m2**6)*m3**2+
     -     (2*m1**4+7*m1**2*m2**2+2*m2**4)*m3**4-(m1**2+m2**2)*m3**6)*
     -     Pi**4+(3*m1**6+(m2**2-m3**2)**3-m1**4*(3*m2**2+7*m3**2) - 
     -     m1**2*(m2**4+4*m2**2*m3**2 - 5*m3**4))*
     -     Idiv2(m3**2,m2**2,m1**2))/(2.*(m1 - m2 - m3)*(m1+m2 - m3)*
     -     (m1 - m2+m3)*(m1+m2+m3))
      end if
      
      return
      end
      
      
      
      
      double complex function TFIdv9fin(m1,m2,m3)
      
      implicit none
      
      double precision A0
      
      double precision m1,m2,m3
      
      double complex Ifin
      
      double precision Ae
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      TFIdv9fin=(Pi**4*(m1**2*A0(m1**2)*(A0(m2**2) - A0(m3**2)) - 
     -     (m1**2+2*m2**2)*A0(m2**2)*A0(m3**2)+m1**2*m2**2*Ae(m1**2)- 
     -     m1**2*m3**2*Ae(m1**2)+m1**4*Ae(m2**2)-m1**2*m3**2*Ae(m2**2)- 
     -     2*m2**2*m3**2*Ae(m2**2)-(m1**4+m1**2*m2**2+2*m2**4)*
     -     Ae(m3**2))+m1**2*(m1**2+m2**2-m3**2)*
     -     Ifin(m3**2,m2**2,m1**2))/2.
      
      return
      end
      
      
      double complex function TFIdv9div1(m1,m2,m3)
      
      implicit none
      
      double precision A0
      
      double precision m1,m2,m3
      
      double precision Idiv1
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      TFIdv9div1=(Pi**4*(m1**2*(m2 - m3)*(m2+m3)*A0(m1**2)+
     -     (m1**4 - (m1**2+2*m2**2)*m3**2)*A0(m2**2) - 
     -     (m1**4+m1**2*m2**2+2*m2**4)*A0(m3**2))+
     -     m1**2*(m1**2+m2**2 - m3**2)*Idiv1(m3**2,m2**2,m1**2))/2.
      
      return
      end
      
      
      double complex function TFIdv9div2(m1,m2,m3)
      
      implicit none
      
      double precision m1,m2,m3
      
      double precision Idiv2
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      TFIdv9div2=((m1**4*m2**2 - (m1**4+m1**2*m2**2+2*m2**4)*m3**2)*Pi**4+
     -    m1**2*(m1**2+m2**2 - m3**2)*Idiv2(m3**2,m2**2,m1**2))/2.
      
      return
      end
      
      
      
      double complex function TFIdv10fin(m1,m2,m3)
      
      implicit none
      
      double precision A0
      
      double precision m1,m2,m3
      
      double complex Ifin
      
      double precision Idiv1,Ae
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      if(m2.eq.m3) then
      TFIdv10fin=(((m1**2-2*m2**2)*Pi**4*(A0(m1**2)*(m2**2-A0(m2**2))-
     -     m2**2*Ae(m1**2)+m1**2*(A0(m2**2) - Ae(m2**2))))/m2**2 + 
     -     2*Pi**4*(-2*m2**2*A0(m2**2)+A0(m2**2)**2 + 2*m2**2*
     -     Ae(m2**2))+m1**2*Idiv1(m2**2,m2**2,m1**2)-(m1**2-2*m2**2)*
     -     Ifin(m2**2,m2**2,m1**2))/(m1**2 - 4*m2**2)
      else
      TFIdv10fin=((m1**2+m2**2 - m3**2)*Pi**4*(A0(m1**2)*(m2**2-
     -     A0(m2**2)) - m2**2*Ae(m1**2)+m1**2*(A0(m2**2)-Ae(m2**2)))+
     -     ((m1**4 - m2**2*m3**2+m3**4 - m1**2*(m2**2+2*m3**2))*Pi**4*
     -     (A0(m1**2)*(m3**2 - A0(m3**2)) - m3**2*Ae(m1**2)+m1**2*
     -     (A0(m3**2) - Ae(m3**2))))/m3**2+(((m2**2-m3**2)**2-m1**2*
     -     (m2**2+m3**2))*Pi**4*(A0(m2**2)*(m3**2 - A0(m3**2))-m3**2*
     -     Ae(m2**2)+m2**2*(A0(m3**2) - Ae(m3**2))))/m3**2+
     -     (m1**2+m2**2 - m3**2)**2*Idiv1(m3**2,m2**2,m1**2) - 
     -     (m1**4+m2**4 - 2*(m1**2+m2**2)*m3**2+m3**4)*
     -     Ifin(m3**2,m2**2,m1**2))/
     -     ((m1 - m2 - m3)*(m1+m2 - m3)*(m1 - m2+m3)*(m1+m2+m3))
      end if
      
      return
      end
      
      
      
      double complex function TFIdv11fin(m1,m2,m3)
      
      implicit none
      
      double precision A0
      
      double precision m1,m2,m3
      
      double complex Ifin
      
      double precision Idiv1,Ae,Idiv2
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      TFIdv11fin= (3*m1**8*m2**4*Pi**4-7*m1**6*m2**6*Pi**4+5*m1**4*
     -     m2**8*Pi**4-m1**2*m2**10*Pi**4 - m1**10*m3**2*Pi**4+
     -     7*m1**8*m2**2*m3**2*Pi**4-12*m1**6*m2**4*m3**2*Pi**4+
     -     3*m1**4*m2**6*m3**2*Pi**4+3*m1**2*m2**8*m3**2*Pi**4+
     -     4*m1**8*m3**4*Pi**4 -11*m1**6*m2**2*m3**4*Pi**4 - 5*m1**4*
     -     m2**4*m3**4*Pi**4-2*m1**2*m2**6*m3**4*Pi**4-6*m1**6*m3**6*
     -     Pi**4+m1**4*m2**2*m3**6*Pi**4 -2*m1**2*m2**4*m3**6*Pi**4+
     -     4*m1**4*m3**8*Pi**4+3*m1**2*m2**2*m3**8*Pi**4 -m1**2*m3**10*
     -     Pi**4+3*m1**10*Pi**4*A0(m3**2)-17*m1**8*m2**2*Pi**4*
     -     A0(m3**2)+29*m1**6*m2**4*Pi**4*A0(m3**2) -19*m1**4*m2**6*
     -     Pi**4*A0(m3**2)+4*m1**2*m2**8*Pi**4*A0(m3**2)-12*m1**8*
     -     m3**2*Pi**4*A0(m3**2)+25*m1**6*m2**2*m3**2*Pi**4*A0(m3**2)+
     -     10*m1**4*m2**4*m3**2*Pi**4*A0(m3**2) - 11*m1**2*m2**6*m3**2*
     -     Pi**4*A0(m3**2)+18*m1**6*m3**4*Pi**4*A0(m3**2)+m1**4*m2**2*
     -     m3**4*Pi**4*A0(m3**2)+13*m1**2*m2**4*m3**4*Pi**4*A0(m3**2) -
     -     12*m1**4*m3**6*Pi**4*A0(m3**2)-9*m1**2*m2**2*m3**6*Pi**4*
     -     A0(m3**2)+3*m1**2*m3**8*Pi**4*A0(m3**2)+m1**2*m2**2*Pi**4*
     -     A0(m2**2)*(-5*m1**6+(m2**2 - m3**2)**2*(3*m2**2+m3**2)+
     -     m1**4*(13*m2**2+11*m3**2) - m1**2*(11*m2**4+2*m2**2*m3**2+
     -     7*m3**4)-2*(m1**4+2*(m2**2 - m3**2)**2 - 3*m1**2*(m2**2+
     -     m3**2))*A0(m3**2))+Pi**4*A0(m1**2)*(-5*m1**6*m2**4+13*m1**4*
     -     m2**6 - 11*m1**2*m2**8+3*m2**10+3*m1**8*m3**2 - 17*m1**6*
     -     m2**2*m3**2+28*m1**4*m2**4*m3**2-m1**2*m2**6*m3**2-9*m2**8*
     -     m3**2 - 12*m1**6*m3**4+25*m1**4*m2**2*m3**4+23*m1**2*m2**4*
     -     m3**4+6*m2**6*m3**4+18*m1**4*m3**6+m1**2*m2**2*m3**6+
     -     6*m2**4*m3**6 - 12*m1**2*m3**8-9*m2**2*m3**8+3*m3**10+
     -     2*m2**2*(m1**6-(m2**2-m3**2)**3 - m1**4*(3*m2**2+2*m3**2)+
     -     3*m1**2*(m2**4 - m2**2*m3**2))*A0(m2**2) -2*(m1**8+m3**2*
     -     (-m2**2+m3**2)**3 - m1**6*(5*m2**2+4*m3**2)+m1**4*(8*m2**4+
     -     7*m2**2*m3**2+6*m3**4)+m1**2*(-4*m2**6+7*m2**4*m3**2+m2**2*
     -     m3**4 - 4*m3**6))*A0(m3**2))+2*m1**6*m2**4*Pi**4*Ae(m1**2) -
     -     6*m1**4*m2**6*Pi**4*Ae(m1**2)+6*m1**2*m2**8*Pi**4*Ae(m1**2)-
     -     2*m2**10*Pi**4*Ae(m1**2) -2*m1**8*m3**2*Pi**4*Ae(m1**2)+
     -     10*m1**6*m2**2*m3**2*Pi**4*Ae(m1**2)-20*m1**4*m2**4*m3**2*
     -     Pi**4*Ae(m1**2)+2*m1**2*m2**6*m3**2*Pi**4*Ae(m1**2)+
     -     6*m2**8*m3**2*Pi**4*Ae(m1**2)+8*m1**6*m3**4*Pi**4*Ae(m1**2)- 
     -     14*m1**4*m2**2*m3**4*Pi**4*Ae(m1**2) - 14*m1**2*m2**4*m3**4*
     -     Pi**4*Ae(m1**2) -4*m2**6*m3**4*Pi**4*Ae(m1**2) - 12*m1**4*
     -     m3**6*Pi**4*Ae(m1**2) -2*m1**2*m2**2*m3**6*Pi**4*Ae(m1**2) - 
     -     4*m2**4*m3**6*Pi**4*Ae(m1**2)+8*m1**2*m3**8*Pi**4*Ae(m1**2)+
     -     6*m2**2*m3**8*Pi**4*Ae(m1**2) -2*m3**10*Pi**4*Ae(m1**2)+
     -     2*m1**8*m2**2*Pi**4*Ae(m2**2)-6*m1**6*m2**4*Pi**4*Ae(m2**2)+
     -     6*m1**4*m2**6*Pi**4*Ae(m2**2)-2*m1**2*m2**8*Pi**4*Ae(m2**2)-
     -     6*m1**6*m2**2*m3**2*Pi**4*Ae(m2**2)+2*m1**2*m2**6*m3**2*
     -     Pi**4*Ae(m2**2)+6*m1**4*m2**2*m3**4*Pi**4*Ae(m2**2)+
     -     2*m1**2*m2**4*m3**4*Pi**4*Ae(m2**2)-2*m1**2*m2**2*m3**6*
     -     Pi**4*Ae(m2**2) -2*m1**10*Pi**4*Ae(m3**2)+10*m1**8*m2**2*
     -     Pi**4*Ae(m3**2)-18*m1**6*m2**4*Pi**4*Ae(m3**2)+14*m1**4*
     -     m2**6*Pi**4*Ae(m3**2) -4*m1**2*m2**8*Pi**4*Ae(m3**2)+8*m1**8
     -     *m3**2*Pi**4*Ae(m3**2)-14*m1**6*m2**2*m3**2*Pi**4*Ae(m3**2)- 
     -     8*m1**4*m2**4*m3**2*Pi**4*Ae(m3**2)+10*m1**2*m2**6*m3**2*
     -     Pi**4*Ae(m3**2) - 12*m1**6*m3**4*Pi**4*Ae(m3**2) -2*m1**4*
     -     m2**2*m3**4*Pi**4*Ae(m3**2) -10*m1**2*m2**4*m3**4*Pi**4*
     -     Ae(m3**2)+8*m1**4*m3**6*Pi**4*Ae(m3**2)+6*m1**2*m2**2*m3**6*
     -     Pi**4*Ae(m3**2)-2*m1**2*m3**8*Pi**4*Ae(m3**2)-6*m1**8*m2**2*
     -     Idiv1(m3**2,m2**2,m1**2)+
     -     16*m1**6*m2**4*Idiv1(m3**2,m2**2,m1**2) - 
     -     14*m1**4*m2**6*Idiv1(m3**2,m2**2,m1**2)+
     -     4*m1**2*m2**8*Idiv1(m3**2,m2**2,m1**2)+
     -     16*m1**6*m2**2*m3**2*Idiv1(m3**2,m2**2,m1**2) - 
     -     4*m1**4*m2**4*m3**2*Idiv1(m3**2,m2**2,m1**2) - 
     -     4*m1**2*m2**6*m3**2*Idiv1(m3**2,m2**2,m1**2) - 
     -     14*m1**4*m2**2*m3**4*Idiv1(m3**2,m2**2,m1**2) - 
     -     4*m1**2*m2**4*m3**4*Idiv1(m3**2,m2**2,m1**2)+
     -     4*m1**2*m2**2*m3**6*Idiv1(m3**2,m2**2,m1**2)+
     -     4*m1**8*m2**2*Idiv2(m3**2,m2**2,m1**2) - 
     -     8*m1**6*m2**4*Idiv2(m3**2,m2**2,m1**2)+
     -     4*m1**4*m2**6*Idiv2(m3**2,m2**2,m1**2) - 
     -     8*m1**6*m2**2*m3**2*Idiv2(m3**2,m2**2,m1**2)+
     -     8*m1**4*m2**4*m3**2*Idiv2(m3**2,m2**2,m1**2)+
     -     4*m1**4*m2**2*m3**4*Idiv2(m3**2,m2**2,m1**2)+
     -     2*m1**8*m2**2*Ifin(m3**2,m2**2,m1**2) - 
     -     6*m1**6*m2**4*Ifin(m3**2,m2**2,m1**2)+
     -     6*m1**4*m2**6*Ifin(m3**2,m2**2,m1**2) - 
     -     2*m1**2*m2**8*Ifin(m3**2,m2**2,m1**2) - 
     -     6*m1**6*m2**2*m3**2*Ifin(m3**2,m2**2,m1**2)+
     -     2*m1**2*m2**6*m3**2*Ifin(m3**2,m2**2,m1**2)+
     -     6*m1**4*m2**2*m3**4*Ifin(m3**2,m2**2,m1**2)+
     -     2*m1**2*m2**4*m3**4*Ifin(m3**2,m2**2,m1**2) - 
     -     2*m1**2*m2**2*m3**6*Ifin(m3**2,m2**2,m1**2))/(2.*m1**2*
     -     (m1+m2-m3)**2*(m1 - m2+m3)**2*(-m1+m2+m3)**2*(m1+m2+m3)**2)
      
      return
      end
      
      
      
      double complex function TFIdv11div1(m1,m2,m3)
      
      implicit none
      
      double precision A0
      
      double precision m1,m2,m3
      
      double precision Idiv1, Idiv2
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      TFIdv11div1=(-2*(m1**8*m3**2+(m2**2 - m3**2)**4*(m2**2+m3**2) - 
     -     m1**6*(m2**4+5*m2**2*m3**2+4*m3**4) -m1**2*(m2**2 - 
     -     m3**2)**2*(3*m2**4+7*m2**2*m3**2+4*m3**4)+m1**4*(3*m2**6+
     -     10*m2**4*m3**2+7*m2**2*m3**4+6*m3**6))*Pi**4*A0(m1**2)
     -     +m1**2*(-5*m1**6*m2**4*Pi**4+13*m1**4*m2**6*Pi**4 - 
     -     11*m1**2*m2**8*Pi**4+3*m2**10*Pi**4+3*m1**8*m3**2*Pi**4 - 
     -     17*m1**6*m2**2*m3**2*Pi**4+34*m1**4*m2**4*m3**2*Pi**4 -11*
     -     m1**2*m2**6*m3**2*Pi**4-5*m2**8*m3**2*Pi**4-12*m1**6*m3**4*
     -     Pi**4+25*m1**4*m2**2*m3**4*Pi**4+13*m1**2*m2**4*m3**4*Pi**4- 
     -     2*m2**6*m3**4*Pi**4+18*m1**4*m3**6*Pi**4+m1**2*m2**2*m3**6*
     -     Pi**4+10*m2**4*m3**6*Pi**4 - 12*m1**2*m3**8*Pi**4 - 9*m2**2*
     -     m3**8*Pi**4+3*m3**10*Pi**4+2*m2**2*(m1**6-3*m1**4*(m2**2+
     -     m3**2)-(m2**2-m3**2)**2*(m2**2+m3**2)+3*m1**2*(m2**4+m3**4))
     -     *Pi**4*A0(m2**2)-2*(m1**8 - m1**6*(5*m2**2+4*m3**2)+
     -     (m2**2 - m3**2)**2*(2*m2**4 - m2**2*m3**2+m3**4)+
     -     m1**4*(9*m2**4+7*m2**2*m3**2+6*m3**4)+
     -     m1**2*(-7*m2**6+4*m2**4*m3**2+m2**2*m3**4 - 4*m3**6))*Pi**4*
     -     A0(m3**2)+2*m1**6*m2**2*Idiv1(m3**2,m2**2,m1**2) - 
     -     6*m1**4*m2**4*Idiv1(m3**2,m2**2,m1**2)+
     -     6*m1**2*m2**6*Idiv1(m3**2,m2**2,m1**2) - 
     -     2*m2**8*Idiv1(m3**2,m2**2,m1**2) - 
     -     6*m1**4*m2**2*m3**2*Idiv1(m3**2,m2**2,m1**2)+
     -     2*m2**6*m3**2*Idiv1(m3**2,m2**2,m1**2)+
     -     6*m1**2*m2**2*m3**4*Idiv1(m3**2,m2**2,m1**2)+
     -     2*m2**4*m3**4*Idiv1(m3**2,m2**2,m1**2) - 
     -     2*m2**2*m3**6*Idiv1(m3**2,m2**2,m1**2) - 
     -     6*m1**6*m2**2*Idiv2(m3**2,m2**2,m1**2)+
     -     16*m1**4*m2**4*Idiv2(m3**2,m2**2,m1**2) - 
     -     14*m1**2*m2**6*Idiv2(m3**2,m2**2,m1**2)+
     -     4*m2**8*Idiv2(m3**2,m2**2,m1**2)+
     -     16*m1**4*m2**2*m3**2*Idiv2(m3**2,m2**2,m1**2) - 
     -     4*m1**2*m2**4*m3**2*Idiv2(m3**2,m2**2,m1**2) - 
     -     4*m2**6*m3**2*Idiv2(m3**2,m2**2,m1**2) - 
     -     14*m1**2*m2**2*m3**4*Idiv2(m3**2,m2**2,m1**2) - 
     -     4*m2**4*m3**4*Idiv2(m3**2,m2**2,m1**2)+
     -     4*m2**2*m3**6*Idiv2(m3**2,m2**2,m1**2)))/
     -     (2.*m1**2*(m1+m2 - m3)**2*(m1 - m2+m3)**2*(-m1+m2+m3)**2*
     -     (m1+m2+m3)**2)
     
      return
      end
      
      
      double complex function TFIdv11div2(m1,m2,m3)
      
      implicit none
      
      double precision m1,m2,m3
      
      double precision Idiv2
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      TFIdv11div2=(-((m1**8*m3**2-m1**6*(m2**4+5*m2**2*m3**2+4*m3**4)+
     -    (m2**2 - m3**2)**2*(m2**6+m2**4*m3**2 - m2**2*m3**4+m3**6)+
     -     m1**4*(3*m2**6+11*m2**4*m3**2+7*m2**2*m3**4+6*m3**6)+
     -     m1**2*(-3*m2**8 - 4*m2**6*m3**2+4*m2**4*m3**4+m2**2*m3**6 - 
     -     4*m3**8))*Pi**4)+m2**2*(m1**6 - 3*m1**4*(m2**2+m3**2) - 
     -     (m2**2 - m3**2)**2*(m2**2+m3**2)+3*m1**2*(m2**4+m3**4))*
     -     Idiv2(m3**2,m2**2,m1**2))/((m1+m2 - m3)**2*
     -     (m1 - m2+m3)**2*(-m1+m2+m3)**2*(m1+m2+m3)**2)
     
      return
      end
      
      
      
c      
c myTFI[D, 0, {0, 0, 0, 0,1}, {{3, m1_}, {1, m2_}, {0, 0}, {0, 0}, {1, m3_}}] := 
c TFIdv12fin[m1, m2, m3]+TFIdv12div1[m1, m2, m3]*pol +TFIdv12div2[m1, m2, m3]*pol^2     
c      
      double complex function TFIdv12fin(m1,m2,m3)
      
      implicit none
     
      double precision m1,m2,m3
      
      double complex Ifin
      
      double precision A0,Idiv1,Idiv2,Ae
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      TFIdv12fin= ((Pi**4*(2*m1**4*m2**2*(-m1**2+m2**2+m3**2)**2+
     -     m1**2*(-3*m1**6+m1**4*(7*m2**2+6*m3**2)+(m2**3-m2*m3**2)**2-
     -     m1**2*(5*m2**4+3*m3**4))*A0(m2**2)+
     -     A0(m1**2)*(m2**2*(-3*m1**6+m1**4*(7*m2**2+6*m3**2)+
     -     (m2**3-m2*m3**2)**2-m1**2*(5*m2**4+3*m3**4))+
     -     ((m1**2-m2**2)**3-2*(m1**4+2*m1**2*m2**2-m2**4)*m3**2+
     -     (m1-m2)*(m1+m2)*m3**4)*A0(m2**2))+((m1**2-m2**2)**3-2*
     -     (m1**4+2*m1**2*m2**2-m2**4)*m3**2+(m1-m2)*(m1+m2)*m3**4)*
     -     (m2**2*Ae(m1**2)+m1**2*Ae(m2**2))))/m1**2+(Pi**4*(m1**2*
     -     m3**2*(m1**6+3*m1**4*(m2-m3)*(m2+m3)+
     -     (m2**2-m3**2)**3+m1**2*(-5*m2**4-6*m2**2*m3**2+3*m3**4))+
     -     m1**2*(-2*m1**6-m1**4*(m2**2-6*m3**2)-(3*m2**2-2*m3**2)*
     -     (m2**2-m3**2)**2+m1**2*(6*m2**4+8*m2**2*m3**2-6*m3**4))*
     -     A0(m3**2)+A0(m1**2)*(-((2*m1**6+m1**4*m2**2-6*m1**2*m2**4+
     -     3*m2**6)*m3**2)+2*(3*m1**4+4*m1**2*m2**2+4*m2**4)*m3**4-
     -     (6*m1**2+7*m2**2)*m3**6+2*m3**8+(m1**2+m2**2-m3**2)*
     -     (m1**4-3*m1**2*m2**2+2*m2**4-(2*m1**2+3*m2**2)*m3**2+m3**4)*
     -     A0(m3**2))+(m1**2+m2**2-m3**2)*(m1**4+2*m2**4-3*m2**2*m3**2+
     -     m3**4-m1**2*(3*m2**2+2*m3**2))*(m3**2*Ae(m1**2)+m1**2*
     -     Ae(m3**2))))/m1**2+Pi**4*(2*m2**2*m3**2*(m2**4-(m1**2-
     -     m3**2)**2)+m2**2*(3*m1**4-4*m1**2*m2**2+m2**4-2*(3*m1**2+
     -     2*m2**2)*m3**2+3*m3**4)*A0(m3**2)+A0(m2**2)*(m3**2*(3*m1**4-
     -     4*m1**2*m2**2+m2**4-2*(3*m1**2+2*m2**2)*m3**2+3*m3**4)-
     -     (m1**4-4*m1**2*m2**2+3*m2**4-2*(m1**2+2*m2**2)*m3**2+m3**4)*
     -     A0(m3**2))-(m1**4-4*m1**2*m2**2+3*m2**4-2*(m1**2+2*m2**2)*
     -     m3**2+m3**4)*(m3**2*Ae(m2**2)+m2**2*Ae(m3**2)))-(m1-m2-m3)*
     -     (m1+m2-m3)*(m1-m2+m3)*(m1+m2+m3)*(3*m1**2-m2**2-3*m3**2)*
     -     Idiv1(m3**2,m2**2,m1**2)+2*(m1**2+m2**2-m3**2)*(-m1**2+
     -     m2**2+m3**2)**2* Idiv2(m3**2,m2**2,m1**2)+((m1**2-m2**2)**3-
     -     (3*m1**4+m2**4)*m3**2+3*(m1**2+m2**2)*m3**4-m3**6)*
     -     Ifin(m3**2,m2**2,m1**2))/(2.*(m1+m2-m3)**2*(m1-m2+m3)**2*
     -     (-m1+m2+m3)**2*(m1+m2+m3)**2)

      
      if(abs(m1).le.1d-12) write(*,*) '(m1)<0!!'
      if(abs(m1 - m2 - m3).le.1d-12) write(*,*) '(m1 - m2 - m3)<0!!'
      if(abs(m1+m2 - m3).le.1d-12) write(*,*) '(m1+m2 - m3)<0!!'
      if(abs(m1+m2+m3).le.1d-12) write(*,*) '(m1+m2+m3)<0!!'
      if(abs(m1 - m2+m3).le.1d-12) write(*,*) '(m1 - m2+m3)<0!!'
     
      return
      end
      
      
      double complex function TFIdv12div1(m1,m2,m3)
      
      implicit none
     
      double precision m1,m2,m3
      
      double precision A0,Idiv1,Idiv2
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      TFIdv12div1=((-(m2**2 - m3**2)**4+m1**6*(m2**2+m3**2) - 
     -     m1**4*(3*m2**4+4*m2**2*m3**2+3*m3**4)+
     -     m1**2*(3*m2**6 - 5*m2**4*m3**2 - m2**2*m3**4+3*m3**6))*
     -     Pi**4*A0(m1**2)+m1**2*(((-3*m1**2+m2**2)*(-(m1**2*m2)+
     -     m2**3)**2 -2*(m1**6 - 4*m1**4*m2**2 - m1**2*m2**4+2*m2**6)*
     -     m3**2+(6*m1**4 - m1**2*m2**2+5*m2**4)*m3**4 - 2*(3*m1**2+
     -     2*m2**2)*m3**6+2*m3**8)*Pi**4+((m1**2 - m2**2)**3 - 
     -     (3*m1**4+m2**4)*m3**2+3*(m1**2+m2**2)*m3**4 - m3**6)*
     -     (Pi**4*(A0(m2**2)+A0(m3**2))+Idiv1(m3**2,m2**2,m1**2)) - 
     -     (m1 - m2 - m3)*(m1+m2 - m3)*(m1 - m2+m3)*(m1+m2+m3)*
     -     (3*m1**2 - m2**2 - 3*m3**2)*Idiv2(m3**2,m2**2,m1**2)))/
     -     (2.*m1**2*(m1+m2 - m3)**2*(m1 - m2+m3)**2*(-m1+m2+m3)**2*
     -     (m1+m2+m3)**2)
     
      return
      end
      
      
      double complex function TFIdv12div2(m1,m2,m3)
      
      implicit none
     
      double precision m1,m2,m3
      
      double precision Idiv2
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      TFIdv12div2=((m2**2*(m1**2 - m2**2)**3+(m1**6 - 5*m1**4*m2**2 -
     -     m1**2*m2**4+m2**6)*m3**2+(-3*m1**4+m1**2*m2**2 - 2*m2**4)*
     -     m3**4+3*(m1**2+m2**2)*m3**6 - m3**8)*Pi**4+((m1**2 - 
     -     m2**2)**3 - (3*m1**4+m2**4)*m3**2+3*(m1**2+m2**2)*m3**4 - 
     -     m3**6)*Idiv2(m3**2,m2**2,m1**2))/(2.*(m1+m2 - m3)**2*(m1 - 
     -     m2+m3)**2*(-m1+m2+m3)**2*(m1+m2+m3)**2)
     
      return
      end
      
      
      
c      
c myTFI[D, 0, {1, 0, 0, 0,1}, {{3, m1_}, {1, m2_}, {0, 0}, {0, 0}, {1, m3_}}] := 
c TFIdv13fin[m1, m2, m3]+TFIdv13div1[m1, m2, m3]*pol+TFIdv13div2[m1, m2, m3]*pol^2     
c      
      double complex function TFIdv13fin(m1,m2,m3)
      
      implicit none
      

      
      double precision m1,m2,m3
      
      double complex Ifin
      
      double precision Idiv1,Idiv2,Ae,A0
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      TFIdv13fin=(Pi**4*(2*m1**4*m2**2*(-m1**2+m2**2+m3**2)**2 - 
     -     m1**2*((5*m1**2 - 3*m2**2)*(m1**2 - m2**2)**2+
     -     4*(-3*m1**4+m1**2*m2**2+m2**4)*m3**2+(9*m1**2+m2**2)*m3**4 - 
     -     2*m3**6)*A0(m2**2)+A0(m1**2)*(m2**2*((m1**2 - m2**2)**2*
     -     (-5*m1**2+3*m2**2)-4*(-3*m1**4+m1**2*m2**2+m2**4)*m3**2 - 
     -     (9*m1**2+m2**2)*m3**4+2*m3**6)+(3*(m1**2 - m2**2)**3+4*
     -     (-2*m1**4+m2**4)*m3**2+(7*m1**2+m2**2)*m3**4 - 2*m3**6)*
     -     A0(m2**2))+(3*(m1**2 - m2**2)**3+4*(-2*m1**4+m2**4)*m3**2+
     -     (7*m1**2+m2**2)*m3**4 - 2*m3**6)*(m2**2*Ae(m1**2)+m1**2*
     -     Ae(m2**2)))+Pi**4*(m1**2*m3**2*(m1**6+3*m1**4*(m2 - m3)*
     -     (m2+m3)+(m2**2 - m3**2)**3+m1**2*(-5*m2**4 - 6*m2**2*m3**2+
     -     3*m3**4))+m1**2*(-2*m1**6 - (7*m2**2 - 2*m3**2)*(m2**2 - 
     -     m3**2)**2+m1**4*(-5*m2**2+6*m3**2)+2*m1**2*(7*m2**4+8*m2**2*
     -     m3**2 - 3*m3**4))*A0(m3**2)+A0(m1**2)*
     -     (m3**2*(-2*m1**6 - (7*m2**2 - 2*m3**2)*(m2**2 - m3**2)**2+
     -     m1**4*(-5*m2**2+6*m3**2)+2*m1**2*(7*m2**4+8*m2**2*m3**2 - 
     -     3*m3**4))+(m1**6+m1**4*(2*m2**2 - 3*m3**2)+(m2**2-m3**2)**2*
     -     (6*m2**2 - m3**2)+m1**2*(-9*m2**4-10*m2**2*m3**2+3*m3**4))*
     -     A0(m3**2))+(m1**6+m1**4*(2*m2**2-3*m3**2)+(m2**2-m3**2)**2*
     -     (6*m2**2 - m3**2)+m1**2*(-9*m2**4-10*m2**2*m3**2+3*m3**4))*
     -     (m3**2*Ae(m1**2)+m1**2*Ae(m3**2)))+Pi**4*(-2*m1**2*m2**2*
     -     m3**2*(-m2**4+(m1**2 - m3**2)**2)+m2**2*(5*m1**6+2*(m2**2 - 
     -     m3**2)**3 - 6*m1**4*(m2**2+2*m3**2) -m1**2*(m2**4+8*m2**2*
     -     m3**2 - 9*m3**4))*A0(m3**2)+A0(m2**2)*(m3**2*(5*m1**6+2*
     -     (m2**2 - m3**2)**3 -6*m1**4*(m2**2+2*m3**2) - m1**2*
     -     (m2**4+8*m2**2*m3**2 - 9*m3**4))+(-3*m1**6 - 2*(m2**2 - 
     -     m3**2)**3+m1**4*(6*m2**2+8*m3**2) -m1**2*(m2**4 - 8*m2**2*
     -     m3**2+7*m3**4))*A0(m3**2)) -(3*m1**6+2*(m2**2 - m3**2)**3 -
     -     2*m1**4*(3*m2**2+4*m3**2)+m1**2*(m2**4-8*m2**2*m3**2+7*
     -     m3**4))*(m3**2*Ae(m2**2)+m2**2*Ae(m3**2))) - 
     -     (m1 - m2 - m3)*(m1+m2 - m3)*(m1 - m2+m3)*(m1+m2+m3)*
     -     (5*m1**4 - 2*m2**4+2*m3**4 - m1**2*(m2**2+7*m3**2))*
     -     Idiv1(m3**2,m2**2,m1**2)+2*m1**2*(m1**2+m2**2 - m3**2)*
     -     (-m1**2+m2**2+m3**2)**2*Idiv2(m3**2,m2**2,m1**2)+
     -     (3*m1**8+2*m3**2*(-m2**2+m3**2)**3-m1**6*(9*m2**2+11*m3**2)- 
     -     3*m1**2*(m2 - m3)*(m2+m3)*(m2**4 -3*m3**4)+3*m1**4*
     -     (3*m2**4+2*m2**2*m3**2+5*m3**4))*Ifin(m3**2,m2**2,m1**2))/
     -    (2.*(m1+m2-m3)**2*(m1-m2+m3)**2*(-m1+m2+m3)**2*(m1+m2+m3)**2)
     
      if(abs(m1 - m2 - m3).le.1d-12) write(*,*) '(m1 - m2 - m3)<0!!'
      if(abs(m1+m2 - m3).le.1d-12) write(*,*) '(m1+m2 - m3)<0!!'
      if(abs(m1+m2+m3).le.1d-12) write(*,*) '(m1+m2+m3)<0!!'
      if(abs(m1 - m2+m3).le.1d-12) write(*,*) '(m1 - m2+m3)<0!!'
     
      return
      end
      
      
      double complex function TFIdv13div1(m1,m2,m3)
      
      implicit none
      

      
      double precision m1,m2,m3
      
      double precision Idiv1,Idiv2,A0
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      TFIdv13div1= ((2*m2**2*m3**2*(m2**2 - m3**2)**3-m1**8*(5*m2**2+
     -     2*m3**2)+m1**6*(13*m2**4+12*m2**2*m3**2+6*m3**4) - 
     -     m1**4*(11*m2**6 - 4*m2**4*m3**2+5*m2**2*m3**4+6*m3**6)+
     -     m1**2*(3*m2**8-12*m2**6*m3**2+7*m2**4*m3**4+2*m3**8))*Pi**4+
     -     (-((m2**2-m3**2)**3*(3*m2**2-m3**2))+m1**6*(3*m2**2+m3**2)+
     -     3*m1**2*(m2 - m3)*(m2+m3)*(3*m2**4 - m3**4) - 
     -     3*m1**4*(3*m2**4+2*m2**2*m3**2+m3**4))*Pi**4*A0(m1**2)+
     -     (3*m1**8+2*m3**2*(-m2**2+m3**2)**3-m1**6*(9*m2**2+11*m3**2)- 
     -     3*m1**2*(m2 - m3)*(m2+m3)*(m2**4 - 3*m3**4)+
     -     3*m1**4*(3*m2**4+2*m2**2*m3**2+5*m3**4))*Pi**4*A0(m2**2)+
     -     m1**8*Pi**4*A0(m3**2) - m1**6*m2**2*Pi**4*A0(m3**2) - 
     -     3*m1**4*m2**4*Pi**4*A0(m3**2)+5*m1**2*m2**6*Pi**4*A0(m3**2)- 
     -     2*m2**8*Pi**4*A0(m3**2) - 3*m1**6*m3**2*Pi**4*A0(m3**2) - 
     -     2*m1**4*m2**2*m3**2*Pi**4*A0(m3**2) - 5*m1**2*m2**4*m3**2*
     -     Pi**4*A0(m3**2)+6*m2**6*m3**2*Pi**4*A0(m3**2)+3*m1**4*m3**4*
     -     Pi**4*A0(m3**2)+m1**2*m2**2*m3**4*Pi**4*A0(m3**2) - 6*m2**4*
     -     m3**4*Pi**4*A0(m3**2)-m1**2*m3**6*Pi**4*A0(m3**2)+2*m2**2*
     -     m3**6*Pi**4*A0(m3**2)+3*m1**8*Idiv1(m3**2,m2**2,m1**2) - 
     -     9*m1**6*m2**2*Idiv1(m3**2,m2**2,m1**2)+
     -     9*m1**4*m2**4*Idiv1(m3**2,m2**2,m1**2) - 
     -     3*m1**2*m2**6*Idiv1(m3**2,m2**2,m1**2) - 
     -     11*m1**6*m3**2*Idiv1(m3**2,m2**2,m1**2)+
     -     6*m1**4*m2**2*m3**2*Idiv1(m3**2,m2**2,m1**2)+
     -     3*m1**2*m2**4*m3**2*Idiv1(m3**2,m2**2,m1**2) - 
     -     2*m2**6*m3**2*Idiv1(m3**2,m2**2,m1**2)+
     -     15*m1**4*m3**4*Idiv1(m3**2,m2**2,m1**2)+
     -     9*m1**2*m2**2*m3**4*Idiv1(m3**2,m2**2,m1**2)+
     -     6*m2**4*m3**4*Idiv1(m3**2,m2**2,m1**2) - 
     -     9*m1**2*m3**6*Idiv1(m3**2,m2**2,m1**2) - 
     -     6*m2**2*m3**6*Idiv1(m3**2,m2**2,m1**2)+2*m3**8*
     -     Idiv1(m3**2,m2**2,m1**2)-(m1 - m2 - m3)*(m1+m2 - m3)*
     -     (m1 - m2+m3)*(m1+m2+m3)*(5*m1**4 - 2*m2**4+2*m3**4 - m1**2*
     -     (m2**2+7*m3**2))*Idiv2(m3**2,m2**2,m1**2))/(2.*
     -     (m1+m2-m3)**2*(m1 - m2+m3)**2*(-m1+m2+m3)**2*(m1+m2+m3)**2)
     
      return
      end
      
      
      double complex function TFIdv13div2(m1,m2,m3)
      
      implicit none
     
      double precision m1,m2,m3
      
      double precision Idiv2
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      TFIdv13div2=((-2*m2**2*m3**2*(m2**2 - m3**2)**3+m1**8*(3*m2**2+
     -     m3**2) -3*m1**6*(3*m2**4+3*m2**2*m3**2+m3**4)+
     -     m1**4*(9*m2**6 - 3*m2**4*m3**2+5*m2**2*m3**4+3*m3**6) - 
     -     m1**2*(3*m2**8 - 9*m2**6*m3**2+4*m2**4*m3**4+m2**2*m3**6+
     -     m3**8))*Pi**4+(3*m1**8+2*m3**2*(-m2**2+m3**2)**3 - m1**6*
     -     (9*m2**2+11*m3**2)-3*m1**2*(m2-m3)*(m2+m3)*(m2**4-3*m3**4)+
     -     3*m1**4*(3*m2**4+2*m2**2*m3**2+5*m3**4))*
     -     Idiv2(m3**2,m2**2,m1**2))/(2.*(m1+m2 - m3)**2*
     -     (m1 - m2+m3)**2*(-m1+m2+m3)**2*(m1+m2+m3)**2)
     
      return
      end
      
      
      double complex function TFIdv14fin(m1,m2,m3)
      
      implicit none
      
      double precision A0
      
      double precision m1,m2,m3
      
      double complex Ifin
      
      double precision Idiv1,Ae
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      TFIdv14fin=-((2*Pi**4*(A0(m1**2)*(-m2**2+A0(m2**2))+m2**2*
     -     Ae(m1**2)+m1**2*(-A0(m2**2)+Ae(m2**2)))+
     -     ((-m1**2+m2**2+m3**2)*Pi**4*(A0(m1**2)*(m3**2 - A0(m3**2))
     -     - m3**2*Ae(m1**2)+m1**2*(A0(m3**2) - Ae(m3**2))))/m3**2+
     -     ((m1**2 - m2**2+m3**2)*Pi**4*
     -     (A0(m2**2)*(m3**2 - A0(m3**2)) - m3**2*Ae(m2**2)+
     -     m2**2*(A0(m3**2) - Ae(m3**2))))/m3**2 - 
     -     (m1**2+m2**2 - m3**2)*
     -     (2*Idiv1(m3**2,m2**2,m1**2) - Ifin(m3**2,m2**2,m1**2)))/
     -     ((m1 - m2 - m3)*(m1+m2 - m3)*(m1 - m2+m3)*(m1+m2+m3)))
      
      return
      end
     
      
      double complex function TFIdv15fin(m1,m2,m3)
      
      implicit none
      
      double precision A0
      
      double precision m1,m2,m3
      
      double complex Ifin
      
      double precision Ae
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      TFIdv15fin=-(m1**2*Pi**4*A0(m1**2)*A0(m3**2)) - m2**2*Pi**4*
     -  A0(m2**2)*A0(m3**2) - 
     -  m1**2*m3**2*Pi**4*Ae(m1**2) - m2**2*m3**2*Pi**4*Ae(m2**2) - 
     -  m1**4*Pi**4*Ae(m3**2) - m2**4*Pi**4*Ae(m3**2)+
     -  m1**2*m2**2*Ifin(m3**2,m2**2,m1**2)
      
      return
      end
      
      
      double complex function TFIdv15div1(m1,m2,m3)
      
      implicit none
      
      double precision A0
      
      double precision m1,m2,m3
      
      double precision Idiv1
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      TFIdv15div1=-(m1**2*m3**2*Pi**4*A0(m1**2)) - m2**2*m3**2*Pi**4*
     -     A0(m2**2) -m1**4*Pi**4*A0(m3**2) - m2**4*Pi**4*A0(m3**2)+
     -     m1**2*m2**2*Idiv1(m3**2,m2**2,m1**2)
     
      return
      end
      
      
      double complex function TFIdv15div2(m1,m2,m3)
      
      implicit none
      
      double precision m1,m2,m3
      
      double precision Idiv2
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      TFIdv15div2=-((m1**4+m2**4)*m3**2*Pi**4)+m1**2*m2**2*
     -     Idiv2(m3**2,m2**2,m1**2)
     
      return
      end
      
      
      double complex function TFIdv16fin(m1,m2,m3)
      
      implicit none
      
      double precision A0
      
      double precision m1,m2,m3
      
      double complex Ifin
      
      double precision Idiv1,Ae
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      if(m1.eq.m3) then
      TFIdv16fin=((-8*m1**2 + m2**2)*Pi**4*A0(m1**2)**2-m1**2*m2**2*
     -    Pi**4*A0(m2**2)+Pi**4*A0(m1**2)*(8*m1**4-m2**4+m2**2*
     -    A0(m2**2))-16*m1**4*Pi**4*Ae(m1**2) + 2*m1**2*m2**2*Pi**4*
     -    Ae(m1**2)+m2**4*Pi**4*Ae(m1**2)+m1**2*m2**2*Pi**4*Ae(m2**2)- 
     -    2*m1**2*m2**2*Idiv1(m2**2,m1**2,m1**2)-m2**2*(-5*m1**2+
     -    m2**2)*Ifin(m2**2,m1**2,m1**2))/(4*m1**2 - m2**2)
      else
      TFIdv16fin=(m2**2*(-m1**2+m2**2-m3**2)*Pi**4*
     -    (A0(m1**2)*(m2**2-A0(m2**2))+m1**2*A0(m2**2)-m2**2*Ae(m1**2)-
     -    m1**2*Ae(m2**2))+2*m1**2*m2**2*Pi**4*
     -    (A0(m2**2)*(m3**2-A0(m3**2))+m2**2*A0(m3**2)-m3**2*Ae(m2**2)-
     -    m2**2*Ae(m3**2))+Pi**4*(m1**2*(m1**4-m2**2*m3**2+m3**4-m1**2*
     -    (3*m2**2+2*m3**2))*A0(m3**2)+A0(m1**2)*(m3**2*(m1**4-m2**2*
     -    m3**2+m3**4-m1**2*(3*m2**2+2*m3**2))-(2*m1**4+m2**4-3*m2**2*
     -    m3**2+2*m3**4-m1**2*(5*m2**2+4*m3**2))*A0(m3**2))-(2*m1**4+
     -    m2**4-3*m2**2*m3**2+2*m3**4-m1**2*(5*m2**2+4*m3**2))*(m3**2*
     -    Ae(m1**2)+m1**2*Ae(m3**2)))+m2**2*(2*m1**2*(-m1**2+m2**2+
     -    m3**2)*Idiv1(m3**2,m2**2,m1**2)+(2*m1**4+(m2**2-m3**2)**2-
     -    3*m1**2*(m2**2+m3**2))*Ifin(m3**2,m2**2,m1**2)))/
     -    ((m1-m2-m3)*(m1+m2-m3)*(m1-m2+m3)*(m1+m2+m3))
      end if
      
      return
      end
      
      
      double complex function TFIdv16div1(m1,m2,m3)
      
      implicit none
      
      double precision A0
      
      double precision m1,m2,m3
      
      double precision Idiv1, Idiv2
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      if(m1.eq.m3) then
      TFIdv16div1=(m1**2*(4*m1**4 - m2**4)*Pi**4 + 
     -    (-16*m1**4 + 2*m1**2*m2**2 + m2**4)*Pi**4*A0(m1**2) + 
     -    m2**2*(m1**2*Pi**4*A0(m2**2) + (5*m1**2 - m2**2)*
     -    Idiv1(m2**2,m1**2,m1**2)-2*m1**2*Idiv2(m2**2,m1**2,m1**2)))/
     -    (4*m1**2 - m2**2)
      else
      TFIdv16div1=(-(m1**4*m2**4*Pi**4)+m1**2*m2**6*Pi**4+m1**6*m3**2*
     -     Pi**4-3*m1**4*m2**2*m3**2*Pi**4+m1**2*m2**4*m3**2*Pi**4-
     -     2*m1**4*m3**4*Pi**4-m1**2*m2**2*m3**4*Pi**4+m1**2*m3**6*
     -     Pi**4+(-m2**6-2*m1**4*m3**2+3*m2**2*m3**4-2*m3**6+
     -     m1**2*(m2**4+5*m2**2*m3**2+4*m3**4))*Pi**4*A0(m1**2)+
     -     m1**2*m2**2*(m1**2-m2**2-m3**2)*Pi**4*A0(m2**2)-
     -     2*m1**6*Pi**4*A0(m3**2)+5*m1**4*m2**2*Pi**4*A0(m3**2)-
     -     3*m1**2*m2**4*Pi**4*A0(m3**2)+4*m1**4*m3**2*Pi**4*A0(m3**2)+
     -     3*m1**2*m2**2*m3**2*Pi**4*A0(m3**2)-2*m1**2*m3**4*Pi**4*
     -     A0(m3**2)+2*m1**4*m2**2*Idiv1(m3**2,m2**2,m1**2)-
     -     3*m1**2*m2**4*Idiv1(m3**2,m2**2,m1**2)+m2**6*
     -     Idiv1(m3**2,m2**2,m1**2)-3*m1**2*m2**2*m3**2*
     -     Idiv1(m3**2,m2**2,m1**2)-2*m2**4*m3**2*
     -     Idiv1(m3**2,m2**2,m1**2)+m2**2*m3**4*
     -     Idiv1(m3**2,m2**2,m1**2)-2*m1**4*m2**2*
     -     Idiv2(m3**2,m2**2,m1**2)+2*m1**2*m2**4*
     -     Idiv2(m3**2,m2**2,m1**2)+2*m1**2*m2**2*m3**2*
     -     Idiv2(m3**2,m2**2,m1**2))/((m1-m2-m3)*(m1+m2-m3)*
     -     (m1-m2+m3)*(m1+m2+m3))
      end if
     
      return
      end
      
      
      double complex function TFIdv16div2(m1,m2,m3)
      
      implicit none
      
      double precision m1,m2,m3
      
      double precision Idiv2
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      if(m1.eq.m3) then
      TFIdv16div2=(m1**2*(-8*m1**4 + m1**2*m2**2 + m2**4)*Pi**4 + 
     -    (5*m1**2*m2**2 - m2**4)*Idiv2(m2**2,m1**2,m1**2))/
     -    (4*m1**2 - m2**2)
      else
      TFIdv16div2=(m1**2*(-m2**6 - 2*m1**4*m3**2 - 2*m2**4*m3**2+
     -     3*m2**2*m3**4 - 2*m3**6+m1**2*(m2**4+5*m2**2*m3**2+
     -     4*m3**4))*Pi**4+m2**2*(2*m1**4+(m2**2 - m3**2)**2 - 
     -     3*m1**2*(m2**2+m3**2))*Idiv2(m3**2,m2**2,m1**2))/
     -     ((m1 - m2 - m3)*(m1+m2 - m3)*(m1 - m2+m3)*(m1+m2+m3))
      end if
     
      return
      end
      
      
      double complex function TFIdv17fin(m1,m2,m3)
      
      implicit none
      
      double precision A0
      
      double precision m1,m2,m3
      
      double complex Ifin
      
      double precision Idiv1,Ae,Idiv2
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      TFIdv17fin=(-2*m1**8*m2**2*Pi**4+2*m1**6*m2**4*Pi**4+2*m1**4*
     -     m2**6*Pi**4-2*m1**2*m2**8*Pi**4-2*m1**8*m3**2*Pi**4+2*
     -     m1**6*m2**2*m3**2*Pi**4+2*m1**2*m2**6*m3**2*Pi**4-2*m2**8*
     -     m3**2*Pi**4+6*m1**6*m3**4*Pi**4+4*m1**4*m2**2*m3**4*Pi**4+
     -     4*m1**2*m2**4*m3**4*Pi**4+6*m2**6*m3**4*Pi**4-6*m1**4*m3**6*
     -     Pi**4-6*m1**2*m2**2*m3**6*Pi**4-6*m2**4*m3**6*Pi**4+2*m1**2*
     -     m3**8*Pi**4+2*m2**2*m3**8*Pi**4+3*m1**8*Pi**4*A0(m3**2)-
     -     4*m1**6*m2**2*Pi**4*A0(m3**2)+2*m1**4*m2**4*Pi**4*A0(m3**2)-
     -     4*m1**2*m2**6*Pi**4*A0(m3**2)+3*m2**8*Pi**4*A0(m3**2)-
     -     9*m1**6*m3**2*Pi**4*A0(m3**2)-3*m1**4*m2**2*m3**2*Pi**4*
     -     A0(m3**2)-3*m1**2*m2**4*m3**2*Pi**4*A0(m3**2)-9*m2**6*m3**2*
     -     Pi**4*A0(m3**2)+9*m1**4*m3**4*Pi**4*A0(m3**2)+10*m1**2*
     -     m2**2*m3**4*Pi**4*A0(m3**2)+9*m2**4*m3**4*Pi**4*A0(m3**2)-
     -     3*m1**2*m3**6*Pi**4*A0(m3**2)-3*m2**2*m3**6*Pi**4*A0(m3**2)-
     -     Pi**4*A0(m2**2)*(-2*m1**8+3*m3**2*(-m2**2+m3**2)**3+
     -     m1**6*(2*m2**2+3*m3**2)+m1**4*(2*m2**4+3*m2**2*m3**2+3*
     -     m3**4)+m1**2*(-2*m2**6+5*m2**4*m3**2+4*m2**2*m3**4-7*m3**6)+
     -     (m1**6+3*m1**4*(m2**2-m3**2)+(m2**2-m3**2)**3+m1**2*(-5*
     -     m2**4+2*m2**2*m3**2+3*m3**4))*A0(m3**2))-Pi**4*A0(m1**2)*
     -     (-2*m1**6*m2**2+2*m1**4*m2**4+2*m1**2*m2**6-2*m2**8-
     -     3*m1**6*m3**2+5*m1**4*m2**2*m3**2+3*m1**2*m2**4*m3**2+3*
     -     m2**6*m3**2+9*m1**4*m3**4+4*m1**2*m2**2*m3**4+3*m2**4*m3**4-
     -     9*m1**2*m3**6-7*m2**2*m3**6+3*m3**8-8*m1**2*m2**2*m3**2*
     -     A0(m2**2)+(m1**6+(m2**2-m3**2)**3-m1**4*(5*m2**2+3*m3**2)+
     -     m1**2*(3*m2**4+2*m2**2*m3**2+3*m3**4))*A0(m3**2))-
     -     m1**6*m3**2*Pi**4*Ae(m1**2)+5*m1**4*m2**2*m3**2*Pi**4*
     -     Ae(m1**2)+5*m1**2*m2**4*m3**2*Pi**4*Ae(m1**2)-m2**6*m3**2*
     -     Pi**4*Ae(m1**2)+3*m1**4*m3**4*Pi**4*Ae(m1**2)-2*m1**2*m2**2*
     -     m3**4*Pi**4*Ae(m1**2)+3*m2**4*m3**4*Pi**4*Ae(m1**2)-3*m1**2*
     -     m3**6*Pi**4*Ae(m1**2)-3*m2**2*m3**6*Pi**4*Ae(m1**2)+m3**8*
     -     Pi**4*Ae(m1**2)-m1**6*m3**2*Pi**4*Ae(m2**2)+5*m1**4*m2**2*
     -     m3**2*Pi**4*Ae(m2**2)+5*m1**2*m2**4*m3**2*Pi**4*Ae(m2**2)-
     -     m2**6*m3**2*Pi**4*Ae(m2**2)+3*m1**4*m3**4*Pi**4*Ae(m2**2)-
     -     2*m1**2*m2**2*m3**4*Pi**4*Ae(m2**2)+3*m2**4*m3**4*Pi**4*
     -     Ae(m2**2)-3*m1**2*m3**6*Pi**4*Ae(m2**2)-3*m2**2*m3**6*Pi**4*
     -     Ae(m2**2)+m3**8*Pi**4*Ae(m2**2)-m1**8*Pi**4*Ae(m3**2)+
     -     4*m1**6*m2**2*Pi**4*Ae(m3**2)-6*m1**4*m2**4*Pi**4*Ae(m3**2)+
     -     4*m1**2*m2**6*Pi**4*Ae(m3**2)-m2**8*Pi**4*Ae(m3**2)+3*m1**6*
     -     m3**2*Pi**4*Ae(m3**2)+m1**4*m2**2*m3**2*Pi**4*Ae(m3**2)+
     -     m1**2*m2**4*m3**2*Pi**4*Ae(m3**2)+3*m2**6*m3**2*Pi**4*
     -     Ae(m3**2)-3*m1**4*m3**4*Pi**4*Ae(m3**2)-6*m1**2*m2**2*m3**4*
     -     Pi**4*Ae(m3**2)-3*m2**4*m3**4*Pi**4*Ae(m3**2)+m1**2*m3**6*
     -     Pi**4*Ae(m3**2)+m2**2*m3**6*Pi**4*Ae(m3**2)+m1**8*
     -     Idiv1(m3**2,m2**2,m1**2)-2*m1**4*m2**4*
     -     Idiv1(m3**2,m2**2,m1**2)+m2**8*Idiv1(m3**2,m2**2,m1**2)-
     -     8*m1**4*m2**2*m3**2*Idiv1(m3**2,m2**2,m1**2)-
     -     8*m1**2*m2**4*m3**2*Idiv1(m3**2,m2**2,m1**2)-
     -     6*m1**4*m3**4*Idiv1(m3**2,m2**2,m1**2)-
     -     6*m2**4*m3**4*Idiv1(m3**2,m2**2,m1**2)+
     -     8*m1**2*m3**6*Idiv1(m3**2,m2**2,m1**2)+
     -     8*m2**2*m3**6*Idiv1(m3**2,m2**2,m1**2)-
     -     3*m3**8*Idiv1(m3**2,m2**2,m1**2)-2*m1**8*
     -     Idiv2(m3**2,m2**2,m1**2)+4*m1**4*m2**4*
     -     Idiv2(m3**2,m2**2,m1**2)- 2*m2**8*Idiv2(m3**2,m2**2,m1**2)+
     -     4*m1**6*m3**2*Idiv2(m3**2,m2**2,m1**2)-
     -     4*m1**4*m2**2*m3**2*Idiv2(m3**2,m2**2,m1**2)-
     -     4*m1**2*m2**4*m3**2*Idiv2(m3**2,m2**2,m1**2)+
     -     4*m2**6*m3**2*Idiv2(m3**2,m2**2,m1**2)+
     -     8*m1**2*m2**2*m3**4*Idiv2(m3**2,m2**2,m1**2)-
     -     4*m1**2*m3**6*Idiv2(m3**2,m2**2,m1**2)-
     -     4*m2**2*m3**6*Idiv2(m3**2,m2**2,m1**2)+2*m3**8*
     -     Idiv2(m3**2,m2**2,m1**2)+m1**8*Ifin(m3**2,m2**2,m1**2)-
     -     4*m1**6*m2**2*Ifin(m3**2,m2**2,m1**2)+6*m1**4*m2**4*
     -     Ifin(m3**2,m2**2,m1**2)-4*m1**2*m2**6*
     -     Ifin(m3**2,m2**2,m1**2)+m2**8*Ifin(m3**2,m2**2,m1**2)-
     -     5*m1**6*m3**2*Ifin(m3**2,m2**2,m1**2)+
     -     9*m1**4*m2**2*m3**2*Ifin(m3**2,m2**2,m1**2)+
     -     9*m1**2*m2**4*m3**2*Ifin(m3**2,m2**2,m1**2)-
     -     5*m2**6*m3**2*Ifin(m3**2,m2**2,m1**2)+
     -     9*m1**4*m3**4*Ifin(m3**2,m2**2,m1**2)+
     -     2*m1**2*m2**2*m3**4*Ifin(m3**2,m2**2,m1**2)+
     -     9*m2**4*m3**4*Ifin(m3**2,m2**2,m1**2)-
     -     7*m1**2*m3**6*Ifin(m3**2,m2**2,m1**2)-
     -     7*m2**2*m3**6*Ifin(m3**2,m2**2,m1**2)+2*m3**8*
     -     Ifin(m3**2,m2**2,m1**2))/(2.*(m1+m2-m3)**2*(m1-m2+m3)**2*
     -     (-m1+m2+m3)**2*(m1+m2+m3)**2)
     
      return
      end
      
      
      double complex function TFIdv17div1(m1,m2,m3)
      
      implicit none
      
      double precision A0
      
      double precision m1,m2,m3
      
      double precision Idiv1, Idiv2
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      TFIdv17div1=(2*m1**8*m2**2*Pi**4-2*m1**6*m2**4*Pi**4-2*m1**4*
     -    m2**6*Pi**4+2*m1**2*m2**8*Pi**4+3*m1**8*m3**2*Pi**4-6*
     -    m1**6*m2**2*m3**2*Pi**4-2*m1**4*m2**4*m3**2*Pi**4-6*m1**2*
     -    m2**6*m3**2*Pi**4+3*m2**8*m3**2*Pi**4-9*m1**6*m3**4*Pi**4-
     -    5*m1**4*m2**2*m3**4*Pi**4-5*m1**2*m2**4*m3**4*Pi**4-
     -    9*m2**6*m3**4*Pi**4+9*m1**4*m3**6*Pi**4+12*m1**2*m2**2*m3**6*
     -    Pi**4+9*m2**4*m3**6*Pi**4-3*m1**2*m3**8*Pi**4-3*m2**2*m3**8*
     -    Pi**4-m3**2*(m1**6+(m2**2-m3**2)**3-m1**4*(5*m2**2+3*m3**2)+
     -    m1**2*(-5*m2**4+2*m2**2*m3**2+3*m3**4))*Pi**4*A0(m1**2)-
     -    m3**2*(m1**6+(m2**2-m3**2)**3-m1**4*(5*m2**2+3*m3**2)+
     -    m1**2*(-5*m2**4+2*m2**2*m3**2+3*m3**4))*Pi**4*A0(m2**2)-
     -    m1**8*Pi**4*A0(m3**2)+4*m1**6*m2**2*Pi**4*A0(m3**2)-
     -    6*m1**4*m2**4*Pi**4*A0(m3**2)+4*m1**2*m2**6*Pi**4*A0(m3**2)-
     -    m2**8*Pi**4*A0(m3**2)+3*m1**6*m3**2*Pi**4*A0(m3**2)+
     -    m1**4*m2**2*m3**2*Pi**4*A0(m3**2)+m1**2*m2**4*m3**2*Pi**4*
     -    A0(m3**2)+3*m2**6*m3**2*Pi**4*A0(m3**2)-3*m1**4*m3**4*Pi**4*
     -    A0(m3**2)-6*m1**2*m2**2*m3**4*Pi**4*A0(m3**2)-3*m2**4*m3**4*
     -    Pi**4*A0(m3**2)+m1**2*m3**6*Pi**4*A0(m3**2)+m2**2*m3**6*
     -    Pi**4*A0(m3**2)+m1**8*Idiv1(m3**2,m2**2,m1**2)-4*m1**6*m2**2*
     -    Idiv1(m3**2,m2**2,m1**2)+6*m1**4*m2**4*
     -    Idiv1(m3**2,m2**2,m1**2)-4*m1**2*m2**6*
     -    Idiv1(m3**2,m2**2,m1**2)+m2**8*Idiv1(m3**2,m2**2,m1**2)-
     -    5*m1**6*m3**2*Idiv1(m3**2,m2**2,m1**2)+
     -    9*m1**4*m2**2*m3**2*Idiv1(m3**2,m2**2,m1**2)+
     -    9*m1**2*m2**4*m3**2*Idiv1(m3**2,m2**2,m1**2)-
     -    5*m2**6*m3**2*Idiv1(m3**2,m2**2,m1**2)+
     -    9*m1**4*m3**4*Idiv1(m3**2,m2**2,m1**2)+
     -    2*m1**2*m2**2*m3**4*Idiv1(m3**2,m2**2,m1**2)+
     -    9*m2**4*m3**4*Idiv1(m3**2,m2**2,m1**2)-
     -    7*m1**2*m3**6*Idiv1(m3**2,m2**2,m1**2)-
     -    7*m2**2*m3**6*Idiv1(m3**2,m2**2,m1**2)+2*m3**8*
     -    Idiv1(m3**2,m2**2,m1**2)+m1**8*Idiv2(m3**2,m2**2,m1**2)-
     -    2*m1**4*m2**4*Idiv2(m3**2,m2**2,m1**2)+m2**8*
     -    Idiv2(m3**2,m2**2,m1**2)-8*m1**4*m2**2*m3**2*
     -    Idiv2(m3**2,m2**2,m1**2)-8*m1**2*m2**4*m3**2*
     -    Idiv2(m3**2,m2**2,m1**2)-6*m1**4*m3**4*
     -    Idiv2(m3**2,m2**2,m1**2)-6*m2**4*m3**4*
     -    Idiv2(m3**2,m2**2,m1**2)+8*m1**2*m3**6*
     -    Idiv2(m3**2,m2**2,m1**2)+8*m2**2*m3**6*
     -    Idiv2(m3**2,m2**2,m1**2)-3*m3**8*Idiv2(m3**2,m2**2,m1**2))/
     -    (2.*(m1+m2-m3)**2*(m1-m2+m3)**2*(-m1+m2+m3)**2*(m1+m2+m3)**2)
     
      return
      end
      
      double complex function TFIdv17div2(m1,m2,m3)
      
      implicit none
      
      double precision m1,m2,m3
      
      double precision Idiv2
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      TFIdv17div2=(m3**2*(-m1**8-m2**2*(m2**2-m3**2)**3+m1**6*(4*m2**2+
     -    3*m3**2)+m1**4*(2*m2**4+m2**2*m3**2-3*m3**4)+
     -    m1**2*(4*m2**6+m2**4*m3**2-6*m2**2*m3**4+m3**6))*Pi**4+
     -    (m1**8+(m2**2-2*m3**2)*(m2**2-m3**2)**3-m1**6*(4*m2**2+5*
     -    m3**2)+m1**4*(6*m2**4+9*m2**2*m3**2+9*m3**4)+
     -    m1**2*(-4*m2**6+9*m2**4*m3**2+2*m2**2*m3**4-7*m3**6))*
     -    Idiv2(m3**2,m2**2,m1**2))/(2.*(m1+m2-m3)**2*(m1-m2+m3)**2*
     -    (-m1+m2+m3)**2*(m1+m2+m3)**2)
     
      return
      end
      
      
      double complex function TFIdv18fin(m1,m2,m3)
      
      implicit none
      
      double precision A0
      
      double precision m1,m2,m3
      
      double complex Ifin
      
      double precision Idiv1,Ae
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      TFIdv18fin=-(Pi**4*(2*m1**2*(m1**4-m2**4-2*m1**2*m3**2+m3**4)*
     -    A0(m2**2)+A0(m1**2)*(2*m2**2*(m1**4-m2**4-2*m1**2*m3**2+
     -    m3**4)+(-3*m1**4+m2**4+2*m2**2*m3**2-3*m3**4+
     -    2*m1**2*(m2**2+3*m3**2))*A0(m2**2))-(3*m1**4-m2**4-2*m2**2*
     -    m3**2+3*m3**4-2*m1**2*(m2**2+3*m3**2))*(m2**2*Ae(m1**2)+
     -    m1**2*Ae(m2**2)))+Pi**4*((-2*m2**2*(m1**2+m2**2-m3**2)**2+
     -    (3*m1**4+2*m1**2*(m2**2-3*m3**2)+3*(m2**2-m3**2)**2)*
     -    A0(m2**2))*A0(m3**2)+m3**2*(-2*(m1**2+m2**2-m3**2)**2*
     -    A0(m2**2)+(3*m1**4+2*m1**2*(m2**2-3*m3**2)+3*(m2**2-
     -    m3**2)**2)*Ae(m2**2))+m2**2*(3*m1**4+2*m1**2*(m2**2-3*m3**2)+
     -    3*(m2**2-m3**2)**2)*Ae(m3**2))+Pi**4*(-2*m1**2*(m1**4-m2**4+
     -    m3**4-2*m1**2*(2*m2**2+m3**2))*A0(m3**2)+A0(m1**2)*(-2*m3**2*
     -    (m1**4-m2**4+m3**4-2*m1**2*(2*m2**2+m3**2))+(5*m1**4+m2**4-
     -    6*m2**2*m3**2+5*m3**4-2*m1**2*(7*m2**2+5*m3**2))*A0(m3**2))+
     -    (5*m1**4+m2**4-6*m2**2*m3**2+5*m3**4-2*m1**2*(7*m2**2+
     -    5*m3**2))*(m3**2*Ae(m1**2)+m1**2*Ae(m3**2)))-
     -    (m1**2+m2**2-m3**2)*(-2*(m1**4-m2**4-2*m1**2*m3**2+m3**4)*
     -    Idiv1(m3**2,m2**2,m1**2)+(3*m1**4+m2**4-4*m2**2*m3**2+
     -    3*m3**4-2*m1**2*(2*m2**2+3*m3**2))*Ifin(m3**2,m2**2,m1**2)))/
     -    (4.*(m1-m2-m3)*(m1+m2-m3)*(m1-m2+m3)*(m1+m2+m3))
     
      return
      end
      
      
      double complex function TFIdv18div1(m1,m2,m3)
      
      implicit none
      
      double precision A0
      
      double precision m1,m2,m3
      
      double precision Idiv1, Idiv2
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      TFIdv18div1=-(Pi**4*(m2**2*(-3*m1**4+m2**4+2*m2**2*m3**2-3*m3**4+
     -    2*m1**2*(m2**2+3*m3**2))*A0(m1**2)+m1**2*(2*m2**2*(m1**4-
     -    m2**4-2*m1**2*m3**2+m3**4)+(-3*m1**4+m2**4+2*m2**2*m3**2-
     -    3*m3**4+2*m1**2*(m2**2+3*m3**2))*A0(m2**2)))+Pi**4*
     -    (m3**2*(3*m1**4+2*m1**2*(m2**2-3*m3**2)+3*(m2**2-m3**2)**2)*
     -    A0(m2**2)+m2**2*(-2*m3**2*(m1**2+m2**2-m3**2)**2+
     -    (3*m1**4+2*m1**2*(m2**2-3*m3**2)+3*(m2**2-m3**2)**2)*
     -    A0(m3**2)))+Pi**4*(m3**2*(5*m1**4+m2**4-6*m2**2*m3**2+
     -    5*m3**4-2*m1**2*(7*m2**2+5*m3**2))*A0(m1**2)+m1**2*(-2*m3**2*
     -    (m1**4-m2**4+m3**4-2*m1**2*(2*m2**2+m3**2))+(5*m1**4+m2**4-
     -    6*m2**2*m3**2+5*m3**4-2*m1**2*(7*m2**2+5*m3**2))*A0(m3**2)))-
     -    (m1**2+m2**2-m3**2)*((3*m1**4+m2**4-4*m2**2*m3**2+3*m3**4-
     -    2*m1**2*(2*m2**2+3*m3**2))*Idiv1(m3**2,m2**2,m1**2)-2*
     -    (m1**4-m2**4-2*m1**2*m3**2+m3**4)*Idiv2(m3**2,m2**2,m1**2)))/
     -    (4.*(m1-m2-m3)*(m1+m2-m3)*(m1-m2+m3)*(m1+m2+m3))
     
      return
      end
      
      double complex function TFIdv18div2(m1,m2,m3)
      
      implicit none
 
      double precision m1,m2,m3
      
      double precision Idiv2
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      TFIdv18div2=((m1**6*(3*m2**2-5*m3**2)-3*m2**2*m3**2*(m2**2-
     -    m3**2)**2+m1**4*(-2*m2**4+5*m2**2*m3**2+10*m3**4)-
     -    m1**2*(m2**6+5*m2**4*m3**2-15*m2**2*m3**4+5*m3**6))*Pi**4+
     -    (3*m1**6+(m2**2-3*m3**2)*(m2**2-m3**2)**2-m1**4*(m2**2+
     -    9*m3**2)-3*m1**2*(m2**4+2*m2**2*m3**2-3*m3**4))*
     -    Idiv2(m3**2,m2**2,m1**2))/
     -    (4.*(m1-m2-m3)*(m1+m2-m3)*(m1-m2+m3)*(m1+m2+m3))
     
      return
      end
      
      
c      
c myTFI[D, 0, {0, 1, 0, 0,0}, {{2, m1_}, {1, m2_}, {0, 0}, {0, 0}, {1, m3_}}] := 
c TFIdv19fin[m1, m2, m3]+TFIdv19div1[m1, m2, m3]*pol+TFIdv19div2[m1, m2, m3]*pol^2     
c      
      double complex function TFIdv19fin(m1,m2,m3)
      
      implicit none
      
      double precision m1,m2,m3
      
      double complex Ifin
      
      double precision A0,Ae,Idiv1
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      TFIdv19fin= ((-2*m2**2*(m1**2-m2**2+m3**2)*Pi**4*
     -     (A0(m1**2)*(m2**2-A0(m2**2))-m2**2*Ae(m1**2)+
     -     m1**2*(A0(m2**2)-Ae(m2**2))))/m1**2+(2*(m1**4-3*m1**2*
     -     m2**2-(2*m1**2+m2**2)*m3**2+m3**4)*Pi**4*
     -     (A0(m1**2)*(m3**2-A0(m3**2))-m3**2*Ae(m1**2)+
     -     m1**2*(A0(m3**2)-Ae(m3**2))))/m1**2+
     -     4*m2**2*Pi**4*(A0(m2**2)*(m3**2-A0(m3**2))-m3**2*Ae(m2**2)+
     -     m2**2*(A0(m3**2)-Ae(m3**2)))+
     -     2*m2**2*(-m1**2+m2**2+m3**2)*
     -     (2*Idiv1(m3**2,m2**2,m1**2)-Ifin(m3**2,m2**2,m1**2)))/
     -     (2.*(m1-m2-m3)*(m1+m2-m3)*(m1-m2+m3)*(m1+m2+m3))
     
      if(abs(m1-m2-m3).le.1d-12) write(*,*) '(m1-m2-m3)<0!!'
      if(abs(m1+m2-m3).le.1d-12) write(*,*) '(m1+m2-m3)<0!!'
      if(abs(m1+m2+m3).le.1d-12) write(*,*) '(m1+m2+m3)<0!!'
      if(abs(m1-m2+m3).le.1d-12) write(*,*) '(m1-m2+m3)<0!!'
     
      return
      end
      
      
      double complex function TFIdv19div1(m1,m2,m3)
      
      implicit none
     
      double precision m1,m2,m3
      
      double precision A0,Idiv1,Idiv2
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      TFIdv19div1=((m2**2*(m1**2-m2**2+m3**2)*Pi**4*
     -     (2*m2**2*A0(m1**2)+2*m1**2*(-m2**2+A0(m2**2))))/m1**2+
     -     (2*(m1**4-3*m1**2*m2**2-(2*m1**2+m2**2)*m3**2+m3**4)*Pi**4*
     -     (-(m3**2*A0(m1**2))+m1**2*(m3**2-A0(m3**2))))/m1**2+
     -     4*m2**2*Pi**4*(-(m3**2*A0(m2**2))+m2**2*(m3**2-A0(m3**2)))-
     -     2*m2**2*(-m1**2+m2**2+m3**2)*
     -     (Idiv1(m3**2,m2**2,m1**2)-2*Idiv2(m3**2,m2**2,m1**2)))/
     -     (2.*(m1-m2-m3)*(m1+m2-m3)*(m1-m2+m3)*(m1+m2+m3))

     
      return
      end
      
      
      double complex function TFIdv19div2(m1,m2,m3)
      
      implicit none
     
      double precision m1,m2,m3
      
      double precision Idiv2
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      TFIdv19div2=-(((-(m1**2*m2**4)+m2**6+(m1**4-3*m1**2*m2**2+m2**4)*
     -     m3**2-(2*m1**2+m2**2)*m3**4+m3**6)*Pi**4+
     -     m2**2*(-m1**2+m2**2+m3**2)*Idiv2(m3**2,m2**2,m1**2))/
     -     ((m1-m2-m3)*(m1+m2-m3)*(m1-m2+m3)*(m1+m2+m3)))

     
      return
      end
      
      
      double complex function TFIdv20fin(m1,m2,m3)
      
      implicit none
      
      double precision A0
      
      double precision m1,m2,m3
      
      double complex Ifin
      
      double precision Idiv1,Ae,Idiv2
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      TFIdv20fin=-(Pi**4*((2*m1**2*(-3*m1**6+(m2**2-m3**2)**3+m1**4*
     -     (7*m2**2+3*m3**2)+m1**2*(-5*m2**4+4*m2**2*m3**2+m3**4))+
     -     2*(m1**6-(m2**2-m3**2)**3-3*m1**4*(m2**2+m3**2)+
     -     m1**2*(3*m2**4-4*m2**2*m3**2+m3**4))*A0(m1**2))*A0(m2**2)+
     -     m2**2*(4*m1**4*(m1**4-2*m1**2*m2**2+m2**4-m3**4)-
     -     2*(3*m1**6-(m2**2-m3**2)**3-m1**4*(7*m2**2+3*m3**2)+
     -     m1**2*(5*m2**4-4*m2**2*m3**2-m3**4))*A0(m1**2)+
     -     2*(m1**6-(m2**2-m3**2)**3-3*m1**4*(m2**2+m3**2)+
     -     m1**2*(3*m2**4-4*m2**2*m3**2+m3**4))*Ae(m1**2))+
     -     2*m1**2*(m1**6-(m2**2-m3**2)**3-3*m1**4*(m2**2+m3**2)+
     -     m1**2*(3*m2**4-4*m2**2*m3**2+m3**4))*Ae(m2**2))-
     -     4*m1**2*Pi**4*(m1**4*m2**2*m3**2-m2**6*m3**2+2*m2**4*m3**4-
     -     m2**2*m3**6+m1**2*m2**2*(-m1**2+m2**2+m3**2)*A0(m3**2)+
     -     A0(m2**2)*(m1**2*m3**2*(-m1**2+m2**2+m3**2)+
     -     ((m2**2-m3**2)**2-m1**2*(m2**2+m3**2))*A0(m3**2))-
     -     m1**2*m2**2*m3**2*Ae(m2**2)+m2**4*m3**2*Ae(m2**2)-
     -     m1**2*m3**4*Ae(m2**2)-2*m2**2*m3**4*Ae(m2**2)+m3**6*
     -     Ae(m2**2)-m1**2*m2**4*Ae(m3**2)+m2**6*Ae(m3**2)-m1**2*m2**2*
     -     m3**2*Ae(m3**2)-2*m2**4*m3**2*Ae(m3**2)+m2**2*m3**4*
     -     Ae(m3**2))+Pi**4*((2*m1**2*(-3*m1**6-(m2**2-m3**2)**3+m1**4*
     -     (3*m2**2+7*m3**2)+m1**2*(m2**4+4*m2**2*m3**2-5*m3**4))+
     -     2*(m1**6+(m2**2-m3**2)**3-3*m1**4*(m2**2+m3**2)+
     -     m1**2*(m2**4-4*m2**2*m3**2+3*m3**4))*A0(m1**2))*A0(m3**2)+
     -     m3**2*(4*m1**4*(m1**4-m2**4-2*m1**2*m3**2+m3**4)-
     -     2*(3*m1**6+(m2**2-m3**2)**3-m1**4*(3*m2**2+7*m3**2)-
     -     m1**2*(m2**4+4*m2**2*m3**2-5*m3**4))*A0(m1**2)+
     -     2*(m1**6+(m2**2-m3**2)**3-3*m1**4*(m2**2+m3**2)+
     -     m1**2*(m2**4-4*m2**2*m3**2+3*m3**4))*Ae(m1**2))+
     -     2*m1**2*(m1**6+(m2**2-m3**2)**3-3*m1**4*(m2**2+m3**2)+
     -     m1**2*(m2**4-4*m2**2*m3**2+3*m3**4))*Ae(m3**2))+
     -     2*(-2*m1**8+(m2**2-m3**2)**4+3*m1**6*(m2**2+m3**2)-
     -     3*m1**2*(m2**2-m3**2)**2*(m2**2+m3**2)+
     -     m1**4*(m2**4+6*m2**2*m3**2+m3**4))*Idiv1(m3**2,m2**2,m1**2)+
     -     4*m1**2*(m1**6-m1**2*(m2**2-m3**2)**2-m1**4*(m2**2+m3**2)+
     -     (m2**2-m3**2)**2*(m2**2+m3**2))*Idiv2(m3**2,m2**2,m1**2)+
     -     2*(-(m2**2-m3**2)**4+m1**6*(m2**2+m3**2)+
     -     3*m1**2*(m2**2-m3**2)**2*(m2**2+m3**2)-3*m1**4*(m2**2+
     -     m3**2)**2)*Ifin(m3**2,m2**2,m1**2))/
     -     (2.*(m1+m2-m3)**2*(m1-m2+m3)**2*(-m1+m2+m3)**2*(m1+m2+m3)**2)
     
      return
      end
      
      
      double complex function TFIdv20div1(m1,m2,m3)
      
      implicit none
      
      double precision A0
      
      double precision m1,m2,m3
      
      double precision Idiv1, Idiv2
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      TFIdv20div1=-((-3*m1**8*m2**2*Pi**4+7*m1**6*m2**4*Pi**4-5*m1**4*m2**6*Pi**4+
     -     m1**2*m2**8*Pi**4-3*m1**8*m3**2*Pi**4+8*m1**6*m2**2*m3**2*Pi**4+
     -     3*m1**4*m2**4*m3**2*Pi**4-4*m1**2*m2**6*m3**2*Pi**4+
     -     7*m1**6*m3**4*Pi**4+3*m1**4*m2**2*m3**4*Pi**4+
     -     6*m1**2*m2**4*m3**4*Pi**4-5*m1**4*m3**6*Pi**4-
     -     4*m1**2*m2**2*m3**6*Pi**4+m1**2*m3**8*Pi**4+
     -     (-(m2**2-m3**2)**4+m1**6*(m2**2+m3**2)+
     -        3*m1**2*(m2**2-m3**2)**2*(m2**2+m3**2)-3*m1**4*(m2**2+m3**2)**2)*
     -      Pi**4*A0(m1**2)+m1**2*
     -      (m1**6-3*m1**4*(m2**2+m3**2)-(m2**2-m3**2)**2*(m2**2+m3**2)+
     -        m1**2*(3*m2**4-2*m2**2*m3**2+3*m3**4))*Pi**4*A0(m2**2)+
     -     m1**8*Pi**4*A0(m3**2)-3*m1**6*m2**2*Pi**4*A0(m3**2)+
     -     3*m1**4*m2**4*Pi**4*A0(m3**2)-m1**2*m2**6*Pi**4*A0(m3**2)-
     -     3*m1**6*m3**2*Pi**4*A0(m3**2)-2*m1**4*m2**2*m3**2*Pi**4*A0(m3**2)+
     -     m1**2*m2**4*m3**2*Pi**4*A0(m3**2)+3*m1**4*m3**4*Pi**4*A0(m3**2)+
     -     m1**2*m2**2*m3**4*Pi**4*A0(m3**2)-m1**2*m3**6*Pi**4*A0(m3**2)+
     -     m1**6*m2**2*Idiv1(m3**2,m2**2,m1**2)-
     -     3*m1**4*m2**4*Idiv1(m3**2,m2**2,m1**2)+
     -     3*m1**2*m2**6*Idiv1(m3**2,m2**2,m1**2)-m2**8*Idiv1(m3**2,m2**2,m1**2)+
     -     m1**6*m3**2*Idiv1(m3**2,m2**2,m1**2)-
     -     6*m1**4*m2**2*m3**2*Idiv1(m3**2,m2**2,m1**2)-
     -     3*m1**2*m2**4*m3**2*Idiv1(m3**2,m2**2,m1**2)+
     -     4*m2**6*m3**2*Idiv1(m3**2,m2**2,m1**2)-
     -     3*m1**4*m3**4*Idiv1(m3**2,m2**2,m1**2)-
     -     3*m1**2*m2**2*m3**4*Idiv1(m3**2,m2**2,m1**2)-
     -     6*m2**4*m3**4*Idiv1(m3**2,m2**2,m1**2)+
     -     3*m1**2*m3**6*Idiv1(m3**2,m2**2,m1**2)+
     -     4*m2**2*m3**6*Idiv1(m3**2,m2**2,m1**2)-m3**8*Idiv1(m3**2,m2**2,m1**2)-
     -     2*m1**8*Idiv2(m3**2,m2**2,m1**2)+3*m1**6*m2**2*Idiv2(m3**2,m2**2,m1**2)+
     -     m1**4*m2**4*Idiv2(m3**2,m2**2,m1**2)-
     -     3*m1**2*m2**6*Idiv2(m3**2,m2**2,m1**2)+m2**8*Idiv2(m3**2,m2**2,m1**2)+
     -     3*m1**6*m3**2*Idiv2(m3**2,m2**2,m1**2)+
     -     6*m1**4*m2**2*m3**2*Idiv2(m3**2,m2**2,m1**2)+
     -     3*m1**2*m2**4*m3**2*Idiv2(m3**2,m2**2,m1**2)-
     -     4*m2**6*m3**2*Idiv2(m3**2,m2**2,m1**2)+
     -     m1**4*m3**4*Idiv2(m3**2,m2**2,m1**2)+
     -     3*m1**2*m2**2*m3**4*Idiv2(m3**2,m2**2,m1**2)+
     -     6*m2**4*m3**4*Idiv2(m3**2,m2**2,m1**2)-
     -     3*m1**2*m3**6*Idiv2(m3**2,m2**2,m1**2)-
     -     4*m2**2*m3**6*Idiv2(m3**2,m2**2,m1**2)+m3**8*Idiv2(m3**2,m2**2,m1**2))/
     -   ((m1+m2-m3)**2*(m1-m2+m3)**2*(-m1+m2+m3)**2*(m1+m2+m3)**2))
     
      return
      end
      
      double complex function TFIdv20div2(m1,m2,m3)
      
      implicit none
      
      double precision m1,m2,m3
      
      double precision Idiv2
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      TFIdv20div2=(-(m1**2*(m1**6*(m2**2+m3**2)-3*m1**4*(m2**2+
     -      m3**2)**2-(m2**2-m3**2)**2*(m2**4+m3**4)+
     -      m1**2*(3*m2**6-m2**4*m3**2-m2**2*m3**4+3*m3**6))*Pi**4)+
     -      ((m2**2-m3**2)**4-m1**6*(m2**2+m3**2)-3*m1**2*(m2**2-
     -      m3**2)**2*(m2**2+m3**2)+3*m1**4*(m2**2+m3**2)**2)*
     -      Idiv2(m3**2,m2**2,m1**2))/
     -      ((m1+m2-m3)**2*(m1-m2+m3)**2*(-m1+m2+m3)**2*(m1+m2+m3)**2)
     
      return
      end
      
      
      
      double complex function TFIdv21fin(m1,m2,m3)
      
      implicit none
      
      double precision A0
      
      double precision m1,m2,m3
      
      double complex Ifin
      
      double precision Idiv1,Ae,Idiv2
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      TFIdv21fin=(-(m1**8*m2**2*Pi**4)+m1**6*m2**4*Pi**4+m1**4*m2**6*
     -    Pi**4-m1**2*m2**8*Pi**4-m1**6*m2**2*m3**2*Pi**4+2*m1**4*
     -    m2**4*m3**2*Pi**4-m1**2*m2**6*m3**2*Pi**4+m1**4*m2**2*m3**4*
     -    Pi**4+m1**2*m2**4*m3**4*Pi**4+m1**2*m2**2*m3**6*Pi**4-m1**8*
     -    Pi**4*A0(m3**2)+8*m1**6*m2**2*Pi**4*A0(m3**2)-14*m1**4*m2**4*
     -    Pi**4*A0(m3**2)+8*m1**2*m2**6*Pi**4*A0(m3**2)-m2**8*Pi**4*
     -    A0(m3**2)+3*m1**6*m3**2*Pi**4*A0(m3**2)-9*m1**4*m2**2*m3**2*
     -    Pi**4*A0(m3**2)-9*m1**2*m2**4*m3**2*Pi**4*A0(m3**2)+3*m2**6*
     -    m3**2*Pi**4*A0(m3**2)-3*m1**4*m3**4*Pi**4*A0(m3**2)-3*m2**4*
     -    m3**4*Pi**4*A0(m3**2)+m1**2*m3**6*Pi**4*A0(m3**2)+m2**2*
     -    m3**6*Pi**4*A0(m3**2)+Pi**4*A0(m2**2)*(m1**8+m3**2*(-m2**2+
     -    m3**2)**3-m1**6*(m2**2+4*m3**2)+m1**4*(-m2**4+m2**2*m3**2+
     -    6*m3**4)+m1**2*(m2**6+3*m2**2*m3**4-4*m3**6)+
     -    (-2*m1**6+(m2**2-m3**2)**3+3*m1**4*(m2**2+m3**2)-
     -    2*m1**2*(m2**4-m2**2*m3**2))*A0(m3**2))+
     -    Pi**4*A0(m1**2)*(m1**6*m2**2-m1**4*m2**4-m1**2*m2**6+m2**8-
     -    m1**6*m3**2+m1**2*m2**4*m3**2-4*m2**6*m3**2+3*m1**4*m3**4+3*
     -    m1**2*m2**2*m3**4+6*m2**4*m3**4-3*m1**2*m3**6-4*m2**2*m3**6+
     -    m3**8+m3**2*(3*m1**4+3*(m2**2-m3**2)**2-2*m1**2*(m2**2+
     -    3*m3**2))*A0(m2**2)+(m1**6-2*m2**6+3*m2**4*m3**2-m3**6-m1**4*
     -    (2*m2**2+3*m3**2)+m1**2*(3*m2**4+2*m2**2*m3**2+3*m3**4))*
     -    A0(m3**2))+m1**6*m3**2*Pi**4*Ae(m1**2)+m1**4*m2**2*m3**2*
     -    Pi**4*Ae(m1**2)+m1**2*m2**4*m3**2*Pi**4*Ae(m1**2)+
     -    m2**6*m3**2*Pi**4*Ae(m1**2)-3*m1**4*m3**4*Pi**4*Ae(m1**2)-
     -    4*m1**2*m2**2*m3**4*Pi**4*Ae(m1**2)-3*m2**4*m3**4*Pi**4*
     -    Ae(m1**2)+3*m1**2*m3**6*Pi**4*Ae(m1**2)+3*m2**2*m3**6*Pi**4*
     -    Ae(m1**2)-m3**8*Pi**4*Ae(m1**2)+m1**6*m3**2*Pi**4*Ae(m2**2)+
     -    m1**4*m2**2*m3**2*Pi**4*Ae(m2**2)+m1**2*m2**4*m3**2*Pi**4*
     -    Ae(m2**2)+m2**6*m3**2*Pi**4*Ae(m2**2)-3*m1**4*m3**4*Pi**4*
     -    Ae(m2**2)-4*m1**2*m2**2*m3**4*Pi**4*Ae(m2**2)-3*m2**4*m3**
     -    4*Pi**4*Ae(m2**2)+3*m1**2*m3**6*Pi**4*Ae(m2**2)+3*m2**2*
     -    m3**6*Pi**4*Ae(m2**2)-m3**8*Pi**4*Ae(m2**2)+m1**8*Pi**4*
     -    Ae(m3**2)-4*m1**6*m2**2*Pi**4*Ae(m3**2)+6*m1**4*m2**4*Pi**4*
     -    Ae(m3**2)-4*m1**2*m2**6*Pi**4*Ae(m3**2)+m2**8*Pi**4*
     -    Ae(m3**2)-3*m1**6*m3**2*Pi**4*Ae(m3**2)+5*m1**4*m2**2*m3**2*
     -    Pi**4*Ae(m3**2)+5*m1**2*m2**4*m3**2*Pi**4*Ae(m3**2)-3*m2**6*
     -    m3**2*Pi**4*Ae(m3**2)+3*m1**4*m3**4*Pi**4*Ae(m3**2)+3*m2**4*
     -    m3**4*Pi**4*Ae(m3**2)-m1**2*m3**6*Pi**4*Ae(m3**2)-m2**2*
     -    m3**6*Pi**4*Ae(m3**2)-2*m1**8*Idiv1(m3**2,m2**2,m1**2)+
     -    10*m1**6*m2**2*Idiv1(m3**2,m2**2,m1**2)-16*m1**4*m2**4*
     -    Idiv1(m3**2,m2**2,m1**2)+10*m1**2*m2**6*
     -    Idiv1(m3**2,m2**2,m1**2)-2*m2**8*Idiv1(m3**2,m2**2,m1**2)+
     -    6*m1**6*m3**2*Idiv1(m3**2,m2**2,m1**2)-
     -    10*m1**4*m2**2*m3**2*Idiv1(m3**2,m2**2,m1**2)-
     -    10*m1**2*m2**4*m3**2*Idiv1(m3**2,m2**2,m1**2)+
     -    6*m2**6*m3**2*Idiv1(m3**2,m2**2,m1**2)-
     -    6*m1**4*m3**4*Idiv1(m3**2,m2**2,m1**2)-
     -    2*m1**2*m2**2*m3**4*Idiv1(m3**2,m2**2,m1**2)-
     -    6*m2**4*m3**4*Idiv1(m3**2,m2**2,m1**2)+
     -    2*m1**2*m3**6*Idiv1(m3**2,m2**2,m1**2)+
     -    2*m2**2*m3**6*Idiv1(m3**2,m2**2,m1**2)-
     -    4*m1**6*m2**2*Idiv2(m3**2,m2**2,m1**2)+
     -    8*m1**4*m2**4*Idiv2(m3**2,m2**2,m1**2)-
     -    4*m1**2*m2**6*Idiv2(m3**2,m2**2,m1**2)+
     -    4*m1**2*m2**2*m3**4*Idiv2(m3**2,m2**2,m1**2)+
     -    2*m1**8*Ifin(m3**2,m2**2,m1**2)-
     -    8*m1**6*m2**2*Ifin(m3**2,m2**2,m1**2)+
     -    12*m1**4*m2**4*Ifin(m3**2,m2**2,m1**2)-
     -    8*m1**2*m2**6*Ifin(m3**2,m2**2,m1**2)+
     -    2*m2**8*Ifin(m3**2,m2**2,m1**2)-
     -    7*m1**6*m3**2*Ifin(m3**2,m2**2,m1**2)+
     -    9*m1**4*m2**2*m3**2*Ifin(m3**2,m2**2,m1**2)+
     -    9*m1**2*m2**4*m3**2*Ifin(m3**2,m2**2,m1**2)-
     -    7*m2**6*m3**2*Ifin(m3**2,m2**2,m1**2)+
     -    9*m1**4*m3**4*Ifin(m3**2,m2**2,m1**2)+
     -    4*m1**2*m2**2*m3**4*Ifin(m3**2,m2**2,m1**2)+
     -    9*m2**4*m3**4*Ifin(m3**2,m2**2,m1**2)-
     -    5*m1**2*m3**6*Ifin(m3**2,m2**2,m1**2)-
     -    5*m2**2*m3**6*Ifin(m3**2,m2**2,m1**2)+
     -    m3**8*Ifin(m3**2,m2**2,m1**2))/
     -    ((m1+m2-m3)**2*(m1-m2+m3)**2*(-m1+m2+m3)**2*(m1+m2+m3)**2)
      
      return
      end
      
      
      double complex function TFIdv21div1(m1,m2,m3)
      
      implicit none
      
      double precision A0
      
      double precision m1,m2,m3
      
      double precision Idiv1, Idiv2
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      TFIdv21div1=(m1**8*m2**2*Pi**4-m1**6*m2**4*Pi**4-m1**4*m2**6*
     -    Pi**4+m1**2*m2**8*Pi**4-m1**8*m3**2*Pi**4+2*m1**6*m2**2*
     -    m3**2*Pi**4-6*m1**4*m2**4*m3**2*Pi**4+2*m1**2*m2**6*m3**2*
     -    Pi**4-m2**8*m3**2*Pi**4+3*m1**6*m3**4*Pi**4+3*m2**6*m3**4*
     -    Pi**4-3*m1**4*m3**6*Pi**4-4*m1**2*m2**2*m3**6*Pi**4-3*m2**4*
     -    m3**6*Pi**4+m1**2*m3**8*Pi**4+m2**2*m3**8*Pi**4+m3**2*
     -    (m1**6+m1**4*(m2**2-3*m3**2)+(m2**2-m3**2)**3+
     -    m1**2*(m2**4-4*m2**2*m3**2+3*m3**4))*Pi**4*A0(m1**2)+
     -    m3**2*(m1**6+m1**4*(m2**2-3*m3**2)+(m2**2-m3**2)**3+
     -    m1**2*(m2**4-4*m2**2*m3**2+3*m3**4))*Pi**4*A0(m2**2)+
     -    m1**8*Pi**4*A0(m3**2)-4*m1**6*m2**2*Pi**4*A0(m3**2)+
     -    6*m1**4*m2**4*Pi**4*A0(m3**2)-4*m1**2*m2**6*Pi**4*A0(m3**2)+
     -    m2**8*Pi**4*A0(m3**2)-3*m1**6*m3**2*Pi**4*A0(m3**2)+
     -    5*m1**4*m2**2*m3**2*Pi**4*A0(m3**2)+5*m1**2*m2**4*m3**2*
     -    Pi**4*A0(m3**2)-3*m2**6*m3**2*Pi**4*A0(m3**2)+3*m1**4*m3**4*
     -    Pi**4*A0(m3**2)+3*m2**4*m3**4*Pi**4*A0(m3**2)-m1**2*m3**6*
     -    Pi**4*A0(m3**2)-m2**2*m3**6*Pi**4*A0(m3**2)+2*m1**8*
     -    Idiv1(m3**2,m2**2,m1**2)-8*m1**6*m2**2*
     -    Idiv1(m3**2,m2**2,m1**2)+12*m1**4*m2**4*
     -    Idiv1(m3**2,m2**2,m1**2)-8*m1**2*m2**6*
     -    Idiv1(m3**2,m2**2,m1**2)+2*m2**8*Idiv1(m3**2,m2**2,m1**2)-
     -    7*m1**6*m3**2*Idiv1(m3**2,m2**2,m1**2)+
     -    9*m1**4*m2**2*m3**2*Idiv1(m3**2,m2**2,m1**2)+
     -    9*m1**2*m2**4*m3**2*Idiv1(m3**2,m2**2,m1**2)-
     -    7*m2**6*m3**2*Idiv1(m3**2,m2**2,m1**2)+
     -    9*m1**4*m3**4*Idiv1(m3**2,m2**2,m1**2)+
     -    4*m1**2*m2**2*m3**4*Idiv1(m3**2,m2**2,m1**2)+
     -    9*m2**4*m3**4*Idiv1(m3**2,m2**2,m1**2)-
     -    5*m1**2*m3**6*Idiv1(m3**2,m2**2,m1**2)-
     -    5*m2**2*m3**6*Idiv1(m3**2,m2**2,m1**2)+
     -    m3**8*Idiv1(m3**2,m2**2,m1**2)-
     -    2*m1**8*Idiv2(m3**2,m2**2,m1**2)+
     -    10*m1**6*m2**2*Idiv2(m3**2,m2**2,m1**2)-
     -    16*m1**4*m2**4*Idiv2(m3**2,m2**2,m1**2)+
     -    10*m1**2*m2**6*Idiv2(m3**2,m2**2,m1**2)-
     -    2*m2**8*Idiv2(m3**2,m2**2,m1**2)+
     -    6*m1**6*m3**2*Idiv2(m3**2,m2**2,m1**2)-
     -    10*m1**4*m2**2*m3**2*Idiv2(m3**2,m2**2,m1**2)-
     -    10*m1**2*m2**4*m3**2*Idiv2(m3**2,m2**2,m1**2)+
     -    6*m2**6*m3**2*Idiv2(m3**2,m2**2,m1**2)-
     -    6*m1**4*m3**4*Idiv2(m3**2,m2**2,m1**2)-
     -    2*m1**2*m2**2*m3**4*Idiv2(m3**2,m2**2,m1**2)-
     -    6*m2**4*m3**4*Idiv2(m3**2,m2**2,m1**2)+
     -    2*m1**2*m3**6*Idiv2(m3**2,m2**2,m1**2)+
     -    2*m2**2*m3**6*Idiv2(m3**2,m2**2,m1**2))/
     -    ((m1+m2-m3)**2*(m1-m2+m3)**2*(-m1+m2+m3)**2*(m1+m2+m3)**2)
     
      return
      end
      
      double complex function TFIdv21div2(m1,m2,m3)
      
      implicit none
      
      double precision m1,m2,m3
      
      double precision Idiv2
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      TFIdv21div2=(-(m3**2*(-m1**8-m2**2*(m2**2-m3**2)**3+m1**6*(m2**2+
     -     3*m3**2)+ m1**4*(-4*m2**4+m2**2*m3**2-3*m3**4)+
     -     m1**2*(m2**6+m2**4*m3**2-3*m2**2*m3**4+m3**6))*Pi**4)+
     -     (2*m1**8+(m2**2-m3**2)**3*(2*m2**2-m3**2)-m1**6*(8*m2**2+
     -     7*m3**2)+3*m1**4*(4*m2**4+3*m2**2*m3**2+3*m3**4)+
     -     m1**2*(-8*m2**6+9*m2**4*m3**2+4*m2**2*m3**4-5*m3**6))*
     -    Idiv2(m3**2,m2**2,m1**2))/((m1+m2-m3)**2*(m1-m2+m3)**2*
     -    (-m1+m2+m3)**2*(m1+m2+m3)**2)
      return
      end
      
      
      
      double complex function TFIdv22fin(m1,m2,m3)
      
      implicit none
      
      double precision A0
      
      double precision m1,m2,m3
      
      double complex Ifin
      
      double precision Idiv1,Ae,Idiv2
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      TFIdv22fin=(-(m1**8*m2**2*Pi**4)+m1**6*m2**4*Pi**4+m1**4*m2**6*
     -    Pi**4-m1**2*m2**8*Pi**4-m1**6*m2**2*m3**2*Pi**4+2*m1**4*
     -    m2**4*m3**2*Pi**4-m1**2*m2**6*m3**2*Pi**4+m1**4*m2**2*m3**4*
     -    Pi**4+m1**2*m2**4*m3**4*Pi**4+m1**2*m2**2*m3**6*Pi**4-m1**8*
     -    Pi**4*A0(m3**2)+7*m1**6*m2**2*Pi**4*A0(m3**2)-11*m1**4*m2**4*
     -    Pi**4*A0(m3**2)+5*m1**2*m2**6*Pi**4*A0(m3**2)+3*m1**6*m3**2*
     -    Pi**4*A0(m3**2)-8*m1**4*m2**2*m3**2*Pi**4*A0(m3**2)-7*m1**2*
     -    m2**4*m3**2*Pi**4*A0(m3**2)-3*m1**4*m3**4*Pi**4*A0(m3**2)+
     -    m1**2*m2**2*m3**4*Pi**4*A0(m3**2)+m1**2*m3**6*Pi**4*
     -    A0(m3**2)+m1**2*Pi**4*A0(m2**2)*(2*m2**2*
     -    (m1**4+m2**4-m2**2*m3**2-m1**2*(2*m2**2+m3**2))-(3*m1**4+
     -    m2**4-4*m2**2*m3**2+3*m3**4-2*m1**2*(2*m2**2+3*m3**2))*
     -    A0(m3**2))+Pi**4*A0(m1**2)*(2*m1**4*m2**4-4*m1**2*m2**6+2*
     -    m2**8-m1**6*m3**2+m1**4*m2**2*m3**2+
     -    3*m1**2*m2**4*m3**2-7*m2**6*m3**2+3*m1**4*m3**4+4*m1**2*
     -    m2**2*m3**4+9*m2**4*m3**4-3*m1**2*m3**6-5*m2**2*m3**6+
     -    m3**8+(m1**6-3*m1**4*m2**2-(m2**2-2*m3**2)*(m2**2-m3**2)**2+
     -    3*m1**2*(m2**4-m3**4))*A0(m2**2)+(m1**6-3*m1**4*m3**2-
     -    (-(m2**2*m3)+m3**3)**2-m1**2*(m2**4+2*m2**2*m3**2-3*m3**4))*
     -    A0(m3**2))+m1**6*m2**2*Pi**4*Ae(m1**2)-3*m1**4*m2**4*Pi**4*
     -    Ae(m1**2)+3*m1**2*m2**6*Pi**4*Ae(m1**2)-m2**8*Pi**4*
     -    Ae(m1**2)+m1**6*m3**2*Pi**4*Ae(m1**2)-m1**2*m2**4*m3**2*
     -    Pi**4*Ae(m1**2)+4*m2**6*m3**2*Pi**4*Ae(m1**2)-3*m1**4*m3**4*
     -    Pi**4*Ae(m1**2)-5*m1**2*m2**2*m3**4*Pi**4*Ae(m1**2)-6*m2**4*
     -    m3**4*Pi**4*Ae(m1**2)+3*m1**2*m3**6*Pi**4*Ae(m1**2)+4*m2**2*
     -    m3**6*Pi**4*Ae(m1**2)-m3**8*Pi**4*Ae(m1**2)+m1**8*Pi**4*
     -    Ae(m2**2)-3*m1**6*m2**2*Pi**4*Ae(m2**2)+3*m1**4*m2**4*Pi**4*
     -    Ae(m2**2)-m1**2*m2**6*Pi**4*Ae(m2**2)-3*m1**6*m3**2*Pi**4*
     -    Ae(m2**2)+4*m1**4*m2**2*m3**2*Pi**4*Ae(m2**2)+3*m1**2*m2**4*
     -    m3**2*Pi**4*Ae(m2**2)+3*m1**4*m3**4*Pi**4*Ae(m2**2)-m1**2*
     -    m2**2*m3**4*Pi**4*Ae(m2**2)-m1**2*m3**6*Pi**4*Ae(m2**2)+
     -    m1**8*Pi**4*Ae(m3**2)-3*m1**6*m2**2*Pi**4*Ae(m3**2)+
     -    3*m1**4*m2**4*Pi**4*Ae(m3**2)-m1**2*m2**6*Pi**4*Ae(m3**2)-
     -    3*m1**6*m3**2*Pi**4*Ae(m3**2)+4*m1**4*m2**2*m3**2*Pi**4*
     -    Ae(m3**2)+3*m1**2*m2**4*m3**2*Pi**4*Ae(m3**2)+3*m1**4*m3**4*
     -    Pi**4*Ae(m3**2)-m1**2*m2**2*m3**4*Pi**4*Ae(m3**2)-
     -    m1**2*m3**6*Pi**4*Ae(m3**2)-2*m1**8*Idiv1(m3**2,m2**2,m1**2)+
     -    8*m1**6*m2**2*Idiv1(m3**2,m2**2,m1**2)-
     -    10*m1**4*m2**4*Idiv1(m3**2,m2**2,m1**2)+
     -    4*m1**2*m2**6*Idiv1(m3**2,m2**2,m1**2)+
     -    6*m1**6*m3**2*Idiv1(m3**2,m2**2,m1**2)-
     -    8*m1**4*m2**2*m3**2*Idiv1(m3**2,m2**2,m1**2)-
     -    6*m1**2*m2**4*m3**2*Idiv1(m3**2,m2**2,m1**2)-
     -    6*m1**4*m3**4*Idiv1(m3**2,m2**2,m1**2)+
     -    2*m1**2*m3**6*Idiv1(m3**2,m2**2,m1**2)-
     -    4*m1**6*m2**2*Idiv2(m3**2,m2**2,m1**2)+
     -    8*m1**4*m2**4*Idiv2(m3**2,m2**2,m1**2)-
     -    4*m1**2*m2**6*Idiv2(m3**2,m2**2,m1**2)+
     -    4*m1**2*m2**2*m3**4*Idiv2(m3**2,m2**2,m1**2)+
     -    m1**8*Ifin(m3**2,m2**2,m1**2)-
     -    3*m1**6*m2**2*Ifin(m3**2,m2**2,m1**2)+
     -    3*m1**4*m2**4*Ifin(m3**2,m2**2,m1**2)-
     -    m1**2*m2**6*Ifin(m3**2,m2**2,m1**2)-
     -    3*m1**6*m3**2*Ifin(m3**2,m2**2,m1**2)+
     -    4*m1**4*m2**2*m3**2*Ifin(m3**2,m2**2,m1**2)+
     -    3*m1**2*m2**4*m3**2*Ifin(m3**2,m2**2,m1**2)+
     -    3*m1**4*m3**4*Ifin(m3**2,m2**2,m1**2)-
     -    m1**2*m2**2*m3**4*Ifin(m3**2,m2**2,m1**2)-
     -    m1**2*m3**6*Ifin(m3**2,m2**2,m1**2))/(m1**2*(m1+m2-m3)**2*
     -    (m1-m2+m3)**2*(-m1+m2+m3)**2*(m1+m2+m3)**2)
      
      return
      end
      
      
      double complex function TFIdv22div1(m1,m2,m3)
      
      implicit none
      
      double precision A0
      
      double precision m1,m2,m3
      
      double precision Idiv1, Idiv2
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      TFIdv22div1=((-(m2**2-m3**2)**4+m1**6*(m2**2+m3**2)-3*m1**4*
     -     (m2**4+m3**4)+m1**2*(3*m2**6-m2**4*m3**2-5*m2**2*m3**4+
     -     3*m3**6))*Pi**4*A0(m1**2)-m1**2*(-2*m1**4*m2**4*Pi**4+
     -     4*m1**2*m2**6*Pi**4-2*m2**8*Pi**4+
     -     m1**6*m3**2*Pi**4-4*m1**4*m2**2*m3**2*Pi**4+5*m1**2*m2**4*
     -     m3**2*Pi**4+2*m2**6*m3**2*Pi**4-3*m1**4*m3**4*Pi**4+
     -     2*m1**2*m2**2*m3**4*Pi**4-m2**4*m3**4*Pi**4+3*m1**2*m3**6*
     -     Pi**4+2*m2**2*m3**6*Pi**4-m3**8*Pi**4+
     -     (-m1**6+m2**6-3*m2**4*m3**2+m2**2*m3**4+m3**6+
     -     3*m1**4*(m2**2+m3**2)-m1**2*(3*m2**4+4*m2**2*m3**2+3*
     -     m3**4))*Pi**4*A0(m2**2)+(-m1**6+m2**6-3*m2**4*m3**2+m2**2*
     -     m3**4+m3**6+3*m1**4*(m2**2+m3**2)-m1**2*(3*m2**4+4*m2**2*
     -     m3**2+3*m3**4))*Pi**4*A0(m3**2)-
     -     m1**6*Idiv1(m3**2,m2**2,m1**2)+
     -     3*m1**4*m2**2*Idiv1(m3**2,m2**2,m1**2)-
     -     3*m1**2*m2**4*Idiv1(m3**2,m2**2,m1**2)+
     -     m2**6*Idiv1(m3**2,m2**2,m1**2)+
     -     3*m1**4*m3**2*Idiv1(m3**2,m2**2,m1**2)-
     -     4*m1**2*m2**2*m3**2*Idiv1(m3**2,m2**2,m1**2)-
     -     3*m2**4*m3**2*Idiv1(m3**2,m2**2,m1**2)-
     -     3*m1**2*m3**4*Idiv1(m3**2,m2**2,m1**2)+
     -     m2**2*m3**4*Idiv1(m3**2,m2**2,m1**2)+
     -     m3**6*Idiv1(m3**2,m2**2,m1**2)+
     -     2*m1**6*Idiv2(m3**2,m2**2,m1**2)-
     -     8*m1**4*m2**2*Idiv2(m3**2,m2**2,m1**2)+
     -     10*m1**2*m2**4*Idiv2(m3**2,m2**2,m1**2)-
     -     4*m2**6*Idiv2(m3**2,m2**2,m1**2)-
     -     6*m1**4*m3**2*Idiv2(m3**2,m2**2,m1**2)+
     -     8*m1**2*m2**2*m3**2*Idiv2(m3**2,m2**2,m1**2)+
     -     6*m2**4*m3**2*Idiv2(m3**2,m2**2,m1**2)+
     -     6*m1**2*m3**4*Idiv2(m3**2,m2**2,m1**2)-
     -     2*m3**6*Idiv2(m3**2,m2**2,m1**2)))/(m1**2*(m1+m2-m3)**2*
     -     (m1-m2+m3)**2*(-m1+m2+m3)**2*(m1+m2+m3)**2)
     
      return
      end
      
      
      double complex function TFIdv22div2(m1,m2,m3)
      
      implicit none
      
      double precision m1,m2,m3
      
      double precision Idiv2
      
      double precision pi
      parameter (pi = 2d0*Asin(1d0))
      
      TFIdv22div2=((-m2**8+3*m2**6*m3**2-2*m2**4*m3**4+m2**2*m3**6-
     -    m3**8+m1**6*(m2**2+m3**2)-3*m1**4*(m2**4+m2**2*m3**2+m3**4)+
     -    m1**2*(3*m2**6+3*m2**4*m3**2+m2**2*m3**4+3*m3**6))*Pi**4+
     -    (m1**6-m2**6+3*m2**4*m3**2-m2**2*m3**4-m3**6-
     -    3*m1**4*(m2**2+m3**2)+m1**2*(3*m2**4+4*m2**2*m3**2+3*m3**4))*
     -    Idiv2(m3**2,m2**2,m1**2))/((m1+m2-m3)**2*(m1-m2+m3)**2*
     -    (-m1+m2+m3)**2*(m1+m2+m3)**2)
     
      return
      end

      
c------------------------------------------------------------------------------------------------c
      
     
